/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.apis;

import java.io.Writer;
import java.util.List;

import org.tencompetence.business.constants.EventType;
import org.tencompetence.business.facade.AssessmentTestFacade;
import org.tencompetence.business.services.PortfolioService;
import org.tencompetence.generated.model.AssessmentTest;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.services.xml.qti.constants.QTIConstants;

import adapters.QtiParser;
import apis.QTIv21test;
import apis.UserTestCase;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.PropsUtil;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.service.ServiceContext;

/**
 * The Class TENCQtiParser. It need the properties tenc.apis.* in portal-ext.properties.
 * 
 * @version $Revision: 1.15 $, $Date: 2009/11/27 16:53:23 $
 */
public class TENCQtiParser extends QtiParser {
	private static final Log LOG = LogFactoryUtil.getLog(TENCQtiParser.class);

	// Database settings
	private static final String TENC_APIS_PREFIX = "tenc.apis.";
	private static final String DRIVER_CLASS_KEY = "driver.class";
	private static final String DATABASE_URL_KEY = "database.url";
	private static final String DATABASE_USERNAME_KEY = "database.username";
	private static final String DATABASE_PASSWORD_KEY = "database.password";

	private static final String EQF_LEVEL_0 = "0";

	private Long userId;
	private Long companyId;
	private Long groupId;

	/**
	 * Instantiates a new TENCQtiParser.
	 * 
	 * @param userId
	 *            the user id
	 * @param runId
	 *            the run id
	 * @param theWriter
	 *            the the writer
	 * @param sourceUrl
	 *            the source url
	 * @param postStr
	 *            the post str
	 * @param language
	 *            the language
	 */
	public TENCQtiParser(final String userId, final int runId, final Writer theWriter, final String sourceUrl,
			final String postStr, final String language) {

		super(userId, runId, theWriter, sourceUrl, postStr, language);

		this.userId = Long.valueOf(userId);
	}

	/**
	 * handleCompletedTest. Out implementation of Spain's code. We need this version to extract the EQF value
	 * (finalScore). Note: if the test somehow fails, the EQF value is set to zero.
	 * 
	 * @param test
	 *            QTIv21test
	 */
	protected void handleCompletedTest(final QTIv21test test) {
		final ServiceContext ctx = new ServiceContext();
		final UserTestCase user = test.getUser();
		String finalScore = EQF_LEVEL_0;

		final List<QTIv21test.Result> results = test.getPlainOutcomeVars();
		for (final QTIv21test.Result result : results) {
			if (result.getKey().equals(QTIConstants.EQF)) {
				final String tempScore = (String) result.getValue();
				finalScore = Validator.isNotNull(tempScore) ? tempScore : EQF_LEVEL_0;
				break;
			}
		}

		ctx.setUserId(userId);
		ctx.setScopeGroupId(groupId);
		ctx.setCompanyId(companyId);

		try {
			final AssessmentTest assessTest = AssessmentTestFacade.getAssessmentTestByIdentifier(user.getTest(), null);
			PortfolioService.addJournalEntry(assessTest, EventType.ATTAINED_TEST, finalScore, ctx);
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	/**
	 * setDBProperties. Properties are read from from portal-ext.properties.
	 * 
	 * @param theTest
	 *            QTIv21test
	 */
	public void setDBProperties(final QTIv21test theTest) {
		try {
			final String driverClassValue = PropsUtil.get(TENC_APIS_PREFIX + DRIVER_CLASS_KEY);
			final String databaseUrlValue = PropsUtil.get(TENC_APIS_PREFIX + DATABASE_URL_KEY);
			final String databaseUsernameValue = PropsUtil.get(TENC_APIS_PREFIX + DATABASE_USERNAME_KEY);
			final String databasePasswordValue = PropsUtil.get(TENC_APIS_PREFIX + DATABASE_PASSWORD_KEY);
			theTest.linkTheTestWithDb(driverClassValue, databaseUrlValue, databaseUsernameValue, databasePasswordValue);
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	/**
	 * setCompanyId.
	 * 
	 * @param companyId
	 *            Long
	 */
	public void setCompanyId(final Long companyId) {
		this.companyId = companyId;
	}

	/**
	 * setGroupId.
	 * 
	 * @param groupId
	 *            Long
	 */
	public void setGroupId(final Long groupId) {
		this.groupId = groupId;
	}
}