/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.assessment.beans;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.faces.context.FacesContext;

import org.tencompetence.business.facade.CompetenceLevelFacade;
import org.tencompetence.business.facade.CompetenceProfileFacade;
import org.tencompetence.business.facade.CompetenceProfileLevelFacade;
import org.tencompetence.business.facade.LevelValueFacade;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.CompetenceProfile;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.generated.model.LevelValue;
import org.tencompetence.portlet.assessment.beans.controller.CompetenceController;
import org.tencompetence.portlet.assessment.beans.controller.CompetenceWrapper;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.goal.beans.GoalListener;

import com.icesoft.faces.component.ext.RowSelectorEvent;
import com.icesoft.faces.context.effects.JavascriptContext;
//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.service.ServiceContext;

/**
 * This is the managed bean for the assessment/assessmentFullPage.jspx.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * 
 * @version $Revision: 1.58 $, $Date: 2009/12/01 09:37:45 $
 */
public class AssessmentFullPage extends GoalListener {

	private static final Log LOG = LogFactoryUtil.getLog(AssessmentFullPage.class);

	private ServiceContext ctx;
	private List<CompetenceWrapper> leafCompetences;
	private CompetenceWrapper selectedCompetence;
	private CompetenceController competenceController;


	/**
	 * Instantiates a new assessment full page.
	 */
	public AssessmentFullPage() {
		// make sure the view gets initialized
		setNeedsInit(true);

		// Add needed js libraries.
		JavascriptContext.includeLib("/js/ratingComponent.js", FacesContext.getCurrentInstance());

		//PortletUtils.setObjectToSession(AssessmentPhaseListener.SET_VIEW, true);

	}

	/**
	 * Gets the List containing the child competence levels.
	 * 
	 * @return the List containing the child competence levels
	 */
	public List<CompetenceWrapper> getChildren() {
		initFields();
		return leafCompetences;
	}

	/**
	 * Gets the competence controller.
	 * 
	 * @return the competence controller
	 */
	public CompetenceController getCompetenceController() {
		initFields();
		return competenceController;
	}
	
	/**
	 * Checks if goals targeted profile has any competence levels associated.
	 * 
	 * @return true, if the goal has no competence levels
	 */
	public boolean isEmptyProfile() {
		initFields();
		return leafCompetences.isEmpty();
	}

	/**
	 * Checks if an element in the list selected.
	 * 
	 * @return true, if an element selected
	 */
	public boolean isElementSelected() {
		initFields();
		return selectedCompetence != null;
	}

	/**
	 * Selection listener called when the selection in the profile level list changes.
	 * 
	 * @param event
	 *            the event describing the selection
	 */
	public void onSelectionChanged(final RowSelectorEvent event) {
		initFields();
		handleSelection(leafCompetences.get(event.getRow()));
	}

	/** {@inheritDoc} */
	@Override
	protected void initFields() {
	
		super.initFields();
	
		if (getNeedsInit()) {
			setNeedsInit(false);
			
			// reset bean
			selectedCompetence = null;
			competenceController = null;
			leafCompetences = new ArrayList<CompetenceWrapper>();
			
			try {
				final Goal goal = getSelectedGoal();
				ctx = PortletUtils.getServiceContext();
	
				if (goal == null) {
					return;
				}
				
				List<CompetenceLevel> levels = null;
	
				final CompetenceProfileLevel profileLevel = CompetenceProfileLevelFacade.getCompetenceProfileLevel(goal
						.getTarget(), ctx);
				final List<LevelValue> values = getLevelValues(profileLevel);
				levels = CompetenceLevelFacade.getHighestLeafCompetenceLevelsByCompetenceProfileId(profileLevel
						.getPrimaryKey(), ctx);
				competenceController = new CompetenceController(values, ctx);
				buildWrappers(levels, values);
				if (!leafCompetences.isEmpty()) {
					handleSelection(leafCompetences.get(0));
				}
	
			} catch (Exception e) {
				PortletUtils.handleError(LOG, e);
			}
		}
	
	}

	private void handleSelection(final CompetenceWrapper competence) {
		selectedCompetence = competence;

		// reset selected state of list items to resemble normal selection list
		for (CompetenceWrapper entry : leafCompetences) {
			entry.setSelected(entry.equals(competence));
		}

		competenceController.setInput(competence);
	}

	/**
	 * Builds the wrappers for the child competences items.
	 * 
	 * @param levels
	 *            the competence levels that should be put on the list
	 * @param values
	 *            the level values of the current competence map
	 */
	private void buildWrappers(final List<CompetenceLevel> levels, final List<LevelValue> values) {
		try {
			final ServiceContext ctx = PortletUtils.getServiceContext();
			leafCompetences = new ArrayList<CompetenceWrapper>();

			if (Validator.isNull(levels)) {
				return;
			}

			for (final CompetenceLevel childLevel : levels) {
				leafCompetences.add(new CompetenceWrapper(childLevel, values, ctx));
			}

		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	private List<LevelValue> getLevelValues(final CompetenceProfileLevel profileLevel) {
		List<LevelValue> values = Collections.emptyList();
		long competenceMapId = -1;

		try {

			CompetenceProfile profile = CompetenceProfileFacade.getCompetenceProfile(profileLevel
					.getCompetenceProfileId(), ctx);
			// get the map id from profile
			competenceMapId = profile.getCompetenceMapId();

			if (competenceMapId != -1) {
				values = LevelValueFacade.getAttainableCompetenceLevelValues(competenceMapId, ctx);
			}

		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}
		return values;
	}

}
