/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.assessment.beans.controller;

import java.util.List;

import org.tencompetence.business.services.PdpService;
import org.tencompetence.business.services.PortfolioService;
import org.tencompetence.generated.model.LevelValue;
import org.tencompetence.portlet.common.components.AssessmentPopupController;
import org.tencompetence.portlet.common.components.RatingComponentController;
import org.tencompetence.portlet.common.utils.PortletUtils;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * The Class CompetenceLevelWrapper represents a row in the list of competence levels.
 * 
 * @version $Revision: 1.5 $, $Date: 2009/11/24 14:10:06 $
 */
public class CompetenceController {
	private static final int MAX_EQF_LEVEL = 8;
	private static final Log LOG = LogFactoryUtil.getLog(CompetenceController.class);

	private CompetenceWrapper competence;

	private RatingComponentController controller;
	private AssessmentPopupController assessmentPopupController;

	private final ServiceContext ctx;
	private final List<LevelValue> values;

	/**
	 * Instantiates a new competence controller.
	 * 
	 * @param values
	 *            the values
	 * @param ctx
	 *            the context
	 */
	public CompetenceController(final List<LevelValue> values, final ServiceContext ctx) {
		this.values = values;
		this.ctx = ctx;
	}

	/**
	 * Sets the input.
	 * 
	 * @param competence
	 *            the new input
	 */
	public void setInput(final CompetenceWrapper competence) {
		this.competence = competence;
		try {
			createController(competence.getAttained());
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	/**
	 * Gets the title.
	 * 
	 * @return the title
	 */
	public String getTitle() {
		return competence != null ? competence.getTitle() : "";
	}

	/**
	 * Checks if the editor input has an associated assessment.
	 * 
	 * @return true, if successful
	 */
	public boolean isAssessmentAvailable() {
		return competence != null ? competence.hasAssessment() : false;
	}

	/**
	 * Gets the rating controller.
	 * 
	 * @return the rating controller
	 */
	public RatingComponentController getRatingController() {
		return controller;
	}

	/**
	 * Gets the assessment popup controller.
	 * 
	 * @return the assessment popup controller
	 */
	public AssessmentPopupController getAssessmentPopupController() {
		if (this.assessmentPopupController == null) {
			try {
				this.assessmentPopupController = new AssessmentPopupController() {

					@Override
					public void handleOk() {
						processAssessmentResult();
					}

					@Override
					public String getAssessmentId() {
						return competence != null ? competence.getAssessmentIdentifier() : null;
					}
				};
			} catch (Exception e) {
				PortletUtils.handleError(LOG, e);
			}
		}
		return this.assessmentPopupController;
	}

	/**
	 * Shows the assessment dialog.
	 * 
	 * @return null as we do not navigate off this page
	 */
	public String showAssessment() {
		getAssessmentPopupController().setVisible(true);
		return null;
	}

	private void saveAttainedLevel(final LevelValue levelValue) {
		try {
			PdpService.setAttainedLevelForUser(competence.getCompetence(), levelValue, ctx.getUserId(), ctx);
			competence.setAttained(levelValue != null ? levelValue.getValue() : null);
			createController(levelValue);

		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	private void createController(final LevelValue levelValue) throws Exception {
		this.controller = new RatingComponentController(values, levelValue, getLevelValueId(competence
				.getRequiredLevel()), true, true, true) {

			/** {@inheritDoc} */
			@Override
			public void ratingComponentClicked() {
				saveAttainedLevel(getAssignedLevel());
			}
		};

	}

	private void createController(final Integer value) throws Exception {
		LevelValue level = value == null ? null : getLevelValueByOrdinal(value);
		createController(level);
	}

	private void processAssessmentResult() {
		if (competence != null) {
			try {
				Integer eqf = PortfolioService.getTestResultForUser(competence.getAssessmentId(), ctx.getUserId(), ctx);

				if (eqf != null && eqf >= 0 && eqf <= MAX_EQF_LEVEL) {
					saveAttainedLevel(convertEQFToLevelValue(eqf));
				}
			} catch (Exception e) {
				PortletUtils.handleError(LOG, e);
			}
		}
	}

	private LevelValue convertEQFToLevelValue(final int eqf) throws Exception {
		int value = Math.round(((values.size() - 1) / (MAX_EQF_LEVEL * 1.0f)) * eqf);

		return getLevelValueByOrdinal(value);
	}

	private LevelValue getLevelValueByOrdinal(final int value) throws Exception {
		for (LevelValue lv : values) {
			if (lv.getValue() == value) {
				return lv;
			}
		}
		throw new Exception("Level value not found");
	}

	private long getLevelValueId(final int value) throws Exception {
		return getLevelValueByOrdinal(value).getLevelValueId();
	}

}
