/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.assessment.beans.controller;

import java.util.List;

import org.tencompetence.business.facade.AssessmentTestFacade;
import org.tencompetence.business.facade.CompetenceFacade;
import org.tencompetence.business.facade.LevelValueFacade;
import org.tencompetence.business.services.PdpService;
import org.tencompetence.generated.model.AssessmentTest;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.LevelValue;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.PortletUtils;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * The Class CompetenceLevelWrapper represents a row in the list of competence levels.
 * 
 * @version $Revision: 1.4 $, $Date: 2009/11/25 07:08:05 $
 */
public class CompetenceWrapper  {
	private static final Log LOG = LogFactoryUtil.getLog(CompetenceWrapper.class);

	private ServiceContext ctx;
	private CompetenceLevel competenceLevel;
	private Competence parentCompetence;

	private boolean selected;
	
	private final Integer required;
	private Integer attained;
	
	private AssessmentTest assessment;

	/**
	 * Instantiates a new competence level wrapper.
	 * 
	 * @param competenceLevel
	 *            the competence level
	 * @param values
	 *            the values
	 * @param ctx
	 *            the ctx
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public CompetenceWrapper(final CompetenceLevel competenceLevel, final List<LevelValue> values,
			final ServiceContext ctx) throws PortalException, SystemException {
		this.competenceLevel = competenceLevel;
		this.ctx = ctx;
		this.required = LevelValueFacade.getLevelValue(competenceLevel.getLevelValueId(), ctx).getValue();
		this.parentCompetence = CompetenceFacade.getCompetence(competenceLevel.getCompetenceId(), ctx);

		final LevelValue attainedLevel = PdpService.getAttainedLevelForUser(ctx.getUserId(), parentCompetence, ctx);
		this.attained = attainedLevel != null ? attainedLevel.getValue() : null;
	}

	/**
	 * Gets the selected.
	 * 
	 * @return the selected
	 */
	public boolean getSelected() {
		return selected;
	}

	/**
	 * Gets the title.
	 * 
	 * @return the title
	 */
	public String getTitle() {
		return parentCompetence.getTitle();
	}
	
	/**
	 * Gets the required level for this competence.
	 * 
	 * @return the required level
	 */
	public int getRequiredLevel() {
		return required;
	}
	
	/**
	 * Gets the competence from this wrapper.
	 * 
	 * @return the competence
	 */
	public Competence getCompetence() {
		return parentCompetence;
	}
	
	/**
	 * Gets the attained.
	 * 
	 * @return the attained
	 */
	public Integer getAttained() {
		return attained;
	}
	

	/**
	 * Gets the competence level.
	 * 
	 * @return the competence level
	 */
	public CompetenceLevel getCompetenceLevel() {
		return competenceLevel;
	}

	/**
	 * Gets the indicator url.
	 * 
	 * @return the indicator url
	 */
	public String getIndicatorUrl() {
		if (attained == null) {
			return "status_notass";
		} else if (attained >= required) {
			return "status_achieved";
		}
	
		return "status_ass";
	}
	
	/**
	 * Gets the alt text for the image.
	 * 
	 * @return the alt text for the image
	 */
	public String getAltText() {
		String labelKey = "assessment.status.assessed";
		if (attained == null) {
			labelKey = "assessment.status.not.assessed";
		} else if (attained >= required) {
			labelKey = "assessment.status.competence.attained";
		}
	
		return Bundle.getLabel(labelKey);
		
	}

	/**
	 * Gets the assessment identifier.
	 * 
	 * @return the assessment identifier
	 */
	public String getAssessmentIdentifier() {
		if (assessment == null && hasAssessment()) {
			try {
				assessment = AssessmentTestFacade.getAssessmentTest(getAssessmentId(), ctx);
			} catch (Exception e) {
				PortletUtils.handleError(LOG, e);
			}
		}
	
		return assessment != null ? assessment.getIdentifier() : null;
	}

	/**
	 * Gets the assessment id.
	 * 
	 * @return the assessment id
	 */
	public long getAssessmentId() {
		return parentCompetence.getTestId();
	}

	/**
	 * Checks if the editor input has an associated assessment.
	 * 
	 * @return true, if successful
	 */
	public boolean hasAssessment() {
		return  parentCompetence.getTestId() != 0;
	}
	
	/**
	 * Sets the attained.
	 * 
	 * @param value
	 *            the new attained
	 */
	public void setAttained(final Integer value) {
		attained = value;
	}

	/**
	 * Sets the selected.
	 * 
	 * @param value
	 *            the new selected
	 */
	public void setSelected(final boolean value) {
		selected = value;
	}

	/** {@inheritDoc} */
	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof CompetenceWrapper)) {
			return false;
		}
		final CompetenceWrapper other = (CompetenceWrapper) obj;

		if (competenceLevel == null) {
			if (other.competenceLevel != null) {
				return false;
			}
		} else if (!competenceLevel.equals(other.competenceLevel)) {
			return false;
		}
		if (parentCompetence == null) {
			if (other.parentCompetence != null) {
				return false;
			}
		} else if (!parentCompetence.equals(other.parentCompetence)) {
			return false;
		}
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public int hashCode() {
		final int prime = 2467;
		int result = 1;
		result = prime * result + ((competenceLevel == null) ? 0 : competenceLevel.hashCode());
		result = prime * result + ((parentCompetence == null) ? 0 : parentCompetence.hashCode());
		return result;
	}

	
}
