/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.components;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.event.ActionEvent;
import javax.portlet.RenderRequest;
import javax.portlet.WindowState;

import org.apache.commons.lang.StringUtils;
import org.tencompetence.business.utils.ClassUtils;
import org.tencompetence.business.utils.EntityUtils;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.EntityPermissionsUtils;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.common.utils.UrlUtils;

import com.icesoft.faces.component.ContextActionEvent;
import com.icesoft.faces.component.DisplayEvent;
import com.icesoft.faces.component.ext.HtmlCommandLink;
import com.icesoft.faces.component.ext.HtmlDataTable;
import com.icesoft.faces.component.ext.UIColumn;
import com.icesoft.faces.component.menubar.MenuItem;
import com.icesoft.faces.component.menupopup.MenuPopup;
import com.liferay.portal.model.BaseModel;

/**
 * This class implements a controller for ICEFace components supporting ice:dataTable with an actionsMenu column.
 * 
 * @param <R>
 *            The generic row item
 * 
 * @author Stef Heyenrath
 * @version $Revision: 1.29 $, $Date: 2009/11/05 16:09:22 $
 */
public class ActionsMenuListController<R extends ActionsMenuRowItem<?>> {

	//private static final Log LOG = LogFactoryUtil.getLog(ActionsMenuListController.class);
	private List<R> items;
	private IConfirmationDialogController confirmController;
	private String maximizedUrl;
	private String normalUrl;
	private Map<String, Map<String, String>> outcomeMapping = null;

	/**
	 * Sets the outcome mapping HashMap. Forward for a specific entity to a specific page.
	 * 
	 * @param mapping
	 *            the mapping
	 */
	public void setOutcomeMapping(final Map<String, Map<String, String>> mapping) {
		outcomeMapping = mapping;
	}

	/**
	 * Set the maximizedUrl (used for returning maximized to the previous page).
	 * 
	 * @param maximizedUrl
	 *            maximizedUrl
	 */
	public void setMaximizedUrl(final String maximizedUrl) {
		this.maximizedUrl = maximizedUrl;
	}

	/**
	 * Set the normalURL (used for returning normal to the previous page).
	 * 
	 * @param normalUrl
	 *            normalURL
	 */
	public void setNormalUrl(final String normalUrl) {
		this.normalUrl = normalUrl;
	}

	/**
	 * Sets the items which acts as input for the ICEFace component.
	 * 
	 * @param items
	 *            the list of items
	 */
	public void setInput(final List<R> items) {
		this.items = items;
	}

	/**
	 * Set the ConfirmationDialogController.
	 * 
	 * @param controller
	 *            this.confirmationController
	 */
	public void setConfirmationDialogController(final IConfirmationDialogController controller) {
		this.confirmController = controller;
	}

	/**
	 * Gets the row items that act as input for this controller.
	 * 
	 * @return the row items that act as input for this controller
	 */
	public List<R> getRowItems() {
		if (items == null) {
			return Collections.emptyList();
		} else {
			return items;
		}
	}

	/**
	 * Gets the number of row items.
	 * 
	 * @return the number of row items
	 */
	public int getRowCount() {
		if (items == null) {
			return 0;
		} else {
			return items.size();
		}
	}

	/**
	 * This listener catches the DisplayEvent from the popupmenu.
	 * 
	 * @param displayEvent
	 *            displayEvent
	 */
	public void displayListener(final DisplayEvent displayEvent) {
		final MenuPopup popup = (MenuPopup) displayEvent.getComponent();
		final ActionsMenuRowItem<?> rowItem = (ActionsMenuRowItem<?>) displayEvent.getContextValue();

		if (!rowItem.isShowActionsMenu()) {
			popup.setRendered(false);
			return;
		}

		for (final UIComponent child : popup.getChildren()) {
			if (child instanceof MenuItem) {
				final MenuItem menuItem = (MenuItem) child;
				checkAllowed(menuItem, rowItem);

				// Do something special for Permissions
				setPermissionLink(menuItem, rowItem);
			}
		}
	}

	/**
	 * Sets the permission link.
	 */
	private void setPermissionLink(final MenuItem menuItem, final ActionsMenuRowItem<?> rowItem) {
		if (!menuItem.isDisabled() && menuItem.getId().equals("permissions")) {
			if (rowItem.getData() instanceof BaseModel<?>) {
				final BaseModel<?> editorInput = (BaseModel<?>) rowItem.getData();
				menuItem.setLink(EntityPermissionsUtils.getEntityPermissionsURL(editorInput));
			}
		}
	}

	/**
	 * Checks if a certain action (view,edit,delete,permissions,subscribe,unsubscribe,copy,study,complete) is allowed on
	 * this rowItem. If not allowed, the menu-option is not rendered.
	 */
	private void checkAllowed(final MenuItem menuItem, final ActionsMenuRowItem<?> rowItem) {
		final String menuId = menuItem.getId();
		final String methodName = "allow" + StringUtils.capitalize(menuId);

		menuItem.setRendered(ClassUtils.isProperty(rowItem, methodName, false));
	}

	/**
	 * This listener catches all click events from the popupmenu.
	 * 
	 * The supported events are : view, edit and delete.
	 * 
	 * @param event
	 *            ActionEvent
	 */
	public void menuActionListener(final ActionEvent event) {

		if (event instanceof ContextActionEvent) {
			final ContextActionEvent contextEvent = (ContextActionEvent) event;
			final ActionsMenuRowItem<?> rowItem = (ActionsMenuRowItem<?>) contextEvent.getContextValue();
			final String id = ((MenuItem) contextEvent.getSource()).getId();

			String outcome;

			if (id.equalsIgnoreCase("view")) {
				outcome = Outcomes.VIEW;
			} else if (id.equalsIgnoreCase("edit")) {
				outcome = Outcomes.EDIT;
			} else {
				outcome = null;
			}

			forwardToNextPage(rowItem, outcome);
		}
	}

	/**
	 * Handles the clickLink event.
	 * 
	 * @param event
	 *            ActionEvent
	 */
	public void clickLink(final ActionEvent event) {
		final HtmlCommandLink link = (HtmlCommandLink) event.getComponent();
		final UIColumn column = (UIColumn) link.getParent();
		final HtmlDataTable table = (HtmlDataTable) column.getParent();
		final ActionsMenuRowItem<?> rowItem = (ActionsMenuRowItem<?>) table.getRowData();

		forwardToNextPage(rowItem, Outcomes.VIEW);
	}

	/**
	 * Foward to next page.
	 * 
	 * When a confirmationController is set, this is used to check the input and display a dialog.
	 * 
	 * @param rowItem
	 *            rowItem
	 * @param outcome
	 *            outcome
	 */
	protected void forwardToNextPage(final ActionsMenuRowItem<?> rowItem, final String outcome) {

		if (outcome != null && rowItem.getData() instanceof BaseModel<?>) {
			final RenderRequest request = PortletUtils.getRenderRequest();
			final BaseModel<?> entity = (BaseModel<?>) rowItem.getData();
			final String className = EntityUtils.getInterfaceFromEntity(entity);
			String newOutcome = outcome;

			if (confirmController == null) {
				BaseEditorBean.passEditorInput(entity);
			} else {
				newOutcome = confirmController.canLeave(outcome, entity);
			}

			newOutcome = getTargetOutcome(className, newOutcome);

			navigateToNextOutcome(request, newOutcome);
		}
	}

	/**
	 * Navigate to next outcome.
	 * 
	 * @param request
	 *            the request
	 * @param newOutcome
	 *            the new outcome
	 */
	protected void navigateToNextOutcome(final RenderRequest request, final String newOutcome) {
		/*
		 * If current page is maximized or maximizedUrl and normalUrl are null, just do a forward.
		 */
		if (WindowState.MAXIMIZED.equals(request.getWindowState()) || maximizedUrl == null || normalUrl == null) {
			PortletUtils.handleNavigation(newOutcome);
		} else if (WindowState.NORMAL.equals(request.getWindowState())) {
			/*
			 * If current page is displayed in normal view, append the new outcome to the url and do a forward.
			 */
			final String maxUrl = UrlUtils.addParamToUrl(maximizedUrl, UrlUtils.TC_TARGET_VIEW, newOutcome);
			PortletUtils.redirect(maxUrl);
		}
	}

	/**
	 * Gets the target outcome.
	 * 
	 * @param className
	 *            the class name
	 * @param outcome
	 *            the new outcome
	 * 
	 * @return the target outcome
	 */
	protected String getTargetOutcome(final String className, final String outcome) {
		String targetOutcome = outcome;
		if (outcomeMapping != null && outcomeMapping.containsKey(outcome)) {
			if (outcomeMapping.get(outcome).containsKey(className)) {
				targetOutcome = outcomeMapping.get(outcome).get(className);
			}
		}

		return targetOutcome;
	}

	/**
	 * Gets the selected items.
	 * 
	 * @return the selected items
	 */
	public List<R> getSelectedItems() {
		final List<R> selectedItems = new ArrayList<R>();
		for (final R item : items) {
			if (item.isSelected()) {
				selectedItems.add(item);
			}
		}
		return selectedItems;
	}

	/**
	 * Gets the selected entities.
	 * 
	 * @param <T>
	 *            BaseModel
	 * @param list
	 *            the list
	 * 
	 * @return the selected entities
	 */
	@SuppressWarnings("unchecked")
	public <T> List<T> getSelectedEntities(final List<T> list) {
		list.clear();

		for (final R item : getSelectedItems()) {
			list.add((T) item.getData());
		}

		return list;
	}

	/**
	 * Gets the entity from action event.
	 * 
	 * @param event
	 *            the event
	 * 
	 * @return the entity from action event
	 */

	public Object getDataFromActionEvent(final ActionEvent event) {
		if (event instanceof ContextActionEvent) {
			final ContextActionEvent contextEvent = (ContextActionEvent) event;
			final ActionsMenuRowItem<?> rowItem = (ActionsMenuRowItem<?>) contextEvent.getContextValue();

			return rowItem.getData();
		}

		return null;
	}
}
