/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.components;

import javax.faces.event.ValueChangeEvent;

import org.tencompetence.business.utils.ClassUtils;
import org.tencompetence.portlet.common.constants.Defines;
import org.tencompetence.portlet.common.utils.EntityPermissionsUtils;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.common.utils.TENCActionKey;

import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.model.BaseModel;

/**
 * This class acts as a base class for backing bean for editors. This class keeps track of an editor input which is
 * passed through the session map. The type of the expected editor input should be defined through the generic.
 * Implementing class should call the {@code checkEditorInput} method on each and every getter method. The
 * <code>checkEditorInput</code> will call the <code>initFields</code> method when the editor input has changed. This
 * gives the implementing class the opportunity to (re-)initialize its fields. Furthermore, this class has an dirty
 * flag, which should be set by any implementing class to keep track of changes in any of the data fields.
 * 
 * @param <T>
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @version $Revision: 1.14 $, $Date: 2009/11/24 12:59:54 $
 */
public abstract class BaseEditorBean<T extends BaseModel<T>> extends PermissionsBean implements IFormState {

	/** The log. */
	private static final Log LOG = LogFactoryUtil.getLog(BaseEditorBean.class);

	private T editorInput;

	/** The title. */
	private String title = StringPool.BLANK;

	/** The description. */
	private String description = StringPool.BLANK;

	private boolean dirty;
	private ConfirmationController confirmController;

	private boolean allowView;
	private boolean allowUpdate;
	private boolean allowDelete;
	private boolean allowPermissions;

	/**
	 * Gets the confirmation dialog controller.
	 * 
	 * @return the confirmation controller
	 */
	public ConfirmationController getConfirmationController() {
		if (confirmController == null) {
			confirmController = new ConfirmationController();
		}

		return confirmController;
	}

	/**
	 * Gets the editor input set for this class. This method has been made public for bean compliance and should not be
	 * called directly.
	 * 
	 * @return the editor input
	 */
	public T getEditorInput() {
		return editorInput;
	}

	/**
	 * Sets the editor input for this class. This method has been made public for bean compliance and should not be
	 * called directly.
	 * 
	 * @param editorInput
	 *            the new editor input for this class
	 */
	public void setEditorInput(final T editorInput) {
		this.editorInput = editorInput;
	}

	/**
	 * Returns true if any of the fields have been changed, false otherwise.
	 * 
	 * @return true, if is dirty, false otherwise
	 */
	public boolean isDirty() {
		return dirty;
	}

	/**
	 * Sets the dirty flag for this bean.
	 * 
	 * @param dirty
	 *            the new dirty value
	 */
	public void setDirty(final boolean dirty) {
		this.dirty = dirty;
	}

	/**
	 * Ensures this backing bean has a valid editor input and the fields are synchronized with the editor input if
	 * needed.
	 */
	@SuppressWarnings("unchecked")
	protected void checkEditorInput() {

		final boolean initFields = PortletUtils.getObjectFromSession(Defines.INIT_FIELDS, false);

		if (initFields || editorInput == null) {
			final Object object = fetchEditorInput();
			if (object != null) {
				setDirty(false);
				// this.editable = true;

				try {
					this.editorInput = (T) object;

					// Only check permissions when this is not a new entity.
					if (!ClassUtils.isProperty(editorInput, "new", true)) {
						allowUpdate = checkModelPermission(editorInput, TENCActionKey.UPDATE);
						allowView = checkModelPermission(editorInput, TENCActionKey.VIEW);
						allowDelete = checkModelPermission(editorInput, TENCActionKey.DELETE);
						allowPermissions = checkModelPermission(editorInput, TENCActionKey.PERMISSIONS);
					}

					initFields(editorInput);
					PortletUtils.getSessionMap().put(Defines.INIT_FIELDS, false);
				} catch (RuntimeException e) {
					// make sure TypeCast exception are properly handled
					PortletUtils.handleError(LOG, new IllegalArgumentException("Invalid editor input", e));
				}
			}
		}
	}

	/**
	 * Saves the editorInput. Calls the abstract doSave method to do the actual saving.
	 * <p>
	 * This method should be hooked to an {@code action} in a {@code ice:CommandButton} or {@code ice:CommandLink}.
	 * 
	 * @return the viewId
	 */
	public String save() {
		try {
			doSave(editorInput);

		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		}

		setDirty(false);

		return getViewId();
	}

	/**
	 * Cancels the editorInput.
	 * 
	 * This method should be hooked to an {@code action} in a {@code ice:CommandButton} or {@code ice:CommandLink}.
	 * 
	 * @return the viewId
	 */
	public String cancel() {
		return getViewId();
	}

	/**
	 * Go to the edit permissions page form this entity.
	 * 
	 * @return null
	 */
	public String permissions() {
		PortletUtils.redirect(getPermissionsURL());
		return null;
	}

	/**
	 * Gets the previous viewId using the NavigationDataHolder. When no previous NavigationData can be found, just
	 * return null and jump back to the normal windowstate of smallView.
	 * 
	 * @return viewId
	 */
	protected String getViewId() {
		// Get previous page
		final NavigationData data = NavigationDataHolder.getInstance().getPreviousNavigationData();

		if (data == null) {
			// go back to small view
			PortletUtils.redirect(NavigationDataHolder.getInstance().getNormalWindowStateURL());
		} else {
			BaseEditorBean.passEditorInput(data.getEditorInput());
			return data.getViewId();
		}

		return null;
	}

	/**
	 * Gets the can navigate back.
	 * 
	 * @return the can navigate back
	 */
	public boolean getCanNavigateBack() {
		final NavigationDataHolder dataHolder = NavigationDataHolder.getInstance();
		return !dataHolder.isEmpty();
	}

	/**
	 * Initialize the fields of this bean. This method is called whenever the editor input has been changed or set.
	 * 
	 * @param editorInput
	 *            the new editor input for this bean
	 */
	protected abstract void initFields(T editorInput);

	/**
	 * Checks if is editable.
	 * 
	 * @return true, if is editable
	 */
	public boolean isEditable() {
		if (editorInput == null || editorInput.isNew()) {
			return true;
		} else {
			return allowUpdate;
		}
	}

	/**
	 * Use this method in the jspx to correctly initialize this bean.
	 * 
	 * @return null
	 */
	public String getPageInit() {
		checkEditorInput();
		return null;
	}

	/**
	 * Do save.
	 * 
	 * @param editorInput
	 *            the editor input
	 * 
	 * @throws SystemException
	 *             the system exception
	 * 
	 * @throws PortalException
	 *             PortalException
	 */
	protected abstract void doSave(T editorInput) throws SystemException, PortalException;

	/**
	 * Gets the title.
	 * 
	 * @return the title
	 */
	public String getTitle() {
		checkEditorInput();
		return title;
	}

	/**
	 * Convenience getter of the description. Because the normal getter does something extra which needs to be done when
	 * a get comes from the GUI.
	 * 
	 * @return the description
	 */
	protected String getDescriptionForInternalUse() {
		return description;
	}

	/**
	 * Convenience getter of the title. Because the normal getter does something extra which needs to be done when a get
	 * comes from the GUI.
	 * 
	 * @return the title
	 */
	protected String getTitleForInternalUse() {
		return title;
	}

	/**
	 * Sets the title.
	 * 
	 * @param title
	 *            the new title
	 */
	public void setTitle(final String title) {
		this.title = title;
	}

	/**
	 * Gets the description.
	 * 
	 * @return the description
	 */
	public String getDescription() {
		checkEditorInput();
		return description;
	}

	/**
	 * Sets the description.
	 * 
	 * @param description
	 *            the new description
	 */
	public void setDescription(final String description) {
		final String normalized = PortletUtils.normalizeLineBreak(description);

		// because carriage return and new line characters are modified after
		// persistence. Therefore we have to
		// normalize the String by removing the \r character.
		if (!this.description.equals(normalized)) {
			setDirty(true);
			this.description = normalized;
		}
	}

	/**
	 * Convenience setter of the description. Because the normal setter does something extra which needs to be done when
	 * a get comes from the GUI.
	 * 
	 * @param description
	 *            The description
	 */
	protected void setDescriptionForInternalUse(final String description) {
		this.description = description;
	}

	/**
	 * This method passes the editor input from one view to another.
	 * 
	 * @param value
	 *            the editor input to pass around
	 */
	public static void passEditorInput(final Object value) {
		PortletUtils.setObjectToSession(Defines.EDITOR_INPUT_PARAM_NAME, value);
		PortletUtils.setObjectToSession(Defines.INIT_FIELDS, true);
	}

	/**
	 * Fetches the editor input.
	 * 
	 * @return the editor input, or null if no editor input was set
	 */
	public static BaseModel<?> fetchEditorInput() {
		final Object result = PortletUtils.getObjectFromSession(Defines.EDITOR_INPUT_PARAM_NAME);
		if (result instanceof BaseModel<?>) {
			return (BaseModel<?>) result;
		}

		return null;
	}

	/**
	 * Valuechange listener for the valuesChanged action.
	 * 
	 * This funcion can be called after values have been changed to set the necessary properties of the BaseEditorBean.
	 * What happens: Dirty becomes true
	 * 
	 * @param event
	 *            the event
	 */
	public void valuesChanged(final ValueChangeEvent event) {
		setDirty(true);
	}

	/**
	 * Checks if the add button visible. This is only so of the editor input is already persisted on the server.
	 * 
	 * @return true, if the add button is visible
	 */
	public boolean isAddButtonVisible() {
		return editorInput != null && !editorInput.isNew();
	}

	/**
	 * getPermissionsURL.
	 * 
	 * @return URL
	 */
	public String getPermissionsURL() {
		return EntityPermissionsUtils.getEntityPermissionsURL(getEditorInput());
	}

	/**
	 * Checks if the user can update this entity.
	 * 
	 * @return true, if this is readonly mode.
	 */
	public boolean isReadOnly() {
		if (editorInput == null || editorInput.isNew()) {
			return false;
		} else {
			return !allowUpdate;
		}
	}

	/**
	 * @return AllowView
	 */
	public boolean isAllowView() {
		return allowView;
	}

	/**
	 * @return AllowUpdate
	 */
	public boolean isAllowUpdate() {
		return allowUpdate;
	}

	/**
	 * @return AllowDelete
	 */
	public boolean isAllowDelete() {
		return allowDelete;
	}

	/**
	 * @return AllowPermissions
	 */
	public boolean isAllowPermissions() {
		return allowPermissions;
	}

	/**
	 * The {@code ConfirmationController} is the controller class for the Confirmation Dialog. It implements the
	 * required {@code IConfirmationDialogController} interface.
	 * 
	 * @see IConfirmationDialogController
	 */
	public class ConfirmationController implements IConfirmationDialogController {

		private boolean visible = false;
		private BaseModel<?> nextInput;
		private String nextView;

		/** {@inheritDoc} */
		public boolean isPopupVisible() {
			return visible;
		}

		/**
		 * Sets the popup visible. This method has been made public for bean compliance and should not be called
		 * directly.
		 * 
		 * @param value
		 *            the new popup visible
		 */
		public void setPopupVisible(final boolean value) {
			visible = value;
		}

		/** {@inheritDoc} */
		public String okPressed() {
			visible = false;
			passEditorInput(nextInput);
			return nextView;
		}

		/** {@inheritDoc} */
		public String cancelPressed() {
			visible = false;
			return null;
		}

		/**
		 * Intercepts navigation from this view to another by checking if the current editor is dirty. If so it asks the
		 * user for a confirmation to discard the changes made.
		 * 
		 * @param nextView
		 *            the id of the view to navigate to
		 * @param nextInput
		 *            the editor input for the view being navigated to
		 * 
		 * @return the string
		 */
		public String canLeave(final String nextView, final BaseModel<?> nextInput) {
			if (isDirty()) {
				visible = true;
				this.nextView = nextView;
				this.nextInput = nextInput;
				return null;
			} else {
				passEditorInput(nextInput);
				return nextView;
			}
		}

	}
}
