/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.components;

import javax.faces.context.FacesContext;

import org.tencompetence.portlet.common.utils.PortletUtils;

import com.liferay.portal.model.BaseModel;

/**
 * This class is the managed bean for the breadcrumbs.jspx. It implements a history of visited pages which must include
 * this back button. Backtracking to previous pages is enabled through a back button and a drop menu of visited pages.
 * Furthermore quick navigation to the map, competences and competence profiles is provided via three hyper links.
 * <p />
 * A confirmation dialog is shown whenever a user leaves a pages by any of the aforementioned methods, which has
 * modified data. For this to work a page:
 * <ul>
 * <li>needs to have a backing bean that extends the <code>BaseEditorBean</code> class</li>
 * <li>finally a navigation rule must be added where the &lt;from-outcome&gt; tag corresponds with the
 * &lt;to-view-id&gt; tag and the &lt;from-view-id&gt; has value *
 * </ul>
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @author Robert Habets
 * @version $Revision: 1.4 $, $Date: 2009/10/13 15:07:55 $
 */
public class BreadcrumbsController {

	/*
	 * This fields contains the backing bean that extends the <code>BaseEditorBean</code>. It will be the
	 * f:setPropertyActionListener on any of the actions defined in the view.
	 */
	private BaseEditorBean<? extends BaseModel<?>> editor;

	/* A controller for the confirmation dialog */
	private PopupController controller;

	/**
	 * Breadcrumb clicked.
	 * 
	 * @return the string with the viewId
	 */
	public String breadcrumbClicked() {

		final String selectedCrumb = FacesContext.getCurrentInstance().getExternalContext()
				.getRequestParameterMap().get("item");

		/*
		 * check if there are changes, if so show popup and handle buttons else update NavigationDataHolder and jump
		 * directly to right page
		 */
		return getPopupController().canLeave(selectedCrumb, null);
	}

	/**
	 * Sets the <code>BaseEditorBean</code> for the back button. The <code>BaseEditorBean</code> is used to check the
	 * changed state of the current page.
	 * 
	 * @param editor
	 *            the editor
	 */
	public void setBaseEditorBean(final BaseEditorBean<? extends BaseModel<?>> editor) {
		this.editor = editor;
	}

	/**
	 * Gets the popup controller associated with the confirmation dialog.
	 * 
	 * @return the popup controller associated with the confirmation dialog
	 */
	public PopupController getPopupController() {
		if (controller == null) {
			controller = new PopupController();
		}

		return controller;
	}

	/**
	 * Returns the <code>BaseEditorBean</code> associated with this back button.
	 * 
	 * @return the editor
	 */
	public BaseEditorBean<? extends BaseModel<?>> getEditor() {
		return this.editor;
	}

	/**
	 * This method is used to record the current viewId + EditorInput and add this to the stack.
	 * 
	 * @return null
	 */
	public String getRecordData() {
		final String viewId = PortletUtils.getViewId();
		NavigationDataHolder.getInstance().recordNavigation(new NavigationData(BaseEditorBean.fetchEditorInput(), viewId));
		return null;
	}

	/**
	 * The Class PopupController acts as a controller for the confirmation dialog. It implements all necessary methods
	 * of the IConfirmationDialogController plus two overloaded initialization <code>canLeave</code>methods. One for
	 * forward navigation by the three navigati shortcut links and one for backward navigation by the back button and
	 * history menu.
	 */
	public class PopupController implements IConfirmationDialogController {

		private boolean visible = false;
		private String nextOutcome;
	

		/** {@inheritDoc} */
		public boolean isPopupVisible() {
			return visible;
		}

		/**
		 * Sets the popup visible.
		 * 
		 * @param value
		 *            the new popup visible
		 */
		public void setPopupVisible(final boolean value) {
			visible = value;
		}

		/** {@inheritDoc} */
		public String okPressed() {
			visible = false;
			goBack(this.nextOutcome);
			
			return nextOutcome;
		}

		/** {@inheritDoc} */
		public String cancelPressed() {
			visible = false;
			return null;
		}

		/**
		 * This method initializes the controller for backward navigation.
		 * 
		 * @param nextOutcome
		 *            the view id of the next page
		 * 
		 * @param nextInput
		 *            the editor input for the view being navigated to
		 * 
		 * @return the string
		 */
		public String canLeave(final String nextOutcome, final BaseModel<?> nextInput) {
			if (editor != null && editor.isDirty()) {
				visible = true;
				this.nextOutcome = nextOutcome;
				return null;
			} else {
				goBack(nextOutcome);
				return nextOutcome;
			}
		}

		/**
		 * Removes all elements from the stack from the NavigationDataHolder up until the selected viewId.
		 * 
		 * @param selectedCrumb
		 */
		private void goBack(final String selectedCrumb) {
			NavigationData navData = NavigationDataHolder.getInstance().peekLastRecordedNavigationData();
			while ((navData != null) && (!selectedCrumb.equalsIgnoreCase(navData.getViewId()))) {
				navData = NavigationDataHolder.getInstance().getLastRecordedNavigationData();
			}
			
			if (navData != null) {
				BaseEditorBean.passEditorInput(navData.getEditorInput());
			}
		}
	}
}
