/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.components;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.business.model.learningpath.ILPElement;
import org.tencompetence.business.model.learningpath.ILearningAction;
import org.tencompetence.business.model.learningpath.ILearningPath;
import org.tencompetence.business.model.learningpath.MetaDataEntry;
import org.tencompetence.business.utils.DateUtils;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.ImageUtils;

import com.liferay.portal.kernel.util.StringPool;

/**
 * This abstract class implements the controller for the lpElement facelet.
 * <p />
 * It provides the facelet with the data from the lpElement passed to the controller. It optionally shows a action
 * button. Default this button is shown when:
 * <ul>
 * <li>true is passed for the hasButton parameter in the class constructor</li>
 * <li>the element shows is either a {@code ILearningAction} or a {@code ILearningPath}</li>
 * <li>the element is not completed</li>
 * </ul>
 * 
 * When the user clicks the button the abstract handleClickEvent method is called. Concrete implementations of this
 * controller need to subclass and implement this method to handle the event.
 * <p />
 * 
 * @author Harrie Martens
 * @version $Revision: 1.7 $, $Date: 2009/11/23 11:29:55 $
 */

public abstract class ElementController {

	private static final int DESCRIPTION_THRESHOLD = 500;

	private ILPElement element;
	private boolean hasButton;
	private String description;
	private boolean descriptionExpanded;
	private boolean metadataExpanded;
	private boolean showImage;
	private boolean portfolioView;

	/**
	 * Instantiates a new element controller.
	 * 
	 * @param element
	 *            the ILPElement that is shown in the facelet.
	 * @param hasButton
	 *            when true the facelet may show the action button
	 * @param showImage
	 *            the show image
	 */
	public ElementController(final ILPElement element, final boolean hasButton, final boolean showImage) {
		this.hasButton = hasButton;
		this.metadataExpanded = true;
		this.showImage = showImage;
		setInput(element);
	}

	/**
	 * Instantiates a new element controller.
	 * 
	 * @param element
	 *            the ILPElement that is shown in the facelet.
	 * @param hasButton
	 *            when true the facelet may show the action button
	 * @param showImage
	 *            the show image
	 * @param isPortfolioView
	 *            is the controller used in the portfolio portlet(s)
	 */
	public ElementController(final ILPElement element, final boolean hasButton, final boolean showImage,
			final boolean isPortfolioView) {
		this.hasButton = hasButton;
		this.metadataExpanded = true;
		this.showImage = showImage;
		this.portfolioView = isPortfolioView;
		setInput(element);
	}

	/**
	 * Gets the title.
	 * 
	 * @return the title
	 */
	public String getTitle() {
		return this.element != null ? this.element.getTitle() : StringPool.BLANK;
	}

	/**
	 * Gets the description.
	 * 
	 * @return the description
	 */
	public String getDescription() {
		if (!this.descriptionExpanded) {
			String result = description.substring(0, DESCRIPTION_THRESHOLD);
			int i = result.lastIndexOf(' ');
			if (i != -1) {
				result = result.substring(0, i);
			}
			return result + "...";
		}

		return this.description;
	}

	/**
	 * Gets the 16x16 image url.
	 * 
	 * @return the image url
	 */
	public String getImageUrl16() {
		return ImageUtils.getImageUrl16(element);
	}

	/**
	 * Gets the 32x32 image url.
	 * 
	 * @return the image url
	 */
	public String getImageUrl32() {
		return ImageUtils.getImageUrl32(element);
	}

	/**
	 * Gets the 48x48 image url.
	 * 
	 * @return the image url
	 */
	public String getImageUrl48() {
		return ImageUtils.getImageUrl48(element);
	}

	/**
	 * Gets the default (140x135) image url.
	 * <p>
	 * The correct icon is determined by the type of the element and whether it is completed or subscribed.
	 * 
	 * @return the image url
	 */
	public String getImageUrl() {
		return ImageUtils.getImageUrl(element);
	}

	/**
	 * Gets the legenda for the icon.
	 * 
	 * @return the legenda
	 */
	public String getLegenda() {
		String result = StringPool.BLANK;
		if (element != null) {
			if (element.isCompleted()) {
				result = Bundle.getLabel("elementcontroller.completed") + "<br />"
						+ DateUtils.dateToString(element.getCompletedDate());
			} else if (element.isSubscribed()) {
				result = Bundle.getLabel("elementcontroller.subscribed") + "<br />"
						+ DateUtils.dateToString(element.getSubscribeDate());
			}
		}

		return result;
	}

	/**
	 * Gets the internationalized text for the Subscribe / Unsubscribe button.
	 * 
	 * @return the subscribe text
	 */
	public String getButtonCaption() {
		String result = StringPool.BLANK;

		if (element != null) {
			result = Bundle.getLabel("elementcontroller." + (element.isSubscribed() ? "unsubscribe" : "subscribe"));
		}

		return result;
	}

	/**
	 * Gets the meta data entries from the element.
	 * <p>
	 * Returns an empty list if the element is null.
	 * 
	 * @return the meta data
	 */
	public List<MetaDataEntry> getMetadata() {
		return element != null ? element.getMetaData() : new ArrayList<MetaDataEntry>(0);
	}

	/**
	 * Gets the toggle description text.
	 * 
	 * @return the toggle description text
	 */
	public String getToggleDescriptionText() {
		return Bundle.getLabel("elementcontroller.description." + (this.descriptionExpanded ? "hide" : "show"));
	}

	/**
	 * Gets the toggle meta data description.
	 * 
	 * @return the toggle meta data description
	 */
	public String getToggleMetadataDescription() {
		return Bundle.getLabel("elementcontroller.metadata." + (this.metadataExpanded ? "hide" : "show"));
	}

	/**
	 * Gets the element associated with this controller.
	 * 
	 * @return the element
	 */
	public ILPElement getElement() {
		return this.element;
	}

	/**
	 * Checks if the description is longer than the threshold.
	 * 
	 * @return true, if the description is longer than the threshold.
	 */
	public boolean isLongDescription() {
		return this.description.length() > DESCRIPTION_THRESHOLD;
	}

	/**
	 * Checks if the meta data is shown.
	 * 
	 * @return true, if the meta data is shown
	 */
	public boolean isMetadataExpanded() {
		return this.metadataExpanded;
	}

	/**
	 * Checks if the facelet requires a action button.
	 * 
	 * @return true, if the button is required
	 */
	public boolean isButtonRequired() {
		return (hasButton
		// && ((!(element instanceof ILearningPlan))
				&& (element instanceof ILearningPath || element instanceof ILearningAction) && !element.isCompleted());
	}

	/**
	 * The server faces action handler called when the action button is clicked.
	 * <p>
	 * This method calls the abstract {@code handleClickEvent()} method to perform the actual handling.
	 * 
	 * @return the navigation string as returned by the {@code handleSubscribeEvent}
	 */
	public String doClick() {
		return handleClickEvent(element);
	}

	/**
	 * The server faces action handler for the show / hide description command.
	 * <p>
	 * Toggles the state of the long description text.
	 * 
	 * @return null as navigation is not required
	 */
	public String doToggleDescription() {
		this.descriptionExpanded = !this.descriptionExpanded;
		return null;
	}

	/**
	 * The server faces action handler for the show / hide meta data command.
	 * 
	 * @return null as navigation is not required
	 */
	public String doToggleMetadata() {
		this.metadataExpanded = !this.metadataExpanded;
		return null;
	}

	/**
	 * Sets the meta data expanded flag.
	 * 
	 * @param value
	 *            the value for the meta data expanded flag
	 */
	protected void setMetadataExpanded(final boolean value) {
		this.metadataExpanded = value;
	}

	/**
	 * Sets the ILPElement input for this controller.
	 * <p>
	 * This input should not be null, otherwise an IllegalArgumentException is thrown
	 * 
	 * @param input
	 *            the new input for the controller
	 */
	public void setInput(final ILPElement input) {
		this.element = input;
		this.description = element != null ? (element.getDescription() == null ? StringPool.BLANK : element
				.getDescription()) : StringPool.BLANK;
		this.descriptionExpanded = this.description.length() <= DESCRIPTION_THRESHOLD;
	}

	/**
	 * Checks if is show title description.
	 * 
	 * @return true, if is show title description
	 */
	public boolean isPortfolioView() {
		return portfolioView;
	}

	/**
	 * Abstract method to handle the click event of the action button.
	 * <p>
	 * implementing classes should implement this method to do the actual handling of the event.
	 * 
	 * @param element
	 *            the element of this controller
	 * 
	 * @return the navigation string.
	 */
	public abstract String handleClickEvent(final ILPElement element);

	/**
	 * Do we need to show the image ?
	 * 
	 * @return true, if is show image
	 */
	public boolean isShowImage() {
		return showImage;
	}

	/**
	 * Gets the date of subscription.
	 * 
	 * @return the date subscribed
	 */
	public String getDateSubscribed() {
		String result = "-";
		if (element != null) {
			if (element.isSubscribed()) {
				result = DateUtils.dateToString(element.getSubscribeDate());
			}
		}
		return result;
	}

	/**
	 * Gets the date completed.
	 * 
	 * @return the date completed
	 */
	public String getDateCompleted() {
		String result = "-";
		if (element != null) {
			if (element.isCompleted()) {
				result = DateUtils.dateToString(element.getCompletedDate());
			}
		}
		return result;

	}
}