/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.components;

import java.util.EmptyStackException;
import java.util.List;
import java.util.Stack;

import org.tencompetence.portlet.common.utils.PortletUtils;

/**
 * 
 * This class implements the history of visited pages that include a back button.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * 
 * @version $Revision: 1.5 $, $Date: 2009/10/13 15:07:55 $
 * @see org.tencompetence.portlet.common.components.BreadcrumbsController
 */
public class NavigationDataHolder {
	private final Stack<NavigationData> stack = new Stack<NavigationData>();
	private String normalWindowStateURL = null;
	private String maximizedWindowStateURL = null;

	/**
	 * Records navigation to the passed view.
	 * 
	 * @param currentEntry
	 *            the current entry
	 */
	public void recordNavigation(final NavigationData currentEntry) {

		final NavigationData lastEntry = peek();

		// only add new pages, or same page with different input to the stack of
		// pages
		if (lastEntry != null) {
			if (lastEntry.equals(currentEntry)) {
				// we always push the new entry, so remove the last one
				// (effectively do nothing)
				stack.pop();
			} else if (lastEntry.getViewId().equals(currentEntry.getViewId()) && lastEntry.getEditorInput() == null) {
				// if editor input was not set, replace entry on history by
				// new one
				stack.pop();
			}
		}
		// add entry to history
		stack.push(currentEntry);
	}

	/**
	 * Returns stack of all available navigation data.
	 * 
	 * @return stack of all available navigation data.
	 */
	public List<NavigationData> getNavigationData() {
		return stack;
	}

	/**
	 * Gets the last recorded navigation data from the history.
	 * 
	 * @return the last recorded navigation data or null if no previous view was available
	 */
	public NavigationData getLastRecordedNavigationData() {
		NavigationData lastNavData = null;
		try {
			// Get the last recorded (current page)
			lastNavData = stack.pop();
		} catch (EmptyStackException em) {
			lastNavData = null;
		}
		return lastNavData;
	}

	/**
	 * Gets the previous navigation data from the history.
	 * 
	 * @return the previous navigation data or null if no previous view was available
	 */
	public NavigationData getPreviousNavigationData() {

		// Get current page by removing it from the stack
		getLastRecordedNavigationData();

		// Get the previous page by peeking to the stack
		return peekLastRecordedNavigationData();
	}

	/**
	 * Peek last recorded navigation data.
	 * 
	 * @return the navigation data
	 */
	public NavigationData peekLastRecordedNavigationData() {
		return peek();
	}

	/**
	 * Checks if history is empty.
	 * 
	 * @return true if the history is empty, false otherwise
	 */
	public boolean isEmpty() {
		return !(stack.size() > 0);
	}

	/**
	 * Clears the history.
	 */
	public void clear() {
		stack.clear();
	}

	private NavigationData peek() {
		try {
			return stack.peek();
		} catch (EmptyStackException emptyStackExc) {
			return null;
		}
	}

	/**
	 * Static method to retrieve the NavigationDataHolder from the context.
	 * 
	 * @return NavigationDataHolder object
	 */
	public static NavigationDataHolder getInstance() {
		return PortletUtils.getManagedBean("#{navigationDataHolder}", NavigationDataHolder.class);
	}

	/**
	 * Sets the NormalWindowStateURL from the SmallView.jspx.
	 * 
	 * @param url
	 *            as String
	 */
	public void setNormalWindowStateURL(final String url) {
		if (normalWindowStateURL == null) {
			normalWindowStateURL = url;
		}
	}

	/**
	 * Gets the NormalWindowStateURL from the SmallView.jspx.
	 * 
	 * @return PortletURL
	 */
	public String getNormalWindowStateURL() {
		return normalWindowStateURL;
	}

	/**
	 * Sets the MaximizedWindowStateURL from the SmallView.jspx.
	 * 
	 * @param url
	 *            as string
	 */
	public void setMaximizedWindowStateURL(final String url) {
		if (maximizedWindowStateURL == null) {
			maximizedWindowStateURL = url;
		}
	}

	/**
	 * Gets the MaximizedWindowStateURL from the SmallView.jspx.
	 * 
	 * @return PortletURL
	 */
	public String getMaximizedWindowStateURL() {
		return maximizedWindowStateURL;
	}
}
