/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.components;

import org.tencompetence.business.utils.EntityUtils;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.common.utils.TENCActionKey;

//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.model.BaseModel;
import com.liferay.portal.security.permission.PermissionChecker;
import com.liferay.portal.service.permission.PortletPermissionUtil;
import com.liferay.portal.theme.ThemeDisplay;

/**
 * Permission bean class.
 * @author Anton Finders
 * @author Stef Heijenrath
 * @version $Revision: 1.14 $, $Date: 2009/11/24 14:33:59 $
 */
public class PermissionsBean {
	private static final Log LOG = LogFactoryUtil.getLog(PermissionsBean.class);

	private boolean expanded = false;

	/**
	 * Check if a specific action is allowed.
	 * 
	 * @param action
	 *            The Action
	 * @return true / false
	 */
	public boolean checkPortletPermission(final String action) {
		final ThemeDisplay themeDisplay = PortletUtils.getThemeDisplay();
		final PermissionChecker permissionChecker = themeDisplay.getPermissionChecker();
		final long plid = themeDisplay.getPlid();

		try {
			return PortletPermissionUtil.contains(permissionChecker, plid, PortletUtils.getLiferayPortletId(), action);
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
	}

	/**
	 * Check if a specific action is allowed.
	 * 
	 * @param action
	 *            The Action
	 * @return true / false
	 */
	public boolean checkPortletPermission(final TENCActionKey action) {
		return checkPortletPermission(action.getAction());
	}

	/**
	 * Check if the user has permission to do an defined action on this resource.
	 * 
	 * @param <T>
	 *            Generic Type
	 * @param entity
	 *            BaseModel<T> entity
	 * @param action
	 *            Action
	 * @return true/false
	 */
	public <T> boolean checkModelPermission(final BaseModel<T> entity, final String action) {
		final ThemeDisplay themeDisplay = PortletUtils.getThemeDisplay();
		final PermissionChecker permissionChecker = themeDisplay.getPermissionChecker();

		final String entityClass = entity.getClass().getInterfaces()[0].getName();
		final String entityId = entity.getPrimaryKeyObj().toString();

		long ownerId = 0;
		long companyId = 0;
		long groupId = 0;

		try {
			ownerId = EntityUtils.getUserIdFromEntity(entity);
			companyId = EntityUtils.getCompanyIdFromEntity(entity);
			groupId = EntityUtils.getGroupIdFromEntity(entity);
		} catch (SystemException se) {
			LOG.error("Unable to get User, Company or Group from entity : " + entity);
			return false;
		}

		// First check if the user is the owner.
		final boolean ownerPermission = permissionChecker.hasOwnerPermission(companyId, entityClass, entityId, ownerId,
				action);

		if (ownerPermission) {
			if (LOG.isDebugEnabled()) {
				LOG.debug("User " + ownerId + " is the owner from entity : " + entity);
			}
			return true;
		} else {
			// Check for other permissions.
			return permissionChecker.hasPermission(groupId, entityClass, entityId, action);
		}
	}

	/**
	 * Check if the user has permission to do an defined action on this resource.
	 * 
	 * @param <T>
	 *            Generic Type
	 * @param entity
	 *            BaseModel<T> entity
	 * @param action
	 *            Action
	 * @return true/false
	 */
	public <T> boolean checkModelPermission(final BaseModel<T> entity, final TENCActionKey action) {
		return checkModelPermission(entity, action.getAction());
	}

	/**
	 * Checks if AdvancedPermissions is expanded.
	 * 
	 * @return isExpanded
	 */
	public boolean isExpanded() {
		return expanded;
	}

	/**
	 * Toggles between expanded state of advanced Permissions on the page.
	 * 
	 * @return null as we remain on the same page
	 */
	public String toggle() {
		expanded = !expanded;
		return null;
	}

	/**
	 * Gets the toggle text for the hide/show additional Permissions links.
	 * 
	 * @return the toggle text
	 */
	public String getToggleText() {
		final String sign = expanded ? "-" : "+";
		return Bundle.getLabel("permissionList.advancedPermissions", new Object[] { sign });
	}
}
