/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.components;

import java.util.ArrayList;
import java.util.List;

import javax.faces.event.ActionEvent;

import org.tencompetence.business.facade.LevelValueFacade;
import org.tencompetence.generated.model.LevelValue;
import org.tencompetence.portlet.common.utils.PortletUtils;

import com.icesoft.faces.component.ext.HtmlInputHidden;
//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;

/**
 * This class implements a controller for the slider. The controller maintains the selection of the assigned.selected
 * level. When a user clicks on a level the assigned level will be set. The required level is used to set the minimum
 * level a user should accomplish. *
 * 
 * @author Leon Schaeps
 * @version $Revision: 1.10 $, $Date: 2009/11/24 12:38:00 $
 */
public abstract class RatingComponentController {
	private static final Log LOG = LogFactoryUtil.getLog(RatingComponentController.class);

	private static final int IMAGE_WIDTH = 9;

	private final List<RatingComponentValue> ratingValues = new ArrayList<RatingComponentValue>();
	private LevelValue assignedLevel;
	private final boolean editable;
	private final boolean showRequiredLevelIcon;
	private final boolean showNullLevelIcon;
	private final int totalLevels;
	private final boolean showPopup;

	/**
	 * Instantiates a new rating component controller.
	 * 
	 * @param levels
	 *            The levels which must be used to show
	 * @param requiredLevelId
	 *            Minimum level a user should accomplish.
	 * @param editable
	 *            To indicate if the ratingcomponent is editable.
	 * @param assignedLevel
	 *            the assigned level
	 * @param large
	 *            Should the size of rating component be small or large
	 * @param showPopup
	 *            the show popup
	 */
	public RatingComponentController(final List<LevelValue> levels, final LevelValue assignedLevel,
			final long requiredLevelId, final boolean editable, final boolean large, final boolean showPopup) {

		final List<LevelValue> levelValues = levels;
		this.assignedLevel = assignedLevel;
		this.editable = editable;
		this.showPopup = showPopup;

		totalLevels = levels.size();

		showRequiredLevelIcon = requiredLevelId > 0;
		showNullLevelIcon = !showRequiredLevelIcon && assignedLevel == null && !large;

		// Add the levels as RatingComponentValues to the slidercontroller
		for (final LevelValue level : levelValues) {
			boolean isOn = (assignedLevel != null) && (level.getValue() <= assignedLevel.getValue());
			final RatingComponentValue rcv = new RatingComponentValue(level, isOn, large);

			// Check the required field
			if (requiredLevelId > 0 && level.getLevelValueId() == requiredLevelId) {
				rcv.setRequired(true);
			}
			ratingValues.add(rcv);
		}
	}

	/**
	 * Gets the level values.
	 * 
	 * @return the level values
	 */
	public List<RatingComponentValue> getLevelValues() {
		return ratingValues;
	}

	/**
	 * Click link. Action event to handle when clicked on a value in the frontend.
	 * 
	 * @param event
	 *            the action event
	 */
	public void clickLink(final ActionEvent event) {
		final HtmlInputHidden hidden = (HtmlInputHidden) event.getComponent().getChildren().get(3);
		setAssignedLevel(getLevelValue(Long.valueOf(hidden.getValue().toString())));
		ratingComponentClicked();
	}

	/**
	 * Rating component clicked. Implement the action to be done when clicked on the ratingcomponent by the user
	 */
	public abstract void ratingComponentClicked();

	/**
	 * Gets the assigned level.
	 * 
	 * @return the assigned level
	 */
	public LevelValue getAssignedLevel() {
		return assignedLevel;
	}

	/**
	 * Sets the assigned level.
	 * 
	 * @param level
	 *            The level to be assigned
	 */
	public void setAssignedLevel(final LevelValue level) {

		assignedLevel = level;

		for (final RatingComponentValue rating : ratingValues) {
			if (assignedLevel != null && (rating.getLevel().getValue() <= assignedLevel.getValue())) {
				rating.setImageType(true);
			} else {
				rating.setImageType(false);
			}
		}
	}

	/**
	 * Gets the level value.
	 * 
	 * @param levelId
	 *            the level id
	 * 
	 * @return the level value
	 */
	private LevelValue getLevelValue(final long levelId) {
		LevelValue levelValue = null;
		if (levelId > 0) {
			try {
				levelValue = LevelValueFacade.getLevelValue(levelId, PortletUtils.getServiceContext());
			} catch (PortalException e) {
				PortletUtils.handleError(LOG, e);
			} catch (SystemException e) {
				PortletUtils.handleError(LOG, e);
			}
		}
		return levelValue;
	}

	/**
	 * Checks if is editable.
	 * 
	 * @return true, if is editable
	 */
	public boolean isEditable() {
		return editable;

	}

	/**
	 * Checks if we need to show the Required Level Icon.
	 * 
	 * @return true / false
	 */
	public boolean isShowRequiredLevelIcon() {
		return showRequiredLevelIcon;
	}

	/**
	 * Checks if we need to the Null Level icon.
	 * 
	 * @return true / false
	 */
	public boolean isShowNullLevelIcon() {
		return showNullLevelIcon;
	}

	/**
	 * Gets the Total Width.
	 * 
	 * @return the Total Width
	 */
	public int getTotalWidth() {
		return (totalLevels * IMAGE_WIDTH) - 2;
	}

	/**
	 * Checks if we need to show the popup.
	 * 
	 * @return true / false
	 */
	public boolean isShowPopup() {
		return showPopup;
	}
}