/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.common.components;

import org.tencompetence.generated.model.LevelValue;
import org.tencompetence.portlet.common.constants.Defines;

import com.liferay.portal.kernel.util.StringPool;

/**
 * The Class RatingComponentValue is a levelvalue with some extra fields needed for the ratingcomponent. Extra fields
 * like images to turn/off this value. Is the level the minimum required level etc.
 * 
 * @author Leon Schaeps
 * @version $Revision: 1.11 $, $Date: 2009/11/24 12:36:31 $
 */
public class RatingComponentValue {

	/** Link to image if ratingvalue is turned on. */
	private static final String SLEVEL_ON = "/images/components/ratingcomponent/rating_on1.png";

	/** Link to image if ratingvalue is turned on for level is zero. */
	private static final String SLEVEL_ON_L0 = "/images/components/ratingcomponent/rating_on_Lev02.png";

	/** Link to image if ratingvalue is turned on for level is zero. */
	// private static final String SLEVEL_ON_L0_NOT_ASSIGNED = "/images/components/ratingcomponent/rating_on_Lev0.png";

	/** Link to small image if ratingvalue is turned on. */
	private static final String SLEVEL_SMALL_ON = "/images/components/ratingcomponent/rating_on_small.png";

	/** Link to image if ratingvalue is turned off. */
	private static final String SLEVEL_OFF = "/images/components/ratingcomponent/rating_off1.png";

	/** Link to image if ratingvalue is turned off. */
	private static final String SLEVEL_OFF_L0 = "/images/components/ratingcomponent/rating_off_Lev02.png";

	/** Link to small image if ratingvalue is turned off. */
	private static final String SLEVEL_SMALL_OFF = "/images/components/ratingcomponent/rating_off_small.png";

	/** Link to image if ratingvalue is hovered. */
	private static final String SLEVEL_HOVER = "/images/components/ratingcomponent/rating_hover1.png";

	/** Link to image if ratingvalue is hovered. */
	private static final String SLEVEL_HOVER_L0 = "/images/components/ratingcomponent/rating_hover_Lev02.png";

	/** Link to small image if ratingvalue is hovered. */
	private static final String SLEVEL_SMALL_HOVER = "/images/components/ratingcomponent/rating_hover_small.png";

	/** Link to image if ratingvalue is required. */
	private static final String REQUIRED_SRC = "/images/components/ratingcomponent/req2.gif";

	/** Link to small image if ratingvalue is required. */
	private static final String REQUIRED_SMALL_SRC = "/images/components/ratingcomponent/req_small.gif";

	/** Link to image if ratingvalue is not required. */
	private static final String NOT_REQUIRED_SRC = "/images/components/ratingcomponent/not_req1.gif";

	/** Link to small image if ratingvalue is not required. */
	private static final String NOT_REQUIRED_SMALL_SRC = "/images/components/ratingcomponent/not_req_small.gif";

	private final LevelValue level;

	/** The url src. Needed for the default url of the image in the jspx file */
	private String urlSrc = StringPool.BLANK;

	/** Is the ratingvalue displayed as large or small. */
	private boolean isLarge;

	/** Link to the actual image for the level (can be on/off). */
	private String imgSrc;

	/** Link to the actual required image for the level. */
	private String reqSrc;

	/**
	 * Gets the url, needed for the default url of the image in the jspx file.
	 * 
	 * @return the url
	 */
	public String getUrlSrc() {
		return urlSrc;
	}

	/**
	 * Instantiates a new rating component value.
	 * 
	 * @param level
	 *            the level
	 * @param imageOn
	 *            the image on
	 * @param large
	 *            Should the small or large images be used
	 */
	public RatingComponentValue(final LevelValue level, final boolean imageOn, final boolean large) {
		isLarge = large;
		// default not required
		setRequired(false);

		this.level = level;
		setImageType(imageOn);

	}

	/**
	 * Sets the required.
	 * 
	 * @param required
	 *            Possibility to set the required value. Needed to show which level is the minimum level.
	 */
	public void setRequired(final boolean required) {
		if (required) {
			if (isLarge) {
				reqSrc = REQUIRED_SRC;
			} else {
				reqSrc = REQUIRED_SMALL_SRC;
			}
		} else {
			if (isLarge) {
				reqSrc = NOT_REQUIRED_SRC;
			} else {
				reqSrc = NOT_REQUIRED_SMALL_SRC;
			}
		}
	}

	/**
	 * Gets the level.
	 * 
	 * @return the level
	 */
	public LevelValue getLevel() {
		return level;
	}

	/**
	 * Gets the req src.
	 * 
	 * @return the req src
	 */
	public String getReqSrc() {
		return reqSrc;
	}

	/**
	 * Sets the img src.
	 * 
	 * @param imgSrc
	 *            the new img src. Not really needed, only implemented for avoiding errors at the front-end.
	 */
	public void setImgSrc(final String imgSrc) {
		// needed for frontend
	}

	/**
	 * Gets the img src.
	 * 
	 * @return the img src
	 */
	public String getImgSrc() {
		return imgSrc;
	}

	/**
	 * Sets the hover src.
	 * 
	 * @param hoverSrc
	 *            the new hover src. Not really needed, only implemented for avoiding errors at the front-end.
	 */
	public void setHoverSrc(final String hoverSrc) {
		// needed for frontend
	}

	/**
	 * Gets the hover src.
	 * 
	 * @return the hover src
	 */
	public String getHoverSrc() {
		String sHover;
		if (isLarge) {
			if (level != null && level.getValue() == 0) {
				sHover = SLEVEL_HOVER_L0;
			} else {
				sHover = SLEVEL_HOVER;
			}
		} else {
			sHover = SLEVEL_SMALL_HOVER;
		}

		return Defines.PORTAL_SRC + sHover;
	}

	/**
	 * Set the image type.
	 * 
	 * @param imageOn
	 *            true or false
	 */
	public final void setImageType(final boolean imageOn) {
		if (imageOn) {
			if (isLarge) {

				if (level != null && level.getValue() == 0) {
					urlSrc = SLEVEL_ON_L0;
				} else {
					urlSrc = SLEVEL_ON;
				}
			} else {
				urlSrc = SLEVEL_SMALL_ON;
			}
		} else {
			if (isLarge) {
				if (level != null && level.getValue() == 0) {
					urlSrc = SLEVEL_OFF_L0;
				} else {
					urlSrc = SLEVEL_OFF;
				}
			} else {
				urlSrc = SLEVEL_SMALL_OFF;
			}
		}

		imgSrc = Defines.PORTAL_SRC + urlSrc;

	}
}
