/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.components;

import java.util.List;

import org.tencompetence.business.facade.CompetenceFacade;
import org.tencompetence.business.facade.LevelValueFacade;
import org.tencompetence.business.services.PdpService;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.LevelValue;
import org.tencompetence.portlet.common.utils.PortletUtils;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * The Class RatingPopupController.
 * @version $Revision: 1.23 $, $Date: 2009/11/24 14:10:06 $
 */
public class RatingPopupController {

	/** The Constant LOG. */
	private static final Log LOG = LogFactoryUtil.getLog(RatingPopupController.class);

	/** The visible. */
	private boolean visible;

	/** The rating controller. */
	private RatingComponentController ratingController;

	/** The dialog input. */
	private IAssessementDialogInput dialogInput;

	/**
	 * Gets the rating controller.
	 * 
	 * @return the rating controller
	 */
	public RatingComponentController getRatingController() {
		return ratingController;
	}

	/**
	 * Instantiates a new rating popup controller.
	 * 
	 * @param dialogInput
	 *            the dialog input
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public RatingPopupController(final IAssessementDialogInput dialogInput) throws SystemException, PortalException {
		super();
		this.dialogInput = dialogInput;
	}

	/**
	 * Checks if is visible.
	 * 
	 * @return true, if is visible
	 */
	public boolean isVisible() {
		return visible;
	}

	/**
	 * Sets the visible of this dialog. When visibility is set to true the <code>getCompetenceLevel</code> of the passed
	 * <code>IAssessmentDialogController</code> is expected to return a valid value.
	 * 
	 * @param visible
	 *            the new visibility value
	 */
	public void setVisible(final boolean visible) {
		if (this.visible != visible) {
			this.visible = visible;
			if (visible) {
				try {
					// ratingController will be refreshed whenever visibility changes from false to true.
					this.ratingController = initController(dialogInput.getCompetenceLevel());
				} catch (SystemException e) {
					PortletUtils.handleError(LOG, e);
				} catch (PortalException e) {
					PortletUtils.handleError(LOG, e);
				}
			}
		}
	}

	/**
	 * Do cancel.
	 * 
	 * @return the string
	 */
	public String doCancel() {
		setVisible(false);
		return null;
	}

	/**
	 * Do save.
	 * 
	 * @return the string
	 */
	public String doSave() {
		setVisible(false);
		this.dialogInput.doSave(this.ratingController.getAssignedLevel());
		return null;
	}

	/**
	 * Inits the controller.
	 * 
	 * @param competenceLevel
	 *            the competence level
	 * 
	 * @return the rating component controller
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	private RatingComponentController initController(final CompetenceLevel competenceLevel) throws SystemException,
			PortalException {
		assert competenceLevel != null : "competenceLevel is null";
		final ServiceContext ctx = PortletUtils.getServiceContext();
		final Competence comp = CompetenceFacade.getCompetence(competenceLevel.getCompetenceId(), ctx);
		final LevelValue attained = PdpService.getAttainedLevelForUser(ctx.getUserId(), comp, ctx);
		final List<LevelValue> values = LevelValueFacade.getAttainableCompetenceLevelValues(comp.getCompetenceMapId(),
				ctx);
//		return new AssessmentRatingController(values, attained == null ? values.get(0) : attained, competenceLevel
		return new AssessmentRatingController(values, attained , competenceLevel.getLevelValueId());
	}

	/**
	 * The Class AssessmentRatingController.
	 */
	private class AssessmentRatingController extends RatingComponentController {

		/**
		 * Instantiates a new assessment rating controller that is used within the ratingPopup.jspx.
		 * 
		 * @param levels
		 *            the levels
		 * @param assignedLevel
		 *            the assigned level
		 * @param requiredLevelId
		 *            the required level id
		 */
		public AssessmentRatingController(final List<LevelValue> levels, final LevelValue assignedLevel,
				final long requiredLevelId) {
			super(levels, assignedLevel, requiredLevelId, true, true, false);
		}

		/** {@inheritDoc} */
		@Override
		public void ratingComponentClicked() {
		}

	}

}
