/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.utils;

import java.text.MessageFormat;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.tencompetence.business.utils.UserLocaleResolver;


import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.StringPool;

/**
 * class to provide localized messages to portlets based on property files.
 * 
 * @author venrooijma
 * @version $Revision: 1.2 $, $Date: 2009/11/24 14:10:06 $
 */
public final class Bundle {

	/** The bundle. */
	private static ResourceBundle resourceBundle;

	/** The Constant BUNDLE_ROOT. */
	private static final String BUNDLE_ROOT = "content.Language";

	private static final Log LOG = LogFactoryUtil.getLog(Bundle.class);

	static {
		// initialize the bundle with the correct properties files.
		resourceBundle = ResourceBundle.getBundle(BUNDLE_ROOT, UserLocaleResolver.getUserLocale());
	}

	private Bundle() {
		// Empty constructor
	}

	/**
	 * get a localized String for [labelKey].
	 * 
	 * @param labelKey
	 *            the label key
	 * 
	 * @return the label
	 */
	public static String getLabel(final String labelKey) {
		return getLabel(labelKey, null);
	}

	/**
	 * get a localized String for [labelKey] with args [args].
	 * 
	 * @param labelKey
	 *            the label key
	 * @param args
	 *            the args
	 * 
	 * @return the label
	 */
	public static String getLabel(final String labelKey, final Object[] args) {
		if (resourceBundle == null) {
			LOG.warn("Labels not setup !!");
			return labelKey;
		} else {
			return getFilledLabelString(labelKey, args);
		}
	}

	/**
	 * Checks if the current bundle is the correct one regarding the user's locale.
	 */
	private static void refreshBundle() {
		final Locale userLocale = UserLocaleResolver.getUserLocale();
		if (!resourceBundle.getLocale().equals(userLocale)) {
			resourceBundle = ResourceBundle.getBundle(BUNDLE_ROOT, userLocale);
		}
	}

	/**
	 * private method to handle the MissingResourceException and empty strings.
	 * 
	 * @param labelKey
	 *            the label key
	 * @param args
	 *            the args
	 * 
	 * @return the filled label string
	 */
	private static String getFilledLabelString(final String labelKey, final Object[] args) {
		refreshBundle();
		String result = labelKey;
		try {
			String tmpString = StringPool.BLANK;
			if (args == null) {
				tmpString = resourceBundle.getString(labelKey);
			} else {
				tmpString = MessageFormat.format(resourceBundle.getString(labelKey), args);
			}

			if (!tmpString.isEmpty()) {
				result = tmpString;
			}
		} catch (MissingResourceException ex) {
			LOG.warn("missing property for key: " + labelKey, ex);
		}

		return result;
	}
}