/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.utils;

import org.tencompetence.business.model.learningpath.ILPElement;
import org.tencompetence.business.model.learningpath.ILearningAction;
import org.tencompetence.business.model.learningpath.ILearningPath;
import org.tencompetence.business.model.learningpath.ILearningPlan;
import org.tencompetence.business.model.learningpath.IParallel;
import org.tencompetence.business.model.learningpath.ISelection;
import org.tencompetence.business.model.learningpath.ISequence;
import org.tencompetence.business.utils.ClassUtils;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.generated.model.Evidence;
import org.tencompetence.generated.model.Goal;

import com.liferay.portal.model.BaseModel;

/**
 * The Class ImageUtils.
 * 
 * Class with some Image / Entity related functions.
 * @version $Revision: 1.7 $, $Date: 2009/11/24 14:10:06 $
 */
public final class ImageUtils {

	private static final int ICON_16 = 16;
	private static final int ICON_32 = 32;
	private static final int ICON_48 = 48;

	private ImageUtils() {
		// Empty constructor
	}

	/**
	 * Gets the image basename.
	 * 
	 * @param entity
	 *            the entity
	 * 
	 * @return the image basename
	 */
	public static String getImageBasename(final Object entity) {
		final String image;
		if (entity instanceof ILearningAction) {
			image = "action";
		} else if (entity instanceof ILearningPlan) {
			image = "plan";
		} else if (entity instanceof ILearningPath) {
			image = "path";
		} else if (entity instanceof ISelection) {
			image = "selection";
		} else if (entity instanceof ISequence) {
			image = "sequence";
		} else if (entity instanceof IParallel) {
			image = "parallel";
		} else if (entity instanceof Competence) {
			image = "competence";
		} else if (entity instanceof CompetenceLevel) {
			image = "level";
		} else if (entity instanceof CompetenceProfileLevel) {
			image = "level";
		} else if (entity instanceof Evidence) {
			image = "evidence";
		} else if (entity instanceof Goal) {
			image = "goal";
		} else {
			image = "action"; // default
		}

		return image;
	}

	/**
	 * Gets the image url.
	 * <p>
	 * The correct icon is determined by the type of the element and whether it is completed or subscribed.
	 * 
	 * @param entity
	 *            the entity
	 * 
	 * @return the image url
	 */
	public static String getImageUrl(final Object entity) {
		return getImageUrl(entity, null, true);
	}

	/**
	 * Gets the 16x16 image url.
	 * 
	 * @param entity
	 *            the entity
	 * 
	 * @return the image url
	 */
	public static String getImageUrl16(final Object entity) {
		return getImageUrl16(entity, false);
	}

	/**
	 * Gets the 16x16 image url.
	 * 
	 * @param entity
	 *            the entity
	 * @param extended
	 *            the extended
	 * 
	 * @return the image url
	 */
	public static String getImageUrl16(final Object entity, final boolean extended) {
		return getImageUrl(entity, ICON_16, extended);
	}

	/**
	 * Gets the 32x32 image url.
	 * 
	 * @param entity
	 *            the entity
	 * 
	 * @return the image url
	 */
	public static String getImageUrl32(final Object entity) {
		return getImageUrl(entity, ICON_32, true);
	}

	/**
	 * Gets the 48x48 image url.
	 * 
	 * @param entity
	 *            the entity
	 * 
	 * @return the image url
	 */
	public static String getImageUrl48(final Object entity) {
		return getImageUrl(entity, ICON_48, true);
	}

	/**
	 * Gets the image url.
	 * <p>
	 * The correct icon is determined by the type of the element and whether it is completed or subscribed.
	 * 
	 * @param entity
	 *            the entity
	 * @param size
	 *            the size from the image
	 * @param extended
	 *            the extended
	 * 
	 * @return the image url
	 */
	public static String getImageUrl(final Object entity, final Integer size, final boolean extended) {

		if (entity != null && (entity instanceof BaseModel<?> || entity instanceof ILPElement)) {
			final String sizePostfix = size != null ? "_" + size : "";
			final String image = getImageBasename(entity);

			String suffix = "";
			// Only possible to display 'completed' or 'subscribed' when the normal big icon is used.
			if (size == null && extended) {
				if (ClassUtils.isProperty(entity, "completed", false)) {
					suffix = "_completed";
				} else if (ClassUtils.isProperty(entity, "subscribed", false)) {
					suffix = "_subscribed";
				}
			}

			return String.format("/images/icons/%s%s%s.png", image, sizePostfix, suffix);
		} else {
			return null;
		}
	}
}