/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.common.utils;

import javax.portlet.PortletMode;
import javax.portlet.PortletModeException;
import javax.portlet.PortletURL;
import javax.portlet.RenderResponse;
import javax.portlet.WindowState;
import javax.portlet.WindowStateException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;

/**
 * The Class URLUtils.
 * 
 * Class with all kind of URL related functions.
 * @version $Revision: 1.8 $, $Date: 2009/11/24 14:10:06 $
 */
public final class UrlUtils {

	private static final Log LOG = LogFactoryUtil.getLog(UrlUtils.class);
	
	/** The Constant TC_TARGET_VIEW. */
	public static final String TC_TARGET_VIEW = "tc_target_view";

	/** The Constant TC_REDIRECT. */
	public static final String TC_REDIRECT = "tc_redirect";

	// Rendergroups needed for the render manager
	/** The Constant RG_LEVEL_VALUES. */
	public static final String RG_LEVEL_VALUES = "RG_LEVEL_VALUES";
	
	private UrlUtils() {
		// Empty constructor
	}

	/**
	 * Add a parameter to an url.
	 * 
	 * @param url
	 *            url
	 * @param paramName
	 *            paramName
	 * @param paramValue
	 *            paramValue
	 * @return url
	 */
	public static String addParamToUrl(final String url, final String paramName, final String paramValue) {
		final String portletId = PortletUtils.getPortletId();
		return url + "&" + portletId + paramName + "=" + paramValue;
	}

	/**
	 * Returns a URL that Liferay uses to render this portlet in normal state. The URL contains information to switch to
	 * the correct JSF view once normalized.
	 * 
	 * @return the Liferay URL to the normal portlet
	 */
	public static String getWindowStateNormalUrl() {
		final RenderResponse response = PortletUtils.getRenderResponse();
		final PortletURL url = response.createRenderURL();

		try {
			url.setWindowState(WindowState.NORMAL);
			return url.toString();
		} catch (WindowStateException e) {
			if (LOG.isErrorEnabled()) {
				LOG.error(e);
			}
			throw new RuntimeException(e);
		}
	}

	/**
	 * Returns a URL that Liferay uses to render this portlet in normal state. The URL contains information to switch to
	 * the correct JSF view once normalized.
	 * 
	 * @param mode
	 *            the portletmode
	 * 
	 * @return the Liferay URL to the normal portlet
	 */
	public static String getWindowStateNormalUrl(final PortletMode mode) {
		final RenderResponse response = PortletUtils.getRenderResponse();
		final PortletURL url = response.createRenderURL();
				
		try {
			url.setWindowState(WindowState.NORMAL);			
			url.setPortletMode(mode);
			return url.toString();
		} catch (PortletModeException e) {
			PortletUtils.handleError(LOG, e);
			throw new RuntimeException(e);
		} catch (WindowStateException e) {
			PortletUtils.handleError(LOG, e);
			throw new RuntimeException(e);
		}
	}

	/**
	 * Returns a URL that Liferay uses to render this portlet in maximized state. The URL contains information to switch
	 * to the correct JSF view once maximized.
	 * 
	 * @return the Liferay URL to the maximized portlet
	 */
	public static String getWindowStateMaximizedUrl() {
		final RenderResponse response = PortletUtils.getRenderResponse();
		final PortletURL url = response.createRenderURL();

		try {
			url.setWindowState(WindowState.MAXIMIZED);
			return url.toString();
		} catch (WindowStateException e) {
			PortletUtils.handleError(LOG, e);
			throw new RuntimeException(e);
		}
	}

	/**
	 * Returns a URL that Liferay uses to render this portlet in normal state. The URL contains information to switch to
	 * the correct JSF view once normalized.
	 * 
	 * @param outcome
	 *            the outcome which is used to show the correct JSF view
	 * 
	 * @return the Liferay URL to the normal portlet
	 */
	public static String getWindowStateNormalUrl(final String outcome) {
		final RenderResponse response = PortletUtils.getRenderResponse();
		final PortletURL url = response.createRenderURL();

		try {
			url.setWindowState(WindowState.NORMAL);
			url.setParameter(UrlUtils.TC_TARGET_VIEW, outcome);
			return url.toString();
		} catch (WindowStateException e) {
			PortletUtils.handleError(LOG, e);
			throw new RuntimeException(e);
		}
	}

	/**
	 * Returns a URL that Liferay uses to render this portlet in maximized state. The URL contains information to switch
	 * to the correct JSF view once maximized.
	 * 
	 * @param outcome
	 *            the outcome which is used to show the correct JSF view
	 * 
	 * @return the Liferay URL to the maximized portlet
	 */
	public static String getWindowStateMaximizedUrl(final String outcome) {
		final RenderResponse response = PortletUtils.getRenderResponse();
		final PortletURL url = response.createRenderURL();

		try {
			url.setWindowState(WindowState.MAXIMIZED);
			url.setParameter(UrlUtils.TC_TARGET_VIEW, outcome);
			return url.toString();
		} catch (WindowStateException e) {
			PortletUtils.handleError(LOG, e);
			throw new RuntimeException(e);
		}
	}
}
