/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.goal.beans;

import javax.faces.event.ActionEvent;
import javax.portlet.PortletPreferences;

import org.tencompetence.business.facade.GoalFacade;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.portlet.assessment.beans.AssessmentPreferences;
import org.tencompetence.portlet.common.components.GoalTitleController;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.PortletUtils;

import com.icesoft.faces.async.render.Renderable;
import com.icesoft.faces.context.DisposableBean;
import com.icesoft.faces.webapp.xmlhttp.PersistentFacesState;
import com.icesoft.faces.webapp.xmlhttp.RenderingException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.StringPool;

/**
 * This is the abstract base class for backing beans that listen to a goal change from the Goal Selector.
 * 
 * @author Stef Heyenrath
 * @author Harrie Martens
 * @version $Revision: 1.7 $, $Date: 2009/12/01 09:37:45 $
 */
public abstract class GoalListener implements Renderable, DisposableBean {
	private static final Log LOG = LogFactoryUtil.getLog(GoalListener.class);

	private boolean needsInit;
	private GoalManager goalManager;
	private Goal selectedGoal = null;
	private PersistentFacesState state;
	private String message = StringPool.BLANK;
	private GoalTitleController goalTitleController;

	/**
	 * Constructor.
	 */
	public GoalListener() {
		// initialize the view
		state = PersistentFacesState.getInstance();
	}

	/**
	 * Gets the goal title controller.
	 * 
	 * @return the goal title controller
	 */
	public GoalTitleController getGoalTitleController() {
		return goalTitleController;
	}

	/**
	 * Gets the GoalManager.
	 * 
	 * @return the GoalManager
	 */
	public GoalManager getGoalManager() {
		return goalManager;
	}

	/**
	 * Gets the message.
	 * 
	 * @return message
	 */
	public String getMessage() {
		return message;
	}

	/**
	 * Get title of goal.
	 * 
	 * @return String title of selected goal.
	 */
	public String getSelectedGoalTitle() {
		initFields();
		return selectedGoal != null ? selectedGoal.getTitle() : StringPool.BLANK;
	}

	/** {@inheritDoc} */
	//@Override
	public PersistentFacesState getState() {
		return state;
	}

	/**
	 * Checks if a goal is selected.
	 * 
	 * @return true, if a goal is selected
	 */
	public boolean isGoalSelected() {
		initFields();
		return selectedGoal != null;
	}

	/**
	 * Setter goalManager.
	 * 
	 * @param goalManager
	 *            the new goalmanager
	 */
	public void setGoalManager(final GoalManager goalManager) {
		this.goalManager = goalManager;
	}

	/** {@inheritDoc} */
	//@Override
	public void renderingException(final RenderingException e) {
		// An FatalRenderingException will occur if a user has closed his browser
		// the update functionality tries to render a view that doesn't exist anymore so only for debugging there is
		// some logging.
		if (LOG.isDebugEnabled()) {
			LOG.debug(e);
		}
	}

	/**
	 * Logout.
	 * 
	 * @param event
	 *            the event
	 */
	public void removeListener(final ActionEvent event) {
		goalManager.removeListener(this);
	}

	/** {@inheritDoc} */
	//@Override
	public void dispose() throws Exception {
		performCleanup();
	}
	
	/**
	 * Gets the needs initialization state.
	 * 
	 * @return the needs initialization state
	 */
	protected boolean getNeedsInit() {
		return needsInit;
	}
	
	/**
	 * Sets the needs initialization state to the passed value.
	 * 
	 * @param value
	 *            the new needs initialization state
	 */
	protected void setNeedsInit(final boolean value) {
		needsInit = value;
	}

	/**
	 * Inits the fields.
	 */
	protected void initFields() {
		final PortletPreferences prefs = PortletUtils.getPortletRequest().getPreferences();
		final String prefGoalId = prefs.getValue(AssessmentPreferences.SELECTED_GOAL, "0");
		Goal newGoal = null;

		if (prefGoalId.equals("0")) {
			// no goal predefined get it from the goalselector
			goalManager.registerListener(this);
			newGoal = goalManager.getSelectedGoal(PortletUtils.getCurrentUserId(), PortletUtils.getGroupId());
		} else {
			// in case of a already registered view remove it
			if (goalManager.hasListener(this)) {
				goalManager.removeListener(this);
			}
			// predefined goal by portlet prefs
			try {
				newGoal = GoalFacade.getGoal(Long.parseLong(prefGoalId), PortletUtils.getServiceContext());
			} catch (Exception e) {
				PortletUtils.handleError(LOG, e);
			}
		}

		if (!identicalGoals(newGoal)) {
			selectedGoal = newGoal;
			needsInit = true;
		}

		// Clear error message
		message = StringPool.BLANK;

		if (selectedGoal == null) {
			message = Bundle.getLabel("error.noGoalSelected");
			return;
		}
		if (!GoalUtil.checkValidTargetType(selectedGoal)) {
			message = Bundle.getLabel("error.noValidTargetSelected");
			return;
		}

		try {
			goalTitleController = new GoalTitleController(selectedGoal, PortletUtils.getServiceContext());
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}

	}

	/**
	 * Get selected goal.
	 * 
	 * @return Selected goal.
	 */
	protected Goal getSelectedGoal() {
		return selectedGoal;
	}

	/**
	 * Sets the message.
	 * 
	 * @param msg
	 *            the new message
	 */
	protected void setMessage(final String msg) {
		message = msg;
	}

	private boolean identicalGoals(final Goal newGoal) {
		if (newGoal == null) {
			return selectedGoal == null;
		}

		if (!newGoal.equals(selectedGoal)) {
			return false;
		}

		return newGoal.getTarget() == selectedGoal.getTarget();
	}

	/**
	 * Perform cleanup.
	 * 
	 * @return true, if successful
	 */
	private boolean performCleanup() {
		try {
			if (goalManager.hasListener(this)) {
				removeListener(null);
			}
			return true;
		} catch (Exception failedCleanup) {
			if (LOG.isErrorEnabled()) {
				LOG.error("Failed to cleanup", failedCleanup);
			}
		}
		return false;
	}
}