/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.goal.controllers;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.business.facade.CompetenceProfileLevelFacade;
import org.tencompetence.business.facade.GoalFacade;
import org.tencompetence.generated.model.CompetenceProfile;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.portlet.common.utils.PortletUtils;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.service.ServiceContext;

/**
 * The Class {@code ProfileLevelEntry} encapsulates a {@code CompetenceProfileLevel} for usage in the paths list.
 * 
 * @author Harrie Martens
 * @version $Revision: 1.4 $, $Date: 2009/11/16 15:22:08 $
 */
public class ProfileEntry implements Comparable<ProfileEntry> {
	private boolean selected;
	private final CompetenceProfile profile;
	private final List<CompetenceProfileLevel> levels = new ArrayList<CompetenceProfileLevel>();
	private Goal goal;
	private boolean active;

	/**
	 * Instantiates a new path data.
	 * 
	 * @param profile
	 *            the profile
	 * @param ctx
	 *            the ctx
	 * 
	 * @throws PortalException
	 *             the portal exception
	 * @throws SystemException
	 *             the system exception
	 */
	public ProfileEntry(final CompetenceProfile profile, final ServiceContext ctx) throws SystemException,
			PortalException {
		if (profile == null) {
			throw new IllegalArgumentException("Null value not allowed for ProfileEntry()");
		}
		this.profile = profile;
		this.selected = false;
		// this.goal = isGoalForProfile(goal) ? goal : null;

		// filter out profile levels with empty title. This is done for Manchester demo.
		for (CompetenceProfileLevel level : CompetenceProfileLevelFacade
				.getCompetenceProfileLevelsByCompetenceProfileId(profile.getPrimaryKey(), ctx)) {
			if (!"".equals(level.getTitle().trim())) {
				levels.add(level);
			}
		}
	}

	/**
	 * Gets the id.
	 * 
	 * @return the id
	 */
	public long getId() {
		return profile.getPrimaryKey();
	}

	/**
	 * Gets the title.
	 * 
	 * @return the title
	 */
	public String getTitle() {
		return profile.getTitle();
	}

	/**
	 * Gets the description.
	 * 
	 * @return the description
	 */
	public String getDescription() {
		return profile.getDescription();
	}

	/**
	 * Gets the selected indicator.
	 * 
	 * @return the selected indicator
	 */
	public boolean getSelected() {
		return this.selected;
	}

	/**
	 * Gets the levels.
	 * 
	 * @return the levels
	 */
	public List<CompetenceProfileLevel> getLevels() {
		return levels;
	}

	/**
	 * Gets the selected level.
	 * 
	 * @return the selected level
	 */
	public Long getSelectedLevel() {
		return goal.getTarget();
	}

	/**
	 * Gets the motivation.
	 * 
	 * @return the motivation
	 */
	public String getMotivation() {
		return goal.getMotivation();
	}
	
	/**
	 * Gets the goal.
	 * 
	 * @return the goal
	 */
	public Goal getGoal() {
		return goal;
	}

	/**
	 * Marks entry as selected in list.
	 * 
	 * @param value
	 *            true if this entry is selected
	 */
	public void setSelected(final boolean value) {
		this.selected = value;
	}

	/**
	 * Sets the selected level.
	 * 
	 * @param level
	 *            the new selected level
	 */
	public void setSelectedLevel(final Long level) {
		goal.setTarget(level);
	}

	/**
	 * Sets the motivation.
	 * 
	 * @param value
	 *            the new motivation
	 */
	public void setMotivation(final String value) {
		goal.setMotivation(value);
	}

	/**
	 * Checks if the user is subscribed to the path.
	 * 
	 * @return true, if the path is subscribed
	 */
	public boolean isSubscribed() {
		return true;
	}

	/**
	 * Checks if this profile is selected as the active goal.
	 * 
	 * @return true, if is active
	 */
	public boolean isActive() {
		return active;
	}

	/**
	 * Checks for levels.
	 * 
	 * @return true, if successful
	 */
	public boolean hasLevels() {
		return !levels.isEmpty();
	}

	/**
	 * Checks if the profile has multiple profile levels.
	 * 
	 * @return true, if there are multiple profile levels
	 */
	public boolean hasMultipleLevels() {
		return levels.size() > 1;
	}

	/**
	 * Finds the associated goal from the passed list of goals.
	 * <p />
	 * Also marks the entry as active if the associated goal is the active goal
	 * 
	 * @param goals
	 *            the list of goals of the user
	 * @param activeGoal
	 *            the active goal
	 * 
	 * @return true, if this entry is associated with the active goal
	 */
	public boolean associateGoal(final List<Goal> goals, final Goal activeGoal) {
		active = false;
		this.goal = null;

		for (Goal goal : goals) {
			if (isGoalForProfile(goal)) {
				this.goal = goal;
				active = goal.equals(activeGoal);
				selected = active;
				break;
			}
		}

		return active;
	}

	/**
	 * Makes this profile the active goal.
	 * <p />
	 * If no goal exists for this profile, a new one is created.
	 * 
	 * @param ctx
	 *            the service context
	 * 
	 * @return the goal
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public Goal activate(final ServiceContext ctx) throws SystemException, PortalException {
		if (goal == null) {
			goal = GoalFacade.createGoal();
			goal.setTitle(getTitle());
			goal.setMotivation("");
			goal.setCompetenceMapId(this.profile.getCompetenceMapId());
			goal.setTarget(levels.get(0).getPrimaryKey());
			goal.setTargetType(CompetenceProfileLevel.class.getName());
			PortletUtils.setOwnership(goal, ctx);
			goal = GoalFacade.addGoal(goal, ctx);
		}
		active = true;

		return goal;
	}

	/**
	 * Mark this profile as not being the active one.
	 */
	public void deactivate() {
		active = false;
	}

	/**
	 * Saves the associated goal.
	 * 
	 * @param ctx
	 *            the service context
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public void save(final ServiceContext ctx) throws SystemException, PortalException {
		GoalFacade.updateGoal(goal, ctx);
	}

	/** {@inheritDoc} */
	//@Override
	public int compareTo(final ProfileEntry that) {
		if (this.equals(that)) {
			// both paths are equal
			return 0;
		}

		if (that == null) {
			return -1;
		}

		/*
		 * if (this.isActive() != that.isActive()) { // selected profile comes first return this.isActive() ? -1 : 1; }
		 */

		// and last sort alphabetically on title
		return this.getTitle().compareToIgnoreCase(that.getTitle());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	/** {@inheritDoc} */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((profile == null) ? 0 : profile.hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	/** {@inheritDoc} */
	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		ProfileEntry other = (ProfileEntry) obj;

		if (profile == null) {
			if (other.profile != null) {
				return false;
			}
		} else if (!profile.equals(other.profile)) {
			return false;
		}
		return true;
	}

	private boolean isGoalForProfile(final Goal goal) {
		if (goal != null) {
			long target = goal.getTarget();
			for (CompetenceProfileLevel level : levels) {
				if (level.getPrimaryKey() == target) {
					return true;
				}
			}
		}

		return false;
	}
}
