/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.modeleditor.common;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import org.tencompetence.portlet.common.utils.PortletUtils;

import com.icesoft.faces.component.ext.RowSelectorEvent;

/**
 * This class implements a controller for ICEFace components supporting ice:rowSelect such as ice:dataTable. The
 * controller maintains the selection on the session scope. A controller instance is identified by a key parameter
 * passed in the constructor. This key should be unique and consistent between different instances of the controller
 * representing the same view component.
 * 
 * @param <T>
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @version $Revision: 1.11 $, $Date: 2009/10/14 12:28:52 $
 */
public class ListController<T extends IRowItem> {

	/**
	 * The Enum SelectionType list the available selection modes of a list.
	 */
	public enum SelectionType {
		/** Single selection. */
		SINGLE,
		/** Multiple selection. */
		MULTIPLE,
		/** Multiple selection supporting standard key commands like Ctrl click. */
		MULTIPLE_ENHANCED
	};

	private List<T> items;
	private final String key;
	private final SelectionType selectionType;

	/**
	 * Instantiates a new list controller with a unique key identifying the associated ICEFace component, such as a
	 * dataTable.
	 * 
	 * @param key
	 *            the key identifying the associated ICEFace component
	 * @param selectionType
	 *            the selection type
	 */
	public ListController(final String key, final SelectionType selectionType) {
		this.key = key;
		this.selectionType = selectionType;
	}

	/**
	 * Sets the items which acts as input for the ICEFace component.
	 * 
	 * @param items
	 *            the list of items
	 */
	public void setInput(final List<T> items) {
		this.items = items;
		// remove obsolete selections from session
		cleanSelections();

		// restore selected state from session
		syncSelection();
	}

	/**
	 * Gets the row items that act as input for this controller.
	 * 
	 * @return the row items that act as input for this controller
	 */
	public List<T> getRowItems() {
		if (items == null) {
			return new ArrayList<T>();
		} else {
			return items;
		}
	}

	/**
	 * Gets a list of the currently selected row items.
	 * 
	 * @return a list of the currently selected row items
	 */
	public List<T> getSelection() {
		final List<T> selection = new ArrayList<T>();
		for (final T item : getRowItems()) {
			if (item.isSelected()) {
				selection.add(item);
			}
		}
		return selection;
	}

	/**
	 * Select listener for the ICEFaces component associated with this controller.
	 * 
	 * @param event
	 *            the event that triggered this listener
	 */
	@SuppressWarnings("unchecked")
	public void selectedListener(final RowSelectorEvent event) {
		HashSet<Long> selectedItems = (HashSet<Long>) PortletUtils.getSessionMap().get(key);
		if (selectedItems == null) {
			selectedItems = new HashSet<Long>();
		}

		if (selectionType == SelectionType.SINGLE) {

			final IRowItem selected = getRowItems().get(event.getRow());
			if (selectedItems.contains(selected.getId())) {
				// now a current selection needs to be deselected so reset the
				// selected
				selectedItems = new HashSet<Long>();
			} else {
				// Reset session to contain only the current single selection
				selectedItems = new HashSet<Long>(1);
				selectedItems.add(selected.getId());
			}

		} else {
			// Add result of event to session variable

			final IRowItem item = getRowItems().get(event.getRow());

			if (item.isSelected() && !selectedItems.contains(item.getId())) {
				// if the item isSelected and not selected before add the
				// selected
				selectedItems.add(item.getId());
			} else {
				// the item is not selected so remove or the item was deselected
				// -> remove
				selectedItems.remove(item.getId());
			}
		}
		PortletUtils.getSessionMap().put(key, selectedItems);
		syncSelection();
	}

	@SuppressWarnings("unchecked")
	private void cleanSelections() {
		final HashSet<Long> pks = new HashSet<Long>();
		for (IRowItem item : getRowItems()) {
			pks.add(Long.valueOf(item.getId()));
		}

		final HashSet<Long> itemsToRemove = new HashSet<Long>(0);
		final HashSet<Long> selectedItems = (HashSet<Long>) PortletUtils.getSessionMap().get(key);
		if (selectedItems != null) {
			for (Long pk : selectedItems) {
				// does the selection still exists?
				if (!pks.contains(pk)) {
					itemsToRemove.add(pk);
				}
			}
			selectedItems.removeAll(itemsToRemove);
		}

		PortletUtils.getSessionMap().put(key, selectedItems);
	}

	@SuppressWarnings("unchecked")
	private void syncSelection() {
		final HashSet<Long> selectedItems = (HashSet<Long>) PortletUtils.getSessionMap().get(key);
		for (IRowItem item : getRowItems()) {
			final Long pk = Long.valueOf(item.getId());
			item.setSelected(selectedItems != null && selectedItems.contains(pk));
		}
	}

	/**
	 * Returns true if the List selection mode is set to <code>SelectionType.MULTIPLE</code>.
	 * 
	 * @return true if the List selection mode is set to <code>SelectionType.MULTIPLE</code>, false otherwise
	 */
	public String getMultiple() {
		return Boolean.toString(selectionType == SelectionType.MULTIPLE);
	}

	/**
	 * Returns true if the List selection mode is set to <code>SelectionType.MULTIPLE_ENHANCED</code>.
	 * 
	 * @return true if the List selection mode is set to <code>SelectionType.MULTIPLE_ENHANCED</code>, false otherwise
	 */
	public String getEnhancedMultiple() {
		return Boolean.toString(selectionType == SelectionType.MULTIPLE_ENHANCED);
	}

	/**
	 * Converts the list of IRowItems to a list of corresponding ID's.
	 * 
	 * @return the list ID's corresponding to the list of IRowItems
	 */
	public List<Long> getIds() {
		final List<Long> result = new ArrayList<Long>(getRowItems().size());
		for (T row : getRowItems()) {
			result.add(row.getId());
		}
		return result;
	}

	/**
	 * Sets the selected items to not selected.
	 */
	public void resetSelection() {

		for (final T item : getRowItems()) {
			item.setSelected(false);
		}

		PortletUtils.getSessionMap().put(key, null);
	}
}
