/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.pdp.beans;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;

import org.tencompetence.business.facade.CompetenceFacade;
import org.tencompetence.business.facade.CompetenceLevelFacade;
import org.tencompetence.business.facade.CompetenceProfileLevelFacade;
import org.tencompetence.business.facade.LevelValueFacade;
import org.tencompetence.business.model.learningpath.ILPElement;
import org.tencompetence.business.model.learningpath.ILearningPath;
import org.tencompetence.business.model.learningpath.ILearningPlan;
import org.tencompetence.business.services.LearningPathService;
import org.tencompetence.business.services.LearningPlanService;
import org.tencompetence.business.services.PdpService;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.generated.model.LevelValue;
import org.tencompetence.portlet.common.components.ElementController;
import org.tencompetence.portlet.common.constants.Defines;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.common.utils.UrlUtils;
import org.tencompetence.portlet.pdp.controllers.Gap;
import org.tencompetence.portlet.pdp.controllers.PathListEntry;

import com.icesoft.faces.component.ext.RowSelectorEvent;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * Enter the class documentation here.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @version $Revision: 1.30 $, $Date: 2009/11/26 13:37:20 $
 */
public class PdpBuildPlan {
	private static final Log LOG = LogFactoryUtil.getLog(PdpBuildPlan.class);
	private final String normalUrl = UrlUtils.getWindowStateNormalUrl();

	private ILearningPlan plan;
	private BuildplanElementController elementController;
	private boolean needsInit;
	private List<PathListEntry> pathList;
	private List<Gap> gapList;
	private ServiceContext ctx;

	/**
	 * Instantiates a new pdp build plan.
	 */
	public PdpBuildPlan() {
		this.needsInit = true;
	}

	/**
	 * Gets the title.
	 * 
	 * @return the title
	 */
	public String getTitle() {
		initFields();
		return plan != null ? plan.getTitle() : "";
	}

	/**
	 * Gets the element controller.
	 * 
	 * @return the element controller
	 */
	public ElementController getElementController() {
		initFields();
		return this.elementController;
	}

	/**
	 * Gets the list of competence gaps.
	 * 
	 * @return the list of competence gaps
	 */
	public List<Gap> getGapList() {
		initFields();
		return this.gapList;
	}

	/**
	 * Gets the list of learning paths.
	 * 
	 * @return the list of learning paths
	 */
	public List<PathListEntry> getPathList() {
		initFields();
		Collections.sort(this.pathList);
		return this.pathList;
	}

	/**
	 * Returns true if a element is selected.
	 * 
	 * @return true, if a element is selected
	 */
	public boolean isElementSelected() {
		initFields();
		return this.elementController.getElement() != null;
	}

	/**
	 * Returns true if there are any plans available.
	 * 
	 * @return true, if there are any plans available
	 */
	public boolean isPlansAvailable() {
		initFields();
		return !this.pathList.isEmpty();
	}

	/**
	 * Selection listener called when the selection in the path list changes.
	 * 
	 * @param event
	 *            the event describing the selection
	 */
	public void selectionListener(final RowSelectorEvent event) {
		initFields();
		handleSelection(pathList.get(event.getRow()));
	}

	/**
	 * Saves the plan and returns to the small view.
	 * 
	 * @return null as navigation is handled directly
	 */
	public String doSave() {
		final List<ILearningPath> selected = new ArrayList<ILearningPath>();
		for (PathListEntry data : pathList) {
			if (data.isPartOfPlan()) {
				selected.add(data.getPath());
			}
		}

		// Always save the selected LP's in the database (also when listsize is 0).
		plan.setChildren(selected);
		try {
			LearningPlanService.updateLearningPlan(plan, ctx);
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}

		PortletUtils.redirect(normalUrl);

		return null;
	}

	/**
	 * Returns to the small view without saving the plan.
	 * 
	 * @return null as navigation is handled directly
	 */
	public String doCancel() {
		PortletUtils.redirect(normalUrl);
		return null;
	}

	private void initFields() {
		if (needsInit) {
			needsInit = false;
			pathList = new ArrayList<PathListEntry>();

			CompetenceProfileLevel target = getTarget();
			createGapList(target);

			List<CompetenceLevel> competenceGaps = new ArrayList<CompetenceLevel>(this.gapList.size());
			for (Gap gap : gapList) {
				competenceGaps.add(gap.getLevel());
			}

			// fill list with available paths
			try {
				for (ILearningPath path : LearningPathService.searchLearningPaths(competenceGaps, ctx)) {
					PathListEntry data = new PathListEntry(path, this.gapList);
					if (!pathList.contains(data)) {
						pathList.add(data);
					}
				}
			} catch (Exception e) {
				PortletUtils.handleError(LOG, e);
			}

			// fill local list with data from plan
			for (ILPElement child : this.plan.getChildren()) {
				if (child instanceof ILearningPath) {
					PathListEntry data = new PathListEntry((ILearningPath) child, this.gapList);

					int index = pathList.indexOf(data);
					if (index == -1) {
						pathList.add(data);
					} else {
						data = pathList.get(index);
					}
					handleAddition(data, true);
				}
			}

			Collections.sort(this.pathList);

			elementController = new BuildplanElementController();

			// select the first path, when available
			if (!pathList.isEmpty()) {
				handleSelection(pathList.get(0));
			}
		}
	}

	private CompetenceProfileLevel getTarget() {
		CompetenceProfileLevel result = null;
		final Object obj = PortletUtils.getSessionMap().get(Defines.EDITOR_INPUT_PARAM_NAME);
		try {
			if (!(obj instanceof Goal)) {
				throw new IllegalArgumentException("Input should be of type Goal");
			}

			Goal goal = (Goal) obj;
			// only accept a competence profile level
			if (goal.getTargetType().equalsIgnoreCase("org.tencompetence.generated.model.CompetenceProfileLevel")) {
				result = CompetenceProfileLevelFacade.getCompetenceProfileLevel(goal.getTarget(), ctx);
			} else {
				throw new Exception("Only targettype CompetenceProfileLevel is a valid goal target");
			}

			this.ctx = PortletUtils.getServiceContext();
			this.plan = LearningPlanService.getLearningPlanForUserByGoalId(ctx.getUserId(), goal.getGoalId(), ctx);
			if (plan == null) {
				this.plan = LearningPlanService.addLearningPlan(ctx.getUserId(), goal, goal.getTitle(), ctx);
			}
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
		return result;
	}

	private void createGapList(final CompetenceProfileLevel target) {
		CPLevel level;
		try {
			level = new CPLevel(target, ctx);
			this.gapList = level.analyzeGaps();
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	private void handleSelection(final PathListEntry selectedPath) {
		// reset selected state of list items to resemble normal selection list
		for (PathListEntry path : pathList) {
			path.setSelected(path.equals(selectedPath));
		}

		// update gap list to show which gaps are covered by the selected path
		for (Gap gap : gapList) {
			gap.setSelected(selectedPath.providesLevel(gap));
		}

		// inform the element viewer
		elementController.setInput(selectedPath);
	}

	private void handleAddition(final PathListEntry path, final boolean isAddedToPlan) {
		if (path.isPartOfPlan() != isAddedToPlan) {
			path.setPartOfPlan(isAddedToPlan);
			for (Gap gap : gapList) {
				if (path.providesLevel(gap)) {
					gap.usePlan(isAddedToPlan);
				}

				Collections.sort(this.pathList);
			}
		}
	}

	/**
	 * The Class BuildplanElementController.
	 */
	public class BuildplanElementController extends ElementController {

		private PathListEntry input;

		/**
		 * Instantiates a new buildplan element controller.
		 */
		public BuildplanElementController() {
			super(null, true, true);
			setMetadataExpanded(false);
		}

		/** {@inheritDoc} */
		@Override
		public String handleClickEvent(final ILPElement element) {
			if (input != null) {
				handleAddition(input, !input.isPartOfPlan());
			}
			return null;
		}

		/** {@inheritDoc} */
		@Override
		public String getButtonCaption() {
			String result = "";

			if (input != null) {
				result = Bundle.getLabel("pdp.buildplan." + (input.isPartOfPlan() ? "remove" : "add"));
			}

			return result;
		}

		/** {@inheritDoc} */
		@Override
		public boolean isButtonRequired() {
			// action button always required
			return true;
		}

		/**
		 * Sets the input for the element viewer.
		 * 
		 * @param input
		 *            the new input
		 */
		public void setInput(final PathListEntry input) {
			this.input = input;

			super.setInput(input != null ? input.getPath() : null);
		}

	};

	/**
	 * The Class CPLevel.
	 */
	private class CPLevel {
		private CompetenceProfileLevel profile;
		private HashMap<Long, CompetenceLevel> leafs;
		private ServiceContext ctx;

		public CPLevel(final CompetenceProfileLevel profileLevel, final ServiceContext ctx) throws SystemException {
			if (profileLevel == null) {
				throw new IllegalArgumentException("null competence profile level not allowed");
			}
			this.profile = profileLevel;
			this.ctx = ctx;
			this.leafs = new HashMap<Long, CompetenceLevel>();

			for (CompetenceLevel level : CompetenceLevelFacade.getCompetenceLevelsByIds(
					profile.getCompetenceLevelIds(), ctx)) {
				fetchLeafs(level);
			}
		}

		private void fetchLeafs(final CompetenceLevel level) throws SystemException {
			List<CompetenceLevel> children = CompetenceLevelFacade.getCompetenceLevelsByIds(level.getChildLevelIds(),
					ctx);
			if (children.size() == 0) {
				leafs.put(level.getPrimaryKey(), level);
			} else {
				for (CompetenceLevel child : children) {
					fetchLeafs(child);
				}
			}
		}

		public List<Gap> analyzeGaps() throws SystemException, PortalException {
			List<Gap> result = new ArrayList<Gap>();
			for (CompetenceLevel level : leafs.values()) {
				Competence competence = CompetenceFacade.getCompetence(level.getCompetenceId(), ctx);
				LevelValue attained = PdpService.getAttainedLevelForUser(ctx.getUserId(), competence, ctx);
				if (attained != null) {
					LevelValue required = LevelValueFacade.getLevelValue(level.getLevelValueId(), ctx);
					if (attained.getValue() < required.getValue()) {
						result.add(new Gap(level));
					}
				} else {
					result.add(new Gap(level));
				}
			}
			return result;
		}

	}

}
