/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.pdp.beans;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.faces.event.ActionEvent;

import org.tencompetence.business.model.learningpath.ICluster;
import org.tencompetence.business.model.learningpath.ICursor;
import org.tencompetence.business.model.learningpath.ILPElement;
import org.tencompetence.business.model.learningpath.ILearningPath;
import org.tencompetence.business.model.learningpath.ISelection;
import org.tencompetence.portlet.common.components.CrumbController;
import org.tencompetence.portlet.common.components.ElementController;
import org.tencompetence.portlet.common.components.ICrumbEntry;
import org.tencompetence.portlet.common.constants.Defines;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.ImageUtils;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.common.utils.UrlUtils;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;

/**
 * Backing bean for the PdpSelectAction.jspx.
 * 
 * @author Hubert Vogten
 * @version $Revision: 1.26 $, $Date: 2009/11/26 13:37:20 $
 */
public class PdpSelectAction {
	private static final Log LOG = LogFactoryUtil.getLog(PdpSelectAction.class);
	private final String normalUrl = UrlUtils.getWindowStateNormalUrl();
	private final BreadCrumbController crumbController;
	private LeafController elementController;
	private ICursor input;

	/**
	 * Instantiates a new pdp select action.
	 */
	public PdpSelectAction() {
		final Object obj = PortletUtils.getSessionMap().get(Defines.EDITOR_INPUT_PARAM_NAME);

		crumbController = new BreadCrumbController();

		if (obj instanceof ICursor) {
			setInput(getHotCursor((ICursor) obj));
		}
	}

	/**
	 * Sets the input.
	 * 
	 * @param input
	 *            the new input
	 */
	public void setInput(final ICursor input) {
		this.input = input;

		if (input != null) {
			ILPElement element = input.getCurrentElement();

			elementController = createController(element, !element.isCompleted());
			crumbController.setInput(input);
		}
	}

	/**
	 * Return the cluster controller associated with this backing bean.
	 * 
	 * @return the cluster controller
	 */
	public LeafController getElementController() {
		return elementController;
	}

	/**
	 * Gets the crumb controller.
	 * 
	 * @return the crumb controller
	 */
	public CrumbController getCrumbController() {
		return crumbController;
	}

	/**
	 * Return the title of the current input.
	 * 
	 * @return the title
	 */
	public String getTitle() {
		return input != null ? input.getLearningPlan().getTitle() : "";
	}

	/**
	 * Returns to the small view without saving the plan.
	 * 
	 * @return null as navigation is handled directly
	 */
	public String doCancel() {
		PortletUtils.redirect(normalUrl);
		return null;
	}

	private LeafController createController(final ILPElement element, final boolean selectable) {
		if (element instanceof ICluster) {
			ICluster cluster = (ICluster) element;
			switch (cluster.getType()) {
			case SEQUENCE:
				return new SequenceController(cluster, selectable);
			case FREE_ORDER:
				return new SelectionController(cluster, selectable);
			default:
				break;
			}
		}
		return new LeafController(element, selectable);
	}

	/**
	 * Returns a <code>Cursor</code> that offers the user a selection, closest to the passed cursor.
	 * 
	 * @param cursor
	 *            the original cursor
	 * 
	 * @return a <code>Cursor</code> that offers the user a selection, closest to the passed cursor
	 */
	private ICursor getHotCursor(final ICursor cursor) {
		boolean isRoot = false;
		while (!isRoot && cursor.getCurrentElement().isCompleted()) {
			isRoot = cursor.getLearningPlan().equals(cursor.pop());
		}
		return cursor;
	}

	private void navigateTo(final ILPElement element) {
		input.push(element);
		setInput(input);
	}

	private void navigateUp(final int steps) {
		if (input != null) {
			for (int i = 0; i < steps; i++) {
				input.pop();
			}
		}
		setInput(input);
	}

	private void persistCursor() {
		try {
			input.getLearningPlan().setCursor(input);
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	/**
	 * The Class LeafController.
	 */
	public class LeafController extends ElementController {
		private final boolean selectable;

		/**
		 * Instantiates a new leaf element controller.
		 * 
		 * @param element
		 *            the element controlled by this controller
		 * @param selectable
		 *            indicator which determines whether the user can selected this element
		 */
		public LeafController(final ILPElement element, final boolean selectable) {
			super(element, selectable, true);
			this.selectable = selectable;
			this.setMetadataExpanded(false);
		}

		/** {@inheritDoc} */
		@Override
		public String handleClickEvent(final ILPElement element) {
			if (element != null) {
				try {
					element.setSubscribed(!element.isSubscribed());
					persistCursor();
				} catch (Exception e) {
					PortletUtils.handleError(LOG, e);
				}
			}

			postSubscribe();

			return null;
		}

		/**
		 * Do the processing after the subscription.
		 */
		protected void postSubscribe() {
			doCancel();
		}

		/**
		 * Method should be called when the command link represented by this controller is clicked.
		 * 
		 * @param e
		 *            the event data
		 */
		public void commandLinkListener(final ActionEvent e) {
			navigateTo(getElement());
		}

		/**
		 * Returns true if the element represented by this controller may be selected by the user.
		 * 
		 * @return true if the element represented by this controller may be selected by the user
		 */
		public boolean isSelectable() {
			return selectable;
		}

		/**
		 * Return true if a list of child elements should be rendered.
		 * 
		 * @return true if a list of child elements should be rendered
		 */
		public boolean isElementList() {
			return false;
		}

		/**
		 * Return true if this element is a leaf.
		 * 
		 * @return true if this element is a leaf
		 */
		public boolean isLeaf() {
			return true;
		}

		/**
		 * Checks if is completed.
		 * 
		 * @return true, if is completed
		 */
		public boolean isCompleted() {
			return getElement().isCompleted();
		}

		/**
		 * Gets the icon for this element.
		 * 
		 * @return the icon
		 */
		public String getIcon() {
			if (isCompleted()) {
				return "/images/icons/check_16.png";
			} else {
				return ImageUtils.getImageUrl16(getElement(), false);
			}
		}

	};

	/**
	 * This class is an abstract controller for <code>ICluster</code> elements.
	 */
	public abstract class ClusterController extends LeafController {

		/**
		 * Instantiates a new cluster controller.
		 * 
		 * @param element
		 *            the element
		 * @param selectable
		 *            indicator whether the element represented by this controller can be selected
		 */
		public ClusterController(final ICluster element, final boolean selectable) {
			super(element, selectable);
		}

		/** {@inheritDoc} */
		@Override
		protected void postSubscribe() {
			PdpSelectAction.this.setInput(getHotCursor(input));
		}

		/**
		 * Gets the info text.
		 * 
		 * @return the info text
		 */
		public abstract String getInfoText();

		/** {@inheritDoc} */
		@Override
		public boolean isLeaf() {
			return false;
		}

		/** {@inheritDoc} */
		public boolean isElementList() {
			if (getElement() instanceof ILearningPath) {
				// LearningPaths and LearningPlans have to be subscribed before a user may continue
				return getElement().isSubscribed();
			}
			return true;
		}

		/**
		 * Returns the child elements of the current input or an empty list if the input was not set.
		 * 
		 * @return the child elements of the current input
		 */
		@SuppressWarnings("unchecked")
		public List<LeafController> getChildren() {
			final List<ILPElement> children = getElement() instanceof ICluster ? ((ICluster) getElement())
					.getChildren() : Collections.EMPTY_LIST;
			final List<LeafController> result = new ArrayList<LeafController>(children.size());

			for (final ILPElement child : children) {
				result.add(createController(child, ((ICluster) getElement()).canPerform(child)));
			}

			return result;
		}
	}

	/**
	 * The controller class for <code>ISequence</code> elements.
	 */
	public class SequenceController extends ClusterController {

		/**
		 * Instantiates a new sequence controller.
		 * 
		 * @param element
		 *            the element
		 * @param selectable
		 *            indicator whether the element represented by this controller can be selected
		 */
		public SequenceController(final ICluster element, final boolean selectable) {
			super(element, selectable);
		}

		/** {@inheritDoc} */
		public String getInfoText() {
			return Bundle.getLabel("pdp.selectaction.sequenceinfo");
		}
	}

	/**
	 * The controller class for <code>ISelection</code> elements.
	 */
	public class SelectionController extends ClusterController {

		/**
		 * Instantiates a new selection controller.
		 * 
		 * @param element
		 *            the element
		 * @param selectable
		 *            indicator whether the element represented by this controller can be selected
		 */
		public SelectionController(final ICluster element, final boolean selectable) {
			super(element, selectable);
		}

		/** {@inheritDoc} */
		public String getInfoText() {
			return Bundle.getLabel("pdp.selectaction.selectioninfo", new Object[] { ((ISelection) getElement())
					.getNumberToSelect() });
		}
	}

	/**
	 * The Class BreadCrumbController.
	 */
	public class BreadCrumbController extends CrumbController {
		private List<ICrumbEntry> crumbs;

		/** {@inheritDoc} */
		@Override
		public List<ICrumbEntry> getCrumbs() {
			return crumbs;
		}

		/** {@inheritDoc} */
		@Override
		public String handleCrumbEvent(final ICrumbEntry selectedCrumb) {
			int position = crumbs.indexOf(selectedCrumb);
			if (position > -1) {
				navigateUp(crumbs.size() - position - 1);
			}

			// stay on same page
			return null;
		}

		/**
		 * Sets the input.
		 * 
		 * @param cursor
		 *            the new input
		 */
		public void setInput(final ICursor cursor) {
			crumbs = new ArrayList<ICrumbEntry>(cursor.getElements().size());
			for (final ILPElement element : cursor.getElements()) {
				crumbs.add(new CrumbEntry(element, formatTitle(element.getTitle())));
			}
		}

	}

	/**
	 * The Class CrumbEntry.
	 */
	public class CrumbEntry implements ICrumbEntry {
		private final String title;

		/**
		 * Instantiates a new crumb entry.
		 * 
		 * @param element
		 *            the element
		 * @param title
		 *            the title
		 */
		public CrumbEntry(final ILPElement element, final String title) {
			this.title = title;
		}

		/** {@inheritDoc} */
		//@Override
		public String getTitle() {
			return title;
		}

		/** {@inheritDoc} */
		//@Override
		public String getViewId() {
			return null;
		}
	}
}