/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.pdp.beans;

import java.util.ArrayList;
import java.util.List;

import javax.portlet.PortletPreferences;

import org.tencompetence.business.facade.CompetenceProfileLevelFacade;
import org.tencompetence.business.facade.GoalFacade;
import org.tencompetence.business.model.learningpath.ILearningAction;
import org.tencompetence.business.model.learningpath.ILearningPlan;
import org.tencompetence.business.services.LearningPlanService;
import org.tencompetence.business.services.PdpService;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.portlet.common.components.GoalTitleController;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.goal.beans.GoalManager;
import org.tencompetence.portlet.goal.beans.GoalUtil;
import org.tencompetence.portlet.pdp.controllers.FilterController;
import org.tencompetence.portlet.pdp.controllers.GenerateStatusType;
import org.tencompetence.portlet.pdp.controllers.PathDetailsController;
import org.tencompetence.portlet.pdp.controllers.PlanGenerator;
import org.tencompetence.portlet.pdp.controllers.Task;
import org.tencompetence.portlet.pdp.controllers.ViewScreenController;

import com.icesoft.faces.async.render.Renderable;
import com.icesoft.faces.context.DisposableBean;
import com.icesoft.faces.webapp.xmlhttp.PersistentFacesState;
import com.icesoft.faces.webapp.xmlhttp.RenderingException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * This is the backing bean for the Activity Navigator (pdp) portlet.
 * 
 * @author venrooijma
 * @author hvo
 * @author Harrie Martens
 * @version $Revision: 1.44 $, $Date: 2009/12/01 09:37:45 $
 */
public class PdpSmallView implements Renderable, DisposableBean {
	private static final int PERCENT_100 = 100;
	private static final Log LOG = LogFactoryUtil.getLog(PdpSmallView.class);
	
	private PersistentFacesState state;
	private ServiceContext ctx;
	private GoalManager goalManager;

	// The controllers
	private GoalTitleController goalTitleController;
	private PlanGenerator generator;
	private ViewScreenController viewScreen;
	private FilterController optionsScreen;
	private PathDetailsController activityScreen;
	
	// The model entities
	private Goal selectedGoal;
	private ILearningPlan currentPlan;
	private CompetenceProfileLevel competenceProfileLevel;
	
	private GenerateStatusType status;
	
	/**
	 * The Enum Screen specifies the different screens in the pdp portlet.
	 */
	private enum Screen {
		MAIN, VIEW, OPTIONS, ACTIVITY
	};

	private Screen activeScreen;
	private boolean feedbackRendered = false;
	private boolean needsInit = true;
	private boolean hasGoalAttained = false;
	private boolean planAvailable = false;
	private boolean smallViewAvailable;
	private String message = "";

	private List<Task> tasks;

	/**
	 * Instantiates a new pdp small view.
	 */
	public PdpSmallView() {
		// initialize the view
		state = PersistentFacesState.getInstance();
		activeScreen = Screen.MAIN;
	}

	/**
	 * getTasks returns all tasks for the current plan.
	 * 
	 * @return List<Task> containing all tasks for the current plan
	 */
	public List<Task> getTasks() {
		return tasks;
	}

	/**
	 * Gets the controller for the options screen.
	 * 
	 * @return the controller for the options screen
	 */
	public FilterController getOptionsController() {
		return optionsScreen;
	}

	/**
	 * Gets the controller for the view route screen.
	 * 
	 * @return the controller for the view route screen
	 */
	public ViewScreenController getViewController() {
		return viewScreen;
	}
	
	/**
	 * Gets the controller for the view activity screen.
	 * 
	 * @return the controller for the view activity screen
	 */
	public PathDetailsController getActivityController() {
		return activityScreen;
	}

	/**
	 * Gets the message.
	 * 
	 * @return message error message in case aborted calculation
	 */
	public String getMessage() {
		return message;
	}

	/**
	 * Gets the feedback describing the outcome of the route generation.
	 * 
	 * @return the feedback for the route generation
	 */
	public String getFeedback() {
		if (status != null) {
			return Bundle.getLabel("pdp.generate.status." + status);
		}
		
		return "";
	}

	/**
	 * Sets the goal manager. 
	 * <p />
	 * The goal manager is injected by Spring.
	 * 
	 * @param goalManager
	 *            the new goal manager
	 */
	public void setGoalManager(final GoalManager goalManager) {
		this.goalManager = goalManager;
	}

	/**
	 * Dummy setter.
	 * 
	 * @param value
	 *            the value
	 */
	public void setSelectedGoal(final String value) {
		System.out.println("In dummy set selected goal");
	}

	/**
	 * Checks if is small view button available.
	 * 
	 * @return true, if is small view button available
	 */
	public boolean isSmallViewAvailable() {
		initFields();
		return smallViewAvailable;
	}

	/**
	 * Checks if the feedback should be rendered.
	 * 
	 * @return true, if the feedback should be rendered
	 */
	public boolean isFeedbackRendered() {
		return feedbackRendered;
	}

	/**
	 * Checks if the main screen is active.
	 * 
	 * @return true, if the main screen is active
	 */
	public boolean isMainActive() {
		return activeScreen == Screen.MAIN;
	}

	/**
	 * Checks if the options screen is active.
	 * 
	 * @return true, if the options screen is active
	 */
	public boolean isOptionsActive() {
		return activeScreen == Screen.OPTIONS;
	}

	/**
	 * Checks if the view route screen is active.
	 * 
	 * @return true, if the view route screen is active
	 */
	public boolean isViewActive() {
		return activeScreen == Screen.VIEW;
	}

	/**
	 * Checks if the view activity screen is active.
	 * 
	 * @return true, if the view activity screen is active
	 */
	public boolean isActivityActive() {
		return activeScreen == Screen.ACTIVITY;
	}

	/**
	 * Checks if the targeted competence profile has associated competence levels.
	 * 
	 * @return true, if the profile has no competence levels
	 */
	public boolean isEmptyProfile() {
		if (competenceProfileLevel != null) {
			return "".equals(competenceProfileLevel.getCompetenceLevelIds().trim());
		}
		return true;
	}

	/**
	 * Checks if a learning plan is available.
	 * 
	 * @return true, if a learning plan is available
	 */
	public boolean isPlanAvailable() {
		initFields();
		return planAvailable;
	}

	/**
	 * Checks if the user has attained the goal.
	 * 
	 * @return true, if the user has attained the goal
	 */
	public boolean isGoalAttained() {
		initFields();
		return hasGoalAttained;
	}

	/**
	 * Checks if all tasks are completed.
	 * 
	 * @return true, if all tasks are completed
	 */
	public boolean isTasksCompleted() {
		boolean result = true;
		for (Task task : tasks) {
			if (!task.isCompleted()) {
				result = false;
			}
		}
		return result;
	}

	/**
	 * Generates a new learning plan for the selected goal.
	 * 
	 * @return null as navigation is handled internally
	 */
	public String doGeneratePlan() {
		feedbackRendered = true;
		status = generator.doGenerate(currentPlan);
		try {
			currentPlan = generator.updatePlan(currentPlan);
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
		planChanged();
		return null;
	}

	/**
	 * Handle the view route button by showing the view screen.
	 * 
	 * @return null, as navigation is handled by the code
	 */
	public String doView() {
		activeScreen = Screen.VIEW;
		return null;
	}

	/**
	 * Handle the options button by showing the options screen.
	 * 
	 * @return null, as navigation is handled by the code
	 */
	public String doOptions() {
		activeScreen = Screen.OPTIONS;
		return null;
	}

	/**
	 * Handle the back button by navigating to main screen.
	 * 
	 * @return null, as navigation is handled by the code
	 */
	public String doBack() {
		activeScreen = Screen.MAIN;
		return null;
	}

	/**
	 * Hides the feedback window.
	 * 
	 * @return null as navigation is handled directly
	 */
	public String doHide() {
		feedbackRendered = false;
		return null;
	}

	/** {@inheritDoc} */
	//@Override
	public PersistentFacesState getState() {
		return state;
	}

	/** {@inheritDoc} */
	//@Override
	public void renderingException(final RenderingException e) {
		// An FatalRenderingException will occur if a user has closed his browser
		// the update functionality tries to render a view that doesn't exist anymore so only for debugging there is
		// some logging.
		if (LOG.isDebugEnabled()) {
			LOG.debug(e);
		}
	}

	/**
	 * Dispose callback called due to a view closing or session invalidation/timeout.
	 * 
	 * clean closed views.
	 * 
	 * @throws Exception
	 *             the exception
	 */
	public void dispose() throws Exception {
		try {
			if (goalManager.hasListener(this)) {
				goalManager.removeListener(this);
				selectedGoal = null;
			}
		} catch (Exception failedCleanup) {
			if (LOG.isErrorEnabled()) {
				LOG.error("Failed to cleanup", failedCleanup);
			}
		}
	}

	/**
	 * Gets the goal title controller.
	 * 
	 * @return the goal title controller
	 */
	private GoalTitleController getGoalTitleController() throws Exception {
		if (goalTitleController == null) {
				goalTitleController = new GoalTitleController(selectedGoal, ctx);
		}
		return goalTitleController;
	}

	private void recalculateProgress(final ILearningPlan plan, final ServiceContext ctx,
			final CompetenceProfileLevel competenceProfileLevel) {

		try {
			PdpService.PDPProgress progress;
			progress = PdpService.getProgressByProfileLevelId(plan, competenceProfileLevel
					.getCompetenceProfileLevelId(), ctx);
			int attained = progress.getAttained();
			
			viewScreen = new ViewScreenController(getGoalTitleController(), currentPlan, attained,
					progress.getPlanned(), progress.getRest());


			hasGoalAttained = (attained == PERCENT_100);
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	private boolean isSameGoal(final Goal newGoal) {
		if (newGoal == null) {
			return selectedGoal == null;
		}

		if (!newGoal.equals(selectedGoal)) {
			return false;
		}

		return newGoal.getTarget() == selectedGoal.getTarget();
	}

	private boolean setInput() {
		Goal newGoal = null;

		// Check if goal is preset via the preferences screen of the portlet
		PortletPreferences prefs = PortletUtils.getPortletRequest().getPreferences();
		String prefGoalId = prefs.getValue(PdpPreferences.SELECTED_GOAL, "0");

		if (!prefGoalId.equals("0")) {
			// Goal is preset in the preferences screen

			// In case of an already registered view remove it
			if (goalManager.hasListener(this)) {
				goalManager.removeListener(this);
			}

			final long goalId = Long.parseLong(prefGoalId);
			try {
				newGoal = GoalFacade.getGoal(goalId, PortletUtils.getServiceContext());
			} catch (Exception e) {
				PortletUtils.handleError(LOG, e);
			}

		} else {
			// no goal preset in the preferences, so get it from the goal manager
			goalManager.registerListener(this);
			newGoal = goalManager.getSelectedGoal(PortletUtils.getCurrentUserId(), PortletUtils.getGroupId());
		}

		if (!isSameGoal(newGoal)) {
			selectedGoal = newGoal;
			needsInit = true;
			return true;
		}

		return false;

	}

	private void initFields() {

		setInput();

		if (!needsInit) {
			return;
		}

		needsInit = false;
		currentPlan = null;
		goalTitleController = null;

		smallViewAvailable = true;
		if (selectedGoal == null) {
			this.smallViewAvailable = false;
			message = Bundle.getLabel("error.noGoalSelected");
			return;
		}
		if (!GoalUtil.checkValidTargetType(selectedGoal)) {
			this.smallViewAvailable = false;
			message = Bundle.getLabel("pdp.error.noValidTargetSelected");
			return;
		}

		try {
			ctx = PortletUtils.getServiceContext();
			
			optionsScreen = new FilterController();
			generator = new PlanGenerator(selectedGoal, optionsScreen, ctx);

			competenceProfileLevel = CompetenceProfileLevelFacade.getCompetenceProfileLevel(selectedGoal.getTarget(),
					ctx);
			currentPlan = LearningPlanService.getLearningPlanForUserByGoalId(selectedGoal.getUserId(), selectedGoal
					.getPrimaryKey(), ctx);

			planChanged();
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	private void planChanged() {
		try {
			tasks = new ArrayList<Task>();
			planAvailable = (currentPlan != null);

			recalculateProgress(currentPlan, ctx, competenceProfileLevel);

			if (planAvailable) {
				tasks.add(new Task(currentPlan) {

					@Override
					public void onActivityCompleted() {
						recalculateProgress(currentPlan, ctx, competenceProfileLevel);
					}

					@Override
					public void onActivityView(final ILearningAction action) {
						if (activityScreen == null) {
							activityScreen = new PathDetailsController(action);
						} else {
						 activityScreen.setInput(action);
						}
						activeScreen = Screen.ACTIVITY;
					}
				});
			}
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}

	}
}