/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.pdp.controllers;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.tencompetence.business.services.FilterOptionsService;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.pdp.util.Filter;
import org.tencompetence.portlet.pdp.util.FilterOption;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * This class is the controller for the pdpFilterOptions.jspx page.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @version $Revision: 1.8 $, $Date: 2009/11/30 14:34:23 $
 */
public class FilterController {
	private static final Log LOG = LogFactoryUtil.getLog(FilterController.class);
	private final Filter filter = new Filter();
	private boolean isCollapsed = true;
	private static final FilterOption[] SUPPORTED_FILTERS = { new FilterOption("provider"),
			new FilterOption("languagelist") };

	/**
	 * Instantiates a new filter controller.
	 */
	public FilterController() {
		initFilterOptions();
	}

	private void initFilterOptions() {

		try {
			ServiceContext ctx = PortletUtils.getServiceContext();
			Map<String, String> filterOptions = FilterOptionsService.readFilterOptions(ctx);
			if (filterOptions != null) {
				
				for (FilterOption option : SUPPORTED_FILTERS) {
					//update filters with stored values
					if (filterOptions.containsKey(option.getKey())) {
						option.setValue(filterOptions.get(option.getKey()));
						filter.setFilterOption(option);
					}
				}
			}
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	/**
	 * Gets the filter value.
	 * 
	 * @return the filter value
	 */
	public String getFilterValue() {
		String filterText = filter.toString();
		return "".equals(filterText) ? Bundle.getLabel("pdp.filter.nofilter") : filterText;
	}

	/**
	 * Returns list of the filter values. Last filter value of the list is marked.
	 * 
	 * @return the filter values
	 */
	public List<FilterOptionLabel> getFilterValues() {
		final List<FilterOptionLabel> result = new ArrayList<FilterOptionLabel>();
		FilterOptionLabel last = null;
		for (FilterOption option : filter.getFilterOptions()) {
			last = new FilterOptionLabel(option.getKey(), option.getValue(), false);
			result.add(last);
		}

		if (last == null) {
			result.add(new FilterOptionLabel("", "", true));
		} else {
			last.setLast(true);
		}

		return result;
	}

	private void saveUserOptions() {
		try {
			ServiceContext ctx = PortletUtils.getServiceContext();
			Map<String, String> map = new HashMap<String, String>();
			for (FilterOption option : filter.getFilterOptions()) {
				map.put(option.getKey(), option.getValue());
			}
			FilterOptionsService.writeFilterOptions(map, ctx);
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	/**
	 * Checks if the goal selection box is collapsed.
	 * 
	 * @return true, if it is collapsed
	 */
	public boolean isCollapsed() {
		return isCollapsed;
	}

	/**
	 * Handle the collapse hyperlink by toggling between the collapse modes.
	 * 
	 * @return null because navigation is not needed
	 */
	public String doCollapse() {
		isCollapsed = !isCollapsed;

		return null;
	}

	/**
	 * Gets the internationalized collapse text.
	 * 
	 * @return the internationalized collapse text
	 */
	public String getCollapseText() {
		return Bundle.getLabel("pdp.filter.collapse." + (isCollapsed ? "open" : "close"));
	}

	/**
	 * Gets the supported filters option, updated with the current filter values.
	 * 
	 * @return the supported filters
	 */
	public List<FilterOption> getSupportedFilters() {
		return Arrays.asList(SUPPORTED_FILTERS);
	}

	/**
	 * Apply filters options to filter.
	 */
	public void doApply() {
		isCollapsed = true;
		for (FilterOption option : SUPPORTED_FILTERS) {
			filter.setFilterOption(option);
		}
		saveUserOptions();
	}

	/**
	 * Gets the filter associated with this controller.
	 * 
	 * @return the filter
	 */
	public Filter getFilter() {
		return filter;
	}

	/**
	 * The Class FilterOptionLabel. Extends normal FilterOption by adding marker for last entry.
	 */
	public class FilterOptionLabel extends FilterOption {
		private boolean last;

		/**
		 * Instantiates a new filter option label.
		 * 
		 * @param key
		 *            the key
		 * @param value
		 *            the value
		 * @param last
		 *            the last
		 */
		public FilterOptionLabel(final String key, final String value, final boolean last) {
			super(key);
			setValue(value);
			this.last = last;
		}

		/**
		 * Checks if is last.
		 * 
		 * @return true, if is last
		 */
		public boolean isLast() {
			return last;
		}

		/**
		 * Sets the last.
		 * 
		 * @param last
		 *            the new last
		 */
		public void setLast(final boolean last) {
			this.last = last;
		}

		/** {@inheritDoc} */
		@Override
		public String toString() {
			if (!"".equals(getKey())) {
				return super.toString();
			} else {
				return Bundle.getLabel("pdp.filter.nofilter");
			}
		}
	}
}
