/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.pdp.controllers;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.business.model.learningpath.ILPElement;
import org.tencompetence.business.model.learningpath.MetaDataEntry;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.ImageUtils;

import com.liferay.portal.kernel.util.StringPool;

/**
 * This abstract class implements the controller for the lpElement facelet.
 * <p />
 * It provides the facelet with the data from the lpElement passed to the controller. It optionally shows a action
 * button. Default this button is shown when:
 * <ul>
 * <li>true is passed for the hasButton parameter in the class constructor</li>
 * <li>the element shows is either a {@code ILearningAction} or a {@code ILearningPath}</li>
 * <li>the element is not completed</li>
 * </ul>
 * 
 * When the user clicks the button the abstract handleClickEvent method is called. Concrete implementations of this
 * controller need to subclass and implement this method to handle the event.
 * <p />
 * 
 * @author Harrie Martens
 * @version $Revision: 1.3 $, $Date: 2009/11/30 08:58:58 $
 */

public class PathDetailsController {

	private static final int DESCRIPTION_THRESHOLD = 500;

	private ILPElement element;
	private String description;
	private boolean descriptionExpanded;
	private boolean metadataExpanded;

	/**
	 * Instantiates a new element controller.
	 * 
	 * @param element
	 *            the ILPElement that is shown in the facelet.
	 */
	public PathDetailsController(final ILPElement element) {
		this.metadataExpanded = false;
		setInput(element);
	}

	/**
	 * Gets the title.
	 * 
	 * @return the title
	 */
	public String getTitle() {
		return this.element != null ? this.element.getTitle() : StringPool.BLANK;
	}

	/**
	 * Gets the description.
	 * 
	 * @return the description
	 */
	public String getDescription() {
		if (!this.descriptionExpanded) {
			String result = description.substring(0, DESCRIPTION_THRESHOLD);
			int i = result.lastIndexOf(' ');
			if (i != -1) {
				result = result.substring(0, i);
			}
			return result + "...";
		}

		return this.description;
	}

	/**
	 * Gets the 48x48 image url.
	 * 
	 * @return the image url
	 */
	public String getImageUrl() {
		return ImageUtils.getImageUrl32(element);
	}

	/**
	 * Gets the meta data entries from the element.
	 * <p>
	 * Returns an empty list if the element is null.
	 * 
	 * @return the meta data
	 */
	public List<MetaDataEntry> getMetadata() {
		return element != null ? element.getMetaData() : new ArrayList<MetaDataEntry>(0);
	}

	/**
	 * Gets the toggle description text.
	 * 
	 * @return the toggle description text
	 */
	public String getToggleDescriptionText() {
		return Bundle.getLabel("elementcontroller.description." + (this.descriptionExpanded ? "hide" : "show"));
	}

	/**
	 * Gets the toggle meta data description.
	 * 
	 * @return the toggle meta data description
	 */
	public String getToggleMetadataDescription() {
		return Bundle.getLabel("elementcontroller.metadata." + (this.metadataExpanded ? "hide" : "show"));
	}

	/**
	 * Gets the element associated with this controller.
	 * 
	 * @return the element
	 */
	public ILPElement getElement() {
		return this.element;
	}

	/**
	 * Checks if the description is longer than the threshold.
	 * 
	 * @return true, if the description is longer than the threshold.
	 */
	public boolean isLongDescription() {
		return this.description.length() > DESCRIPTION_THRESHOLD;
	}

	/**
	 * Checks if the meta data is shown.
	 * 
	 * @return true, if the meta data is shown
	 */
	public boolean isMetadataExpanded() {
		return this.metadataExpanded;
	}

	/**
	 * The server faces action handler for the show / hide description command.
	 * <p>
	 * Toggles the state of the long description text.
	 * 
	 * @return null as navigation is not required
	 */
	public String doToggleDescription() {
		this.descriptionExpanded = !this.descriptionExpanded;
		return null;
	}

	/**
	 * The server faces action handler for the show / hide meta data command.
	 * 
	 * @return null as navigation is not required
	 */
	public String doToggleMetadata() {
		this.metadataExpanded = !this.metadataExpanded;
		return null;
	}

	/**
	 * Sets the meta data expanded flag.
	 * 
	 * @param value
	 *            the value for the meta data expanded flag
	 */
	protected void setMetadataExpanded(final boolean value) {
		this.metadataExpanded = value;
	}

	/**
	 * Sets the ILPElement input for this controller.
	 * <p>
	 * This input should not be null, otherwise an IllegalArgumentException is thrown
	 * 
	 * @param input
	 *            the new input for the controller
	 */
	public void setInput(final ILPElement input) {
		this.element = input;
		this.description = element != null ? (element.getDescription() == null ? StringPool.BLANK : element
				.getDescription()) : StringPool.BLANK;
		this.descriptionExpanded = this.description.length() <= DESCRIPTION_THRESHOLD;
	}

}