/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.pdp.controllers;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.business.model.learningpath.ILearningPath;
import org.tencompetence.portlet.common.utils.PortletUtils;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;


/**
 * The Class {@code PathListEntry} encapsulates a {@code ILearningPath} for usage in the paths list.
 * 
 * @author Harrie Martens
 * @version $Revision: 1.4 $, $Date: 2009/10/28 13:01:21 $
 */
public class PathListEntry implements Comparable<PathListEntry> {
	private static final Log LOG = LogFactoryUtil.getLog(PathListEntry.class);
	private final ILearningPath path;
	private boolean partOfPlan;
	private List<Gap> coveredGaps;
	private boolean selected;

	/**
	 * Instantiates a new path data.
	 * 
	 * @param path
	 *            the path
	 * @param gaps
	 *            the gaps
	 */
	public PathListEntry(final ILearningPath path, final List<Gap> gaps) {
		if (path == null) {
			throw new IllegalArgumentException("Null value not allowed for PathListEntry()");
		}
		this.path = path;
		this.partOfPlan = false;
		this.coveredGaps = scanForCoverage(gaps);
		this.selected = false;
	}

	/**
	 * Gets the title.
	 * 
	 * @return the title
	 */
	public String getTitle() {
		return path.getTitle();
	}

	/**
	 * Gets the coverage description.
	 * <p>
	 * This description indicates the number of covered open gaps and the total number of covered gaps.
	 * 
	 * @return the coverage description
	 */
	public String getCoverage() {
		if (this.isPartOfPlan()) {
			return "   - / " + this.coveredGaps.size();
		}
	
		return getOpenGapCount() + " / " + this.coveredGaps.size();
	}

	/**
	 * Gets the selected indicator.
	 * 
	 * @return the selected indicator
	 */
	public boolean getSelected() {
		return this.selected;
	}

	/**
	 * Checks if the user is subscribed to the path.
	 * 
	 * @return true, if the path is subscribed
	 */
	public boolean isSubscribed() {
		return path.isSubscribed();
	}

	/**
	 * Checks if the path is part of the learning plan.
	 * 
	 * @return true, if the path is part of the plan
	 */
	public boolean isPartOfPlan() {
		return this.partOfPlan;
	}

	/**
	 * Sets the part of plan.
	 * 
	 * @param partOfPlan
	 *            the new part of plan
	 */
	public void setPartOfPlan(final boolean partOfPlan) {
		this.partOfPlan = partOfPlan;
	}

	/**
	 * Checks if the plan covers the gap by providing at least the required competence level.
	 * 
	 * @param gap
	 *            the gap to check
	 * 
	 * @return true, if the plan covers the gap
	 */
	public boolean providesLevel(final Gap gap) {
		boolean result = false;
		try {
			result = path.providesLevel(gap.getLevel());
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
		return result;

	}

	/**
	 * Sets the selected.
	 * 
	 * @param value
	 *            the new selected
	 */
	public void setSelected(final boolean value) {
		this.selected = value;
	}

	/**
	 * Gets the path.
	 * 
	 * @return the path
	 */
	public ILearningPath getPath() {
		return this.path;
	}

	/**
	 * Gets the uri.
	 * 
	 * @return the uri
	 */
	public String getUri() {
		return this.path.getUri();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((path == null) ? 0 : path.hashCode());
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		PathListEntry other = (PathListEntry) obj;

		if (path == null) {
			if (other.path != null) {
				return false;
			}
		} else if (!path.equals(other.path)) {
			return false;
		}
		return true;
	}

	//@Override
	public int compareTo(final PathListEntry that) {
		if (this.equals(that)) {
			// both paths are equal
			return 0;
		}
		
		if (that == null) {
			return -1;
		}
		
		if (this.isPartOfPlan() != that.isPartOfPlan()) {
			// selected paths come first
			return this.isPartOfPlan() ? -1 : 1;
		}
		
		if (this.getOpenGapCount() != that.getOpenGapCount()) {
			// next sort on open gaps covered count
			return this.getOpenGapCount() > that.getOpenGapCount() ? -1 : 1;
		}
		
		if (this.coveredGaps.size() != that.coveredGaps.size()) {
			// next sort on total gaps covered count
			return this.coveredGaps.size() > that.coveredGaps.size() ? -1 : 1;
		}
				
		// and last sort alphabetically on title
		return this.getTitle().compareToIgnoreCase(that.getTitle());
	}

	private List<Gap> scanForCoverage(final List<Gap> gaps) {
		 List<Gap> result = new ArrayList<Gap>();
		 for (Gap gap : gaps) {
			 if (providesLevel(gap)) {
				 result.add(gap);
			 }
		 }
		 return result;
	}

	private int getOpenGapCount() {
		int openGaps = 0;
		for (Gap gap : this.coveredGaps) {
			if (!gap.isCovered()) {
				openGaps++;
			}
		}
		return openGaps;
	}
	
}
