/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.pdp.controllers;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.business.facade.CompetenceFacade;
import org.tencompetence.business.facade.CompetenceLevelFacade;
import org.tencompetence.business.facade.CompetenceProfileLevelFacade;
import org.tencompetence.business.facade.LevelValueFacade;
import org.tencompetence.business.model.learningpath.ILearningPath;
import org.tencompetence.business.model.learningpath.ILearningPlan;
import org.tencompetence.business.model.learningpath.MetaDataEntry;
import org.tencompetence.business.services.LearningPathService;
import org.tencompetence.business.services.LearningPlanService;
import org.tencompetence.business.services.PdpService;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.generated.model.LevelValue;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.pdp.util.FilterOption;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;

import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * The plan generator generates a new learning plan for a specific goal/user.
 * <p>
 * First a gap analysis on the goal is performed, next all relevant learning paths are fetched from the db. A path is
 * considered relevant if it targets at least one of the competences the user is still missing for this goal. In the
 * final step the best paths are combined to create a new learning plan for the user.
 * 
 * 
 * @author Harrie Martens
 * @version $Revision: 1.3 $, $Date: 2009/11/30 14:34:23 $
 */
public class PlanGenerator {
	private static final Log LOG = LogFactoryUtil.getLog(PlanGenerator.class);

	private GoalImpl goal;
	private ServiceContext ctx;
	private FilterController filterController;
	private List<PathEntry> foundPaths;
	private boolean planGenerated;
	private boolean completePlan;

	/**
	 * Instantiates a new pan generator controller.
	 * 
	 * @param goal
	 *            the goal
	 * @param filterController
	 *            the filter controller
	 * @param context
	 *            the context
	 * 
	 * @throws PortalException
	 *             the portal exception
	 * @throws SystemException
	 *             the system exception
	 */
	public PlanGenerator(final Goal goal, final FilterController filterController, final ServiceContext context)
			throws SystemException, PortalException {
		this.goal = new GoalImpl(goal);
		this.filterController = filterController;
		this.ctx = context;
		
		this.planGenerated = false;
		this.completePlan = false;
		this.foundPaths = new ArrayList<PathEntry>();
	}

	/**
	 * Gets the title.
	 * 
	 * @return the title
	 */
	public String getTitle() {
		return goal != null ? goal.getTitle() : "";
	}

	/**
	 * Gets the filter controller.
	 * 
	 * @return the filter controller
	 */
	public FilterController getFilterController() {
		return filterController;
	}

	/**
	 * Gets the generated plan as a list of found paths.
	 * 
	 * @return the generated plan
	 */
	public List<PathEntry> getGeneratedPlan() {
		return foundPaths;
	}

	/**
	 * Checks if is plan generated.
	 * 
	 * @return true, if is plan generated
	 */
	public boolean isPlanGenerated() {
		return planGenerated;
	}

	/**
	 * Checks if is no paths available.
	 * 
	 * @return true, if is no paths available
	 */
	public boolean isNoPathsAvailable() {
		return foundPaths.isEmpty();
	}

	/**
	 * Checks if is complete plan.
	 * 
	 * @return true, if is complete plan
	 */
	public boolean isCompletePlan() {
		return completePlan;
	}

	/**
	 * Checks if is plan available.
	 * 
	 * @return true, if is plan available
	 */
	public boolean isPlanAvailable() {
		return planGenerated && !foundPaths.isEmpty();
	}

	/**
	 * Accepts and saves the generated plan and returns to the small view.
	 * 
	 * @param plan
	 *            the plan
	 * 
	 * @return null as navigation is handled directly
	 * 
	 * @throws PortalException
	 *             the portal exception
	 * @throws SystemException
	 *             the system exception
	 */
	public ILearningPlan updatePlan(final ILearningPlan plan) throws SystemException, PortalException {
		ILearningPlan result = plan;
		// Always save the selected LP's in the database (also when listsize is 0).
		if (plan == null) {
			result = LearningPlanService.addLearningPlan(ctx.getUserId(), goal.goal, goal.getTitle(), ctx);
		}
			
		List<ILearningPath> selected = new ArrayList<ILearningPath>();
		for (PathEntry entry : foundPaths) {
			selected.add(entry.getPath());
		}
		result.setChildren(selected);
		try {
			LearningPlanService.updateLearningPlan(result, ctx);
			result = LearningPlanService.getLearningPlan(result.getUri(), ctx);
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
		return result;
	}

	/**
	 * Generates a new personal development plan for the current goal.
	 * <p>
	 * When generating the plan apply the filter options set by the user.
	 * 
	 * @param originalPlan
	 *            the original plan used to compare the outcome of the process
	 * 
	 * @return the generate status type providing the outcome of the generation proces
	 */
	public GenerateStatusType doGenerate(final ILearningPlan originalPlan) {
		GenerateStatusType result = GenerateStatusType.NONE;
		
		foundPaths = new ArrayList<PathEntry>();
		List<CompetenceLevel> gaps = new ArrayList<CompetenceLevel>();
		List<ILearningPath> availablePaths = new ArrayList<ILearningPath>();

		// Fetch the list of competence gaps for the current goal
		for (CompetenceImpl gap : goal.getGaps()) {
			gaps.add(gap.getRequiredCompetenceLevel());
		}

		// Find learning paths aimed at these gaps
		try {
			for (ILearningPath path : LearningPathService.searchLearningPaths(gaps, ctx)) {

				// Only allow paths that apply to the filter options
				if (passesFilter(path)) {
					availablePaths.add(path);
				}
			}
			completePlan = fillGaps(gaps, availablePaths);
			planGenerated = true;

			if (completePlan) {
				result = GenerateStatusType.COMPLETE; 
			} else {
				// Not all gaps are covered
				result = foundPaths.isEmpty() ? GenerateStatusType.NONE : GenerateStatusType.PARTIAL;
			}
			
			// Check if the generate plan equals the old plan

			// Start of debugging info
			System.out.println("generation result : " + result);
			for (PathEntry path : foundPaths) {
				System.out.println(path.getPath().getUri() + "->" + path.getTitle());
			}
			// End of debugging info

		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
		
		return result;
	}

	private boolean fillGaps(final List<CompetenceLevel> gaps, final List<ILearningPath> paths) throws SystemException,
			PortalException {

		if (gaps.isEmpty()) {
			return true;
		}

		ILearningPath bestPath = null;
		int maxCoverCount = 0;

		// Search for the path covering the most gaps
		for (ILearningPath path : paths) {
			int count = getCoveredCount(path, gaps);
			if (count > maxCoverCount) {
				maxCoverCount = count;
				bestPath = path;
			}
		}

		// Found a best plan
		if (bestPath != null) {
			List<CompetenceLevel> remaining = new ArrayList<CompetenceLevel>();
			for (CompetenceLevel gap : gaps) {
				if (!bestPath.providesLevel(gap)) {
					remaining.add(gap);
				}
			}

			paths.remove(bestPath);
			foundPaths.add(new PathEntry(bestPath));
			return fillGaps(remaining, paths);
		}

		return false;
	}

	private int getCoveredCount(final ILearningPath path, final List<CompetenceLevel> gaps) throws SystemException,
			PortalException {
		int count = 0;

		for (CompetenceLevel gap : gaps) {
			if (path.providesLevel(gap)) {
				count++;
			}
		}

		return count;
	}

	private boolean passesFilter(final ILearningPath path) {
		final List<MetaDataEntry> metadata = path.getMetaData();

		for (FilterOption filter : filterController.getFilter().getFilterOptions()) {
			MetaDataEntry entry = getEntryByKey(filter.getKey(), metadata);
			if (entry != null) {
				if (!entry.getValue().equalsIgnoreCase(filter.getValue())) {
					return false;
				}
			} else {
				return false;
			}
		}
		return true;
	}

	private MetaDataEntry getEntryByKey(final String key, final List<MetaDataEntry> metadata) {
		for (MetaDataEntry entry : metadata) {
			if (entry.getKey().equalsIgnoreCase(key)) {
				return entry;
			}
		}
		return null;
	}

	/**
	 * The Class PathEntry.
	 */
	public class PathEntry {
		private ILearningPath path;
		private boolean selected;

		/**
		 * Instantiates a new path entry.
		 * 
		 * @param path
		 *            the path
		 */
		public PathEntry(final ILearningPath path) {
			if (path == null) {
				throw new IllegalArgumentException("path should not be null");
			}
			this.path = path;
		}

		/**
		 * Gets the path.
		 * 
		 * @return the path
		 */
		public ILearningPath getPath() {
			return path;
		}

		/**
		 * Gets the selected state of this entry.
		 * 
		 * @return true if this entry is selected
		 */
		public boolean getSelected() {
			return selected;
		}

		/**
		 * Gets the title.
		 * 
		 * @return the title
		 */
		public String getTitle() {
			return path.getTitle();
		}

		/**
		 * Sets the selected state.
		 * 
		 * @param selected
		 *            the new selected state
		 */
		public void setSelected(final boolean selected) {
			this.selected = selected;
		}
	}

	/**
	 * The Class GoalImpl.
	 */
	private class GoalImpl {

		private final Goal goal;
		private final List<CompetenceImpl> competences;

		public GoalImpl(final Goal goal) throws SystemException, PortalException {
			if (goal == null) {
				throw new IllegalArgumentException("Null not allowed in GoalImpl()");
			}
			this.goal = goal;
			competences = new ArrayList<CompetenceImpl>();
			determineCompetenceGap();
		}

		public String getTitle() {
			return goal.getTitle();
		}

		public List<CompetenceImpl> getGaps() {

			List<CompetenceImpl> result = new ArrayList<CompetenceImpl>();
			try {
				for (CompetenceImpl competence : competences) {

					if (!competence.isAttained()) {
						result.add(competence);
					}
				}
			} catch (Exception e) {
				PortletUtils.handleError(LOG, e);
			}

			return result;
		}

		/**
		 * Determine competence gap.
		 * 
		 * @throws SystemException
		 *             the system exception
		 * @throws PortalException
		 *             the portal exception
		 */
		private void determineCompetenceGap() throws SystemException, PortalException {
			final List<CompetenceLevel> leafs = new ArrayList<CompetenceLevel>();

			// first fetch all competence leafs
			if (targetsCompetenceProfile()) {
				final CompetenceProfileLevel profile = CompetenceProfileLevelFacade.getCompetenceProfileLevel(goal
						.getTarget(), ctx);
				for (CompetenceLevel level : CompetenceLevelFacade.getCompetenceLevelsByIds(profile
						.getCompetenceLevelIds(), ctx)) {
					fetchLeafs(level, leafs);
				}
			} else if (targetsCompetence()) {
				fetchLeafs(CompetenceLevelFacade.getCompetenceLevel(goal.getTarget(), ctx), leafs);
			}

			// Convert list of leafs to list of competences and filter out the doubles
			for (CompetenceLevel leaf : leafs) {
				CompetenceImpl competence = new CompetenceImpl(leaf);
				int index = competences.indexOf(competence);
				if (index == -1) {
					competences.add(competence);
				} else {
					// Competence already on list
					CompetenceImpl org = competences.get(index);
					// Check if higher level is required
					if (org.required < competence.required) {
						// Store the highest required level
						org.setRequiredCompetenceLevel(competence.getRequiredCompetenceLevel());
					}
				}
			}
		}

		private void fetchLeafs(final CompetenceLevel level, final List<CompetenceLevel> leafs) throws SystemException {
			List<CompetenceLevel> children = CompetenceLevelFacade.getCompetenceLevelsByIds(level.getChildLevelIds(),
					ctx);
			if (children.size() == 0 && !leafs.contains(level)) {
				leafs.add(level);
			} else {
				for (CompetenceLevel child : children) {
					fetchLeafs(child, leafs);
				}
			}
		}

		private boolean targetsCompetenceProfile() {
			return CompetenceProfileLevel.class.getName().equalsIgnoreCase(goal.getTargetType());
		}

		private boolean targetsCompetence() {
			return CompetenceLevel.class.getName().equalsIgnoreCase(goal.getTargetType());
		}

	}

	/**
	 * The Class CompetenceImpl.
	 */
	private class CompetenceImpl {
		private static final int SHIFT_32 = 32;
		private long id;
		private Competence competence;
		private CompetenceLevel requiredLevel;
		private int attained;
		private int required;

		public CompetenceImpl(final CompetenceLevel level) {
			this.id = level.getCompetenceId();
			this.requiredLevel = level;
			this.required = -1;
			this.attained = -1;
		}

		public Competence getCompetence() throws SystemException, PortalException {
			if (competence == null) {
				competence = CompetenceFacade.getCompetence(id, ctx);
			}

			return competence;
		}

		public CompetenceLevel getRequiredCompetenceLevel() {
			return requiredLevel;
		}

		public int getAttainedValue() throws SystemException, PortalException {
			if (attained == -1) {
				LevelValue att = PdpService.getAttainedLevelForUser(ctx.getUserId(), getCompetence(), ctx);
				attained = att != null ? att.getValue() : 0;
			}

			return attained;
		}

		public int getRequiredValue() throws PortalException, SystemException {
			if (required == -1) {
				required = LevelValueFacade.getLevelValue(requiredLevel.getLevelValueId(), ctx).getValue();
			}
			return required;

		}

		public void setRequiredCompetenceLevel(final CompetenceLevel level) {
			requiredLevel = level;
			required = -1;
		}

		public boolean isAttained() throws SystemException, PortalException {
			return getAttainedValue() >= getRequiredValue();
		}

		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + (int) (id ^ (id >>> SHIFT_32));
			return result;
		}

		@Override
		public boolean equals(final Object obj) {
			if (this == obj) {
				return true;
			}
			if (obj == null) {
				return false;
			}
			if (!(obj instanceof CompetenceImpl)) {
				return false;
			}
			CompetenceImpl other = (CompetenceImpl) obj;
			if (id != other.id) {
				return false;
			}
			return true;
		}

	}

}
