/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.pdp.controllers;

import javax.faces.event.ActionEvent;
import javax.faces.event.ActionListener;

import org.tencompetence.business.model.learningpath.ICursor;
import org.tencompetence.business.model.learningpath.ILPElement;
import org.tencompetence.business.model.learningpath.ILearningAction;
import org.tencompetence.business.model.learningpath.ILearningPlan;
import org.tencompetence.business.model.learningpath.impl.CursorImpl;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.portlet.common.components.ActionButtonController;
import org.tencompetence.portlet.common.components.ElementController;
import org.tencompetence.portlet.common.utils.Bundle;
import org.tencompetence.portlet.common.utils.PortletUtils;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;

/**
 * The Task class implements a task from the my plan portlet.
 * 
 * @author Harrie Martens
 * @author Anton Finders
 * @version $Revision: 1.20 $, $Date: 2009/12/01 09:37:45 $
 */
public abstract class Task {

	private static final Log LOG = LogFactoryUtil.getLog(Task.class);
	private final ILearningPlan plan;
	private ICursor cursor;
	private ActionButtonController<Goal> controller;

	/**
	 * Instantiates a new Task for the specified plan.
	 * 
	 * @param plan
	 *            the plan
	 */
	public Task(final ILearningPlan plan) {
		if (plan == null) {
			throw new IllegalArgumentException();
		}
		this.plan = plan;
		setCursor();
	}

	/**
	 * Gets the activity title.
	 * 
	 * @return the activity title
	 */
	public String getActivityTitle() {
		return isActivityAvailable() ? getAction().getTitle() : "";
	}

	/**
	 * Gets the activity url.
	 * 
	 * @return the activity url
	 */
	public String getActivityURL() {
		return getAction().getUri();
	}

	/**
	 * Gets the title.
	 * 
	 * @return the title
	 */
	public String getTitle() {
		return getActivityTitle();
	}
	
	/**
	 * Gets the activity hint.
	 * 
	 * @return the activity hint
	 */
	public String getActivityHint() {
		return isActivityComplete() ? "&nbsp;" : Bundle.getLabel("pdp.task.study.caption");
	}

	/**
	 * Checks if is activity complete.
	 * 
	 * @return true, if is activity complete
	 */
	public boolean isActivityComplete() {
		return (isActivityAvailable() && getAction().isCompleted());
	}

	/**
	 * Checks if activity url is available.
	 * 
	 * @return true, if is activity url available
	 */
	public boolean isActivityURLAvailable() {
		return getActivityURL() != null;
	}

	/**
	 * Checks if a valid activity is available.
	 * <p>
	 * An action is valid if the cursor points to ation that is both subscribed and completed.
	 * 
	 * @return true, if valid action is available
	 */
	public boolean isActivityAvailable() {
		ILearningAction action = getAction();
		// return (action != null && !action.isCompleted() && action.isSubscribed());
		return (action != null);
	}

	/**
	 * Checks if this task completed.
	 * 
	 * @return true, if this task completed
	 */
	public boolean isCompleted() {
		return plan.isCompleted();
	}

	/**
	 * Selects the next available learning action a user has to perform.
	 * 
	 * @return null, stay on the same page
	 */
	public String doSelectNextAction() {
		if (plan != null) {
			ICursor nextActionCursor = plan.getNextLearningAction();
			if (nextActionCursor != null) {
				try {
					// make sure user is subscribed to all parents of the next learning action
					nextActionCursor.subScribe();

					// make this the current cursor for this plan
					plan.setCursor(nextActionCursor);

					this.cursor = nextActionCursor;
				} catch (Exception e) {
					PortletUtils.handleError(LOG, e);
				}
			}
		}
		return null;
	}

	/**
	 * Do view action.
	 * 
	 * @return the string
	 */
	public String doViewAction() {
		if (isActivityAvailable()) {
			onActivityView(getAction());
		}

		return null;
	}

	/**
	 * Complete the current learning action.
	 * 
	 * @return null, stay on the same page
	 */
	public String doCompleteAction() {
		if (isActivityAvailable()) {
			try {
				getAction().setCompleted(true);
				plan.isCompleted();
				onActivityCompleted();
			} catch (Exception e) {
				PortletUtils.handleError(LOG, e);
			}
		}
		return null;
	}

	/**
	 * On activity completed event handler.
	 * <p />
	 * This abstract method must be implemented by subclasses
	 */
	public abstract void onActivityCompleted();
		
	/**
	 * On activity view event handler.
	 * <p />
	 * This abstract method must be implemented by subclasses
	 * 
	 * @param action
	 *            the action for which the the view button was pressed
	 */
	public abstract void onActivityView(final ILearningAction action);
	
	

	/**
	 * getElementController.
	 * 
	 * @return ElementController ElementController
	 */
	public ElementController getElementController() {
		return new ElementController(getAction(), false, true) {

			@Override
			public String handleClickEvent(final ILPElement element) {
				return null;
			}
		};
	}

	/**
	 * Gets the controller.
	 * 
	 * @return the controller
	 */
	public ActionButtonController<Goal> getController() {
		if (controller == null) {
			controller = new ActionButtonController<Goal>();
			controller.addMenuItem(Bundle.getLabel("action.VIEW"), "/images/components/actionsmenu/edit.png",

			new ActionListener() {

				//@Override
				public void processAction(final ActionEvent arg0) {
					doViewAction();
				}
			},

			null);

			controller.addMenuItem(Bundle.getLabel("action.COMPLETE"), "/images/components/actionsmenu/complete.png",
					new ActionListener() {

						//@Override
						public void processAction(final ActionEvent arg0) {
							ILearningAction action = getAction();
							if (action != null) {
								try {
									action.setCompleted(true);
									plan.isCompleted();
									onActivityCompleted();
								} catch (Exception e) {
									PortletUtils.handleError(LOG, e);
								}
							}

						}
					},

					null);

			controller.addMenuItem(Bundle.getLabel("action.UNSUBSCRIBE"),
					"/images/components/actionsmenu/unsubscribe.png", new ActionListener() {

						//@Override
						public void processAction(final ActionEvent arg0) {
							ILearningAction action = getAction();
							if (action != null) {

								try {
									action.setSubscribed(false);
								} catch (Exception e) {
									PortletUtils.handleError(LOG, e);
								}
							}

						}
					},

					null);
		}

		return controller;
	}

	private void setCursor() {
		try {
			cursor = plan.getCursor();
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}
		if (cursor == null) {
			cursor = new CursorImpl(plan);
		}

		if (getAction() == null) {
			doSelectNextAction();
		}

	}

	private ILearningAction getAction() {
		ILPElement element = cursor.getCurrentElement();
		if (element instanceof ILearningAction) {
			return (ILearningAction) element;
		}
		return null;
	}

}