/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.pdp.controllers;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.business.model.learningpath.ILPElement;
import org.tencompetence.business.model.learningpath.ILearningPath;
import org.tencompetence.business.model.learningpath.ILearningPlan;
import org.tencompetence.portlet.common.components.GoalTitleController;

import com.icesoft.faces.component.ext.RowSelectorEvent;

/**
 * This class acts as the controller for the viewScreen.jspx.
 * <p>
 * It provides the view data of the current active Learning Path
 * 
 * @author Harrie Martens
 * @version $Revision: 1.3 $, $Date: 2009/12/01 09:37:45 $
 */
public class ViewScreenController {
	private List<PathEntry> paths;
	private PathDetailsController pathDetailsController;
	private final GoalTitleController goal;
	private final int attained;
	private final int planned;
	private final int notPlanned;

	/**
	 * Instantiates a new view screen controller.
	 * 
	 * @param goal
	 *            the goal title controller for providing the title and level
	 * @param plan
	 *            the current learning plan for achieving the goal
	 * @param attained
	 *            the percentage of competences the user has already attained
	 * @param planned
	 *            the percentage of competences for which the user has a plan
	 * @param notPlanned
	 *            the percentage of competences for which the user has no plan
	 */
	public ViewScreenController(final GoalTitleController goal, final ILearningPlan plan, final int attained,
			final int planned, final int notPlanned) {
		this.goal = goal;
		this.attained = attained;
		this.planned = planned;
		this.notPlanned = notPlanned;

		paths = new ArrayList<PathEntry>();
		if (plan != null) {
			for (ILPElement path : plan.getChildren()) {
				if (path instanceof ILearningPath) {
					paths.add(new PathEntry((ILearningPath) path));
				}
			}
		}
		pathDetailsController = new PathDetailsController(null);
		if (!paths.isEmpty()) {
			handleSelection(paths.get(0));
		}
	}

	/**
	 * Gets the goal controller.
	 * 
	 * @return the goal controller
	 */
	public GoalTitleController getGoal() {
		return goal;
	}

	/**
	 * Gets the list of learning paths that are part of the plan.
	 * 
	 * @return the list of learning paths that are part of the plan
	 */
	public List<PathEntry> getPaths() {
		return paths;
	}

	/**
	 * Checks if a plan is available by checking if it has any associated paths.
	 * 
	 * @return true, if is plan available
	 */
	public boolean isPlanAvailable() {
		return !paths.isEmpty();
	}

	/**
	 * Gets the percentage of attained competence levels for the current goal.
	 * 
	 * @return the attained percentage
	 */
	public String getAttained() {
		return String.valueOf(attained);
	}

	/**
	 * Gets the percentage of planned competence levels for the current goal.
	 * 
	 * @return the planned percentage
	 */
	public String getPlanned() {
		return String.valueOf(planned);
	}

	/**
	 * Gets the percentage of not yet planned competence levels for the current goal.
	 * 
	 * @return the not planned percentage
	 */
	public String getNotPlanned() {
		return String.valueOf(notPlanned);
	}

	/**
	 * Returns true if an element is selected.
	 * 
	 * @return true, if an element is selected
	 */
	public boolean isElementSelected() {
		return pathDetailsController.getElement() != null;
	}

	/**
	 * Gets the element controller.
	 * 
	 * @return the element controller
	 */
	public PathDetailsController getDetailsController() {
		return pathDetailsController;
	}

	/**
	 * Selection listener called when the selection in the path list changes.
	 * 
	 * @param event
	 *            the event describing the selection
	 */
	public void onSelection(final RowSelectorEvent event) {
		handleSelection(paths.get(event.getRow()));
	}

	private void handleSelection(final PathEntry selectedPath) {
		// reset selected state of list items to resemble normal selection list
		for (PathEntry path : paths) {
			path.setSelected(path.equals(selectedPath));
		}

		// inform the element viewer
		pathDetailsController.setInput(selectedPath.getPath());
	}

	/**
	 * The Class PathEntry represents a ILearningPath from the ILearningPlan.
	 */
	public class PathEntry {
		private ILearningPath path;
		private boolean selected;

		/**
		 * Instantiates a new path entry.
		 * 
		 * @param path
		 *            the path
		 */
		public PathEntry(final ILearningPath path) {
			if (path == null) {
				throw new IllegalArgumentException("path should not be null");
			}
			this.path = path;
		}

		/**
		 * Gets the path.
		 * 
		 * @return the path
		 */
		public ILearningPath getPath() {
			return path;
		}

		/**
		 * Gets the selected state of this entry.
		 * 
		 * @return true if this entry is selected
		 */
		public boolean getSelected() {
			return selected;
		}

		/**
		 * Gets the title.
		 * 
		 * @return the title
		 */
		public String getTitle() {
			return path.getTitle();
		}

		/**
		 * Sets the selected state.
		 * 
		 * @param selected
		 *            the new selected state
		 */
		public void setSelected(final boolean selected) {
			this.selected = selected;
		}
	}

}
