/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.portlet.portfolio.common.components;

import java.io.Serializable;
import java.util.List;
import java.util.Stack;
import java.util.UUID;

import org.tencompetence.business.model.learningpath.ILPElement;
import org.tencompetence.business.utils.ClassUtils;
import org.tencompetence.portlet.common.components.CrumbController;
import org.tencompetence.portlet.common.components.ICrumbEntry;
import org.tencompetence.portlet.common.constants.Defines;
import org.tencompetence.portlet.common.utils.PortletUtils;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.model.BaseModel;

/**
 * This class is the controller for the Crumb control used in the <code>PortfolioDetailsController</code> bean. The
 * controller maintains a list of visited entities.
 * 
 * @author sheyenrath
 * @version $Revision: 1.12 $, $Date: 2009/11/02 21:50:07 $
 */
public class PortfolioCrumbController extends CrumbController {
	private static final Log LOG = LogFactoryUtil.getLog(PortfolioCrumbController.class);

	private static final String CRUMB_CONTROLLER = "portfoliocrumbcontroller";
	private static final String CRUMB_DATA_PARAM_NAME = "portfoliocrumbdata";
	private Stack<ICrumbEntry> crumbs;

	/**
	 * Gets the controller from session.
	 * 
	 * @return the controller
	 */
	public static PortfolioCrumbController getController() {
		return PortletUtils.getObjectFromSession(CRUMB_CONTROLLER, new PortfolioCrumbController());
	}

	/**
	 * Process a new editor input and this method should be called by the owning bean when a new editor input has been
	 * set. In case navigation is up the hierarchy, crumbs are removed and for navigation down the hierarchy new crumbs
	 * are added. This is achieved by determining the position of the passed <code>editorInput</code> in the list of
	 * crumbs. As a consequence, each crumb has to be unique.
	 * 
	 * @param editorInput
	 *            the editor input that has been set for the backing bean
	 * @param viewId
	 *            the view id
	 */
	public void processEditorInput(final Object editorInput, final String viewId) {

		crumbs = PortletUtils.getObjectFromSession(CRUMB_DATA_PARAM_NAME, new Stack<ICrumbEntry>());
		final ICrumbEntry input = createCrumbEntry(editorInput, viewId);

		if (input != null && !crumbs.contains(input)) {
			// add the current editor input as last crumb
			crumbs.add(input);
		}
		PortletUtils.setObjectToSession(CRUMB_DATA_PARAM_NAME, crumbs);
	}

	/** {@inheritDoc} */
	@Override
	public List<ICrumbEntry> getCrumbs() {
		return this.crumbs;
	}

	/** {@inheritDoc} */
	@Override
	public String handleCrumbEvent(final ICrumbEntry crumb) {
		try {
			final Object entity = ((PortfolioCrumbEntry) crumb).getEntity();

			// Make sure the page gets refreshed
			PortletUtils.setObjectToSession(Defines.EDITOR_INPUT_PARAM_NAME, entity);
			PortletUtils.setObjectToSession(Defines.INIT_FIELDS, true);

			// Remove current clicked crumb from crumbs
			ICrumbEntry temp = crumbs.peek();
			while (!temp.equals(crumb)) {
				temp = crumbs.pop();
			}

			return crumb.getViewId();
		} catch (Exception e) {
			PortletUtils.handleError(LOG, e);
		}

		return null;
	}

	private ICrumbEntry createCrumbEntry(final Object entity, final String viewId) {
		return new PortfolioCrumbEntry(entity, viewId);
	}

	/**
	 * This utility class is a the data class for the crumbs of the control.
	 * 
	 * @author sheyenrath
	 * @version $Revision: 1.12 $, $Date: 2009/11/02 21:50:07 $
	 */
	public class PortfolioCrumbEntry implements ICrumbEntry, Serializable {
		private static final long serialVersionUID = -4372089634041160701L;

		private final Object entity;
		private String id;
		private final String title;
		private final String viewId;

		/**
		 * Instantiates a new crumb entry.
		 * 
		 * @param entity
		 *            the entity
		 * @param viewId
		 *            the viewId
		 */
		public PortfolioCrumbEntry(final Object entity, final String viewId) {
			title = formatTitle(ClassUtils.getProperty(entity, "title", ""));

			// Default id is the primaryKey from BaseModel<?>
			if (entity instanceof BaseModel<?>) {
				final BaseModel<?> baseModel = (BaseModel<?>) entity;
				id = baseModel.getPrimaryKeyObj().toString();
			} else if (entity instanceof ILPElement) {
				// If this is not present, use the uri.
				final ILPElement element = (ILPElement) entity;
				id = element.getUri();
			} else {
				id = UUID.randomUUID().toString();
			}

			this.viewId = viewId;
			this.entity = entity;
		}

		/**
		 * Gets the id.
		 * 
		 * @return the id
		 */
		public String getId() {
			return id;
		}

		/** {@inheritDoc} */
		//@Override
		public String getTitle() {
			return title;
		}

		/** {@inheritDoc} */
		//@Override
		public String getViewId() {
			return viewId;
		}

		/**
		 * Gets the entity being stubbed by this class.
		 * 
		 * @return the entity being stubbed by this class
		 */
		public Object getEntity() {
			return entity;
		}

		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + ((id == null) ? 0 : id.hashCode());
			return result;
		}

		@Override
		public boolean equals(final Object obj) {
			if (this == obj) {
				return true;
			}
			if (obj == null) {
				return false;
			}
			if (!(obj instanceof PortfolioCrumbEntry)) {
				return false;
			}
			final PortfolioCrumbEntry other = (PortfolioCrumbEntry) obj;
			if (id == null) {
				if (other.id != null) {
					return false;
				}
			} else if (!id.equals(other.id)) {
				return false;
			}
			return true;
		}
	}

	/**
	 * Resets the history of this control.
	 */
	public static void reset() {
		PortletUtils.resetObjectOnSession(CRUMB_DATA_PARAM_NAME);
		PortletUtils.resetObjectOnSession(CRUMB_CONTROLLER);
	}
}
