/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.portfolio.common.components;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.business.facade.JournalEntryFacade;
import org.tencompetence.business.services.PortfolioService;
import org.tencompetence.business.services.PortfolioService.PortfolioSearchResult;
import org.tencompetence.generated.model.PortfolioEntry;
import org.tencompetence.portlet.common.components.ActionsMenuListController;
import org.tencompetence.portlet.common.components.InitEntityController;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.portlet.common.utils.UrlUtils;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * This is a abstract class PortfolioFullView.
 * 
 * @param <T>
 *            GenericType
 * 
 * @author sheyenrath
 * @version $Revision: 1.17 $, $Date: 2009/11/26 13:37:20 $
 */
public abstract class PortfolioFullView<T> extends InitEntityController<T> {
	private static final Log LOG = LogFactoryUtil.getLog(PortfolioFullView.class);

	private final String maxUrl = UrlUtils.getWindowStateMaximizedUrl();
	private final String normalUrl = UrlUtils.getWindowStateNormalUrl();

	private ActionsMenuListController<PortfolioActionsMenuRowItem> historyList;

	private PortfolioHistoryController<T> historyController;
	private PortfolioHeaderController<T> headerController;
	private int selectedTab;

	/**
	 * Gets the primary key.
	 * 
	 * @return the primary key
	 */
	protected abstract String getPrimaryKey();

	@Override
	protected void initFields() {
		initHeaderController();
		initHistoryController();
	}

	private void initHeaderController() {
		headerController = new PortfolioHeaderController<T>(this, getEntity());
	}

	private void initHistoryController() {
		try {
			final ServiceContext ctx = PortletUtils.getServiceContext();
			final String primaryKey = getPrimaryKey();

			final List<PortfolioEntry> portfolioEntries = JournalEntryFacade
					.getEntriesByUCGEntityIdEventTypesDateRange(ctx.getUserId(), 0L, 0L, primaryKey, null, null, null,
							true);

			final List<PortfolioSearchResult> searchEntries = PortfolioService.getFilteredPortfolioSearchResults(
					portfolioEntries, null, ctx);

			final List<PortfolioActionsMenuRowItem> result = new ArrayList<PortfolioActionsMenuRowItem>();

			for (final PortfolioSearchResult entry : searchEntries) {
				final PortfolioActionsMenuRowItem row = new PortfolioActionsMenuRowItem(entry, entry.getEntry()
						.getPrimaryKey(), true);
				result.add(row);
			}

			historyList = new ActionsMenuListController<PortfolioActionsMenuRowItem>();
			historyList.setInput(result);
			historyController = new PortfolioHistoryController<T>(historyList);
		} catch (PortalException e) {
			PortletUtils.handleError(LOG, e);
		} catch (SystemException e) {
			PortletUtils.handleError(LOG, e);
		}
	}

	/**
	 * Gets the PortfolioHistoryController.
	 * 
	 * @return the controller
	 */
	public PortfolioHistoryController<T> getHistoryController() {
		return historyController;
	}

	/**
	 * Gets the PortfolioHeaderController.
	 * 
	 * @return the controller
	 */
	public PortfolioHeaderController<T> getHeaderController() {
		return headerController;
	}

	/**
	 * Back to portfolio small view.
	 * 
	 * @return Outcome
	 */
	public String back() {
		PortletUtils.redirect(normalUrl);
		return null;
	}

	/**
	 * Sets the selected tab.
	 * 
	 * @param selectedTab
	 *            the new selected tab
	 */
	public void setSelectedTab(final int selectedTab) {
		this.selectedTab = selectedTab;
	}

	/**
	 * Gets the selected tab.
	 * 
	 * @return the selected tab
	 */
	public int getSelectedTab() {
		return selectedTab;
	}

	/**
	 * Gets the maximized url.
	 * 
	 * @return the maximized url
	 */
	protected String getMaximizedUrl() {
		return maxUrl;
	}

	/**
	 * Gets the normal url.
	 * 
	 * @return the normal url
	 */
	protected String getNormalUrl() {
		return normalUrl;
	}
}
