/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.qtieditor.common;

import java.util.ArrayList;
import java.util.List;

import javax.faces.event.ActionEvent;
import javax.faces.model.SelectItem;

import org.tencompetence.generated.model.AssessmentItem;
import org.tencompetence.portlet.common.constants.Outcomes;
import org.tencompetence.portlet.common.utils.PortletUtils;
import org.tencompetence.services.xml.qti.elements.AssessmentElementFactory;
import org.tencompetence.services.xml.qti.elements.AssessmentItemInstance;
import org.tencompetence.services.xml.qti.elements.BasicElement;
import org.tencompetence.services.xml.qti.elements.BasicElementList;
import org.tencompetence.services.xml.qti.elements.InlineChoice;
import org.tencompetence.services.xml.qti.elements.InlineChoiceBlock;
import org.tencompetence.services.xml.qti.elements.InlineChoiceInteraction;
import org.tencompetence.services.xml.qti.elements.InterpretationValue;
import org.tencompetence.services.xml.qti.elements.Value;

/**
 * InlineChoiceInteractionItem.
 *
 * @author Teodor Stoev
 * @version $Revision: 1.5 $, $Date: 2009/11/24 14:10:05 $
 */
public class InlineChoiceInteractionItem {
	private static final String INIT_FIELDS = "init.fields";
	private static final String EDITOR_INPUT_PARAM_NAME = "editor.input";
	private InlineChoiceInteraction inlineChoiceInteraction;
	private String answer;
	private AssessmentItem editorInput;
	private InlineChoiceListController controller;
	private List<SelectItem> radioCorrectItems;
	private String selectedCorrect;
	
	/** Constructor.
	 * 
	 * @param inlineChoiceInteraction
	 *            the inline choice interaction
	 *
	 * @param answer
	 *            the answer
	 * 
	 * @param editorInput
	 *            the assessment item
	 */
	public InlineChoiceInteractionItem(final InlineChoiceInteraction inlineChoiceInteraction, final String answer,
			final AssessmentItem editorInput) {
		this.inlineChoiceInteraction = inlineChoiceInteraction;
		this.answer = answer;
		this.editorInput = editorInput;

		controller = new InlineChoiceListController();
		controller.setInput(getInlineChoiceItems());
		
		radioCorrectItems = getListOfRadioCorrectItems();

		InterpretationValue correctResponse = inlineChoiceInteraction.getResponseDeclaration().getCorrectResponse();
		if (correctResponse.getValueList().size() > 0) {
			String correctChoiceId = ((Value) correctResponse.getValueAt(0)).getData();
			for (SelectItem item : radioCorrectItems) {
				String value = (String) item.getValue();
				if (value.equals(correctChoiceId)) {
					selectedCorrect = value;
				}
			}
		}
	}
	
	/**
	 * Gets the controller.
	 *
	 * @return the controller
	 */
	public InlineChoiceListController getController() {
		return controller;
	}
	
	/**
	 * Gets the inline choice interaction.
	 *
	 * @return the inline choice interaction
	 */
	public InlineChoiceInteraction getInlineChoiceInteraction() {
		return inlineChoiceInteraction;
	}
	
	/**
	 * Gets the position of the interaction.
	 *
	 * @return the position
	 */
	public String getPosition() {
		int start = inlineChoiceInteraction.getStart();
		int end = start + inlineChoiceInteraction.getLength();
		
		return start + " - " + end;
	}
	
	/**
	 * Gets the answer.
	 *
	 * @return the answer
	 */
	public String getAnswer() {
		return answer;
	}
	
	/**
	 * Gets the list of correct items to populate the radio buttons control.
	 * 
	 * @return the list of correct items
	 */
	public List<SelectItem> getRadioCorrectItems() {
		return radioCorrectItems;
	}
	
	private List<SelectItem> getListOfRadioCorrectItems() {
		BasicElementList inlineChoiceList = inlineChoiceInteraction.getInlineChoiceList();
		ArrayList<SelectItem> result = new ArrayList<SelectItem>();

		for (int i = 0; i < inlineChoiceList.size(); i++) {
			InlineChoice choice = (InlineChoice) inlineChoiceList.getBasicElementAt(i);
			SelectItem newItem = new SelectItem(choice.getId());
			newItem.setLabel("");
			result.add(newItem);
		}

		return result;
	}
	
	/**
	 * Gets the selected correct answer.
	 * 
	 * @return the selected correct answer
	 */
	public String getSelectedCorrect() {
		return selectedCorrect;
	}

	/**
	 * Sets the selected correct answer.
	 * 
	 * @param correct
	 *            the new correct answer
	 */
	public void setSelectedCorrect(final String correct) {
		updateInteractionCorrectChoice(correct);
		
		selectedCorrect = correct;
	}
	
	/**
	 * Adds an inline choice to an inline choice-interaction. Method called if add button is pressed.
	 *
	 * The current AssessmentItem is modified and passed to the next page
	 * 
	 * @param event
	 *            ActionEvent
	 */
	public void addChoice(final ActionEvent event) {
		AssessmentItemInstance assessmentItem = AssessmentItemInstance.fromXmlString(editorInput.getXmlData());
		InlineChoiceBlock inlineChoiceBlock = (InlineChoiceBlock) assessmentItem.getItemBody().getFirstBlock();
		
		for (BasicElement element : inlineChoiceBlock.getInteractionList()) {
			InlineChoiceInteraction interaction = (InlineChoiceInteraction) element;
			if (interaction.getId().equals(inlineChoiceInteraction.getId())) {
				interaction.setInlineChoiceList(getInlineChoices());
				interaction.addInlineChoice(new InlineChoice(assessmentItem));
				break;
			}
		}

		editorInput.setXmlData(assessmentItem.toXmlString());
		PortletUtils.getSessionMap().put(EDITOR_INPUT_PARAM_NAME, editorInput);
		
		forwardToNextPage(Outcomes.ADD);
	}
	
	private BasicElementList getInlineChoices() {
		BasicElementList result = new BasicElementList();
		
		for (RowItem<InlineChoiceItem> rowItem : controller.getRowItems()) {
			if (rowItem.getData().getInlineChoice() != null) {
				result.add(rowItem.getData().getInlineChoice());
			}
		}
		return result;
	}
	
	private List<RowItem<InlineChoiceItem>> getInlineChoiceItems() {
		BasicElementList inlineChoiceList = inlineChoiceInteraction.getInlineChoiceList();
		List<RowItem<InlineChoiceItem>> result = new ArrayList<RowItem<InlineChoiceItem>>(inlineChoiceList.size());

		for (int i = 0; i < inlineChoiceList.size(); i++) {
			InlineChoice choice = (InlineChoice) inlineChoiceList.getBasicElementAt(i);
			result.add(new RowItem<InlineChoiceItem>(
					new InlineChoiceItem(choice, i), (long) choice.getId().hashCode()));
		}

		return result;
	}
	
	private void updateInteractionCorrectChoice(final String newCorrectChoice) {
		AssessmentItemInstance assessmentItem = AssessmentItemInstance.fromXmlString(editorInput.getXmlData());
		InlineChoiceBlock inlineChoiceBlock = (InlineChoiceBlock) assessmentItem.getItemBody().getFirstBlock();
		
		String oldCorrectChoice = inlineChoiceInteraction.getCorrectChoice().getData();
		int difference = newCorrectChoice.length() - oldCorrectChoice.length();

		String originalString = inlineChoiceBlock.getData();
		originalString = originalString.substring(0, inlineChoiceInteraction.getStart())
			+ newCorrectChoice
			+ originalString.substring(inlineChoiceInteraction.getStart() + inlineChoiceInteraction.getLength());		
		inlineChoiceBlock.setData(originalString);
		
		int size = inlineChoiceBlock.getInteractionList().size();
		for (int i = 0; i < size; i++) {
			InlineChoiceInteraction interaction =
				(InlineChoiceInteraction) inlineChoiceBlock.getInteractionList().getBasicElementAt(i);

			if (interaction.getResponseIdentifier().equals(inlineChoiceInteraction.getResponseIdentifier())) {
				//handle the changed correct choice
				interaction.setLength(interaction.getLength() + difference);

				//handle all blanks after the changed correct choice
				for (int j = i + 1; j < size; j++) {
					interaction = (InlineChoiceInteraction) inlineChoiceBlock.getInteractionList().getBasicElementAt(j);
					interaction.setStart(interaction.getStart() + difference);
				}
				
				interaction.getResponseDeclaration().setFirstCorrectResponse(
						AssessmentElementFactory.IDENTIFIER, newCorrectChoice);
				break;
			}
		}
		
		editorInput.setXmlData(assessmentItem.toXmlString());
		PortletUtils.getSessionMap().put(EDITOR_INPUT_PARAM_NAME, editorInput);
	}
	
	/**
	 * Forward to next page.
	 *
	 * @param outcome
	 *            outcome
	 */
	private void forwardToNextPage(final String outcome) {
		if (outcome != null) {
			PortletUtils.getSessionMap().put(INIT_FIELDS, true);
			PortletUtils.handleNavigation(outcome);
		}
	}
}
