/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.portlet.qtieditor.common;

import org.tencompetence.services.xml.qti.elements.AssessmentElementFactory;
import org.tencompetence.services.xml.qti.elements.FeedbackInline;
import org.tencompetence.services.xml.qti.elements.InterpretationValue;
import org.tencompetence.services.xml.qti.elements.MapEntry;
import org.tencompetence.services.xml.qti.elements.Mapping;
import org.tencompetence.services.xml.qti.elements.OutcomeDeclaration;
import org.tencompetence.services.xml.qti.elements.ResponseDeclaration;
import org.tencompetence.services.xml.qti.elements.SimpleChoice;
import org.tencompetence.services.xml.qti.elements.Value;

/**
 * SimpleChoiceItem.
 *
 * @author Teodor Stoev
 * @version $Revision: 1.9 $, $Date: 2009/11/27 10:33:46 $
 */
public class SimpleChoiceItem {
	private SimpleChoice simpleChoice;
	private int index;
	private boolean correct;
	private String score;

	/** Constructor.
	 * 
	 * @param simpleChoice
	 *            the simple choice
	 *            
	 * @param index
	 *            the index
	 */
	public SimpleChoiceItem(final SimpleChoice simpleChoice, final int index) {
		this.simpleChoice = simpleChoice;
		this.index = index;

		// Determine if correct
		InterpretationValue correctResponses =
			simpleChoice.getChoiceInteraction().getResponseDeclaration().getCorrectResponse();
		for (int i = 0; i < correctResponses.size(); i++) {
			if (correctResponses.getValueAt(i).getData().equals(simpleChoice.getId())) {
				correct = true;
			}
		}

		// Determine if score is defined
		Mapping mapping = simpleChoice.getChoiceInteraction().getResponseDeclaration().getMapping();
		for (int i = 0; i < mapping.size(); i++) {
			MapEntry mapEntry = mapping.getMapEntryByKey(simpleChoice.getId());
			if (mapEntry != null) {
				score = mapEntry.getMappedValueString();
				break;
			}
		}
	}

	/**
	 * Gets the simple choice.
	 *
	 * @return the simple choice
	 */
	public SimpleChoice getSimpleChoice() {
		return simpleChoice;
	}

	/**
	 * Gets the choice text.
	 *
	 * @return the choice text
	 */
	public String getText() {
		return simpleChoice.getData();
	}

	/**
	 * Sets the choice text.
	 *
	 * @param text
	 *            the new text
	 */
	public void setText(final String text) {
		simpleChoice.setData(text);
	}

	/**
	 * Gets the index of the item.
	 *
	 * @return the index
	 */
	public int getIndex() {
		return index;
	}

	/**
	 * Checks if the choice is currently marked as correct.
	 *
	 * @return true / false
	 */
	public boolean isCorrect() {
		return correct;
	}

	/**
	 * Sets the correct value.
	 *
	 * @param correct
	 *            the new correct value
	 */
	public void setCorrect(final boolean correct) {
		this.correct = correct;
		
		InterpretationValue correctResponses =
			simpleChoice.getChoiceInteraction().getResponseDeclaration().getCorrectResponse();
		if (correct) {
			Value value = new Value(AssessmentElementFactory.IDENTIFIER, simpleChoice.getId());
			correctResponses.addValue(value);
		} else {
			Value valueToRemove = null;
			for (int i = 0; i < correctResponses.size(); i++) {
				if (correctResponses.getValueAt(i).getData().equals(simpleChoice.getId())) {
					valueToRemove = correctResponses.getValueAt(i);
				}
			}
			if (valueToRemove != null) {
				correctResponses.removeValue(valueToRemove);
			}
		}
	}

	/**
	 * Gets the score of the item.
	 *
	 * @return the score
	 */
	public String getScore() {
		if (score != null) {
			return score;
		} else {
			return "0.0";
		}
	}

	/**
	 * Sets the score of the item.
	 *
	 * @param score
	 *            the new score
	 */
	public void setScore(final String score) {
		this.score = score;
	}
	
	/**
	 * Gets the feedback of the item.
	 *
	 * @return the score
	 */
	public String getFeedback() {
		if (simpleChoice.getFeedbackInline() != null) {
			return simpleChoice.getFeedbackInline().getData();
		} else {
			return null;
		}
	}

	/**
	 * Sets the feedback of the item.
	 *
	 * @param feedback
	 *            the new feedback
	 */
	public void setFeedback(final String feedback) {
		if (feedback.length() > 0) {
			// When the responseDeclaration is constructed from xml and no modalFeedback is defined,
			// the response's feedback outcome is null. So we first create the feedback outcome.
			ResponseDeclaration responseDeclaration = simpleChoice.getChoiceInteraction().getResponseDeclaration();
			if (responseDeclaration.getFeedbackOutcomeDeclaration() == null) {
				OutcomeDeclaration aOutcomeDeclaration = new OutcomeDeclaration(responseDeclaration.getId() + ".FEEDBACK", 
						AssessmentElementFactory.IDENTIFIER, 
						AssessmentElementFactory.SINGLE,
						simpleChoice.getAssessmentItem());
				responseDeclaration.setFeedbackOutcomeDeclaration(aOutcomeDeclaration);
			}
			
			FeedbackInline feedbackInline = new FeedbackInline(simpleChoice.getAssessmentItem(), simpleChoice);
			feedbackInline.setData(feedback);
			simpleChoice.setFeedbackInline(feedbackInline);
		} else {
			if (simpleChoice.getFeedbackInline() != null) {
				simpleChoice.setFeedbackInline(null);
			}
		}
	}
}