/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.facade;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.business.constants.EventType;
import org.tencompetence.business.exceptions.AssessmentItemException;
import org.tencompetence.business.facade.validators.AssessmentItemValidator;
import org.tencompetence.business.services.PortfolioService;
import org.tencompetence.generated.model.AssessmentItem;
import org.tencompetence.generated.model.AssessmentTest;
import org.tencompetence.generated.service.AssessmentItemLocalServiceUtil;

import com.liferay.counter.service.CounterLocalServiceUtil;
//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.service.ServiceContext;

/**
 * The Class AssesmentItemFacade.
 * 
 * @author venrooijma
 * @version $Id: AssessmentItemFacade.java,v 1.6 2009/10/28 12:59:27 sheyenrath Exp $
 */
public final class AssessmentItemFacade {

	private AssessmentItemFacade() {
	}

	/**
	 * Creates the assessment item.
	 * 
	 * @return the assessment item
	 * @throws SystemException
	 *             the system exception
	 */
	public static AssessmentItem createAssessmentItem() throws SystemException {
		final long id = CounterLocalServiceUtil.increment();

		final AssessmentItem result = AssessmentItemLocalServiceUtil.createAssessmentItem(id);
		result.setNew(true);

		return result;
	}

	/**
	 * Adds the assessment item.
	 * 
	 * @param assessmentItem
	 *            the assessment item
	 * @param ctx
	 *            the ctx
	 * @return the assessment item
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static AssessmentItem addAssessmentItem(final AssessmentItem assessmentItem, final ServiceContext ctx)
			throws SystemException, PortalException {

		validateAssessmentItem(assessmentItem);

		final AssessmentItem result = AssessmentItemLocalServiceUtil.addAssessmentItem(assessmentItem);

		FacadeUtils.addPermissions(result, ctx);

		PortfolioService.addJournalEntry(result, EventType.CREATE, ctx);

		return result;
	}

	/**
	 * Update assessment item.
	 * 
	 * @param assessmentItem
	 *            the assessment item
	 * @param ctx
	 *            the ctx
	 * @return the assessment item
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static AssessmentItem updateAssessmentItem(final AssessmentItem assessmentItem, final ServiceContext ctx)
			throws SystemException, PortalException {
		validateAssessmentItem(assessmentItem);

		final AssessmentItem result = AssessmentItemLocalServiceUtil.updateAssessmentItem(assessmentItem);

		PortfolioService.addJournalEntry(result, EventType.UPDATE, ctx);

		return result;
	}

	private static void validateAssessmentItem(final AssessmentItem assessmentItem) throws SystemException {
		final List<String> errors = new ArrayList<String>();

		if (!AssessmentItemValidator.validate(assessmentItem, errors)) {
			throw new AssessmentItemException("Invalid AssessmentItem : " + errors.toString());
		}
	}

	/**
	 * Gets the assessment item by identifier.
	 * 
	 * @param identifier
	 *            the identifier
	 * @param ctx
	 *            the ctx
	 * @return the assessment item by identifier
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static AssessmentItem getAssessmentItemByIdentifier(final String identifier, final ServiceContext ctx)
			throws SystemException, PortalException {
		final List<AssessmentItem> list = AssessmentItemLocalServiceUtil.getAssessmentItemsByIdentifier(identifier);
		if (list != null) {
			if (list.isEmpty()) {
				return null;
			} else if (list.size() == 1) {
				return list.get(0);
			} else {
				throw new AssessmentItemException("More then 1 AssessmentItem found for identifier : " + identifier);
			}
		}

		return null;
	}

	/**
	 * Gets the assessment items.
	 * 
	 * @param ctx
	 *            the ctx
	 * @return the assessment items
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static List<AssessmentItem> getAssessmentItems(final ServiceContext ctx) throws SystemException,
			PortalException {
		return AssessmentItemLocalServiceUtil.getAssessmentItems();

	}

	/**
	 * Gets the assessment tests by assessment item id.
	 * 
	 * @param assessmentItemId
	 *            the assessment item id
	 * @param ctx
	 *            the ctx
	 * @return the assessment tests by assessment item id
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static List<AssessmentTest> getAssessmentTestsByAssessmentItemId(final long assessmentItemId,
			final ServiceContext ctx) throws SystemException, PortalException {
		return AssessmentItemLocalServiceUtil.getAssessmentTestsByAssessmentItemId(assessmentItemId);

	}

	/**
	 * Delete assessment item.
	 * 
	 * @param item
	 *            the item
	 * @param ctx
	 *            the ctx
	 * @throws SystemException
	 *             the system exception
	 */
	public static void deleteAssessmentItem(final AssessmentItem item, final ServiceContext ctx) throws SystemException {
		AssessmentItemLocalServiceUtil.deleteAssessmentItem(item);
	}

	/**
	 * Gets the assessment items by ids.
	 * 
	 * @param ids
	 *            the ids
	 * @return the assessment items by ids
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<AssessmentItem> getAssessmentItemsByIds(final List<Long> ids) throws SystemException {
		return AssessmentItemLocalServiceUtil.getAssessmentItemsByIds(ids);
	}

}
