/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.facade;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.business.constants.EventType;
import org.tencompetence.business.exceptions.CompetenceException;
import org.tencompetence.business.facade.validators.CompetenceValidator;
import org.tencompetence.business.services.PortfolioService;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.LevelValue;
import org.tencompetence.generated.service.CompetenceLocalServiceUtil;

import com.liferay.counter.service.CounterLocalServiceUtil;
//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.service.ServiceContext;

/**
 * Implement competence functionality.
 * 
 * @author Stef Heyenrath
 * @version $Id: CompetenceFacade.java,v 1.7 2009/10/28 12:59:27 sheyenrath Exp $
 */
public final class CompetenceFacade {

	private CompetenceFacade() {
	};

	/**
	 * Creates a new competence.
	 * <p>
	 * The entity is not persisted in the database.
	 * 
	 * @return the competence
	 * @throws SystemException
	 *             the system exception
	 */
	public static Competence createCompetence() throws SystemException {
		final long id = CounterLocalServiceUtil.increment();

		final Competence result = CompetenceLocalServiceUtil.createCompetence(id);
		result.setNew(true);

		return result;
	}

	/**
	 * Adds the competence.
	 * 
	 * @param competence
	 *            the competence
	 * @param ctx
	 *            the ctx
	 * @return the competence
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the Portal exception
	 */
	public static Competence addCompetence(final Competence competence, final ServiceContext ctx)
			throws SystemException, PortalException {

		final List<String> errors = new ArrayList<String>();
		if (!CompetenceValidator.validate(competence, errors, ctx)) {
			throw new CompetenceException("Invalid Competence : " + errors.toString());
		}
		final Competence result = CompetenceLocalServiceUtil.addCompetence(competence);

		FacadeUtils.addPermissions(result, ctx);
		PortfolioService.addJournalEntry(result, EventType.CREATE, ctx);

		// Now create a CompetenceLevel for all possible attainable level values
		for (final LevelValue value : LevelValueFacade.getAttainableCompetenceLevelValues(competence
				.getCompetenceMapId(), ctx)) {
			final CompetenceLevel competenceLevel = CompetenceLevelFacade.createCompetenceLevel();
			competenceLevel.setUserId(competence.getUserId());
			competenceLevel.setGroupId(competence.getGroupId());
			competenceLevel.setCompanyId(competence.getCompanyId());
			competenceLevel.setCompetenceId(competence.getCompetenceId());
			competenceLevel.setLevelValueId(value.getLevelValueId());
			competenceLevel.setTitle(FacadeUtils.appendTitle(competence, value));
			competenceLevel.setDescription(Constants.DEFAULT_DESCRIPTION);
			competenceLevel.setVersion(Constants.FIRST_VERSION);
			competenceLevel.setVersionTag(Constants.FIRST_VERSION_STR);
			CompetenceLevelFacade.addCompetenceLevel(competenceLevel, ctx);
		}

		return result;
	}

	/**
	 * Gets the competence.
	 * 
	 * @param competenceId
	 *            the competence id
	 * @param ctx
	 *            the ctx
	 * @return the competence
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static Competence getCompetence(final long competenceId, final ServiceContext ctx) throws SystemException,
			PortalException {
		return CompetenceLocalServiceUtil.getCompetence(competenceId);
	}

	/**
	 * Gets the competence.
	 * 
	 * @param competenceIds
	 *            the competenceIds
	 * @param ctx
	 *            the ctx
	 * @return the competence
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static List<Competence> getCompetenceByIds(final List<Long> competenceIds, final ServiceContext ctx)
			throws SystemException, PortalException {
		return CompetenceLocalServiceUtil.getCompetencesByIds(competenceIds);
	}

	/**
	 * Gets the competences count.
	 * 
	 * @param competenceMapId
	 *            the competence map id
	 * @param ctx
	 *            the ctx
	 * @return the competences count
	 * @throws SystemException
	 *             the system exception
	 */
	public static int getCompetencesCount(final long competenceMapId, final ServiceContext ctx) throws SystemException {
		return CompetenceLocalServiceUtil.getCompetencesCount(competenceMapId);
	}

	/**
	 * Gets the count of competences of the specified type in the specified competence map.
	 * 
	 * @param competenceMapId
	 *            the competence map id
	 * @param type
	 *            the type of the competences to count
	 * @param ctx
	 *            the ctx
	 * @return the number of competences found
	 * @throws SystemException
	 *             the system exception
	 */
	public static int getCompetencesCount(final long competenceMapId, final String type, final ServiceContext ctx)
			throws SystemException {
		return CompetenceLocalServiceUtil.getCompetencesCount(competenceMapId, type);
	}

	/**
	 * Gets the competences by competence map id.
	 * 
	 * @param competenceMapId
	 *            the competence map id
	 * @param ctx
	 *            the ctx
	 * @return the competences by competence map id
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<Competence> getCompetencesByCompetenceMapId(final long competenceMapId, final ServiceContext ctx)
			throws SystemException {

		return CompetenceLocalServiceUtil.getCompetencesByCompetenceMapId(competenceMapId);
	}

	/**
	 * Gets the competences by competence map id and type.
	 * 
	 * @param competenceMapId
	 *            the competence map id
	 * @param type
	 *            the type
	 * @param ctx
	 *            the ctx
	 * @return the competences by competence map id and type
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<Competence> getCompetencesByCompetenceMapIdAndType(final long competenceMapId,
			final String type, final ServiceContext ctx) throws SystemException {
		return CompetenceLocalServiceUtil.getCompetencesByCompetenceMapIdAndType(competenceMapId, type);
	}

	/**
	 * Get a Competence without title and description.
	 * 
	 * @param competenceId
	 *            competenceId
	 * @return Competence
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static Competence getBareCompetence(final long competenceId) throws SystemException, PortalException {
		return CompetenceLocalServiceUtil.getBareCompetence(competenceId);
	}

	/**
	 * Get a Competences without title and description for a competenceMap.
	 * 
	 * @param competenceMapId
	 *            competenceMapId
	 * @return List<Competence>
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static List<Competence> getBareCompetencesByCompetenceMapId(final long competenceMapId)
			throws SystemException, PortalException {
		return CompetenceLocalServiceUtil.getBareCompetencesByCompetenceMapId(competenceMapId);
	}

	/**
	 * Update competence.
	 * 
	 * @param competence
	 *            the competence
	 * @param ctx
	 *            the ctx
	 * @return the competence
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static Competence updateCompetence(final Competence competence, final ServiceContext ctx)
			throws SystemException, PortalException {

		final List<String> errors = new ArrayList<String>();

		if (!CompetenceValidator.validate(competence, errors, ctx)) {
			throw new CompetenceException("Invalid Competence : " + errors.toString());
		}

		final Competence result = CompetenceLocalServiceUtil.updateCompetence(competence, false);

		PortfolioService.addJournalEntry(result, EventType.UPDATE, ctx);

		return result;
	}

}
