/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.facade;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.tencompetence.business.constants.EventType;
import org.tencompetence.business.constants.ServiceConstants;
import org.tencompetence.business.exceptions.CompetenceLevelException;
import org.tencompetence.business.exceptions.EndlessLoopException;
import org.tencompetence.business.facade.validators.CompetenceLevelValidator;
import org.tencompetence.business.services.PortfolioService;
import org.tencompetence.generated.model.ActivityWrapper;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.LevelValue;
import org.tencompetence.generated.service.CompetenceLevelLocalServiceUtil;

import com.liferay.counter.service.CounterLocalServiceUtil;
//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.service.ServiceContext;

/**
 * Enter the class documentation here.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @version $Revision: 1.13 $, $Date: 2009/11/16 12:10:57 $
 */
public final class CompetenceLevelFacade {

	/**
	 * Instantiates a new competence level facade.
	 */
	private CompetenceLevelFacade() {
	}

	/**
	 * Creates a new competence level instance.
	 * <p>
	 * The entity is not persisted in the database.
	 * 
	 * @return the competence level
	 * @throws SystemException
	 *             the system exception
	 */
	protected static CompetenceLevel createCompetenceLevel() throws SystemException {
		final long id = CounterLocalServiceUtil.increment();
		final CompetenceLevel result = CompetenceLevelLocalServiceUtil.createCompetenceLevel(id);
		result.setNew(true);

		return result;
	}

	/**
	 * Adds the competence level.
	 * 
	 * @param competenceLevel
	 *            the competence level
	 * @param ctx
	 *            the ctx
	 * @return the competence level
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static CompetenceLevel addCompetenceLevel(final CompetenceLevel competenceLevel, final ServiceContext ctx)
			throws SystemException, PortalException {
		final List<String> errors = new ArrayList<String>();

		if (!CompetenceLevelValidator.validate(competenceLevel, errors, ctx)) {
			throw new CompetenceLevelException("Invalid Competence : " + errors.toString());
		}

		final CompetenceLevel result = CompetenceLevelLocalServiceUtil.addCompetenceLevel(competenceLevel);

		// Also add permissions for the CompetenceLevels
		FacadeUtils.addPermissions(competenceLevel, ctx);
		PortfolioService.addJournalEntry(result, EventType.CREATE, ctx);

		return result;
	}

	/**
	 * Get a CompetenceLevel without title and description.
	 * 
	 * @param competenceLevelId
	 *            competenceId
	 * @return Competence
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static CompetenceLevel getBareCompetenceLevel(final long competenceLevelId) throws SystemException,
			PortalException {
		return CompetenceLevelLocalServiceUtil.getBareCompetenceLevel(competenceLevelId);
	}

	/**
	 * Get CompetenceLevels without title and description for a Competence.
	 * 
	 * @param competenceId
	 *            competenceId
	 * @return List<CompetenceLevel>
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static List<CompetenceLevel> getBareCompetenceLevelsByCompetenceId(final long competenceId)
			throws SystemException, PortalException {
		return CompetenceLevelLocalServiceUtil.getBareCompetenceLevelsByCompetenceId(competenceId);
	}

	/**
	 * Get a CompetenceLevel without title and description.
	 * 
	 * @param competenceId
	 *            the competenceId
	 * @param levelValueId
	 *            the levelValueId
	 * @return Competence
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static CompetenceLevel getBareCompetenceLevelByCompetenceIdLevelValueId(final long competenceId,
			final long levelValueId) throws PortalException, SystemException {
		return CompetenceLevelLocalServiceUtil.getBareCompetenceLevelByCompetenceIdLevelValueId(competenceId,
				levelValueId);
	}

	/**
	 * Get a CompetenceLevels without title and description for a CompetenceMap.
	 * 
	 * @param mapId
	 *            mapId
	 * @return List<CompetenceLevel>
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static List<CompetenceLevel> getBareCompetenceLevelsByCompetenceMapId(final long mapId)
			throws SystemException, PortalException {
		return CompetenceLevelLocalServiceUtil.getBareCompetenceLevelsByCompetenceMapId(mapId);
	}

	/**
	 * Creates a new version for the passed competence level.
	 * 
	 * @param competenceLevel
	 *            the competence level
	 * @param ctx
	 *            the ctx
	 * @return the competence level
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static CompetenceLevel addCompetenceLevelNewVersion(final CompetenceLevel competenceLevel,
			final ServiceContext ctx) throws SystemException, PortalException {

		final List<String> errors = new ArrayList<String>();

		final long primaryKey = competenceLevel.getPrimaryKey();

		// Create a new CompetenceLevel
		final CompetenceLevel newLevel = createCompetenceLevel();

		// Copy all values from current CompetenceLevel
		newLevel.setUserId(competenceLevel.getUserId());
		newLevel.setGroupId(competenceLevel.getGroupId());
		newLevel.setCompanyId(competenceLevel.getCompanyId());
		newLevel.setCompetenceLevelId(newLevel.getPrimaryKey());
		newLevel.setCompetenceId(competenceLevel.getCompetenceId());
		newLevel.setParentId(primaryKey);
		newLevel.setTitle(competenceLevel.getTitle());
		newLevel.setDescription(competenceLevel.getDescription());
		newLevel.setVersion(competenceLevel.getVersion() + 1);
		newLevel.setVersionTag(competenceLevel.getVersionTag());
		newLevel.setIndex(competenceLevel.getIndex());
		newLevel.setLevelValueId(competenceLevel.getLevelValueId());
		newLevel.setChildLevelIds(competenceLevel.getChildLevelIds());
		newLevel.setMetaLevelIds(competenceLevel.getMetaLevelIds());

		// Validate
		if (!CompetenceLevelValidator.validate(newLevel, errors, ctx)) {
			throw new CompetenceLevelException("Invalid CompetenceLevel : " + errors.toString());
		}

		updateCompetenceLevel(newLevel, ctx);

		FacadeUtils.addPermissions(newLevel, ctx);

		PortfolioService.addJournalEntry(newLevel, EventType.CREATE_VERSION, ctx);

		// Update the existing CompetenceLevel
		final CompetenceLevel existingLevel = getBareCompetenceLevel(primaryKey);
		existingLevel.setChildId(newLevel.getPrimaryKey());

		CompetenceLevelLocalServiceUtil.updateCompetenceLevel(existingLevel, false);

		return newLevel;
	}

	/**
	 * Update the competence level.
	 * 
	 * @param competenceLevel
	 *            the competence level
	 * @param ctx
	 *            the ctx
	 * @return the competence level
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static CompetenceLevel updateCompetenceLevel(final CompetenceLevel competenceLevel, final ServiceContext ctx)
			throws SystemException, PortalException {
		final List<String> errors = new ArrayList<String>();

		if (!CompetenceLevelValidator.validate(competenceLevel, errors, ctx)) {
			throw new CompetenceLevelException("Invalid Competence : " + errors.toString());
		}

		final CompetenceLevel result = CompetenceLevelLocalServiceUtil.updateCompetenceLevel(competenceLevel);

		PortfolioService.addJournalEntry(result, EventType.UPDATE, ctx);

		return result;
	}

	/**
	 * Gets the competence level.
	 * 
	 * @param competenceLevelId
	 *            the competence level id
	 * @param ctx
	 *            the ctx
	 * @return the competence level
	 * @throws PortalException
	 *             the portal exception
	 * @throws SystemException
	 *             the system exception
	 */
	public static CompetenceLevel getCompetenceLevel(final long competenceLevelId, final ServiceContext ctx)
			throws PortalException, SystemException {
		final CompetenceLevel result = CompetenceLevelLocalServiceUtil.getCompetenceLevel(competenceLevelId);

		return result;
	}

	/**
	 * Gets the competence levels by ids.
	 * 
	 * @param ids
	 *            the ids
	 * @param ctx
	 *            the ctx
	 * @return the competence levels by ids
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<CompetenceLevel> getCompetenceLevelsByIds(final String ids, final ServiceContext ctx)
			throws SystemException {
		return getCompetenceLevelsByIds(FacadeUtils.decodeEntries(ids), ctx);
	}

	/**
	 * Gets the competence levels by ids.
	 * 
	 * @param ids
	 *            the ids
	 * @param ctx
	 *            the ctx
	 * @return the competence levels by ids
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<CompetenceLevel> getCompetenceLevelsByIds(final List<Long> ids, final ServiceContext ctx)
			throws SystemException {
		return CompetenceLevelLocalServiceUtil.getCompetenceLevelsByIds(ids);
	}

	/**
	 * Get the attainable competence levels. Included level 0.
	 * 
	 * @param competenceId
	 *            competence id
	 * @param ctx
	 *            Context
	 * @return List of competence levels
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<CompetenceLevel> getAttainableCompetenceLevelsByCompetenceId(final long competenceId,
			final ServiceContext ctx) throws SystemException {
		return CompetenceLevelLocalServiceUtil.getCompetenceLevelsByCompetenceId(competenceId);
	}

	/**
	 * Get the required competence levels. Level 0 is excluded.
	 * 
	 * @param competenceId
	 *            competence id
	 * @param ctx
	 *            Context
	 * @return List of competence levels
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the Portal exception
	 */
	public static List<CompetenceLevel> getRequirableCompetenceLevelsByCompetenceId(final long competenceId,
			final ServiceContext ctx) throws SystemException, PortalException {
		final List<CompetenceLevel> result = new ArrayList<CompetenceLevel>();

		for (CompetenceLevel level : CompetenceLevelFacade.getAttainableCompetenceLevelsByCompetenceId(competenceId,
				ctx)) {
			final LevelValue lv = LevelValueFacade.getLevelValue(level.getLevelValueId(), ctx);
			if (lv.getValue() != 0) {
				result.add(level);
			}
		}

		return result;

	}

	/**
	 * Gets the competence levels.
	 * 
	 * @param start
	 *            the start
	 * @param end
	 *            the end
	 * @param ctx
	 *            the ctx
	 * @return the competence levels
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<CompetenceLevel> getCompetenceLevels(final int start, final int end, final ServiceContext ctx)
			throws SystemException {
		return CompetenceLevelLocalServiceUtil.getCompetenceLevels(start, end);
	}

	/**
	 * Gets the competence levels by competence map id.
	 * 
	 * @param mapId
	 *            the map id
	 * @param ctx
	 *            the ctx
	 * @return the competence levels by competence map id
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<CompetenceLevel> getCompetenceLevelsByCompetenceMapId(final long mapId, final ServiceContext ctx)
			throws SystemException {
		return CompetenceLevelLocalServiceUtil.getCompetenceLevelsByCompetenceMapId(mapId);
	}

	/**
	 * Get a list from leaf competenceLevels by a CompetenceProfileId. (If more then one competenceLevel with the same
	 * competence is found, the highest is returned.)
	 * 
	 * @param competenceProfileLevelId
	 *            competence profile level id
	 * @param ctx
	 *            Context
	 * @return List of competence levels
	 * @throws SystemException
	 *             the system exception
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 * @throws EndlessLoopException
	 *             EndlessLoopException
	 */
	public static List<CompetenceLevel> getHighestLeafCompetenceLevelsByCompetenceProfileId(
			final long competenceProfileLevelId, final ServiceContext ctx) throws SystemException, PortalException,
			EndlessLoopException {

		final Map<Long, CompetenceLevel> map = new HashMap<Long, CompetenceLevel>();

		for (final CompetenceLevel leaf : getLeafCompetenceLevelsByCompetenceProfileId(competenceProfileLevelId, ctx)) {
			final Long competenceId = leaf.getCompetenceId();
			final LevelValue value = LevelValueFacade.getLevelValue(leaf.getLevelValueId(), ctx);

			if (map.containsKey(competenceId)) {
				final CompetenceLevel mapLevel = map.get(competenceId);
				final LevelValue mapValue = LevelValueFacade.getLevelValue(mapLevel.getLevelValueId(), ctx);

				// If another leaf has a higher level, add this to the list.
				if (value.getValue() > mapValue.getValue()) {
					map.put(competenceId, leaf);
				}
			} else {
				map.put(competenceId, leaf);
			}
		}

		return new ArrayList<CompetenceLevel>(map.values());
	}

	/**
	 * Get the leaf competenceLevels by a CompetenceProfileId.
	 * 
	 * @param competenceProfileLevelId
	 *            competence profile level id
	 * @param ctx
	 *            Context
	 * @return List of competence levels
	 * @throws SystemException
	 *             the system exception
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 * @throws EndlessLoopException
	 *             EndlessLoopException
	 */
	public static List<CompetenceLevel> getLeafCompetenceLevelsByCompetenceProfileId(
			final long competenceProfileLevelId, final ServiceContext ctx) throws SystemException, PortalException,
			EndlessLoopException {
		final List<CompetenceLevel> leafs = new ArrayList<CompetenceLevel>();

		final List<CompetenceLevel> competenceLevels = CompetenceProfileLevelFacade.getCompetenceLevels(
				competenceProfileLevelId, ctx);
		for (final CompetenceLevel level : competenceLevels) {
			findLeafs(leafs, level, ctx);
		}

		return leafs;
	}

	/**
	 * Method findLeaf. Loop through the childeren of a given CompetenceLevel. See if it's a leaf. If so, add it to the
	 * existing list, otherwise recurse. Return the new list.
	 * 
	 * @param leafs
	 * @param level
	 * @param ctx
	 * @throws SystemException
	 * @throws EndlessLoopException
	 */
	private static void findLeafs(final List<CompetenceLevel> leafs, final CompetenceLevel level,
			final ServiceContext ctx) throws SystemException, EndlessLoopException {
		int counter = 0;
		final List<CompetenceLevel> children = CompetenceLevelFacade.getCompetenceLevelsByIds(level.getChildLevelIds(),
				ctx);

		if (children.size() == 0 && !leafs.contains(level)) {
			leafs.add(level);
		} else {
			for (final CompetenceLevel child : children) {
				if (counter == ServiceConstants.MAX_LOOP_COUNTER) {
					throw new EndlessLoopException("Current competenceLevelId " + child.getCompetenceLevelId());
				}
				findLeafs(leafs, child, ctx);
			}
		}
	}

	/**
	 * Clear activities.
	 * 
	 * @param competenceLevel
	 *            the competence level
	 * @param ctx
	 *            the ctx
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static void clearActivities(final CompetenceLevel competenceLevel, final ServiceContext ctx)
			throws SystemException, PortalException {
		CompetenceLevelLocalServiceUtil.clearActivities(competenceLevel);

		PortfolioService.addJournalEntry(competenceLevel, EventType.UPDATE, ctx);
	}

	/**
	 * Adds the wrapper activities.
	 * 
	 * @param competenceLevel
	 *            the competence level
	 * @param activities
	 *            the activities
	 * @param ctx
	 *            the ctx
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static void addWrapperActivities(final CompetenceLevel competenceLevel,
			final List<ActivityWrapper> activities, final ServiceContext ctx) throws SystemException, PortalException {

		CompetenceLevelLocalServiceUtil.addWrapperActivities(competenceLevel, activities);

		PortfolioService.addJournalEntry(competenceLevel, EventType.UPDATE, ctx);

	}

	/**
	 * Removes the wrapper activities.
	 * 
	 * @param competenceLevel
	 *            the competence level
	 * @param activities
	 *            the activities
	 * @param ctx
	 *            the ctx
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static void removeWrapperActivities(final CompetenceLevel competenceLevel,
			final List<ActivityWrapper> activities, final ServiceContext ctx) throws SystemException, PortalException {

		CompetenceLevelLocalServiceUtil.removeWrapperActivities(competenceLevel, activities);

		PortfolioService.addJournalEntry(competenceLevel, EventType.UPDATE, ctx);
	}

	/**
	 * Gets the wrapper activities.
	 * 
	 * @param competenceLevel
	 *            the competence level
	 * @param ctx
	 *            the ctx
	 * @return the wrapper activities
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<ActivityWrapper> getWrapperActivities(final CompetenceLevel competenceLevel,
			final ServiceContext ctx) throws SystemException {
		return CompetenceLevelLocalServiceUtil.getWrapperActivities(competenceLevel);
	}

}
