/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.facade;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.business.constants.EventType;
import org.tencompetence.business.exceptions.CompetenceMapException;
import org.tencompetence.business.facade.validators.CompetenceMapValidator;
import org.tencompetence.business.services.PortfolioService;
import org.tencompetence.generated.model.CompetenceMap;
import org.tencompetence.generated.model.LevelValue;
import org.tencompetence.generated.service.CompetenceMapLocalServiceUtil;
import org.tencompetence.services.constants.LevelValueType;

import com.liferay.counter.service.CounterLocalServiceUtil;
//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.service.ServiceContext;

/**
 * Implements the CompetenceMap service facade.
 * 
 * @author Harrie Martens
 * @version $Id: CompetenceMapFacade.java,v 1.6 2009/10/28 12:59:27 sheyenrath Exp $
 */
public final class CompetenceMapFacade {

	// Hide the constructor to disallow class creation
	private CompetenceMapFacade() {
	}

	/**
	 * Creates the competence map.
	 * 
	 * @return the competence map
	 * @throws SystemException
	 *             the system exception
	 */
	public static CompetenceMap createCompetenceMap() throws SystemException {
		final long id = CounterLocalServiceUtil.increment();
		final CompetenceMap result = CompetenceMapLocalServiceUtil.createCompetenceMap(id);
		result.setNew(true);

		return result;
	}

	/**
	 * Adds the competence map.
	 * 
	 * @param map
	 *            the map
	 * @param competenceLevelValues
	 *            the competence level values
	 * @param competenceProfileLevelValues
	 *            the competence profile level values
	 * @param ctx
	 *            the ctx
	 * @return the competence map
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static CompetenceMap addCompetenceMap(final CompetenceMap map, final int competenceLevelValues,
			final int competenceProfileLevelValues, final ServiceContext ctx) throws SystemException, PortalException {

		final List<String> errors = new ArrayList<String>();

		if (!CompetenceMapValidator.validate(map, errors)) {
			throw new CompetenceMapException("Invalid CompetenceMap : " + errors.toString());
		}

		if (getCompetenceMapByGroupId(map.getGroupId(), ctx) != null) {
			throw new CompetenceMapException("CompetenceMap already exists for groupId : " + map.getGroupId());
		}

		final CompetenceMap result = CompetenceMapLocalServiceUtil.addCompetenceMap(map);

		// reIndex(map);

		addLevelValuesToMap(result, 0, competenceLevelValues, LevelValueType.Competence, ctx);
		addLevelValuesToMap(result, 1, competenceProfileLevelValues, LevelValueType.CompetenceProfile, ctx);

		FacadeUtils.addPermissions(result, ctx);
		PortfolioService.addJournalEntry(result, EventType.CREATE, ctx);

		return map;
	}

	/**
	 * Gets the competence map.
	 * 
	 * @param competenceMapId
	 *            the competence map id
	 * @param ctx
	 *            the ctx
	 * @return the competence map
	 * @throws PortalException
	 *             the portal exception
	 * @throws SystemException
	 *             the system exception
	 */
	public static CompetenceMap getCompetenceMap(final long competenceMapId, final ServiceContext ctx)
			throws PortalException, SystemException {
		return CompetenceMapLocalServiceUtil.getCompetenceMap(competenceMapId);
	}

	/**
	 * Gets the competence map by company id.
	 * 
	 * @param companyId
	 *            the company id
	 * @param ctx
	 *            the ctx
	 * @return the competence map by company id
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static CompetenceMap getCompetenceMapByCompanyId(final long companyId, final ServiceContext ctx)
			throws SystemException, PortalException {
		final List<CompetenceMap> list = CompetenceMapLocalServiceUtil.getCompetenceMapByCompanyId(companyId);

		if (list != null) {
			if (list.isEmpty()) {
				return null;
			} else if (list.size() == 1) {
				return list.get(0);
			} else {
				throw new CompetenceMapException("More then 1 CompetenceMap found for companyId : " + companyId);
			}
		}

		return null;
	}

	/**
	 * Gets the competence map by group id.
	 * 
	 * @param groupId
	 *            the group id
	 * @param ctx
	 *            the ctx
	 * @return the competence map by group id
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static CompetenceMap getCompetenceMapByGroupId(final long groupId, final ServiceContext ctx)
			throws SystemException, PortalException {

		final List<CompetenceMap> list = CompetenceMapLocalServiceUtil.getCompetenceMapByGroupId(groupId);

		if (list != null) {
			if (list.isEmpty()) {
				return null;
			} else if (list.size() == 1) {
				return list.get(0);
			} else {
				throw new CompetenceMapException("More then 1 CompetenceMap found for groupId : " + groupId);
			}
		}

		return null;
	}

	/**
	 * Update competence map.
	 * 
	 * @param competenceMap
	 *            the competence map
	 * @param ctx
	 *            the ctx
	 * @return the competence map
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static CompetenceMap updateCompetenceMap(final CompetenceMap competenceMap, final ServiceContext ctx)
			throws SystemException, PortalException {

		final List<String> errors = new ArrayList<String>();
		if (!CompetenceMapValidator.validate(competenceMap, errors)) {
			throw new CompetenceMapException("Invalid CompetenceMap : " + errors.toString());
		}

		// reIndex(map);

		final CompetenceMap result = CompetenceMapLocalServiceUtil.updateCompetenceMap(competenceMap);

		PortfolioService.addJournalEntry(result, EventType.UPDATE, ctx);

		return result;
	}

	/**
	 * Update competence profile level values.
	 * 
	 * @param competenceMap
	 *            the competence map
	 * @param numLevels
	 *            the num levels
	 * @param ctx
	 *            the ctx
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static void updateCompetenceProfileLevelValues(final CompetenceMap competenceMap, final int numLevels,
			final ServiceContext ctx) throws SystemException, PortalException {
		final List<LevelValue> levels = LevelValueFacade.getLevelValuesByCompetenceMapIdAndType(competenceMap
				.getPrimaryKey(), LevelValueType.CompetenceProfile.getId(), ctx);
		final int currentLevelCount = levels.size();

		if (numLevels > currentLevelCount) {
			// Extend the range if needed
			addLevelValuesToMap(competenceMap, currentLevelCount + 1, numLevels, LevelValueType.CompetenceProfile, ctx);
		} else if (numLevels < currentLevelCount) {
			// If a value > numLevels, set the value to numLevels
			for (final LevelValue lvlValue : levels) {
				if (lvlValue.getValue() > numLevels) {
					lvlValue.setValue(numLevels);
					LevelValueFacade.updateLevelValue(lvlValue, ctx);
				}
			}
		}

		PortfolioService.addJournalEntry(competenceMap, EventType.UPDATE, ctx);
	}

	/**
	 * Update the CompetenceLevelValues to a new number.
	 * 
	 * @param competenceMap
	 *            The competenceMap
	 * @param numLevels
	 *            The new number of CompetenceLevelValues
	 * @param ctx
	 *            ctx
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             the portal exception
	 */
	public static void updateCompetenceLevelValues(final CompetenceMap competenceMap, final int numLevels,
			final ServiceContext ctx) throws SystemException, PortalException {
		final List<LevelValue> levels = LevelValueFacade.getLevelValuesByCompetenceMapIdAndType(competenceMap
				.getPrimaryKey(), LevelValueType.Competence.getId(), ctx);
		final int currentRequiredLevelCount = levels.size() - 1;

		if (numLevels > currentRequiredLevelCount) {
			// Extend the range if needed
			addLevelValuesToMap(competenceMap, currentRequiredLevelCount + 1, numLevels, LevelValueType.Competence, ctx);
		} else if (numLevels < currentRequiredLevelCount) {
			// If a value > numLevels, set the value to numLevels
			for (final LevelValue lvlValue : levels) {
				if (lvlValue.getValue() > numLevels) {
					lvlValue.setValue(numLevels);
					LevelValueFacade.updateLevelValue(lvlValue, ctx);
				}
			}
		}
		PortfolioService.addJournalEntry(competenceMap, EventType.UPDATE, ctx);
	}

	/**
	 * Update the CompetenceProfileLevelValues to a new number.
	 * 
	 * @param competenceMap
	 *            The competenceMap
	 * @param newLevelCount
	 *            The new number of CompetenceProfileLevelValues
	 * @throws SystemException
	 *             SystemException
	 */
	public static void updateCompetenceProfileLevelValues(final CompetenceMap competenceMap, final int newLevelCount)
			throws SystemException {
	}

	private static void addLevelValuesToMap(final CompetenceMap map, final int minLevel, final int maxLevel,
			final LevelValueType type, final ServiceContext ctx) throws SystemException, PortalException {

		for (int val = minLevel; val <= maxLevel; val++) {
			final LevelValue value = LevelValueFacade.createLevelValue();
			value.setUserId(map.getUserId());
			value.setGroupId(map.getGroupId());
			value.setCompanyId(map.getCompanyId());
			value.setCompetenceMapId(map.getCompetenceMapId());
			value.setTitle(map.getTitle() + " - " + type.toString() + "Level - " + val);
			value.setDescription(Constants.DEFAULT_DESCRIPTION);
			value.setValue(val);
			value.setType(type.getId());
			LevelValueFacade.addLevelValue(value, ctx);
		}
	}

}
