/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.facade;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.business.constants.EventType;
import org.tencompetence.business.exceptions.CompetenceProfileException;
import org.tencompetence.business.facade.validators.CompetenceProfileValidator;
import org.tencompetence.business.services.PortfolioService;
import org.tencompetence.generated.model.CompetenceProfile;
import org.tencompetence.generated.service.CompetenceProfileLocalServiceUtil;

import com.liferay.counter.service.CounterLocalServiceUtil;
//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.util.OrderByComparator;
import com.liferay.portal.service.ServiceContext;

/**
 * Implements the CompetenceProfile service facade.
 * 
 * @author Harrie Martens
 * @version $Id: CompetenceProfileFacade.java,v 1.6 2009/10/28 12:59:27 sheyenrath Exp $
 */
public final class CompetenceProfileFacade {

	// Hide the constructor to prevent class instantiation
	private CompetenceProfileFacade() {
	}

	/**
	 * Creates the competence profile.
	 * 
	 * @return the competence profile
	 * @throws SystemException
	 *             the system exception
	 */
	public static CompetenceProfile createCompetenceProfile() throws SystemException {
		final long id = CounterLocalServiceUtil.increment();
		final CompetenceProfile result = CompetenceProfileLocalServiceUtil.createCompetenceProfile(id);
		result.setNew(true);

		return result;
	}

	/**
	 * Add a CompetenceProfile.
	 * 
	 * @param competenceProfile
	 *            The CompetenceProfile
	 * @param ctx
	 *            The ServiceContext
	 * @return ActivityWrapper
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static CompetenceProfile addCompetenceProfile(final CompetenceProfile competenceProfile,
			final ServiceContext ctx) throws SystemException, PortalException {

		validate(competenceProfile, ctx);
		final CompetenceProfile result = CompetenceProfileLocalServiceUtil.addCompetenceProfile(competenceProfile);

		PortfolioService.addJournalEntry(result, EventType.CREATE, ctx);
		FacadeUtils.addPermissions(result, ctx);

		return result;
	}

	/**
	 * Update competence profile.
	 * 
	 * @param competenceProfile
	 *            the competence profile
	 * @param ctx
	 *            the ctx
	 * @return the competence profile
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static CompetenceProfile updateCompetenceProfile(final CompetenceProfile competenceProfile,
			final ServiceContext ctx) throws SystemException, PortalException {

		validate(competenceProfile, ctx);
		final CompetenceProfile result = CompetenceProfileLocalServiceUtil.updateCompetenceProfile(competenceProfile);

		PortfolioService.addJournalEntry(result, EventType.UPDATE, ctx);

		return result;
	}

	/**
	 * Gets the competence profiles count.
	 * 
	 * @param competenceMapId
	 *            the competence map id
	 * @param ctx
	 *            the ctx
	 * @return the competence profiles count
	 * @throws SystemException
	 *             the system exception
	 */
	public static int getCompetenceProfilesCount(final long competenceMapId, final ServiceContext ctx)
			throws SystemException {
		return CompetenceProfileLocalServiceUtil.getCompetenceProfilesCount(competenceMapId);
	}

	/**
	 * Gets the competence profiles by competence map id.
	 * 
	 * @param competenceMapId
	 *            the competence map id
	 * @param ctx
	 *            the ctx
	 * @return the competence profiles by competence map id
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<CompetenceProfile> getCompetenceProfilesByCompetenceMapId(final long competenceMapId,
			final ServiceContext ctx) throws SystemException {
		return CompetenceProfileLocalServiceUtil.getCompetenceProfilesByCompetenceMapId(competenceMapId);

	}

	/**
	 * Gets the competence profiles by competence map id.
	 * 
	 * @param competenceMapId
	 *            the competence map id
	 * @param start
	 *            the start
	 * @param end
	 *            the end
	 * @param ctx
	 *            the ctx
	 * @return the competence profiles by competence map id
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<CompetenceProfile> getCompetenceProfilesByCompetenceMapId(final long competenceMapId,
			final int start, final int end, final ServiceContext ctx) throws SystemException {
		return CompetenceProfileLocalServiceUtil.getCompetenceProfilesByCompetenceMapId(competenceMapId, start, end);
	}

	/**
	 * Gets the competence profiles by competence map id.
	 * 
	 * @param competenceMapId
	 *            the competence map id
	 * @param start
	 *            the start
	 * @param end
	 *            the end
	 * @param orderByComparator
	 *            the order by comparator
	 * @param ctx
	 *            the ctx
	 * @return the competence profiles by competence map id
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<CompetenceProfile> getCompetenceProfilesByCompetenceMapId(final long competenceMapId,
			final int start, final int end, final OrderByComparator orderByComparator, final ServiceContext ctx)
			throws SystemException {
		return CompetenceProfileLocalServiceUtil.getCompetenceProfilesByCompetenceMapId(competenceMapId, start, end,
				orderByComparator);
	}

	/**
	 * Gets the competence profile.
	 * 
	 * @param competenceProfileId
	 *            the competence profile id
	 * @param ctx
	 *            the ctx
	 * @return the competence profile
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static CompetenceProfile getCompetenceProfile(final long competenceProfileId, final ServiceContext ctx)
			throws SystemException, PortalException {
		return CompetenceProfileLocalServiceUtil.getCompetenceProfile(competenceProfileId);
	}

	private static void validate(final CompetenceProfile competenceProfile, final ServiceContext ctx)
			throws SystemException {
		final List<String> errors = new ArrayList<String>();

		if (!CompetenceProfileValidator.validate(competenceProfile, errors, ctx)) {
			throw new CompetenceProfileException("Invalid CompetenceProfile : " + errors.toString());
		}
	}

}
