/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.facade;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.business.constants.EventType;
import org.tencompetence.business.exceptions.CompetenceProfileLevelException;
import org.tencompetence.business.facade.validators.CompetenceProfileLevelValidator;
import org.tencompetence.business.services.PortfolioService;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.generated.service.CompetenceProfileLevelLocalServiceUtil;

import com.liferay.counter.service.CounterLocalServiceUtil;
//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.service.ServiceContext;

/**
 * Implements the CompetenceProfileLevel service facade.
 * 
 * @author Harrie Martens
 * @version $Id: CompetenceProfileLevelFacade.java,v 1.7 2009/11/11 08:53:19 sheyenrath Exp $
 */
public final class CompetenceProfileLevelFacade {

	// Hide the constructor to prevent class instantiation
	private CompetenceProfileLevelFacade() {
	}

	/**
	 * Creates the competence profile level.
	 * 
	 * @return the competence profile level
	 * @throws SystemException
	 *             the system exception
	 */
	public static CompetenceProfileLevel createCompetenceProfileLevel() throws SystemException {
		final long id = CounterLocalServiceUtil.increment();
		final CompetenceProfileLevel result = CompetenceProfileLevelLocalServiceUtil.createCompetenceProfileLevel(id);
		result.setNew(true);

		return result;
	}

	/**
	 * Adds the competence profile level.
	 * 
	 * @param competenceProfileLevel
	 *            the competence profile level
	 * @param ctx
	 *            the context
	 * @return the competence profile level
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static CompetenceProfileLevel addCompetenceProfileLevel(final CompetenceProfileLevel competenceProfileLevel,
			final ServiceContext ctx) throws SystemException, PortalException {

		competenceProfileLevel.setVersion(Constants.FIRST_VERSION);
		validate(competenceProfileLevel, ctx);
		final CompetenceProfileLevel result = CompetenceProfileLevelLocalServiceUtil
				.addCompetenceProfileLevel(competenceProfileLevel);

		FacadeUtils.addPermissions(result, ctx);
		PortfolioService.addJournalEntry(result, EventType.CREATE, ctx);

		return result;
	}

	/**
	 * Adds the competence profile level new version.
	 * 
	 * @param competenceProfileLevel
	 *            the competence profile level
	 * @param ctx
	 *            the context
	 * @return the competence profile level
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static CompetenceProfileLevel addCompetenceProfileLevelNewVersion(
			final CompetenceProfileLevel competenceProfileLevel, final ServiceContext ctx) throws SystemException,
			PortalException {

		final long primaryKey = competenceProfileLevel.getPrimaryKey();

		final CompetenceProfileLevel newLevel = createCompetenceProfileLevel();

		newLevel.setUserId(competenceProfileLevel.getUserId());
		newLevel.setGroupId(competenceProfileLevel.getGroupId());
		newLevel.setCompanyId(competenceProfileLevel.getCompanyId());
		newLevel.setCompetenceProfileLevelId(newLevel.getPrimaryKey());
		newLevel.setCompetenceProfileId(competenceProfileLevel.getCompetenceProfileId());
		newLevel.setParentId(primaryKey);
		newLevel.setTitle(competenceProfileLevel.getTitle());
		newLevel.setDescription(competenceProfileLevel.getDescription());
		newLevel.setVersion(competenceProfileLevel.getVersion() + 1);
		newLevel.setVersionTag(competenceProfileLevel.getVersionTag());
		newLevel.setIndex(competenceProfileLevel.getIndex());
		newLevel.setLevelValueId(competenceProfileLevel.getLevelValueId());
		newLevel.setCompetenceLevelIds(competenceProfileLevel.getCompetenceLevelIds());

		validate(newLevel, ctx);

		updateCompetenceProfileLevel(newLevel, ctx);

		// Update the existing CompetenceProfileLevel
		final CompetenceProfileLevel existingLevel = getCompetenceProfileLevel(primaryKey, ctx);
		existingLevel.setChildId(newLevel.getPrimaryKey());

		updateCompetenceProfileLevel(existingLevel, ctx);

		FacadeUtils.addPermissions(newLevel, ctx);
		PortfolioService.addJournalEntry(newLevel, EventType.CREATE_VERSION, ctx);

		return newLevel;
	}

	/**
	 * Update competence profile level.
	 * 
	 * @param competenceProfileLevel
	 *            the competence profile level
	 * @param ctx
	 *            the context
	 * @return the competence profile level
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static CompetenceProfileLevel updateCompetenceProfileLevel(
			final CompetenceProfileLevel competenceProfileLevel, final ServiceContext ctx) throws SystemException,
			PortalException {

		validate(competenceProfileLevel, ctx);
		final CompetenceProfileLevel result = CompetenceProfileLevelLocalServiceUtil
				.updateCompetenceProfileLevel(competenceProfileLevel);

		PortfolioService.addJournalEntry(result, EventType.UPDATE, ctx);

		return result;
	}

	/**
	 * Gets the competence profile levels by competence map id.
	 * 
	 * @param mapId
	 *            the map id
	 * @param ctx
	 *            the ctx
	 * @return the competence profile levels by competence map id
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static List<CompetenceProfileLevel> getCompetenceProfileLevelsByCompetenceMapId(final long mapId,
			final ServiceContext ctx) throws SystemException, PortalException {
		return CompetenceProfileLevelLocalServiceUtil.getCompetenceProfileLevelsByCompetenceMapId(mapId);
	}

	/**
	 * Gets the competence profile levels by competence profile id.
	 * 
	 * @param competenceProfileId
	 *            the competence profile id
	 * @param ctx
	 *            the context
	 * @return the competence profile levels by competence profile id
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static List<CompetenceProfileLevel> getCompetenceProfileLevelsByCompetenceProfileId(
			final long competenceProfileId, final ServiceContext ctx) throws SystemException, PortalException {
		return CompetenceProfileLevelLocalServiceUtil
				.getCompetenceProfileLevelsByCompetenceProfileId(competenceProfileId);
	}

	/**
	 * Gets the competence profile level.
	 * 
	 * @param competenceProfileLevelId
	 *            the competence profile level id
	 * @param ctx
	 *            the ctx
	 * @return the competence profile level
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static CompetenceProfileLevel getCompetenceProfileLevel(final long competenceProfileLevelId,
			final ServiceContext ctx) throws SystemException, PortalException {
		return CompetenceProfileLevelLocalServiceUtil.getCompetenceProfileLevel(competenceProfileLevelId);
	}

	/**
	 * Gets the competence profile levels by ids.
	 * 
	 * @param ids
	 *            the ids
	 * @return the competence profile levels by ids
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<CompetenceProfileLevel> getCompetenceProfileLevelsByIds(final List<Long> ids)
			throws SystemException {
		return CompetenceProfileLevelLocalServiceUtil.getCompetenceProfileLevelsByIds(ids);
	}

	/**
	 * Gets the competence levels.
	 * 
	 * @param ctx
	 *            the ctx
	 * @param id
	 *            the id
	 * @return the competence levels
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static List<CompetenceLevel> getCompetenceLevels(final long id, final ServiceContext ctx)
			throws SystemException, PortalException {
		final CompetenceProfileLevel level = getCompetenceProfileLevel(id, ctx);
		final List<Long> ids = FacadeUtils.decodeEntries(level.getCompetenceLevelIds());

		return CompetenceLevelFacade.getCompetenceLevelsByIds(ids, ctx);
	}

	private static void validate(final CompetenceProfileLevel competenceProfileLevel, final ServiceContext ctx)
			throws SystemException {
		final List<String> errors = new ArrayList<String>();

		if (!CompetenceProfileLevelValidator.validate(competenceProfileLevel, errors, ctx)) {
			throw new CompetenceProfileLevelException("Invalid CompetenceProfileLevel : " + errors.toString());
		}
	}

}
