/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.facade;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.business.constants.EventType;
import org.tencompetence.business.exceptions.EvidenceException;
import org.tencompetence.business.facade.validators.EvidenceValidator;
import org.tencompetence.business.services.PortfolioService;
import org.tencompetence.generated.model.Evidence;
import org.tencompetence.generated.service.EvidenceLocalServiceUtil;

import com.liferay.counter.service.CounterLocalServiceUtil;
//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.service.ServiceContext;

/**
 * Implements the Evidence service facade.
 * 
 * @author Harrie Martens
 * @version $Id: EvidenceFacade.java,v 1.9 2009/11/10 09:36:36 sheyenrath Exp $
 */
public final class EvidenceFacade {

	// Hide the constructor to prevent class instantiation
	private EvidenceFacade() {
	}

	/**
	 * Creates the evidence.
	 * 
	 * @return the evidence
	 * @throws SystemException
	 *             the system exception
	 */
	public static Evidence createEvidence() throws SystemException {
		final long id = CounterLocalServiceUtil.increment();
		final Evidence result = EvidenceLocalServiceUtil.createEvidence(id);
		result.setNew(true);

		return result;
	}

	/**
	 * Adds the evidence.
	 * 
	 * @param evidence
	 *            the evidence
	 * @param ctx
	 *            the ctx
	 * @return the evidence
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static Evidence addEvidence(final Evidence evidence, final ServiceContext ctx) throws SystemException,
			PortalException {

		validate(evidence, ctx);
		final Evidence result = EvidenceLocalServiceUtil.addEvidence(evidence);

		FacadeUtils.addPermissions(result, ctx);
		PortfolioService.addJournalEntry(result, EventType.CREATE, ctx);

		return result;
	}

	/**
	 * Update evidence.
	 * 
	 * @param evidence
	 *            the evidence
	 * @param ctx
	 *            the ctx
	 * @return the evidence
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static Evidence updateEvidence(final Evidence evidence, final ServiceContext ctx) throws SystemException,
			PortalException {
		validate(evidence, ctx);
		final Evidence result = EvidenceLocalServiceUtil.updateEvidence(evidence);

		PortfolioService.addJournalEntry(result, EventType.UPDATE, ctx);

		return result;
	}

	/**
	 * Get evidence.
	 * 
	 * @param evidenceId
	 *            the evidenceId
	 * @param ctx
	 *            the ctx
	 * @return the evidence
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static Evidence getEvidence(final long evidenceId, final ServiceContext ctx) throws SystemException,
			PortalException {
		return EvidenceLocalServiceUtil.getEvidence(evidenceId);
	}

	private static void validate(final Evidence evidence, final ServiceContext ctx) throws SystemException {
		final List<String> errors = new ArrayList<String>();

		if (!EvidenceValidator.validate(evidence, errors, ctx)) {
			throw new EvidenceException("Invalid Evidence entity: " + errors.toString());
		}
	}

	/**
	 * Get all Evidences for a specific User and TargetType (=EntityClass).
	 * 
	 * @param userId
	 *            userId
	 * @param targetType
	 *            TargetType
	 * 
	 * @return List from Evidences
	 * 
	 * @throws SystemException
	 *             SystemException
	 */
	public static List<Evidence> getEvidencesByUserTargetTypes(final long userId, final String targetType)
			throws SystemException {
		return EvidenceLocalServiceUtil.getEvidencesByUserTargetType(userId, targetType);
	}

	/**
	 * Get all Evidences for a specific User and TargetTypes (=EntityClasses).
	 * 
	 * @param userId
	 *            userId
	 * @param targetTypes
	 *            TargetTypes
	 * 
	 * @return List from Evidences
	 * 
	 * @throws SystemException
	 *             SystemException
	 */
	public static List<Evidence> getEvidencesByUserTargetTypes(final long userId, final List<String> targetTypes)
			throws SystemException {
		return EvidenceLocalServiceUtil.getEvidencesByUserTargetTypes(userId, targetTypes);
	}

	/**
	 * Get all Evidences for a specific User,TargetType (=EntityClass) and TargetId.
	 * 
	 * @param userId
	 *            userId
	 * @param targetType
	 *            TargetType
	 * @param targetId
	 *            TargetId
	 * 
	 * @return List from Evidences
	 * 
	 * @throws SystemException
	 *             SystemException
	 */
	public static List<Evidence> getEvidencesByUserTargetTypeTargetId(final long userId, final String targetType,
			final Long targetId) throws SystemException {
		return EvidenceLocalServiceUtil.getEvidencesByUserTargetTypeTargetId(userId, targetType, targetId);
	}

	/**
	 * Get a list from entityId's from an Evidences for a specific TargetType (=EntityClasse).
	 * 
	 * @param evidenceId
	 *            evidenceId
	 * @param targetType
	 *            TargetType
	 * 
	 * @return List from Evidences
	 * 
	 * @throws SystemException
	 *             SystemException
	 */
	public static List<Long> getEntityIdsByEvidenceId(final long evidenceId, final String targetType)
			throws SystemException {
		return EvidenceLocalServiceUtil.getEntityIdsByEvidenceId(evidenceId, targetType);
	}
}
