/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.facade;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.business.exceptions.CompetenceMapException;
import org.tencompetence.business.utils.EntityUtils;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.CompetenceMap;
import org.tencompetence.generated.model.CompetenceProfile;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.generated.model.LevelValue;
import org.tencompetence.generated.service.CompetenceLocalServiceUtil;
import org.tencompetence.generated.service.CompetenceProfileLocalServiceUtil;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.model.BaseModel;
import com.liferay.portal.model.ResourceConstants;
import com.liferay.portal.service.ResourceLocalServiceUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * Enter the class documentation here.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * 
 * @version $Revision: 1.10 $, $Date: 2009/10/28 12:59:27 $
 */
public final class FacadeUtils {
	private static final String SUSPENSION_POINTS = "...";

	private FacadeUtils() {

	}

	/**
	 * Add permissions to a BaseModel<T> entity.
	 * 
	 * @param <T>
	 *            Generic type
	 * @param entity
	 *            The ModelEntity
	 * @param ctx
	 *            The ServiceContext
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	protected static <T> void addPermissions(final BaseModel<T> entity, final ServiceContext ctx)
			throws PortalException, SystemException {
		final String className = EntityUtils.getInterfaceFromEntity(entity);
		final String primaryKey = entity.getPrimaryKeyObj().toString();

		addPermissions(className, primaryKey, ctx);
	}

	/**
	 * Add permissions to a className + primaryKey.
	 * 
	 * @param className
	 *            The className
	 * @param primaryKey
	 *            The primaryKey
	 * @param ctx
	 *            The ServiceContext
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	protected static void addPermissions(final String className, final String primaryKey, final ServiceContext ctx)
			throws PortalException, SystemException {
		if (ctx.isAddGroupPermissions() || ctx.isAddGuestPermissions()) {
			ResourceLocalServiceUtil.addResources(ctx.getCompanyId(), ctx.getScopeGroupId(), ctx.getUserId(),
					className, primaryKey, false, ctx.isAddGroupPermissions(), ctx.isAddGuestPermissions());
		} else {
			ResourceLocalServiceUtil.addModelResources(ctx.getCompanyId(), ctx.getScopeGroupId(), ctx.getUserId(),
					className, primaryKey, ctx.getGroupPermissions(), ctx.getGuestPermissions());
		}
	}

	/**
	 * Remove permissions from a BaseModel<T> entity.
	 * 
	 * @param <T>
	 *            Generic type
	 * @param entity
	 *            The ModelEntity
	 * @param ctx
	 *            The ServiceContext
	 * @throws PortalException
	 *             PortalException
	 * @throws SystemException
	 *             SystemException
	 */
	protected static <T> void removePermissions(final BaseModel<T> entity, final ServiceContext ctx)
			throws PortalException, SystemException {

		final String className = EntityUtils.getInterfaceFromEntity(entity);
		final String primaryKey = entity.getPrimaryKeyObj().toString();

		ResourceLocalServiceUtil.deleteResource(ctx.getCompanyId(), className, ResourceConstants.SCOPE_INDIVIDUAL,
				primaryKey);
	}

	/**
	 * Append a compentence title and a levelvalue title and check for the maximal length.
	 * 
	 * @param comp
	 *            the competence
	 * @param value
	 *            the LevelValue
	 * 
	 * @return the title string
	 */
	public static String appendTitle(final Competence comp, final LevelValue value) {
		final String valueTitle = " (" + value.getTitle() + ")";
		String newTitle = comp.getTitle() + valueTitle;
		// check if title is to long
		if (newTitle.length() > Constants.MAX_TITLE_LENGTH) {
			final int endOfSubString = Constants.MAX_TITLE_LENGTH - valueTitle.length() - SUSPENSION_POINTS.length();
			if (endOfSubString > 0) {
				// removing competenceTitle is enough to fix the problem
				newTitle = comp.getTitle().substring(0, endOfSubString);
				newTitle += SUSPENSION_POINTS + valueTitle;
			} else {
				// there should be more characters removed then the comp title is long
				// so removing competenceTitle is not enough to fix the problem
				newTitle = valueTitle.substring(0, Constants.MAX_TITLE_LENGTH - SUSPENSION_POINTS.length());
			}
		}
		return newTitle;
	}

	/**
	 * Create a List<Long> from csv string.
	 * 
	 * @param csvString
	 *            csvString
	 * @return List<Long>
	 */
	public static List<Long> decodeEntries(final String csvString) {

		final List<Long> result = new ArrayList<Long>();

		if (Validator.isNotNull(csvString)) {
			for (final String value : csvString.split(",")) {
				result.add(Long.parseLong(value));
			}
		}

		return result;
	}

	/**
	 * Create a csv string from List<Long>.
	 * 
	 * @param values
	 *            A list
	 * @return CSV String
	 */
	public static String encodeEntries(final List<Long> values) {

		if (values == null || values.isEmpty()) {
			return null;
		} else {
			final StringBuilder sb = new StringBuilder();
			for (final Long value : values) {
				sb.append(value);
				sb.append(',');
			}

			final String result = sb.toString();
			return result.substring(0, result.length() - 1);
		}
	}

	/**
	 * Try to get the competenceMapId from an entity.
	 * 
	 * @param entity
	 *            the entity
	 * 
	 * @param <T>
	 *            The generic entity type
	 * 
	 * @return the competence map id
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static <T> long getCompetenceMapId(final BaseModel<T> entity) throws PortalException, SystemException {

		if (entity instanceof CompetenceMap) {
			return ((CompetenceMap) entity).getPrimaryKey();
		}

		if (entity instanceof Competence) {
			return ((Competence) entity).getCompetenceMapId();
		}

		if (entity instanceof CompetenceProfile) {
			return ((CompetenceProfile) entity).getCompetenceMapId();
		}

		if (entity instanceof CompetenceProfileLevel) {
			final CompetenceProfileLevel cpl = (CompetenceProfileLevel) entity;
			final CompetenceProfile cp = CompetenceProfileLocalServiceUtil.getCompetenceProfile(cpl
					.getCompetenceProfileId());
			return getCompetenceMapId(cp);
		}

		if (entity instanceof CompetenceLevel) {
			final CompetenceLevel cl = (CompetenceLevel) entity;
			final Competence c = CompetenceLocalServiceUtil.getCompetence(cl.getCompetenceId());
			return getCompetenceMapId(c);
		}

		throw new CompetenceMapException("Not able to get the competenceMapId from entity:" + entity);
	}
}
