/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.tencompetence.business.facade;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.business.constants.EventType;
import org.tencompetence.business.exceptions.GoalException;
import org.tencompetence.business.facade.validators.GoalValidator;
import org.tencompetence.business.services.PortfolioService;
import org.tencompetence.generated.model.CompetenceProfile;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.generated.model.PortfolioEntry;
import org.tencompetence.generated.service.GoalLocalServiceUtil;

import com.liferay.counter.service.CounterLocalServiceUtil;
//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.service.ServiceContext;

/**
 * The Class GoalFacade.
 * 
 * @author venrooijma
 * @version $Revision: 1.19 $, $Date: 2009/11/24 14:18:19 $
 */
public final class GoalFacade {

	private GoalFacade() {
		// Private constructor
	}

	/**
	 * Creates the goal.
	 * 
	 * @return the goal
	 * @throws SystemException
	 *             the system exception
	 */
	public static Goal createGoal() throws SystemException {
		final long id = CounterLocalServiceUtil.increment();
		final Goal result = GoalLocalServiceUtil.createGoal(id);
		result.setNew(true);

		return result;
	}

	/**
	 * Updates a Goal. If a goal is previously saved in the system this can either be done with or without an existing
	 * target and targetType. If the TargetType and Target are saved previously the values cannot be updated and the
	 * goal that is returned has the previously saved target and TargetType values. if targettype and target are not
	 * previously saved the new values will be saved if their valid. If the values are left blank in the inputparam they
	 * will not be saved.
	 * 
	 * @param goal
	 *            the goal to update
	 * @param ctx
	 *            the context
	 * @return the updated goal
	 * @throws SystemException
	 *             thrown if a Validation error occurs
	 * @throws PortalException
	 *             the portal exception
	 */
	public static Goal updateGoal(final Goal goal, final ServiceContext ctx) throws SystemException, PortalException {

		final Goal newGoal = goal;

		// validate fields
		final List<String> errors = new ArrayList<String>();
		if (!GoalValidator.validate(newGoal, errors)) {
			throw new GoalException("Invalid Goal : " + errors.toString());
		}

		final Goal result = GoalLocalServiceUtil.updateGoal(goal);

		PortfolioService.addJournalEntry(result, EventType.UPDATE, ctx);

		return result;
	}

	/**
	 * Adds a new goal to the system.
	 * 
	 * @param goal
	 *            the goal to add
	 * @param ctx
	 *            the ServiceContext
	 * @return the added goal
	 * @throws SystemException
	 *             The SystemException thrown if a non valid goal is passed
	 * @throws PortalException
	 *             portalException
	 */
	public static Goal addGoal(final Goal goal, final ServiceContext ctx) throws SystemException, PortalException {

		// if not save if set in new version
		// validate other fields
		final List<String> errors = new ArrayList<String>();
		if (!GoalValidator.validate(goal, errors)) {
			throw new GoalException("Invalid Goal : " + errors.toString());
		}

		final Goal result = GoalLocalServiceUtil.addGoal(goal);

		FacadeUtils.addPermissions(result, ctx);

		PortfolioService.addJournalEntry(result, EventType.CREATE, ctx);

		// If the user creates a goal for himself, add the subscribe event to
		// portfolio.
		if (goal.getUserId() == ctx.getUserId()) {
			PortfolioService.addJournalEntry(result, EventType.SUBSCRIBE, ctx);
		}

		return result;
	}

	/**
	 * Get all Goals which the user has created.
	 * 
	 * @param userId
	 *            The userId
	 * @param ctx
	 *            The ctx
	 * 
	 * @return List<Goal>
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static List<Goal> getGoalsForUser(final long userId, final ServiceContext ctx) throws SystemException,
			PortalException {
		final List<PortfolioEntry> entries = JournalEntryFacade.getEntriesByUserClassEventType(ctx.getUserId(),
				Goal.class.getName(), EventType.CREATE.name());

		return PortfolioService.getEntitiesFromJournalEntries(entries, Goal.class, ctx, false);
	}

	/**
	 * Get all Goals which the user is subscribed to.
	 * 
	 * @param userId
	 *            The userId
	 * @param ctx
	 *            The ctx
	 * 
	 * @return List<Goal>
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static List<Goal> getSubscribedGoalsForUser(final long userId, final ServiceContext ctx)
			throws SystemException, PortalException {
		final List<PortfolioEntry> entries = JournalEntryFacade.getEntriesByUserClassEventType(ctx.getUserId(),
				Goal.class.getName(), EventType.SUBSCRIBE.name());

		return PortfolioService.getEntitiesFromJournalEntries(entries, Goal.class, ctx, false);
	}

	/**
	 * Get all Goals which the user has created and is coupled to the groupId.
	 * 
	 * 
	 * @param userId
	 *            The userId
	 * @param ctx
	 *            The ctx
	 * 
	 * @param mapId
	 *            the liferay groupId
	 * @return List<Goal>
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static List<Goal> getGoalsForUserAndCurrentMap(final long userId, final long mapId, final ServiceContext ctx)
			throws SystemException, PortalException {
		final List<Goal> allGoals = getGoalsForUser(userId, ctx);
		final List<Goal> result = new ArrayList<Goal>();
		for (Goal goal : allGoals) {
			if (isTargetSet(goal)) {
				// goal is coupled to a specific community
				// if the target resolves to the map coupled to the community
				// given by the groupId add the goal
				// atm only competence profile levels can be the target for a
				// goal

				if (goal.getTargetType().equals(CompetenceProfileLevel.class.getName())) {
					final CompetenceProfileLevel level = CompetenceProfileLevelFacade.getCompetenceProfileLevel(goal
							.getTarget(), ctx);
					final CompetenceProfile profile = CompetenceProfileFacade.getCompetenceProfile(level
							.getCompetenceProfileId(), ctx);
					if (mapId == profile.getCompetenceMapId()) {
						result.add(goal);
					}
				} else {
					throw new RuntimeException("the targetType: " + goal.getTargetType() + " is not supported yet");
				}

			} else {
				// goal is not coupled to a specific community yet
				result.add(goal);
			}
		}

		return result;
	}

	private static boolean isTargetSet(final Goal goal) {
		return !"".equalsIgnoreCase(goal.getTargetType()) && goal.getTarget() != 0;
	}

	/**
	 * Gets the goal.
	 * 
	 * @param goalId
	 *            the goal id
	 * @param ctx
	 *            the ctx
	 * @return the goal
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static Goal getGoal(final long goalId, final ServiceContext ctx) throws SystemException, PortalException {
		return GoalLocalServiceUtil.getGoal(goalId);
	}

	/**
	 * Gets the goals by ids.
	 * 
	 * @param ids
	 *            the ids
	 * @return the goals by ids
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<Goal> getGoalsByIds(final List<Long> ids) throws SystemException {
		return GoalLocalServiceUtil.getGoalsByIds(ids);
	}

	/**
	 * Gets the goals by targetType and targetId.
	 * 
	 * @param targetType
	 *            the target type
	 * @param targetId
	 *            the target id
	 * 
	 * @return the goals by ids
	 * 
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<Goal> getGoalsByTargetTypeAndTargetId(final String targetType, final long targetId)
			throws SystemException {
		return GoalLocalServiceUtil.getGoalsByTargetTypeAndTargetId(targetType, targetId);
	}

	/**
	 * Gets a goal by profileLevelId.
	 * 
	 * @param profileLevelId
	 *            profileLevelId
	 * 
	 * @return the goal
	 * 
	 * @throws SystemException
	 *             the system exception
	 */
	public static Goal getGoalByCompetenceProfileLevelId(final long profileLevelId) throws SystemException {
		final List<Goal> goals = getGoalsByTargetTypeAndTargetId(CompetenceProfileLevel.class.getName(), profileLevelId);
		return !goals.isEmpty() ? goals.get(0) : null;
	}
}
