/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.facade;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import org.tencompetence.generated.model.PortfolioEntry;
import org.tencompetence.generated.service.PortfolioEntryLocalServiceUtil;

import com.liferay.counter.service.CounterLocalServiceUtil;
//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.service.ServiceContext;

/**
 * Implements the JournalEntry service facade.
 * 
 * @author Harrie Martens
 * @version $Id: JournalEntryFacade.java,v 1.18 2009/11/09 15:57:53 sheyenrath Exp $
 */
public final class JournalEntryFacade {

	// Hide the constructor to prevent class instantiation
	private JournalEntryFacade() {
	}

	/**
	 * Creates the journal entry.
	 * 
	 * @return the portfolio entry
	 * @throws SystemException
	 *             the system exception
	 */
	public static PortfolioEntry createJournalEntry() throws SystemException {
		final long id = CounterLocalServiceUtil.increment();
		final PortfolioEntry result = PortfolioEntryLocalServiceUtil.createPortfolioEntry(id);
		result.setNew(true);

		return result;
	}

	/**
	 * Adds the journal entry.
	 * 
	 * @param portfolioEntry
	 *            the portfolio entry
	 * @param ctx
	 *            the ctx
	 * @return the portfolio entry
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static PortfolioEntry addJournalEntry(final PortfolioEntry portfolioEntry, final ServiceContext ctx)
			throws SystemException, PortalException {
		return PortfolioEntryLocalServiceUtil.addPortfolioEntry(portfolioEntry);
	}

	/**
	 * Get the most recent PortfolioEntry for a specific User, Company, Group, entityClass, entityId, eventType and
	 * value.
	 * 
	 * @param entityClass
	 *            The entityClass
	 * @param entityId
	 *            The entityId
	 * @param eventType
	 *            The eventType
	 * @param value
	 *            the value
	 * @param userId
	 *            the user id
	 * @param companyId
	 *            the company id
	 * @param groupId
	 *            the group id
	 * 
	 * @return List from PortfolioEntry
	 * 
	 * @throws SystemException
	 *             SystemException
	 */
	public static PortfolioEntry getLatestEntryByUCGClassEntityIdEventTypeValue(final long userId,
			final long companyId, final long groupId, final String entityClass, final String entityId,
			final String eventType, final String value) throws SystemException {
		return PortfolioEntryLocalServiceUtil.getLatestEntryByUserClassEventType(userId, companyId, groupId,
				entityClass, entityId, eventType, value);
	}

	/**
	 * Get a list from the most recent PortfolioEntries for a specific User, Company, Group, entityClass and eventTypes.
	 * 
	 * @param entityClass
	 *            The entityClass
	 * @param events
	 *            The eventTypes
	 * @param userId
	 *            the user id
	 * @param companyId
	 *            the company id
	 * @param groupId
	 *            the group id
	 * 
	 * @return List from PortfolioEntry
	 * 
	 * @throws SystemException
	 *             SystemException
	 */
	public static List<PortfolioEntry> getLatestEntriesByUCGClassEventTypes(final long userId, final long companyId,
			final long groupId, final String entityClass, final List<String> events) throws SystemException {
		return PortfolioEntryLocalServiceUtil.getLatestEntriesByUserClassEventTypes(userId, companyId, groupId,
				entityClass, events);
	}

	/**
	 * Get a list from the most recent PortfolioEntries for a specific User, entityClass and eventType.
	 * 
	 * @param entityClass
	 *            The entityClass
	 * @param event
	 *            event
	 * @param ctx
	 *            the ctx
	 * @return List from PortfolioEntry
	 * @throws SystemException
	 *             SystemException
	 */
	public static List<PortfolioEntry> getLatestEntriesByUserClassEventType(final String entityClass,
			final String event, final ServiceContext ctx) throws SystemException {

		return PortfolioEntryLocalServiceUtil.getLatestEntriesByUserClassEventType(ctx.getUserId(), ctx.getCompanyId(),
				ctx.getScopeGroupId(), entityClass, event);
	}

	/**
	 * Get all entries for a specific User.
	 * 
	 * @param userId
	 *            the user id
	 * @param companyId
	 *            the company id
	 * @param groupId
	 *            the group id
	 * 
	 * @return List from PortfolioEntries
	 * 
	 * @throws SystemException
	 *             SystemException
	 */
	public static List<PortfolioEntry> getEntriesByUCG(final long userId, final long companyId, final long groupId)
			throws SystemException {
		return PortfolioEntryLocalServiceUtil.getEntriesByUser(userId, companyId, groupId);
	}

	/**
	 * Get all entries for a specific User, Company, group, Entity, eventTypes, startDate and endDate.
	 * 
	 * @param entityId
	 *            entityId
	 * @param eventTypes
	 *            EventTypes
	 * @param startDate
	 *            the start date
	 * @param endDate
	 *            the end date
	 * @param sortByDateDescending
	 *            sortByDateDescending
	 * @param userId
	 *            the user id
	 * @param companyId
	 *            the company id
	 * @param groupId
	 *            the group id
	 * 
	 * @return List from PortfolioEntries
	 * 
	 * @throws SystemException
	 *             SystemException
	 */
	public static List<PortfolioEntry> getEntriesByUCGEntityIdEventTypesDateRange(final long userId,
			final long companyId, final long groupId, final String entityId, final List<String> eventTypes,
			final Date startDate, final Date endDate, final boolean sortByDateDescending) throws SystemException {
		return PortfolioEntryLocalServiceUtil.getEntriesByUserEntityIdEventTypesDateRange(userId, companyId, groupId,
				entityId, eventTypes, startDate, endDate, sortByDateDescending);
	}

	/**
	 * Get all Portfolio Entries for a specific User, Class and eventType.
	 * 
	 * @param entityClass
	 *            entityClass
	 * @param eventType
	 *            EventType
	 * @param userId
	 *            the user id
	 * 
	 * @return List from PortfolioEntries
	 * 
	 * @throws SystemException
	 *             SystemException
	 */
	public static List<PortfolioEntry> getEntriesByUserClassEventType(final long userId, final String entityClass,
			final String eventType) throws SystemException {
		return getEntriesByUCGClassEventTypeValue(userId, 0L, 0L, entityClass, eventType, null);
	}

	/**
	 * Get all Portfolio Entries for a specific User, Class and eventType.
	 * 
	 * @param entityClass
	 *            entityClass
	 * @param eventType
	 *            EventType
	 * @param value
	 *            the value
	 * @param userId
	 *            the user id
	 * @param companyId
	 *            the company id
	 * @param groupId
	 *            the group id
	 * 
	 * @return List from PortfolioEntries
	 * 
	 * @throws SystemException
	 *             SystemException
	 */
	public static List<PortfolioEntry> getEntriesByUCGClassEventTypeValue(final long userId, final long companyId,
			final long groupId, final String entityClass, final String eventType, final String value)
			throws SystemException {
		return PortfolioEntryLocalServiceUtil.getEntriesByUserClassEventTypeValue(userId, companyId, groupId,
				entityClass, eventType, value);
	}

	/**
	 * Get all Portfolio Entries for a specific User, Classes, eventTypes, startDate and endDate.
	 * 
	 * @param entityClasses
	 *            entityClasses
	 * @param eventTypes
	 *            EventTypes
	 * @param startDate
	 *            the start date
	 * @param endDate
	 *            the end date
	 * @param userId
	 *            the user id
	 * @param companyId
	 *            the company id
	 * @param groupId
	 *            the group id
	 * 
	 * @return List from PortfolioEntries
	 * 
	 * @throws SystemException
	 *             SystemException
	 */
	public static List<PortfolioEntry> getEntriesByUCGClassesEventTypesDateRange(final long userId,
			final long companyId, final long groupId, final List<String> entityClasses, final List<String> eventTypes,
			final Date startDate, final Date endDate) throws SystemException {

		return PortfolioEntryLocalServiceUtil.getEntriesByUserClassesEventTypesDateRange(userId, companyId, groupId,
				entityClasses, eventTypes, startDate, endDate, true);
	}

	/**
	 * Get all Portfolio Entries for a specific User, eventTypes, startDate and endDate.
	 * 
	 * @param eventTypes
	 *            EventTypes
	 * @param startDate
	 *            the start date
	 * @param endDate
	 *            the end date
	 * @param ctx
	 *            the ctx
	 * 
	 * @return List from PortfolioEntries
	 * 
	 * @throws SystemException
	 *             SystemException
	 */
	public static List<PortfolioEntry> getEntriesByUserEventTypes(final List<String> eventTypes, final Date startDate,
			final Date endDate, final ServiceContext ctx) throws SystemException {
		return PortfolioEntryLocalServiceUtil.getEntriesByUserClassesEventTypesDateRange(ctx.getUserId(), ctx
				.getCompanyId(), ctx.getScopeGroupId(), null, eventTypes, startDate, endDate, true);
	}

	/**
	 * Get all Portfolio Entries for a specific User, Classes, startDate and endDate.
	 * 
	 * @param entityClasses
	 *            entityClasses
	 * @param startDate
	 *            the start date
	 * @param endDate
	 *            the end date
	 * @param userId
	 *            the user id
	 * @param companyId
	 *            the company id
	 * @param groupId
	 *            the group id
	 * 
	 * @return List from PortfolioEntries
	 * 
	 * @throws SystemException
	 *             SystemException
	 */
	public static List<PortfolioEntry> getEntriesByUCGClassesDateRange(final long userId, final long companyId,
			final long groupId, final List<String> entityClasses, final Date startDate, final Date endDate)
			throws SystemException {
		return PortfolioEntryLocalServiceUtil.getEntriesByUserClassesEventTypesDateRange(userId, companyId, groupId,
				entityClasses, null, startDate, endDate, true);
	}

	/**
	 * Create a list from Entity-Id's from a PortfolioEntry list.
	 * 
	 * @param list
	 *            The PortfolioEntrylist
	 * @return List<Long> from Entity-Id's
	 */
	public static List<Long> convertToIdList(final List<PortfolioEntry> list) {
		return convertToIdList(list, null);
	}

	/**
	 * Get all Portfolio Entities for a specific User, Company, Group, Classes, eventTypes, startDate and endDate.
	 * 
	 * @param entityClasses
	 *            entityClasses
	 * @param eventTypes
	 *            EventTypes
	 * @param startDate
	 *            the start date
	 * @param endDate
	 *            the end date
	 * @param userId
	 *            the user id
	 * @param companyId
	 *            the company id
	 * @param groupId
	 *            the group id
	 * 
	 * @return List from PortfolioEntries
	 * 
	 * @throws SystemException
	 *             SystemException
	 */
	public static List<PortfolioEntry> getEntitiesByUCGClassesEventTypes(final long userId, final long companyId,
			final long groupId, final List<String> entityClasses, final List<String> eventTypes, final Date startDate,
			final Date endDate) throws SystemException {
		return PortfolioEntryLocalServiceUtil.getEntitiesByUserClassesEventTypes(userId, companyId, groupId,
				entityClasses, eventTypes, startDate, endDate);
	}

	/**
	 * Create a list from Entity-Id's from a PortfolioEntry list.
	 * 
	 * @param list
	 *            The PortfolioEntrylist
	 * @param entityClass
	 *            The full class name from the entity which is stored in this portfolio entry.
	 * @return List<Long> from Entity-Id's
	 */
	@SuppressWarnings("unchecked")
	public static List<Long> convertToIdList(final List<PortfolioEntry> list, final String entityClass) {

		if (list == null) {
			return Collections.EMPTY_LIST;
		}

		final List<Long> idList = new ArrayList<Long>();
		for (final PortfolioEntry entry : list) {
			if (entityClass == null || (entityClass.equals(entry.getEntityClass()))) {
				idList.add(Long.parseLong(entry.getEntityId()));
			}
		}

		return idList;
	}
}
