/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.facade;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.tencompetence.business.constants.EventType;
import org.tencompetence.business.constants.LearningPathType;
import org.tencompetence.business.exceptions.LearningPathException;
import org.tencompetence.business.facade.validators.LearningPathValidator;
import org.tencompetence.business.services.PortfolioService;
import org.tencompetence.generated.model.LearningPath;
import org.tencompetence.generated.model.PortfolioEntry;
import org.tencompetence.generated.service.LearningPathLocalServiceUtil;

import com.liferay.counter.service.CounterLocalServiceUtil;
//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.model.User;
import com.liferay.portal.service.ServiceContext;
import com.liferay.portal.service.UserLocalServiceUtil;

/**
 * Implement Learning Path functionality.
 * 
 * @author Stef Heyenrath
 * @version $Id: LearningPathFacade.java,v 1.20 2009/11/24 13:42:08 antonfinders Exp $
 */
public final class LearningPathFacade {

	private LearningPathFacade() {
		// private constructor
	}

	/**
	 * Creates a LearningPath.
	 * 
	 * @return LearningPath
	 * 
	 * @throws SystemException
	 *             the system exception
	 */
	public static LearningPath createLearningPath() throws SystemException {
		final long id = CounterLocalServiceUtil.increment();
		final LearningPath entity = LearningPathLocalServiceUtil.createLearningPath(id);
		entity.setNew(true);
		return entity;
	}

	/**
	 * Creates a LearningPath. This method is needed to act as a constructor for flex.
	 * 
	 * @param id
	 *            the id
	 * @return LearningPath
	 * 
	 * @throws SystemException
	 *             the system exception
	 */
	public static LearningPath createLearningPath(final long id) throws SystemException {
		// final long id = CounterLocalServiceUtil.increment();
		final LearningPath entity = LearningPathLocalServiceUtil.createLearningPath(id);
		entity.setNew(true);
		return entity;
	}

	/**
	 * Gets a learning path.
	 * 
	 * @param learningPathId
	 *            the learning path id
	 * @param ctx
	 *            the ctx
	 * 
	 * @return LearningPath
	 * 
	 * @throws PortalException
	 *             the portal exception
	 * @throws SystemException
	 *             the system exception
	 */
	public static LearningPath getLearningPath(final long learningPathId, final ServiceContext ctx)
			throws PortalException, SystemException {

		final LearningPath entity = LearningPathLocalServiceUtil.getLearningPath(learningPathId);

		return entity;
	}

	/**
	 * Get a bare LearningPath with no title, description or xmlData.
	 * 
	 * @param learningPathId
	 *            learningPathId
	 * @return LearningPath
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static LearningPath getBareLearningPath(final long learningPathId) throws SystemException, PortalException {
		return LearningPathLocalServiceUtil.getBareLearningPath(learningPathId);
	}

	/**
	 * Get all bare LearningPaths (paths and plans) with no title, description or xmlData.
	 * 
	 * @return LearningPath
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static List<LearningPath> getAllBareLearningPaths() throws SystemException, PortalException {
		return LearningPathLocalServiceUtil.getAllBareLearningPaths();
	}

	/**
	 * Get all bare LearningPaths with no title, description or xmlData.
	 * 
	 * @return LearningPath
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static List<LearningPath> getBareLearningPaths() throws SystemException, PortalException {
		return LearningPathLocalServiceUtil.getBareLearningPathsByType(LearningPathType.PATH.name());
	}

	/**
	 * Get some bare LearningPaths with no title, description or xmlData.
	 * 
	 * @param ids
	 *            The LearningPath ids
	 * 
	 * @return LearningPath
	 * 
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static List<LearningPath> getBareLearningPathsByIds(final List<Long> ids) throws SystemException,
			PortalException {
		return LearningPathLocalServiceUtil.getBareLearningPathsByIds(ids);
	}

	/**
	 * Get a list from some LearningPaths.
	 * 
	 * @param ids
	 *            The LearningPath ids.
	 * @param ctx
	 *            The ServiceContext
	 * 
	 * @return List
	 * 
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static List<LearningPath> getLearningPathsByIds(final List<Long> ids, final ServiceContext ctx)
			throws SystemException, PortalException {
		return LearningPathLocalServiceUtil.getLearningPathsByIds(ids);
	}

	/**
	 * Get a list from all LearningPaths (paths + plans).
	 * 
	 * @return LearningPath
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static List<LearningPath> getAllLearningPaths() throws SystemException, PortalException {
		return LearningPathLocalServiceUtil.getAllLearningPaths();
	}

	/**
	 * Get a list from all LearningPaths.
	 * 
	 * @return LearningPath
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static List<LearningPath> getLearningPaths() throws SystemException, PortalException {
		return LearningPathLocalServiceUtil.getLearningPathsByType(LearningPathType.PATH.name());
	}

	/**
	 * Get a list from all LearningPaths for a specific group.
	 * 
	 * @param groupId
	 *            the groupId
	 * 
	 * @return LearningPath
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static List<LearningPath> getLearningPathsByGroupId(final long groupId) throws SystemException,
			PortalException {
		return LearningPathLocalServiceUtil.getLearningPathsByGroupAndType(groupId, LearningPathType.PATH.name());
	}

	/**
	 * Get a list from all LearningPlans.
	 * 
	 * @return LearningPath
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static List<LearningPath> getLearningPlans() throws SystemException, PortalException {
		return LearningPathLocalServiceUtil.getLearningPathsByType(LearningPathType.PLAN.name());
	}

	/**
	 * Get a list from all LearningPlans for a specific group.
	 * 
	 * @param groupId
	 *            the groupId
	 * 
	 * @return LearningPath
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static List<LearningPath> getLearningPlansByGroupId(final long groupId) throws SystemException,
			PortalException {
		return LearningPathLocalServiceUtil.getLearningPathsByGroupAndType(groupId, LearningPathType.PLAN.name());
	}

	/**
	 * Get a LearningPath specified by its Uri.
	 * 
	 * @param uri
	 *            The LearningPath Uri.
	 * @param ctx
	 *            The ServiceContext
	 * 
	 * @return LearningPath
	 * 
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static LearningPath getLearningPathByUri(final String uri, final ServiceContext ctx) throws SystemException,
			PortalException {
		return LearningPathLocalServiceUtil.getLearningPathByUriAndType(uri, LearningPathType.PATH.name());
	}

	/**
	 * Get a LearningPlan specified by its Uri.
	 * 
	 * @param uri
	 *            The LearningPlan Uri.
	 * @param ctx
	 *            The ServiceContext
	 * 
	 * @return LearningPath
	 * 
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static LearningPath getLearningPlanByUri(final String uri, final ServiceContext ctx) throws SystemException,
			PortalException {
		return LearningPathLocalServiceUtil.getLearningPathByUriAndType(uri, LearningPathType.PLAN.name());
	}

	/**
	 * Get a bare LearningPath specified by its Uri.
	 * 
	 * @param uri
	 *            The LearningPath Uri.
	 * @param ctx
	 *            The ServiceContext
	 * @param type
	 *            the type
	 * 
	 * @return LearningPath
	 * 
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	private static LearningPath getBareLearningPathByUriAndType(final String uri, final String type,
			final ServiceContext ctx) throws SystemException, PortalException {
		return LearningPathLocalServiceUtil.getBareLearningPathByUriAndType(uri, type);
	}

	/**
	 * Get a list from all owned LearningPaths.
	 * 
	 * @param ctx
	 *            The ServiceContext
	 * @param logInJournal
	 *            Log this actiion in the journal
	 * @param userId
	 *            The userId
	 * 
	 * @return List
	 * 
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static List<LearningPath> getLearningPathsForUser(final long userId, final ServiceContext ctx,
			final boolean logInJournal) throws SystemException, PortalException {
		final List<PortfolioEntry> entries = JournalEntryFacade.getEntriesByUserClassEventType(ctx.getUserId(),
				LearningPath.class.getName(), EventType.CREATE.name());

		return PortfolioService.getEntitiesFromJournalEntries(entries, LearningPath.class, ctx, logInJournal);
	}

	/**
	 * Get a list from all co-edited LearningPaths.
	 * 
	 * @param ctx
	 *            The ServiceContext
	 * @param userId
	 *            The userId
	 * 
	 * @return List
	 * 
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static List<LearningPath> getCoEditedLearningPathsForUser(final long userId, final ServiceContext ctx)
			throws SystemException, PortalException {

		final List<LearningPath> list = getBareLearningPaths();

		if (list.isEmpty()) {
			return Collections.emptyList();
		}

		final List<Long> lpIds = new ArrayList<Long>();
		for (final LearningPath learningPath : list) {
			if (FacadeUtils.decodeEntries(learningPath.getCoeditorIds()).contains(Long.valueOf(userId))) {
				lpIds.add(learningPath.getPrimaryKey());
			}
		}

		return getLearningPathsByIds(lpIds, ctx);
	}

	/**
	 * Add a LearningPath.
	 * 
	 * @param learningPath
	 *            The LearningPath
	 * @param ctx
	 *            The ServiceContext
	 * @return LearningPath
	 * 
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static LearningPath addLearningPath(final LearningPath learningPath, final ServiceContext ctx)
			throws SystemException, PortalException {

		final List<String> errors = new ArrayList<String>();

		if (!LearningPathValidator.validate(learningPath, errors, ctx)) {
			throw new LearningPathException("Invalid learningpath : " + errors.toString());
		}

		// check uri
		final String uri = learningPath.getUri();
		if (Validator.isNull(uri)) {
			throw new LearningPathException("Invalid learningpath : Uri required");
		} else {
			if (getBareLearningPathByUriAndType(learningPath.getUri(), learningPath.getType(), ctx) != null) {
				throw new LearningPathException(
						"Invalid learningpath : There is already an existing LearningPath with uri: " + uri);
			}
		}

		final LearningPath entity = LearningPathLocalServiceUtil.addLearningPath(learningPath);

		FacadeUtils.addPermissions(entity, ctx);

		PortfolioService.addJournalEntry(entity, EventType.CREATE, ctx);

		return entity;
	}

	/**
	 * Copy a LearningPath.
	 * 
	 * @param currentPath
	 *            The currentPath
	 * @param newUri
	 *            newUri
	 * @param ctx
	 *            The ServiceContext
	 * @return LearningPath
	 * 
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static LearningPath copyLearningPath(final LearningPath currentPath, final String newUri,
			final ServiceContext ctx) throws SystemException, PortalException {
		final LearningPath newLP = LearningPathFacade.createLearningPath();

		newLP.setCoeditorIds(currentPath.getCoeditorIds());
		newLP.setCompanyId(currentPath.getCompanyId());
		newLP.setGroupId(currentPath.getGroupId());
		newLP.setUri(newUri);
		newLP.setUserId(currentPath.getUserId());
		newLP.setXmlData(currentPath.getXmlData());

		return addLearningPath(newLP, ctx);
	}

	/**
	 * Delete a LearningPath.
	 * 
	 * @param learningPath
	 *            The LearningPath
	 * @param ctx
	 *            The ServiceContext
	 * 
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	public static void deleteLearningPath(final LearningPath learningPath, final ServiceContext ctx)
			throws SystemException, PortalException {

		FacadeUtils.removePermissions(learningPath, ctx);

		PortfolioService.addJournalEntry(learningPath, EventType.DELETE, ctx);

		LearningPathLocalServiceUtil.deleteLearningPath(learningPath);
	}

	/**
	 * Update learning path.
	 * 
	 * @param learningPath
	 *            the learning path
	 * @param ctx
	 *            the ctx
	 * 
	 * @return the learning path
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static LearningPath updateLearningPath(final LearningPath learningPath, final ServiceContext ctx)
			throws SystemException, PortalException {

		final List<String> errors = new ArrayList<String>();

		if (!LearningPathValidator.validate(learningPath, errors, ctx)) {
			throw new LearningPathException("Invalid learningpath : " + errors.toString());
		}

		final LearningPath entity = LearningPathLocalServiceUtil.updateLearningPath(learningPath);

		PortfolioService.addJournalEntry(entity, EventType.UPDATE, ctx);

		return entity;
	}

	/**
	 * Return a list from coauthors for this Learningpath.
	 * 
	 * @param learningPathId
	 *            the learningPath Id
	 * 
	 * @return List<User>
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static List<User> getCoEditors(final long learningPathId) throws SystemException, PortalException {
		final LearningPath learningPath = getBareLearningPath(learningPathId);
		final List<Long> ids = FacadeUtils.decodeEntries(learningPath.getCoeditorIds());
		final List<User> users = new ArrayList<User>();
		for (final Long userId : ids) {
			users.add(UserLocalServiceUtil.getUserById(userId));
		}

		return users;
	}
}