/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.facade;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.business.constants.EventType;
import org.tencompetence.business.exceptions.LevelValueException;
import org.tencompetence.business.facade.validators.LevelValueValidator;
import org.tencompetence.business.services.PortfolioService;
import org.tencompetence.generated.model.LevelValue;
import org.tencompetence.generated.service.LevelValueLocalServiceUtil;

import com.liferay.counter.service.CounterLocalServiceUtil;
//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.service.ServiceContext;

/**
 * Implements the LevelValue service facade.
 * 
 * @author Harrie Martens
 * @version $Id: LevelValueFacade.java,v 1.6 2009/10/28 12:59:27 sheyenrath Exp $
 */
public final class LevelValueFacade {

	// Hide the constructor to disallow class creation
	private LevelValueFacade() {
	}

	/**
	 * Creates the level value.
	 * 
	 * @return the level value
	 * @throws SystemException
	 *             the system exception
	 */
	public static LevelValue createLevelValue() throws SystemException {
		final long id = CounterLocalServiceUtil.increment();
		final LevelValue result = LevelValueLocalServiceUtil.createLevelValue(id);
		result.setNew(true);

		return result;
	}

	/**
	 * Update level value.
	 * 
	 * @param levelValue
	 *            the level value
	 * @param merge
	 *            the merge
	 * @param ctx
	 *            the ctx
	 * @return the level value
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static LevelValue updateLevelValue(final LevelValue levelValue, final boolean merge, final ServiceContext ctx)
			throws SystemException, PortalException {
		final List<String> errors = new ArrayList<String>();

		if (!LevelValueValidator.validate(levelValue, errors)) {
			throw new LevelValueException("Invalid LevelValue : " + errors.toString());
		}

		final LevelValue lv = LevelValueLocalServiceUtil.updateLevelValue(levelValue, merge);

		PortfolioService.addJournalEntry(lv, EventType.CREATE, ctx);

		return lv;
	}

	/**
	 * Update level value.
	 * 
	 * @param levelValue
	 *            the level value
	 * @param ctx
	 *            the ctx
	 * @return the level value
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static LevelValue updateLevelValue(final LevelValue levelValue, final ServiceContext ctx)
			throws SystemException, PortalException {
		return updateLevelValue(levelValue, false, ctx);
	}

	/**
	 * Gets the competence profile level values.
	 * 
	 * @param competenceMapId
	 *            the competence map id
	 * @param ctx
	 *            the ctx
	 * @return the competence profile level values
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static List<LevelValue> getCompetenceProfileLevelValues(final long competenceMapId, final ServiceContext ctx)
			throws SystemException, PortalException {

		return LevelValueLocalServiceUtil.getCompetenceProfileLevelValues(competenceMapId);
	}

	/**
	 * Gets the required levels. Level 0 is excluded.
	 * 
	 * @param competenceMapId
	 *            the competence map id
	 * @param ctx
	 *            the ctx
	 * @return the competence level values
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static List<LevelValue> getRequirableCompetenceLevelValues(final long competenceMapId,
			final ServiceContext ctx) throws SystemException, PortalException {
		return LevelValueLocalServiceUtil.getRequirableCompetenceLevelValues(competenceMapId);
	}

	/**
	 * Gets the attainable levels. Level 0 is included.
	 * 
	 * @param competenceMapId
	 *            the competence map id
	 * @param ctx
	 *            the ctx
	 * @return the competence level values
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static List<LevelValue> getAttainableCompetenceLevelValues(final long competenceMapId,
			final ServiceContext ctx) throws SystemException, PortalException {
		return LevelValueLocalServiceUtil.getAttainableCompetenceLevelValues(competenceMapId);
	}

	/**
	 * Gets the competence profile level values count.
	 * 
	 * @param competenceMapId
	 *            the competence map id
	 * @param ctx
	 *            the ctx
	 * @return the competence profile level values count
	 * @throws SystemException
	 *             the system exception
	 */
	public static long getCompetenceProfileLevelValuesCount(final long competenceMapId, final ServiceContext ctx)
			throws SystemException {
		return LevelValueLocalServiceUtil.getCompetenceProfileLevelValuesCount(competenceMapId);
	}

	/**
	 * Gets the number of CompetenceLevelValues minus 1. If there are 8 levels created there is an extra level 0
	 * created. This level shoudn't count.
	 * 
	 * @param competenceMapId
	 *            the mapid
	 * @param ctx
	 *            the ctx
	 * @return number of CompetenceLevelValues
	 * @throws SystemException
	 *             the SystemException
	 */
	public static long getRequiredCompetenceLevelValuesCount(final long competenceMapId, final ServiceContext ctx)
			throws SystemException {
		return LevelValueLocalServiceUtil.getRequiredCompetenceLevelValuesCount(competenceMapId);
	}

	/**
	 * Gets the number of CompetenceLevelValues. If there are 8 levels created there is an extra level 0 created. This
	 * level is included in this calculation.
	 * 
	 * @param ctx
	 *            the ctx
	 * @param competenceMapId
	 *            the mapid
	 * @return number of CompetenceLevelValues
	 * @throws SystemException
	 *             the SystemException
	 */
	public static long getAttainedCompetenceLevelValuesCount(final long competenceMapId, final ServiceContext ctx)
			throws SystemException {
		return LevelValueLocalServiceUtil.getAttainedCompetenceLevelValuesCount(competenceMapId);
	}

	/**
	 * Gets the level value.
	 * 
	 * @param levelValueId
	 *            the level value id
	 * @param ctx
	 *            the ctx
	 * @return the level value
	 * @throws PortalException
	 *             the portal exception
	 * @throws SystemException
	 *             the system exception
	 */
	public static LevelValue getLevelValue(final long levelValueId, final ServiceContext ctx) throws PortalException,
			SystemException {
		return LevelValueLocalServiceUtil.getLevelValue(levelValueId);
	}

	/**
	 * Gets the level values by competence map id and type.
	 * 
	 * @param primaryKey
	 *            the primary key
	 * @param id
	 *            the id
	 * @param ctx
	 *            the ctx
	 * @return the level values by competence map id and type
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<LevelValue> getLevelValuesByCompetenceMapIdAndType(final long primaryKey, final Integer id,
			final ServiceContext ctx) throws SystemException {
		return LevelValueLocalServiceUtil.geLevelValuesByCompetenceMapIdAndType(primaryKey, id);
	}

	/**
	 * Adds the level value.
	 * 
	 * @param value
	 *            the value
	 * @param ctx
	 *            the ctx
	 * @return the level value
	 * @throws SystemException
	 *             the system exception
	 */
	protected static LevelValue addLevelValue(final LevelValue value, final ServiceContext ctx) throws SystemException {
		return LevelValueLocalServiceUtil.addLevelValue(value);
	}
}
