/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.facade.validators;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.List;

import org.tencompetence.business.facade.CompetenceMapFacade;
import org.tencompetence.business.facade.Constants;
import org.tencompetence.business.utils.EntityUtils;
import org.tencompetence.generated.NoSuchCompetenceMapException;

import com.liferay.portal.NoSuchCompanyException;
import com.liferay.portal.NoSuchGroupException;
import com.liferay.portal.NoSuchUserException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.model.BaseModel;
import com.liferay.portal.service.CompanyLocalServiceUtil;
import com.liferay.portal.service.GroupLocalServiceUtil;
import com.liferay.portal.service.ServiceContext;
import com.liferay.portal.service.UserLocalServiceUtil;

/**
 * The Class GenericValidator is the common root for all validators.
 * 
 * @author sheyenrath
 * @author venrooijma
 * @author Harrie Martens
 * @version $Id: GenericValidator.java,v 1.6 2009/10/07 10:02:24 sheyenrath Exp $
 */
public class GenericValidator {

	private static final Log LOG = LogFactoryUtil.getLog(GenericValidator.class);

	/**
	 * Default constructor.
	 */
	protected GenericValidator() {
	}

	/**
	 * Check if a specific method does exists on the entity and check the return value from this method.
	 * 
	 * @param object
	 *            The class
	 * @param methodName
	 *            The method
	 * @return result
	 */
	protected static boolean checkMethod(final BaseModel<?> object, final String methodName) {
		try {
			final Method method = object.getClass().getMethod(methodName, new Class[] {});
			if (method != null) {
				return Validator.isNotNull(method.invoke(object, new Object[] {}));
			}
		} catch (Exception e) {
			return false;
		}

		return false;
	}

	/**
	 * Check if the Title and Description are valid.
	 * 
	 * @param entity
	 *            Entity
	 * @param errors
	 *            A String list from the errors
	 * @return result
	 */
	protected static boolean validateTitleAndDescription(final BaseModel<?> entity, final List<String> errors) {
		boolean valid = true;

		valid = validateTitle(entity, errors);

		if (!checkMethod(entity, "getDescription")) {
			errors.add("Description required");
			valid = false;
		}

		return valid;
	}

	/**
	 * Validate title.
	 * 
	 * @param entity
	 *            the entity
	 * @param errors
	 *            the errors
	 * @return true, if successful
	 */
	protected static boolean validateTitle(final BaseModel<?> entity, final List<String> errors) {
		boolean valid = true;

		if (!checkMethod(entity, "getTitle")) {
			errors.add("Title required");
			valid = false;
		}

		Method method;
		try {
			method = entity.getClass().getMethod("getTitle", new Class[] {});
			if (method != null) {
				String result = (String) method.invoke(entity, new Object[] {});
				if (result.length() > Constants.MAX_TITLE_LENGTH) {
					errors.add("Title longer then " + Constants.MAX_TITLE_LENGTH + " characters");
					valid = false;
				}
			}
		} catch (SecurityException e) {
			valid = false;
			doDebugLog(e);
		} catch (NoSuchMethodException e) {
			valid = false;
			doDebugLog(e);
		} catch (IllegalArgumentException e) {
			valid = false;
			doDebugLog(e);
		} catch (IllegalAccessException e) {
			valid = false;
			doDebugLog(e);
		} catch (InvocationTargetException e) {
			valid = false;
			doDebugLog(e);
		}

		return valid;
	}

	private static void doDebugLog(final Exception e) {
		if (LOG.isDebugEnabled()) {
			LOG.debug(e);
		}
	}

	/**
	 * Validate common aspects of BaseModel entities.
	 * <p>
	 * The following checks are performed:
	 * <ul>
	 * <li>check if entity has a primary key</li>
	 * <li>check if entity has a title, this check is only performed if the entity has a getTitle method</li>
	 * <li>check if entity has a description, this check is only performed if the entity has a getDescription method</li>
	 * <li>Check if entity has a valid user id</li>
	 * <li>Check if entity has a valid company id</li>
	 * <li>Check if entity has a valid group id</li>
	 * </ul>
	 * 
	 * @param entity
	 *            Entity
	 * @param errors
	 *            A String list from the errors
	 * @return result
	 */
	protected static boolean validateGeneralPropertiesEntity(final BaseModel<?> entity, final List<String> errors) {
		boolean valid = true;

		if (!checkMethod(entity, "getPrimaryKey")) {
			errors.add("PrimaryKey required");
			valid = false;
		}

		valid &= validateTitleAndDescription(entity, errors);
		valid &= validateUCG(entity, errors);

		return valid;
	}

	/**
	 * Validate User, Company and Group.
	 * 
	 * @param entity
	 *            Entity
	 * @param errors
	 *            A String list from the errors
	 * @return true if validation was successful
	 */
	protected static boolean validateUCG(final BaseModel<?> entity, final List<String> errors) {
		boolean valid = true;

		// check user id
		try {
			long userId = EntityUtils.getUserIdFromEntity(entity);
			if (userId == 0) {
				errors.add("UserId required");
				valid = false;
			} else {
				try {
					UserLocalServiceUtil.getUserById(userId);
				} catch (NoSuchUserException e) {
					errors.add("User is invalid");
					valid = false;
				}
			}
		} catch (Exception e) {
			errors.add("Exception occurred when checking user id: " + e.getMessage());
			valid = false;
		}

		// check company id
		try {
			long companyId = EntityUtils.getCompanyIdFromEntity(entity);
			if (companyId == 0) {
				errors.add("CompanyId required");
				valid = false;
			} else {
				try {
					CompanyLocalServiceUtil.getCompanyById(companyId);
				} catch (NoSuchCompanyException e) {
					errors.add("Company is invalid");
					valid = false;
				}
			}
		} catch (Exception e) {
			errors.add("Exception occurred when checking company id: " + e.getMessage());
			valid = false;
		}

		// check group id
		try {
			final long groupId = EntityUtils.getGroupIdFromEntity(entity);

			if (groupId == 0) {
				errors.add("GroupId required");
				valid = false;
			} else {
				try {
					GroupLocalServiceUtil.getGroup(groupId);
				} catch (NoSuchGroupException e) {
					errors.add("Group is invalid");
					valid = false;
				}
			}
		} catch (Exception e) {
			errors.add("Exception occurred when checking group id: " + e.getMessage());
			valid = false;
		}

		return valid;
	}

	/**
	 * Validates a comma separated entity String.
	 * 
	 * @param errors
	 *            A String list containing the detected errors
	 * @param entities
	 *            a comma separated list of entity id's
	 * @return true if the list of id's is valid, false otherwise SystemException
	 */
	protected static boolean validateEntities(final String entities, final List<String> errors) {
		boolean valid = true;

		if (entities != null && !"".equals(entities.trim())) {
			String[] ids = entities.split(",");
			for (String id : ids) {
				if ("".equals(id)) {
					errors.add("Invalid id passed");
					valid = false;
				} else {
					try {
						Long number = Long.parseLong(id);
						if (number <= 0) {
							errors.add(String.format("id '%s' is not a valid number", id));
							valid = false;
						}
					} catch (NumberFormatException e) {
						errors.add(String.format("id '%s' is not a number", id));
						valid = false;
					}
				}
			}
		}

		return valid;
	}

	/**
	 * Validate competence map.
	 * 
	 * @param errors
	 *            the errors
	 * @param ctx
	 *            the ctx
	 * @param mapId
	 *            the map id
	 * @return true, if successful
	 */
	protected static boolean validateCompetenceMap(final long mapId, final List<String> errors,
			final ServiceContext ctx) {
		boolean valid = true;
		try {
			CompetenceMapFacade.getCompetenceMap(mapId, ctx);
		} catch (NoSuchCompetenceMapException e) {
			errors.add(String.format("CompetenceMap %d does not exist", mapId));
			valid = false;
		} catch (Exception e) {
			errors.add("Exception occurred when checking CompetenceMap: " + e.getMessage());
			valid = false;
		}
		return valid;
	}

}
