/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.model.learningpath.impl;

import java.util.ArrayList;
import java.util.List;

import org.tencompetence.business.constants.ClusterType;
import org.tencompetence.business.model.learningpath.ICluster;
import org.tencompetence.business.model.learningpath.ICursor;
import org.tencompetence.business.model.learningpath.ILPElement;
import org.tencompetence.business.model.learningpath.ILearningPath;
import org.tencompetence.services.jaxb.model.lp.ExternalLearningPath;
import org.tencompetence.services.jaxb.model.lp.LearningAction;
import org.tencompetence.services.jaxb.model.lp.LearningActionsCluster;
import org.tencompetence.services.jaxb.model.lp.LearningActionsCluster.ExternalLearningPathRef;
import org.tencompetence.services.jaxb.model.lp.LearningActionsCluster.LearningActionRef;
import org.tencompetence.services.jaxb.model.lp.LearningActionsCluster.LearningActionsClusterRef;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;

import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.service.ServiceContext;

/**
 * The class ClusterImpl.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @author Stef Heyenrath
 * 
 * @version $Revision: 1.5 $, $Date: 2009/11/30 12:05:50 $
 */
public abstract class ClusterImpl extends LPElementImpl implements ICluster {
	protected final LearningActionsCluster cluster;

	/**
	 * Instantiates a new cluster impl.
	 * 
	 * @param learningPath
	 *            the learning path
	 * @param cluster
	 *            the cluster
	 * @param ctx
	 *            the ctx
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public ClusterImpl(final LearningActionsCluster cluster, final ILearningPath learningPath, final ServiceContext ctx)
			throws SystemException, PortalException {
		super(learningPath, ctx);
		if (cluster == null) {
			throw new IllegalArgumentException("Cluster should not be null");
		}
		this.cluster = cluster;

		init();
	}
	
	/**
	 * Gets the LearningActionsCluster.
	 * 
	 * @return the cluster
	 */
	protected LearningActionsCluster getCluster() {
		return cluster;
	}

	/** {@inheritDoc} */
	@Override
	public List<ILPElement> getChildren() {
		final List<ILPElement> result = new ArrayList<ILPElement>();
		for (final Object ref : cluster.getRefs()) {
			final ILPElement element = getElementFromRef(ref);
			if (element != null) {
				result.add(element);
			}
		}
		return result;
	}

	/** {@inheritDoc} */
	@Override
	public String getDescription() {
		return cluster.getMetadata().getDescription();
	}

	/** {@inheritDoc} */
	@Override
	public String getTitle() {
		return cluster.getMetadata().getTitle();
	}

	/** {@inheritDoc} */
	@Override
	public String getId() {
		return cluster.getId();
	}

	/** {@inheritDoc} */
	@Override
	public String getUri() {
		return cluster.getId();
	}

	/** {@inheritDoc} */
	@Override
	public boolean isCompleted() {
		int numberCompleted = 0;

		for (final ILPElement element : getChildren()) {
			if (element.isCompleted()) {
				++numberCompleted;
			}
		}
		return numberCompleted >= getNumberToSelect();
	}

	/** {@inheritDoc} */
	public boolean canPerform(final ILPElement element) {
		switch (getType()) {
		case SEQUENCE:
			boolean selectable = false;
			boolean processed = false;
			for (final ILPElement child : getChildren()) {
				selectable = !child.isCompleted() && !selectable && !processed;
				if (child.equals(element)) {
					return selectable;
				}
				if (selectable) {
					processed = true;
				}
			}
			break;

		case FREE_ORDER:
			for (final ILPElement child : getChildren()) {
				if (child.equals(element)) {
					return !child.isCompleted();
				}
			}
			break;
		case PARALLEL:
			break;
		default:
			break;
		}
		return false;
	}

	@Override
	public ClusterType getType() {
		return ClusterType.valueOfType(cluster.getType());
	}

	/** {@inheritDoc} */
	public ICursor getNextLearningAction(final ICursor cursor) {
		cursor.push(this);
		for (final ILPElement lpe : getChildren()) {
			if (!lpe.isCompleted()) {
				final ICursor result = ((LPElementImpl) lpe).getNextLearningAction(cursor);
				if (result != null) {
					return result;
				}
			}
		}
		return null;
	}

	/** {@inheritDoc} */
	@Override
	protected Object getJAXBElement() {
		return cluster;
	}
	
	/** {@inheritDoc} */
	@Override
	public boolean isAdvisedOrder() {
		return cluster.isAdvisedOrder();
	}

	private ILPElement getElementFromRef(final Object ref) {
		Object obj;
		if (ref instanceof ExternalLearningPathRef) {
			obj = ((ExternalLearningPathRef) ref).getRef();
			if (obj instanceof ExternalLearningPath) {
				return getLearningPath().getLPElementById(((ExternalLearningPath) obj).getId());
			}
		} else if (ref instanceof LearningActionsClusterRef) {
			obj = ((LearningActionsClusterRef) ref).getRef();
			if (obj instanceof LearningActionsCluster) {
				return getLearningPath().getLPElementById(((LearningActionsCluster) obj).getId());
			}
		} else if (ref instanceof LearningActionRef) {
			obj = ((LearningActionRef) ref).getRef();
			if (obj instanceof LearningAction) {
				return getLearningPath().getLPElementById(((LearningAction) obj).getId());
			}
		}
		return null;
	}
}
