/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.model.learningpath.impl;

import java.util.Arrays;
import java.util.List;
import java.util.Stack;

import org.tencompetence.business.model.learningpath.ICursor;
import org.tencompetence.business.model.learningpath.ILPElement;
import org.tencompetence.business.model.learningpath.ILearningPath;
import org.tencompetence.business.model.learningpath.ILearningPlan;

import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;

/**
 * The Cursor is a book mark for a Learning plan. It represents a location in the hierarchy of the learning plan.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @version $Revision: 1.4 $, $Date: 2009/11/17 22:07:54 $
 */
public class CursorImpl implements ICursor {
	private static final String ELEMENT_SEPARATOR = ">";
	private final Stack<ILPElement> cursorStack = new Stack<ILPElement>();
	private final ILearningPlan plan;

	/**
	 * Instantiates a new cursor.
	 * 
	 * @param cursorData
	 *            the cursor data
	 * @param plan
	 *            the plan
	 * 
	 * @throws PortalException
	 *             the portal exception
	 * @throws SystemException
	 *             the system exception
	 */
	public CursorImpl(final ILearningPlan plan, final String cursorData)
			throws PortalException, SystemException {
		this.plan = plan;
		valueOf(cursorData);
	}

	/**
	 * Instantiates a new cursor.
	 * 
	 * @param plan
	 *            the plan
	 */
	public CursorImpl(final ILearningPlan plan) {
		if (plan == null) {
			throw new IllegalArgumentException("Null value not allowed");
		}
		this.plan = plan;
		cursorStack.push(plan);
	}

	/** {@inheritDoc} */
	public void push(final ILPElement element) {
		cursorStack.push(element);
	}

	/** {@inheritDoc} */
	public ILPElement pop() {
		if (cursorStack.size() > 1) {
			return cursorStack.pop();
		}

		return cursorStack.get(0);
	}

	/** {@inheritDoc} */
	public ILPElement getCurrentElement() {
		return cursorStack.peek();
	}

	/** {@inheritDoc} */
	@Override
	public List<ILPElement> getElements() {
		return cursorStack;
	}

	/** {@inheritDoc} */
	public ILearningPlan getLearningPlan() {
		return (ILearningPlan) cursorStack.get(0);
	}

	/** {@inheritDoc} */
	@Override
	public String toString() {
		final StringBuffer result = new StringBuffer();
		boolean first = true;
		for (final ILPElement element : cursorStack) {
			if (first) {
				first = false;
				result.append(((ILearningPlan) element).getUri());
			} else {
				result.append(ELEMENT_SEPARATOR);
				result.append(element.getUri());
			}
		}

		return result.toString();
	}

	private void valueOf(final String cursorData) throws PortalException, SystemException {
		ILearningPath path = null;
		ILPElement element = null;
		for (final String uri : Arrays.asList(cursorData.split(ELEMENT_SEPARATOR))) {
			if (path == null) {
				// first id represents the uri of the learning plan
				// element = LearningPlanService.getLearningPlan(uri, ctx);
				element = plan;
			} else {
				element = path.getLPElementByUri(uri);
			}

			if (element instanceof ILearningPath) {
				path = (ILearningPath) element;
			} else if (element == null) {
				// invalid element reference encountered, so stop here
				return;
			}
			cursorStack.push(element);
		}
	}

	/**
	 * Subscribes to all elements of the cursor that require a subscription but are not yet subscribed.
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public void subScribe() throws SystemException, PortalException {
		for (final ILPElement element : cursorStack) {
			if (element.canSubscribe() && !element.isSubscribed()) {
				element.setSubscribed(true);
			}
		}
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (toString().hashCode());
		return result;
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		CursorImpl other = (CursorImpl) obj;
		if (!toString().equals(other.toString())) {
			return false;
		}
		return true;
	}
}
