/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.model.learningpath.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;

import org.tencompetence.business.constants.ClusterType;
import org.tencompetence.business.exceptions.LearningPathException;
import org.tencompetence.business.facade.CompetenceFacade;
import org.tencompetence.business.facade.CompetenceLevelFacade;
import org.tencompetence.business.facade.CompetenceProfileLevelFacade;
import org.tencompetence.business.facade.LevelValueFacade;
import org.tencompetence.business.model.learningpath.ICluster;
import org.tencompetence.business.model.learningpath.ICompetenceLevel;
import org.tencompetence.business.model.learningpath.ICursor;
import org.tencompetence.business.model.learningpath.ILPElement;
import org.tencompetence.business.model.learningpath.ILearningAction;
import org.tencompetence.business.model.learningpath.ILearningPath;
import org.tencompetence.business.model.learningpath.MetaDataEntry;
import org.tencompetence.business.services.LearningPathService;
import org.tencompetence.business.services.PdpService;
import org.tencompetence.business.utils.EntityUtils;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.generated.model.LevelValue;
import org.tencompetence.services.exceptions.TENCServiceException;
import org.tencompetence.services.jaxb.LPMarshaller;
import org.tencompetence.services.jaxb.model.lp.CompetenceLevelXML;
import org.tencompetence.services.jaxb.model.lp.ExternalLearningPath;
import org.tencompetence.services.jaxb.model.lp.Finish;
import org.tencompetence.services.jaxb.model.lp.LearningAction;
import org.tencompetence.services.jaxb.model.lp.LearningActionsCluster;
import org.tencompetence.services.jaxb.model.lp.LearningPathXML;
import org.tencompetence.services.jaxb.model.lp.ObjectFactory;
import org.tencompetence.services.jaxb.model.lp.LearningActionsCluster.ExternalLearningPathRef;
import org.tencompetence.services.jaxb.model.lp.LearningPathXML.LearningPathDesign.DesignLearningActions.LearningActionRef;
import org.tencompetence.services.jaxb.model.lp.LearningPathXML.LearningPathDesign.DesignLearningActions.LearningActionsClusterRef;
import org.tencompetence.services.jaxb.model.lp.LearningPathXML.Metadata.Costs;
import org.tencompetence.services.jaxb.model.lp.LearningPathXML.Metadata.LPRunInformation;
import org.tencompetence.services.jaxb.model.lp.LearningPathXML.Metadata.LPRunInformation.Location;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * The class LearningPathImpl.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @author Stef Heyenrath
 * 
 * @version $Revision: 1.16 $, $Date: 2009/11/30 14:00:14 $
 */
public class LearningPathImpl extends LPElementImpl implements ILearningPath {
	private static final Log LOG = LogFactoryUtil.getLog(LearningPathImpl.class);
	private LearningPathXML xmlPath;
	private List<ICompetenceLevel> internalLevels = new ArrayList<ICompetenceLevel>();
	private List<CompetenceLevel> levels = new ArrayList<CompetenceLevel>();
	private CompetenceProfileLevel finishCompetenceProfileLevel;
	private ICluster rootCluster;

	private HashMap<String, ILPElement> elementsById = new HashMap<String, ILPElement>();
	private HashMap<String, ILPElement> elementsByUri = new HashMap<String, ILPElement>();

	/**
	 * The Constructor.
	 * 
	 * @param xml
	 *            the xml
	 * @param ctx
	 *            the ctx
	 * @throws PortalException
	 *             the portal exception
	 * @throws SystemException
	 *             the system exception
	 */
	public LearningPathImpl(final String xml, final ServiceContext ctx) throws PortalException, SystemException {
		super(null, ctx);
		setXmlData(xml);
		init();
	}

	@Override
	public List<MetaDataEntry> getMetaData() {
		final List<MetaDataEntry> metadata = super.getMetaData();

		if (xmlPath.getMetadata() != null) {
			final Costs costs = xmlPath.getMetadata().getCosts();
			if (costs != null) {
				addMetadataEntry(metadata, "costs.currency", costs.getCurrency());
				addMetadataEntry(metadata, "costs.value", costs.getValue());
			}

			final List<LPRunInformation> runs = xmlPath.getMetadata().getRunInformation();
			if (runs != null) {
				for (final LPRunInformation ri : runs) {
					final List<Location> locations = ri.getLocation();
					if (locations != null) {
						for (final Location location : locations) {
							addMetadataEntry(metadata, "locations", location);
						}
					}

					addMetadataEntry(metadata, "startDate", ri.getStartDate());
					addMetadataEntry(metadata, "endDate", ri.getEndDate());
				}
			}
		}
		return metadata;
	}

	/** {@inheritDoc} */
	@Override
	public void setXmlData(final String xmlData) {
		try {
			xmlPath = LPMarshaller.getInstance().unMarshal(xmlData);
		} catch (TENCServiceException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		try {
			createInternalModel();
		} catch (PortalException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SystemException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/** {@inheritDoc} */
	@Override
	public ILPElement getLPElementByUri(final String uri) {
		return elementsByUri.get(uri);
	}

	/** {@inheritDoc} */
	@Override
	public ILPElement getLPElementById(final String id) {
		return elementsById.get(id);
	}

	/** {@inheritDoc} */
	@Override
	public String getDescription() {
		return xmlPath.getMetadata().getDescription();
	}

	/** {@inheritDoc} */
	@Override
	public String getTitle() {
		return xmlPath.getMetadata().getTitle();
	}

	/** {@inheritDoc} */
	@Override
	public List<? extends ILPElement> getChildren() {
		return rootCluster != null ? rootCluster.getChildren() : new ArrayList<ILPElement>();
	}

	/** {@inheritDoc} */
	@Override
	public ExternalLearningPath createJAXBRepresentation() {
		final ObjectFactory of = new ObjectFactory();

		final ExternalLearningPath lp = of.createExternalLearningPath();
		lp.setId(getId());
		lp.setUri(getUri());

		return lp;
	}

	/** {@inheritDoc} */
	@Override
	public String getId() {
		return xmlPath.getId();
	}

	/** {@inheritDoc} */
	@Override
	public String getUri() {
		return xmlPath.getMetadata().getUri();
	}

	/** {@inheritDoc} */
	@Override
	public List<CompetenceLevel> getCompetenceLevels() {
		return Collections.unmodifiableList(levels);
	}

	/** {@inheritDoc} */
	@Override
	public boolean providesLevel(final CompetenceLevel competenceLevel) {

		if (levels.size() == 0 || competenceLevel == null) {
			return false;
		}

		// check if the levels contains the given competenceLevel
		if (levels.contains(competenceLevel)) {
			return true;
		}

		ICompetenceLevel checkLevel = null;
		try {
			checkLevel = new CompetenceLevelImpl(competenceLevel, getContext());
		} catch (SystemException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (PortalException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		for (final ICompetenceLevel level : internalLevels) {
			if (checkLevel.getCompetence().equals(level.getCompetence())
					&& level.getLevelValue() >= checkLevel.getLevelValue()) {
				return true;
			}
		}

		return false;
	}

	/** {@inheritDoc} */
	@Override
	public int hashCode() {
		final int prime = 2011;
		int result = 1;
		result = prime * result + ((xmlPath == null) ? 0 : xmlPath.hashCode());
		return result;
	}

	/** {@inheritDoc} */
	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof LearningPathImpl)) {
			return false;
		}
		final LearningPathImpl other = (LearningPathImpl) obj;
		if (xmlPath == null) {
			if (other.xmlPath != null) {
				return false;
			}
		} else if (!xmlPath.getMetadata().getUri().equals(other.xmlPath.getMetadata().getUri())) {
			return false;
		}
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public CompetenceProfileLevel getFinishCompetenceProfileLevel() {
		return finishCompetenceProfileLevel;
	}

	/** {@inheritDoc} */
	@Override
	public ClusterType getType() {
		return rootCluster != null ? rootCluster.getType() : ClusterType.NONE;
	}

	/** {@inheritDoc} */
	@Override
	public int getNumberToSelect() {
		return rootCluster != null ? rootCluster.getNumberToSelect() : -1;
	}

	/** {@inheritDoc} */
	@Override
	public boolean canPerform(final ILPElement element) {
		return rootCluster != null ? rootCluster.canPerform(element) : false;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isCompleted() {
		// calculate completed state and compare this with the value from the portfolio
		// update the portfolio with the new value when necessary
		boolean completed = rootCluster != null ? rootCluster.isCompleted() : false;
		if (super.isCompleted() != completed) {
			try {
				setCompleted(completed);
			} catch (Exception e) {
				LOG.error(e);
			}
		}
		return completed;
	}

	/** {@inheritDoc} */
	@Override
	public void setCompleted(final boolean value) {
		super.setCompleted(value);
		if (value) {
			final ServiceContext ctx = getContext();

			// set the minimum attained level for all targeted competences by this learning path
			for (final CompetenceLevel level : getCompetenceLevels()) {
				Competence competence = null;
				try {
					competence = CompetenceFacade.getCompetence(level.getCompetenceId(), ctx);
				} catch (SystemException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (PortalException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				LevelValue levelValue = null;
				try {
					levelValue = LevelValueFacade.getLevelValue(level.getLevelValueId(), ctx);
				} catch (PortalException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (SystemException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				try {
					PdpService.setMinimumAttainedLevelForUser(ctx.getUserId(), competence, levelValue, ctx);
				} catch (SystemException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (PortalException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public List<ILearningAction> getLearningActions() {
		return getLearningActions(this);
	}

	/** {@inheritDoc} */
	public ICursor getNextLearningAction(final ICursor cursor) {
		cursor.push(this);
		for (final ILPElement lpe : getChildren()) {
			if (!lpe.isCompleted()) {
				final ICursor result = ((LPElementImpl) lpe).getNextLearningAction(cursor);
				if (result != null) {
					return result;
				}
			}
		}
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public boolean canSubscribe() {
		return true;
	}

	/** {@inheritDoc} */
	@Override
	protected void init() throws SystemException, PortalException {
		super.init();
		initLearningPath();
	}

	/**
	 * initLearningPath.
	 * 
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             PortalException
	 */
	protected void initLearningPath() throws SystemException, PortalException {
		initCompetenceProfileLevel();
		initCompetenceLevels();
		initRootCluster();
	}

	/** {@inheritDoc} */
	@Override
	protected Object getJAXBElement() {
		return xmlPath;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isAdvisedOrder() {
		return rootCluster != null ? rootCluster.isAdvisedOrder() : true;
	}

	private void initCompetenceProfileLevel() throws SystemException, PortalException {
		// get the finishcompetenceprofilelevel from the xml
		final String finishUri = xmlPath.getLearningPathDesign().getFinish().getUri();
		if (finishUri != null) {
			final Long finishCompetenceProfileLevelId = EntityUtils.decode(finishUri);
			if (finishCompetenceProfileLevelId > 0) {
				finishCompetenceProfileLevel = CompetenceProfileLevelFacade.getCompetenceProfileLevel(
						finishCompetenceProfileLevelId, getContext());
			} else {
				throw new LearningPathException(
						"Invalid learningPath: invalid or no competence profile level with uri " + finishUri);
			}
		}
	}

	private ILPElement getElementFromRef(final Object ref) {
		Object obj;
		if (ref instanceof ExternalLearningPathRef) {
			obj = ((ExternalLearningPathRef) ref).getRef();
			if (obj instanceof ExternalLearningPath) {
				return getLPElementById(((ExternalLearningPath) obj).getId());
			}
		} else if (ref instanceof LearningActionsClusterRef) {
			obj = ((LearningActionsClusterRef) ref).getRef();
			if (obj instanceof LearningActionsCluster) {
				return getLPElementById(((LearningActionsCluster) obj).getId());
			}
		} else if (ref instanceof LearningActionRef) {
			obj = ((LearningActionRef) ref).getRef();
			if (obj instanceof LearningAction) {
				return getLPElementById(((LearningAction) obj).getId());
			}
		}
		return null;
	}

	private void addLPElementToLists(final ILPElement element) {
		elementsByUri.put(element.getUri(), element);
		elementsById.put(element.getId(), element);
	}

	private void createInternalModel() throws PortalException, SystemException {
		for (final Object obj : xmlPath.getLearningActions().getObjects()) {
			if (obj instanceof LearningAction) {
				final LearningAction action = (LearningAction) obj;
				addLPElementToLists(new LearningActionImpl(action, this, getContext()));
			} else if (obj instanceof LearningActionsCluster) {
				final LearningActionsCluster cluster = (LearningActionsCluster) obj;
				final ClusterType type = ClusterType.valueOfType(cluster.getType());
				switch (type) {
				case SEQUENCE:
					addLPElementToLists(new SequenceImpl(cluster, this, getContext()));
					break;
				case FREE_ORDER:
					addLPElementToLists(new SelectionImpl(cluster, this, getContext()));
					break;
				case PARALLEL:
					addLPElementToLists(new ParallelImpl(cluster, this, getContext()));
					break;
				default:
					throw new IllegalArgumentException("Cluster type not supported");
				}
			} else if (obj instanceof ExternalLearningPath) {
				final ExternalLearningPath extPath = (ExternalLearningPath) obj;
				final ILearningPath path = LearningPathService.getLearningPath(extPath.getUri(), getContext());

				// The elementsByUri/elementsById lookup for an ExternalLearningPath differs from other entities.
				elementsByUri.put(extPath.getUri(), path);
				elementsById.put(extPath.getId(), path);
			}
		}
	}

	private void initCompetenceLevels() throws SystemException, PortalException {
		// get the competencelevels from the xml
		if (xmlPath.getLearningPathDesign().getFinish().getCompetenceLevelRef() != null) {
			final List<Long> competenceLevelIds = new ArrayList<Long>();
			for (Finish.CompetenceLevelRef ref : xmlPath.getLearningPathDesign().getFinish().getCompetenceLevelRef()) {
				final CompetenceLevelXML xmlLevel = (CompetenceLevelXML) ref.getRef();
				Long id = EntityUtils.decode(xmlLevel.getUri());
				if (id > 0) {
					competenceLevelIds.add(id);
				} else {
					throw new LearningPathException("Invalid learningPath: invalid or no competence level with uri "
							+ xmlLevel.getUri());
				}
			}
			levels = CompetenceLevelFacade.getCompetenceLevelsByIds(competenceLevelIds, getContext());
			for (CompetenceLevel level : levels) {
				internalLevels.add(new CompetenceLevelImpl(level, getContext()));
			}
		}
	}

	private void initRootCluster() {
		final Object ref = xmlPath.getLearningPathDesign().getLearningActions().getRefs().get(0);
		if (ref instanceof LearningActionsClusterRef) {
			rootCluster = (ICluster) getElementFromRef((LearningActionsClusterRef) ref);
		} else {
			rootCluster = null;
		}
	}

	private List<ILearningAction> getLearningActions(final ILPElement element) {
		final List<ILearningAction> actions = new ArrayList<ILearningAction>();

		if (element instanceof ICluster) {
			final List<? extends ILPElement> children = ((ICluster) element).getChildren();
			for (final ILPElement child : children) {
				actions.addAll(getLearningActions(child));
			}
		} else if (element instanceof ILearningAction) {
			actions.add((ILearningAction) element);
		}

		return actions;
	}
}