/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.services;

import java.util.List;

import org.tencompetence.business.constants.ServiceConstants;
import org.tencompetence.business.exceptions.EndlessLoopException;
import org.tencompetence.business.facade.CompetenceFacade;
import org.tencompetence.business.facade.CompetenceLevelFacade;
import org.tencompetence.business.facade.CompetenceProfileLevelFacade;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.generated.model.LevelValue;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;

import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.service.ServiceContext;

/**
 * AssessmentCalculation class.
 * 
 * @author Leon Schaeps
 * @author Anton Finders
 * 
 * @version $Revision: 1.4 $, $Date: 2009/11/24 14:18:19 $
 */
public class AssessmentCalculation {
	private static final int PERCENT_100 = 100;
	private static final Log LOG = LogFactoryUtil.getLog(AssessmentCalculation.class);
	private double total;
	private double attained;
	private int counter;

	/**
	 * Calculate the rating op the attained competencelevels.
	 * 
	 * @param selectedGoal
	 *            Goal to calculate the attained levels
	 * @param ctx
	 *            Context
	 * @return int rating
	 */
	public int getAssessmentProgress(final Goal selectedGoal, final ServiceContext ctx) {
		total = 0.0;
		attained = 0.0;
		counter = 0;

		try {
			if (selectedGoal == null) {
				return 0;
			}
			if (selectedGoal.getTargetType().equalsIgnoreCase(CompetenceProfileLevel.class.getName())) {
				List<CompetenceLevel> list = CompetenceProfileLevelFacade.getCompetenceLevels(selectedGoal.getTarget(),
						ctx);
				for (CompetenceLevel level : list) {
					getProcessProgress(level, ctx);
				}
			}
		} catch (EndlessLoopException e) {
			LOG.warn("getProcessProgress - EndlessLoopException: ", e);
			return -1;
		} catch (PortalException e) {
			LOG.error("getProgressRate - PortalException: ", e);
		} catch (SystemException e) {
			LOG.error("getProgressRate - SystemException: ", e);
		}

		if (total == 0) {
			return 0;
		}
		return (int) Math.round(attained / total * PERCENT_100);
	}

	/**
	 * Calculate the number of attained competence levels leafs.
	 * 
	 * @param competenceLevel
	 *            the competence level
	 * @param ctx
	 *            the ctx
	 * @param counter
	 *            the counter
	 * @param total
	 *            the total
	 * @param attained
	 *            the attained
	 * 
	 * @throws EndlessLoopException
	 *             the endless loop exception
	 */
	private void getProcessProgress(final CompetenceLevel competenceLevel, final ServiceContext ctx)
			throws EndlessLoopException {
		counter++;

		List<CompetenceLevel> childs;
		try {
			if (counter == ServiceConstants.MAX_LOOP_COUNTER) {
				throw new EndlessLoopException("Current competenceLevelId " + competenceLevel.getCompetenceLevelId());
			}
			childs = CompetenceLevelFacade.getCompetenceLevelsByIds(competenceLevel.getChildLevelIds(), ctx);
			if (childs.size() > 0) { // folder
				for (CompetenceLevel child : childs) {
					getProcessProgress(child, ctx);
				}
			} else { // leaf
				final Competence childCompetence = CompetenceFacade.getCompetence(competenceLevel.getCompetenceId(), ctx);
				final LevelValue levelValue = PdpService.getAttainedLevelForUser(ctx.getUserId(), childCompetence, ctx);
				if (levelValue != null) {
					attained++;
				}
				total++;
			}
		} catch (SystemException e) {
			LOG.error("getProcessProgress - SystemException: ", e);
		} catch (PortalException e) {
			LOG.error("getProcessProgress - PortalException: ", e);
		}
		return;
	}
}
