/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.services;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.tencompetence.business.constants.ClusterType;
import org.tencompetence.business.constants.EventType;
import org.tencompetence.business.constants.LearningPathType;
import org.tencompetence.business.exceptions.LearningPlanException;
import org.tencompetence.business.facade.CompetenceLevelFacade;
import org.tencompetence.business.facade.CompetenceProfileLevelFacade;
import org.tencompetence.business.facade.GoalFacade;
import org.tencompetence.business.facade.JournalEntryFacade;
import org.tencompetence.business.facade.LearningPathFacade;
import org.tencompetence.business.model.learningpath.ILPElement;
import org.tencompetence.business.model.learningpath.ILearningPath;
import org.tencompetence.business.model.learningpath.ILearningPlan;
import org.tencompetence.business.model.learningpath.impl.LearningPlanImpl;
import org.tencompetence.business.utils.EntityUtils;
import org.tencompetence.business.utils.StringUtils;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.CompetenceProfileLevel;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.generated.model.LearningPath;
import org.tencompetence.generated.model.PortfolioEntry;
import org.tencompetence.services.exceptions.TENCServiceException;
import org.tencompetence.services.jaxb.LPMarshaller;
import org.tencompetence.services.jaxb.model.lp.CompetenceLevelXML;
import org.tencompetence.services.jaxb.model.lp.ExternalLearningPath;
import org.tencompetence.services.jaxb.model.lp.Finish;
import org.tencompetence.services.jaxb.model.lp.LearningActionsCluster;
import org.tencompetence.services.jaxb.model.lp.LearningPathXML;
import org.tencompetence.services.jaxb.model.lp.ObjectFactory;
import org.tencompetence.services.jaxb.model.lp.LearningActionsCluster.ClusterMetadata;
import org.tencompetence.services.jaxb.model.lp.LearningActionsCluster.ExternalLearningPathRef;
import org.tencompetence.services.jaxb.model.lp.LearningPathXML.CompetenceLevels;
import org.tencompetence.services.jaxb.model.lp.LearningPathXML.LearningActions;
import org.tencompetence.services.jaxb.model.lp.LearningPathXML.LearningPathDesign;
import org.tencompetence.services.jaxb.model.lp.LearningPathXML.Metadata;
import org.tencompetence.services.jaxb.model.lp.LearningPathXML.LearningPathDesign.DesignLearningActions.LearningActionsClusterRef;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.service.ServiceContext;

/**
 * The class LearningPlanService.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @author Stef Heyenrath
 * 
 * @version $Revision: 1.5 $, $Date: 2009/11/09 15:57:53 $
 */
public final class LearningPlanService {

	private LearningPlanService() {
		// private constructor
	}

	/**
	 * Gets the learning plan.
	 * 
	 * @param uri
	 *            the uri
	 * @param ctx
	 *            the ctx
	 * 
	 * @return the learning path
	 * 
	 * @throws PortalException
	 *             the portal exception
	 * @throws SystemException
	 *             the system exception
	 */
	public static ILearningPlan getLearningPlan(final String uri, final ServiceContext ctx) throws PortalException,
			SystemException {

		final LearningPath path = LearningPathFacade.getLearningPlanByUri(uri, ctx);
		return path != null ? new LearningPlanImpl(path.getXmlData(), ctx) : null;
	}

	/**
	 * Gets the learning plan.
	 * 
	 * @param ctx
	 *            the ctx
	 * @param userId
	 *            the user id
	 * @param goalId
	 *            the goal id
	 * 
	 * @return the learning path
	 * 
	 * @throws PortalException
	 *             the portal exception
	 * @throws SystemException
	 *             the system exception
	 */
	public static ILearningPlan getLearningPlanForUserByGoalId(final long userId, final long goalId,
			final ServiceContext ctx) throws PortalException, SystemException {

		final String uri = getLearningPlanUriForUserByGoalId(userId, goalId, ctx);
		return getLearningPlan(uri, ctx);
	}

	/**
	 * Gets the learning plan uri from the portfolioentry.
	 * 
	 * @param ctx
	 *            the ctx
	 * @param userId
	 *            the user id
	 * @param goalId
	 *            the goal id
	 * 
	 * @return the uri
	 * 
	 * @throws PortalException
	 *             the portal exception
	 * @throws SystemException
	 *             the system exception
	 */
	public static String getLearningPlanUriForUserByGoalId(final long userId, final long goalId,
			final ServiceContext ctx) throws PortalException, SystemException {

		final List<PortfolioEntry> entries = JournalEntryFacade.getEntriesByUCGClassEventTypeValue(userId, 0L, 0L,
				ILearningPlan.class.getName(), EventType.GOAL_LINKED.name(), String.valueOf(goalId));

		if (entries.size() == 0) {
			return null;
		} else if (entries.size() == 1) {
			return entries.get(0).getEntityId();
		} else {
			throw new LearningPlanException("More than one (" + entries.size() + ") learning plan found for goal "
					+ goalId);
		}
	}

	/**
	 * Get all ILearningPlans for a user.
	 * 
	 * @param ctx
	 *            the ctx
	 * @param userId
	 *            the user id
	 * 
	 * @return the list<ILearningPlan>
	 * 
	 * @throws PortalException
	 *             the portal exception
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<ILearningPlan> getLearningPlansByUser(final long userId, final ServiceContext ctx)
			throws SystemException, PortalException {
		return mapList(LearningPathFacade.getLearningPathsForUser(userId, ctx, false), ctx);
	}

	private static List<ILearningPlan> mapList(final List<LearningPath> list, final ServiceContext ctx)
			throws PortalException, SystemException {
		final List<ILearningPlan> resultList = new ArrayList<ILearningPlan>();
		for (final LearningPath path : list) {
			resultList.add(new LearningPlanImpl(path.getXmlData(), ctx));
		}

		return resultList;
	}

	/**
	 * Create and save a ILearningPlan with required values.
	 * 
	 * @param userId
	 *            the userId
	 * @param ctx
	 *            the ctx
	 * @param goal
	 *            the goal
	 * @param title
	 *            the title
	 * 
	 * @return the ILearningPlan
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	@SuppressWarnings("unchecked")
	public static ILearningPlan addLearningPlan(final long userId, final Goal goal, final String title,
			final ServiceContext ctx) throws SystemException, PortalException {

		if (getLearningPlanUriForUserByGoalId(userId, goal.getGoalId(), ctx) != null) {
			throw new LearningPlanException("A learningPlan with goalId " + goal.getGoalId() + " already exists");
		}

		final long profileLevelId = goal.getTarget();
		final CompetenceProfileLevel profileLevel = CompetenceProfileLevelFacade.getCompetenceProfileLevel(
				profileLevelId, ctx);
		final List<CompetenceLevel> levels = CompetenceLevelFacade.getCompetenceLevelsByIds(profileLevel
				.getCompetenceLevelIds(), ctx);

		final LearningPath path = LearningPathFacade.createLearningPath();
		path.setUserId(ctx.getUserId());
		path.setCompanyId(ctx.getCompanyId());
		path.setGroupId(ctx.getScopeGroupId());
		path.setUri(EntityUtils.encode(path));
		path.setCoeditorIds(null);
		path.setType(LearningPathType.PLAN.name());

		final String planTitle = Validator.isNotNull(title) ? title : goal.getTitle();
		final String xmlData = createInitialLearningPlanXML(path.getUri(), profileLevel, levels,
				Collections.EMPTY_LIST, planTitle);
		path.setXmlData(xmlData);

		LearningPathFacade.addLearningPath(path, ctx);

		final ILearningPlan plan = new LearningPlanImpl(path.getXmlData(), ctx);

		PortfolioService.addJournalEntry(ctx.getUserId(), ILearningPlan.class.getName(), plan.getUri(),
				EventType.CREATE, null, ctx);

		// add journalentry with link between ILearningPlan and Goal.
		PortfolioService.addJournalEntry(ctx.getUserId(), ILearningPlan.class.getName(), plan.getUri(),
				EventType.GOAL_LINKED, String.valueOf(goal.getGoalId()), ctx);

		return plan;
	}

	/**
	 * Update learning plan.
	 * 
	 * @param plan
	 *            the plan
	 * @param ctx
	 *            the ctx
	 * 
	 * @return the ILearningPlan
	 * 
	 * @throws PortalException
	 *             the portal exception
	 * @throws SystemException
	 *             the system exception
	 */
	public static ILearningPlan updateLearningPlan(final ILearningPlan plan, final ServiceContext ctx)
			throws PortalException, SystemException {

		final String uri = plan.getUri();
		final LearningPath learningPath = LearningPathFacade.getLearningPlanByUri(uri, ctx);

		final String xmlData = updateExistingLearningPlanXML(learningPath.getXmlData(), plan.getChildren());
		learningPath.setXmlData(xmlData);

		LearningPathFacade.updateLearningPath(learningPath, ctx);

		PortfolioService.addJournalEntry(ctx.getUserId(), ILearningPlan.class.getName(), plan.getUri(),
				EventType.UPDATE, null, ctx);

		plan.setXmlData(learningPath.getXmlData());

		return plan;
	}

	/**
	 * Create a LearningPath XML String with required values.
	 * 
	 * @param paths
	 *            the selected learning paths
	 * @param title
	 *            the goalTitle
	 * @param uri
	 *            the uri
	 * @param initialLevels
	 *            the levels
	 * @param initialProfileLevel
	 *            the initial profile level
	 * 
	 * @return the ILearningPath
	 * 
	 * @throws TENCServiceException
	 *             the TENC service exception
	 */
	private static String createInitialLearningPlanXML(final String uri,
			final CompetenceProfileLevel initialProfileLevel, final List<CompetenceLevel> initialLevels,
			final List<? extends ILPElement> paths, final String title) throws TENCServiceException {
		final ObjectFactory of = new ObjectFactory();
		final LearningPathXML xmlPath = of.createLP();
		xmlPath.setId(StringUtils.randomId());

		// MetaData
		final Metadata lpMeta = of.createLPMetadata();
		lpMeta.setTitle(title);
		lpMeta.setUri(uri);

		xmlPath.setMetadata(lpMeta);

		// LearningPathDesign, Start and Finish
		final LearningPathDesign design = of.createLPDesign();
		final Finish finish = of.createFinish();
		finish.setId(StringUtils.randomId());

		if (initialProfileLevel != null) {
			finish.setUri(EntityUtils.encode(initialProfileLevel));
		}

		// Add all competenceLevels from the competenceProfile
		if (initialLevels != null) {
			final CompetenceLevels xmlLevels = of.createLPCompetenceLevels();
			for (final CompetenceLevel level : initialLevels) {

				// Add CompetenceLevel to xmlPath
				final CompetenceLevelXML xmlLevel = of.createCompetenceLevelXML();
				xmlLevel.setId(StringUtils.randomId());
				xmlLevel.setUri(EntityUtils.encode(level));
				xmlLevels.getCompetenceLevel().add(xmlLevel);

				// Add CompetenceLevel to Finish Ref
				final Finish.CompetenceLevelRef ref = of.createFinishCompetenceLevelRef();
				ref.setRef(xmlLevel);
				finish.getCompetenceLevelRef().add(ref);
			}

			xmlPath.setCompetenceLevels(xmlLevels);
		}

		design.setFinish(finish);
		design.setLearningActions(of.createDesignLAs());

		// LearningActions
		final LearningActions la = of.createLPLearningActions();
		xmlPath.setLearningActions(la);

		// create learning paths and add to learning actions
		int numPaths = 0;
		for (final ILPElement element : paths) {
			if (element instanceof ILearningPath) {
				la.getObjects().add(((ILearningPath) element).createJAXBRepresentation());
				numPaths++;
			}
		}

		// create cluster and add to learning actions
		final ClusterMetadata metadata = of.createLACClusterMetadata();
		metadata.setTitle(ClusterType.FREE_ORDER.getType() + " " + StringUtils.randomId());

		final LearningActionsCluster cluster = of.createLAC();
		cluster.setMetadata(metadata);
		cluster.setId(StringUtils.randomId());
		cluster.setType(ClusterType.FREE_ORDER.getType());
		cluster.setNumberToSelect(numPaths);
		la.getObjects().add(cluster);

		xmlPath.setLearningPathDesign(design);

		// add learning path references to cluster
		for (final Object obj : la.getObjects()) {
			if (obj instanceof ExternalLearningPath) {
				final ExternalLearningPathRef ref = of.createLACExternalLearningPathRef();
				ref.setRef(obj);
				cluster.getRefs().add(ref);
			}
		}

		// create the design and add the clusterRef
		final LearningActionsClusterRef clusterRef = of.createDesignLAsLearningActionsClusterRef();
		clusterRef.setRef(cluster);
		design.getLearningActions().getRefs().add(clusterRef);

		return LPMarshaller.getInstance().marshal(xmlPath);
	}

	/**
	 * Create a LearningPath XML String with required values.
	 * 
	 * @param paths
	 *            the selected learning paths
	 * @param xmlData
	 *            the xml data
	 * 
	 * @return the ILearningPath
	 * 
	 * @throws TENCServiceException
	 *             the TENC service exception
	 */
	private static String updateExistingLearningPlanXML(final String xmlData, final List<? extends ILPElement> paths)
			throws TENCServiceException {
		final ObjectFactory of = new ObjectFactory();
		final LearningPathXML xmlPath = LPMarshaller.getInstance().unMarshal(xmlData);

		// Cleanup LearningActions (remove all ExternalLearningPaths)
		LearningActions la = xmlPath.getLearningActions();
		LearningActionsCluster cluster = null;
		for (final Object action : la.getObjects()) {
			if (action instanceof LearningActionsCluster) {
				cluster = (LearningActionsCluster) action;
				cluster.getRefs().clear();
				break;
			}
		}

		la = of.createLPLearningActions();
		// Create learning paths and add to learning actions
		int numPaths = 0;
		for (final ILPElement element : paths) {
			if (element instanceof ILearningPath) {
				final ExternalLearningPath path = ((ILearningPath) element).createJAXBRepresentation();
				la.getObjects().add(path);

				// Add LearningPath as ref to the cluster
				final ExternalLearningPathRef ref = of.createLACExternalLearningPathRef();
				ref.setRef(path);
				cluster.getRefs().add(ref);
				numPaths++;
			}
		}

		// Update the cluster
		cluster.setNumberToSelect(numPaths);

		// Add cluster to actions
		la.getObjects().add(cluster);
		xmlPath.setLearningActions(la);

		return LPMarshaller.getInstance().marshal(xmlPath);
	}

	/**
	 * Gets the goal for LearningPlan.
	 * 
	 * @param plan
	 *            the plan
	 * @param ctx
	 *            the ctx
	 * 
	 * @return the goal for learning plan
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static Goal getGoalForLearningPlan(final ILearningPlan plan, final ServiceContext ctx)
			throws SystemException, PortalException {

		final PortfolioEntry entry = JournalEntryFacade.getLatestEntryByUCGClassEntityIdEventTypeValue(ctx.getUserId(),
				0L, 0L, ILearningPlan.class.getName(), plan.getUri(), EventType.GOAL_LINKED.name(), null);

		if (entry != null) {
			final long goalId = Long.parseLong(entry.getValue());
			return GoalFacade.getGoal(goalId, ctx);
		}

		return null;
	}
}
