/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.services;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.tencompetence.business.constants.EventType;
import org.tencompetence.business.exceptions.CompetenceException;
import org.tencompetence.business.exceptions.EndlessLoopException;
import org.tencompetence.business.exceptions.LevelValueException;
import org.tencompetence.business.facade.CompetenceFacade;
import org.tencompetence.business.facade.CompetenceLevelFacade;
import org.tencompetence.business.facade.JournalEntryFacade;
import org.tencompetence.business.facade.LevelValueFacade;
import org.tencompetence.business.model.learningpath.ILPElement;
import org.tencompetence.business.model.learningpath.ILearningPath;
import org.tencompetence.business.model.learningpath.ILearningPlan;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.CompetenceLevel;
import org.tencompetence.generated.model.LevelValue;
import org.tencompetence.generated.model.PortfolioEntry;
import org.tencompetence.services.constants.LevelValueType;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.service.ServiceContext;
import com.liferay.portal.service.UserLocalServiceUtil;

/**
 * PdpService class.
 * 
 * @author jal
 * @version $Revision: 1.6 $, $Date: 2009/11/24 14:18:19 $
 */
public final class PdpService {
	private static final double PERCENT_100 = 100;

	private PdpService() {
		// private constructor
	}

	/**
	 * The object containing the PDP progress.
	 * 
	 * @author jal
	 * 
	 */
	public static class PDPProgress {
		private int attained;
		private int planned;
		private int rest;

		/**
		 * Constructor.
		 * 
		 * @param attained
		 *            attained
		 * @param planned
		 *            planned
		 * @param rest
		 *            rest
		 */
		public PDPProgress(final int attained, final int planned, final int rest) {
			this.attained = attained;
			this.planned = planned;
			this.rest = rest;
		}

		/**
		 * get attained.
		 * 
		 * @return attained attained
		 */
		public int getAttained() {
			return attained;
		}

		/**
		 * get planned.
		 * 
		 * @return planned planned
		 */
		public int getPlanned() {
			return planned;
		}

		/**
		 * get rest.
		 * 
		 * @return rest rest
		 */
		public int getRest() {
			return rest;
		}
	}

	/**
	 * getProgressByProfileLevelId.
	 * 
	 * @param plan
	 *            ILearningPlan
	 * @param competenceProfileLevelId
	 *            long
	 * @param ctx
	 *            ServiceContext
	 * 
	 * @throws PortalException
	 *             PortalException
	 * @throws SystemException
	 *             SystemException
	 * @throws EndlessLoopException
	 *             EndlessLoopException
	 * @return PDPProgress
	 */
	public static PDPProgress getProgressByProfileLevelId(final ILearningPlan plan,
			final long competenceProfileLevelId, final ServiceContext ctx) throws SystemException, PortalException,
			EndlessLoopException {
		int attained = 0;
		int planned = 0;
		int rest = (int) PERCENT_100;

		// store the levels already attained, we need them in planCoversLevel()
		List<CompetenceLevel> levelsYetToCheck = new ArrayList<CompetenceLevel>();
		double levelsAttained = 0;
		double levelsCoveredByPlan = 0;
		double levelsNotAttainedNotCovered = 0;
		double total = 0;

		List<CompetenceLevel> levels = CompetenceLevelFacade.getLeafCompetenceLevelsByCompetenceProfileId(
				competenceProfileLevelId, ctx);
		total = levels.size();
		if (plan != null && total != 0) {
			for (CompetenceLevel level : levels) {
				// get the relevant competence levels attained. Green part
				Competence competence = CompetenceFacade.getCompetence(level.getCompetenceId(), ctx);
				LevelValue attainedLevel = PdpService.getAttainedLevelForUser(ctx.getUserId(), competence, ctx);
				LevelValue requiredLevel = LevelValueFacade.getLevelValue(level.getLevelValueId(), ctx);
				if (attainedLevel != null && (attainedLevel.getValue() >= requiredLevel.getValue())) {
					levelsAttained++;
				} else {
					levelsYetToCheck.add(level);
				}
			}

			// get the number of levels covered by the plan (but not attained yet). Blue part
			levelsCoveredByPlan = numberOfCoveredLevelsByPlan(plan, levelsYetToCheck);

			// not attained and not covered by plan. Red part
			levelsNotAttainedNotCovered = total - (levelsAttained + levelsCoveredByPlan);

			attained = (int) Math.round(levelsAttained / total * PERCENT_100);
			planned = (int) Math.round(levelsCoveredByPlan / total * PERCENT_100);
			rest = (int) Math.round(levelsNotAttainedNotCovered / total * PERCENT_100);
		}

		return new PDPProgress(attained, planned, rest);
	}

	/**
	 * Check if a plan covers a certain competenceLevel and also if it isn't attained already.
	 * 
	 * @param plan
	 * @param level
	 * @param levelsAttained
	 * @return true is the level is covered by the plan and not attained yet
	 * @throws PortalException
	 * @throws SystemException
	 */
	private static int numberOfCoveredLevelsByPlan(final ILearningPlan plan,
			final List<CompetenceLevel> levelsYetToCheck) throws SystemException, PortalException {
		List<CompetenceLevel> foundLevels = new ArrayList<CompetenceLevel>();
		List<? extends ILPElement> children = plan.getChildren();
		// loop through the children of this plan. If the child is a lerning path, see which levels it provides
		for (ILPElement child : children) {
			if (child instanceof ILearningPath) {
				ILearningPath lpChild = (ILearningPath) child;
				for (CompetenceLevel level : levelsYetToCheck) {
					if (lpChild.providesLevel(level)) {
						// check if the level is already found. If not, add it
						if (!foundLevels.contains(level)) {
							foundLevels.add(level);
						}
					}
				}
			}
		}
		return foundLevels.size();
	}

	/**
	 * Get Attained Competence Level for a user.
	 * 
	 * @param userId
	 *            the user id
	 * @param competence
	 *            competence
	 * @param ctx
	 *            the ctx
	 * 
	 * @return the attained level for user
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static LevelValue getAttainedLevelForUser(final long userId, final Competence competence,
			final ServiceContext ctx) throws SystemException, PortalException {

		LevelValue levelValue = null;

		checkInputValues(competence, ctx.getUserId(), ctx);

		final PortfolioEntry portfolioEntry = PortfolioService.getLatestEntryByUserEventType(userId, competence,
				EventType.ATTAINED_COMPETENCE_LEVEL, null);

		if (portfolioEntry != null) {
			final long levelValueId = Long.valueOf(portfolioEntry.getValue());
			levelValue = LevelValueFacade.getLevelValue(levelValueId, ctx);
		}

		return levelValue;
	}

	/**
	 * Get all Attained Competences for a user (Latest versions).
	 * 
	 * @param userId
	 *            userId (In case the current user gets portfolio-entries for another user)
	 * @param ctx
	 *            ServiceContext
	 * @return Map<Competence, LevelValue>
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static Map<Competence, LevelValue> getAttainedLevelsForUser(final long userId, final ServiceContext ctx)
			throws SystemException, PortalException {

		final Map<Competence, LevelValue> result = new HashMap<Competence, LevelValue>();

		final List<PortfolioEntry> portfolioEntries = JournalEntryFacade.getLatestEntriesByUserClassEventType(
				Competence.class.getName(), EventType.ATTAINED_COMPETENCE_LEVEL.name(), ctx);

		for (final PortfolioEntry portfolioEntry : portfolioEntries) {
			final Competence competence = (Competence) CompetenceFacade.getCompetence(Long.valueOf(portfolioEntry
					.getEntityId()), ctx);
			final long levelValueId = Long.valueOf(portfolioEntry.getValue());
			final LevelValue levelValue = LevelValueFacade.getLevelValue(levelValueId, ctx);
			result.put(competence, levelValue);
		}

		return result;
	}

	private static void checkInputValues(final Competence competence, final long userId, final ServiceContext ctx)
			throws SystemException, PortalException {

		if (Validator.isNull(competence)) {
			throw new CompetenceException("Competence invalid");
		} else {
			CompetenceFacade.getCompetence(competence.getPrimaryKey(), ctx);
		}

		if (Validator.isNull(userId)) {
			throw new SystemException("User is invalid");
		} else {
			UserLocalServiceUtil.getUserById(userId);
		}
	}

	/**
	 * Set Attained Competence Level for a user. (Only possible on Leaf!)
	 * 
	 * @param competence
	 *            competence
	 * @param levelValue
	 *            levelValue
	 * @param userId
	 *            userId (In case the current user creates a portfolio-entry for another user)
	 * @param ctx
	 *            the ctx
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static void setAttainedLevelForUser(final Competence competence, final LevelValue levelValue,
			final long userId, final ServiceContext ctx) throws SystemException, PortalException {

		if (levelValue == null || levelValue.getType() != LevelValueType.Competence.getId()) {
			throw new LevelValueException("Levelvalue invalid");
		} else {
			LevelValueFacade.getLevelValue(levelValue.getPrimaryKey(), ctx);
		}

		// Try to get the CompetenceLevel
		final CompetenceLevel competenceLevel = CompetenceLevelFacade.getBareCompetenceLevelByCompetenceIdLevelValueId(
				competence.getPrimaryKey(), levelValue.getPrimaryKey());

		// Only allowed to set an attained level on a Leaf CompetenceLevel!
		if (competenceLevel != null && Validator.isNull(competenceLevel.getChildLevelIds())) {
			setAttainedLevel(competence, levelValue, userId, ctx);
			new UserLevelCalculation().computeAllCompetenceLevels(userId, competence.getCompetenceMapId(), ctx);
		}
	}

	/**
	 * Set Attained Competence Level for a user to at least the passed level value. The level will not be decreased by
	 * this method. (Only possible on Leaf!).
     *
	 * @param userId
	 *            userId (In case the current user creates a portfolio-entry for another user) 
	 * @param competence
	 *            competence
	 * @param levelValue
	 *            levelValue
	 * @param ctx
	 *            the ctx
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static void setMinimumAttainedLevelForUser(final long userId, final Competence competence,
			final LevelValue levelValue, final ServiceContext ctx) throws SystemException, PortalException {

		final LevelValue oldValue = PdpService.getAttainedLevelForUser(userId, competence, ctx);

		// The setAttainedLevelForUser method will be under 3 circumstances:
		// 1) there was no value set before,
		// 2) a null value was passed as new level (let setAttainedLevelForUser handle it),
		// 3) the new value is higher than the old one
		if (oldValue == null || levelValue == null || (levelValue.getValue() > oldValue.getValue())) {
			setAttainedLevelForUser(competence, levelValue, userId, ctx);
		}
	}

	/**
	 * Sets the attained level.
	 * 
	 * @param competence
	 *            the competence
	 * @param levelValue
	 *            the level value
	 * @param userId
	 *            the user id
	 * @param ctx
	 *            the ctx
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	protected static void setAttainedLevel(final Competence competence, final LevelValue levelValue, final long userId,
			final ServiceContext ctx) throws SystemException, PortalException {
		PortfolioService.addJournalEntry(userId, competence, EventType.ATTAINED_COMPETENCE_LEVEL, String
				.valueOf(levelValue.getPrimaryKey()), ctx);
	}
}
