/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.services;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.tencompetence.business.constants.EventType;
import org.tencompetence.business.facade.AssessmentTestFacade;
import org.tencompetence.business.facade.CompetenceFacade;
import org.tencompetence.business.facade.GoalFacade;
import org.tencompetence.business.facade.JournalEntryFacade;
import org.tencompetence.business.facade.LearningPathFacade;
import org.tencompetence.business.model.learningpath.ILPElement;
import org.tencompetence.business.model.learningpath.ILearningAction;
import org.tencompetence.business.model.learningpath.ILearningPath;
import org.tencompetence.business.model.learningpath.ILearningPlan;
import org.tencompetence.business.utils.ClassUtils;
import org.tencompetence.business.utils.EntityUtils;
import org.tencompetence.generated.model.AssessmentTest;
import org.tencompetence.generated.model.Competence;
import org.tencompetence.generated.model.Goal;
import org.tencompetence.generated.model.LearningPath;
import org.tencompetence.generated.model.LevelValue;
import org.tencompetence.generated.model.PortfolioEntry;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.model.BaseModel;
import com.liferay.portal.service.ServiceContext;
import com.liferay.portal.service.UserLocalServiceUtil;

/**
 * Enter the class documentation here.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @version $Revision: 1.13 $, $Date: 2009/11/30 12:37:50 $
 */
public final class PortfolioService {

	private static final Log LOG = LogFactoryUtil.getLog(PortfolioService.class);

	private PortfolioService() {
		// private constructor
	}

	/**
	 * The Enum SubscribeType.
	 */
	public enum SubscribeType {

		/** Only possible to subscribe once for an entity. */
		ONCE,

		/** Possible to subscribe multiple times for an entity. */
		MULTIPLE
	}

	/**
	 * The Class SearchResult.
	 */
	public static class PortfolioSearchResult {
		private PortfolioEntry entry;
		private Object obj;

		/**
		 * Instantiates a new search result.
		 * 
		 * @param portfolio
		 *            the portfolio
		 * @param obj
		 *            the obj
		 */
		PortfolioSearchResult(final PortfolioEntry portfolio, final Object obj) {
			this.entry = portfolio;
			this.obj = obj;
		}

		/**
		 * Gets the obj.
		 * 
		 * @return the obj
		 */
		public Object getObj() {
			return obj;
		}

		/**
		 * Gets the entry.
		 * 
		 * @return the entry
		 */
		public PortfolioEntry getEntry() {
			return entry;
		}
	}

	/**
	 * Get the most recent PortfolioEntry for a specific User, entityClass, entityId, eventType and value.
	 * 
	 * @param userId
	 *            the user id
	 * @param className
	 *            the class name
	 * @param entityId
	 *            the entity id
	 * @param type
	 *            the type
	 * @param value
	 *            the value
	 * 
	 * @return the latest entry
	 * 
	 * @throws SystemException
	 *             the system exception
	 */
	public static PortfolioEntry getLatestEntryByUserEventType(final long userId, final String className,
			final String entityId, final EventType type, final String value) throws SystemException {
		return JournalEntryFacade.getLatestEntryByUCGClassEntityIdEventTypeValue(userId, 0L, 0L, className, entityId,
				type.name(), value);
	}

	/**
	 * Gets the latest entry by User, entity, eventType and value.
	 * 
	 * @param userId
	 *            the user id
	 * @param entity
	 *            the entity
	 * @param type
	 *            the type
	 * @param value
	 *            the value
	 * @return the latest entry by user event type
	 * 
	 * @throws SystemException
	 *             the system exception
	 */
	public static PortfolioEntry getLatestEntryByUserEventType(final long userId, final BaseModel<?> entity,
			final EventType type, final String value) throws SystemException {
		return JournalEntryFacade.getLatestEntryByUCGClassEntityIdEventTypeValue(userId, 0L, 0L, EntityUtils
				.getInterfaceFromEntity(entity), entity.getPrimaryKeyObj().toString(), type.name(), value);
	}

	/**
	 * Add a portfolio Entry for a BaseModel<T> entity.
	 * 
	 * @param <T>
	 *            Generic BaseModel<T> type
	 * 
	 * @param entity
	 *            The ModelEntity
	 * @param eventType
	 *            The eventType
	 * @param ctx
	 *            The ServiceContext
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             the portal exception
	 */
	public static <T> void addJournalEntry(final BaseModel<T> entity, final EventType eventType,
			final ServiceContext ctx) throws SystemException, PortalException {
		addJournalEntry(ctx.getUserId(), entity, eventType, null, ctx);
	}

	/**
	 * Add a portfolio Entry for a BaseModel<T> entity with a value.
	 * 
	 * @param <T>
	 *            Generic BaseModel<T> type
	 * 
	 * @param entity
	 *            The ModelEntity
	 * @param eventType
	 *            The eventType
	 * @param value
	 *            The value
	 * @param ctx
	 *            The ServiceContext
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             the portal exception
	 */
	public static <T> void addJournalEntry(final BaseModel<T> entity, final EventType eventType, final String value,
			final ServiceContext ctx) throws SystemException, PortalException {
		addJournalEntry(ctx.getUserId(), entity, eventType, value, ctx);
	}

	/**
	 * Add a portfolio Entry for a BaseModel<T> entity.
	 * 
	 * @param <T>
	 *            Generic Type
	 * 
	 * @param userId
	 *            userId (in case the current user creates a portfolio-entry for another user)
	 * @param entity
	 *            The ModelEntity
	 * @param eventType
	 *            The eventType
	 * @param value
	 *            The value
	 * @param ctx
	 *            The ServiceContext
	 * 
	 * @return the portfolio entry
	 * 
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             the portal exception
	 */
	protected static <T> PortfolioEntry addJournalEntry(final long userId, final BaseModel<T> entity,
			final EventType eventType, final String value, final ServiceContext ctx) throws SystemException,
			PortalException {
		return addJournalEntry(userId, EntityUtils.getInterfaceFromEntity(entity),
				entity.getPrimaryKeyObj().toString(), eventType, value, ctx);
	}

	/**
	 * Add a portfolio Entry for an entity.
	 * 
	 * @param userId
	 *            userId (in case the current user creates a portfolio-entry for another user)
	 * @param className
	 *            The className
	 * @param eventType
	 *            The eventType
	 * @param value
	 *            The value
	 * @param ctx
	 *            The ServiceContext
	 * @param entityId
	 *            the entity id
	 * 
	 * @return the portfolio entry
	 * 
	 * @throws SystemException
	 *             SystemException
	 * @throws PortalException
	 *             the portal exception
	 */
	public static PortfolioEntry addJournalEntry(final long userId, final String className, final String entityId,
			final EventType eventType, final String value, final ServiceContext ctx) throws SystemException,
			PortalException {
		final PortfolioEntry portfolioEntry = JournalEntryFacade.createJournalEntry();

		portfolioEntry.setGroupId(ctx.getScopeGroupId());
		portfolioEntry.setCompanyId(ctx.getCompanyId());
		portfolioEntry.setUserId(userId);
		portfolioEntry.setEventType(eventType.name());
		portfolioEntry.setJournalDate(new Date());
		portfolioEntry.setEntityClass(className);
		portfolioEntry.setEntityId(entityId);
		if (value != null) {
			portfolioEntry.setValue(value);
		}

		return JournalEntryFacade.addJournalEntry(portfolioEntry, ctx);
	}

	private static void checkInputValues(final long assessmentId, final long userId, final ServiceContext ctx)
			throws SystemException, PortalException {

		AssessmentTestFacade.getAssessmentTest(assessmentId, ctx);

		if (Validator.isNull(userId)) {
			throw new SystemException("User is invalid");
		} else {
			UserLocalServiceUtil.getUserById(userId);
		}
	}

	/**
	 * Gets the entity from portfolio.
	 * 
	 * @param portfolio
	 *            the portfolio
	 * @param ctx
	 *            the ctx
	 * @param logInJournal
	 *            Log this action in the journal
	 * @return BaseModel<?>
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static BaseModel<?> getEntityFromJournalEntry(final PortfolioEntry portfolio, final ServiceContext ctx,
			final boolean logInJournal) throws SystemException, PortalException {
		return EntityUtils.getEntityFromClassAndId(portfolio.getEntityClass(), Long.parseLong(portfolio.getEntityId()),
				ctx, logInJournal);
	}

	/**
	 * The Class LPElementCache.
	 */
	private static class LPElementCache {
		private final HashMap<String, ILPElement> cache = new HashMap<String, ILPElement>();
		private final ServiceContext ctx;

		/**
		 * Instantiates a new LPElementCache.
		 * 
		 * @param ctx
		 *            the ServiceContext
		 */
		public LPElementCache(final ServiceContext ctx) {
			this.ctx = ctx;
		}

		/**
		 * Gets the LPElement from journal entry.
		 * 
		 * @param portfolio
		 *            the portfolio
		 * @param ctx
		 *            the ctx
		 * 
		 * @return the lP element from journal entry
		 * 
		 * @throws SystemException
		 *             the system exception
		 * @throws PortalException
		 *             the portal exception
		 */
		public ILPElement getLPElementFromJournalEntry(final PortfolioEntry portfolio, final ServiceContext ctx)
				throws PortalException, SystemException {
			return getLPElementFromCache(portfolio.getEntityClass(), portfolio.getEntityId(), portfolio.getValue());
		}

		private ILPElement getLPElementFromCache(final String className, final String uri, final String value)
				throws PortalException, SystemException {

			// Check if we can find the ILPElement in our cache.
			if (!cache.containsKey(uri)) {
				if (ILearningPlan.class.getName().equals(className)) {
					cache.put(uri, LearningPlanService.getLearningPlan(uri, ctx));
				} else if (ILearningPath.class.getName().equals(className)) {
					cache.put(uri, LearningPathService.getLearningPath(uri, ctx));
				} else if (ILearningAction.class.getName().equals(className)) {
					final ILearningPath learningPath = (ILearningPath) getLPElementFromCache(ILearningPath.class
							.getName(), value, null);
					final ILPElement element = learningPath.getLPElementByUri(uri);

					// http://tst.tencompetence.org/ticket/504
					if (element != null) {
						cache.put(uri, learningPath.getLPElementByUri(uri));
					}
				}
			}

			return cache.get(uri);
		}
	}

	/**
	 * Gets a list from BaseModel entities (can be mixed) from journal entries.
	 * 
	 * @param entries
	 *            The PortfolioEntries
	 * @param ctx
	 *            The ctx
	 * @param logInJournal
	 *            Log this action in the journal
	 * @return List<BaseModel<?>>
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static List<BaseModel<?>> getMixedEntitiesFromJournalEntries(final List<PortfolioEntry> entries,
			final ServiceContext ctx, final boolean logInJournal) throws SystemException, PortalException {

		final List<BaseModel<?>> list = new ArrayList<BaseModel<?>>();

		for (final PortfolioEntry entry : entries) {
			final BaseModel<?> entity = getEntityFromJournalEntry(entry, ctx, logInJournal);
			if (entity != null) {
				list.add(entity);
			}
		}

		return list;
	}

	/**
	 * Gets the filtered search results.
	 * 
	 * @param entries
	 *            the entries
	 * @param filter
	 *            the filter
	 * @param ctx
	 *            the ctx
	 * 
	 * @return the filtered search results
	 * 
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<PortfolioSearchResult> getFilteredPortfolioSearchResults(final List<PortfolioEntry> entries,
			final String filter, final ServiceContext ctx) throws SystemException {
		final List<PortfolioSearchResult> resultList = new ArrayList<PortfolioSearchResult>();

		// Do some caching...
		final Map<String, Object> cachedEntities = new HashMap<String, Object>();
		final LPElementCache cache = new LPElementCache(ctx);

		for (final PortfolioEntry entry : entries) {
			try {
				final String key = entry.getEntityId();
				final Object entity;

				// First try to find the entity in the cache
				if (cachedEntities.containsKey(key)) {
					entity = cachedEntities.get(key);
				} else if (isBaseModelEntity(entry.getEntityClass())) {
					entity = getEntityFromJournalEntry(entry, ctx, false);
					cachedEntities.put(key, entity);
				} else if (isLPElement(entry.getEntityClass())) {
					entity = cache.getLPElementFromJournalEntry(entry, ctx);
					cachedEntities.put(key, entity);
				} else {
					throw new IllegalArgumentException("Classname not found : " + entry.getEntityClass());
				}

				// http://tst.tencompetence.org/ticket/504
				if (entity != null) {
					if (Validator.isNull(filter)) {
						resultList.add(new PortfolioSearchResult(entry, entity));
					} else {
						final String title = ClassUtils.getProperty(entity, "title", StringPool.BLANK);
						if (title.toLowerCase().contains(filter.toLowerCase())) {
							resultList.add(new PortfolioSearchResult(entry, entity));
						}
					}
				}
			} catch (PortalException e) {
				LOG.debug(e);
			}
		}
		return resultList;
	}

	/**
	 * Gets the entries filtered on title by User, Company, Group, classes, eventTypes and dateRange.
	 * 
	 * @param userId
	 *            the user id
	 * @param companyId
	 *            the company id [ When 0 is passed, there is no filtering on company ]
	 * @param groupId
	 *            the group id [ When 0 is passed, there is no filtering on group (community) ]
	 * @param entityClasses
	 *            the entity classes
	 * @param eventTypes
	 *            the event types
	 * @param maxDate
	 *            the date (most recent date of both dates, minDate < maxDate)
	 * @param minDate
	 *            the date (less recent date of both dates, minDate < maxDate)
	 * @param filter
	 *            the filter
	 * @param ctx
	 *            the ctx
	 * 
	 * 
	 * @return SearchResult the filtered entries by user classes event types
	 * 
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<PortfolioSearchResult> getFilteredEntriesByUCGClassesEventTypesDateRange(final long userId,
			final long companyId, final long groupId, final List<String> entityClasses, final List<String> eventTypes,
			final Date minDate, final Date maxDate, final String filter, final ServiceContext ctx)
			throws SystemException {

		final List<PortfolioEntry> entries = JournalEntryFacade.getEntriesByUCGClassesEventTypesDateRange(userId,
				companyId, groupId, entityClasses, eventTypes, minDate, maxDate);
		return getFilteredPortfolioSearchResults(entries, filter, ctx);
	}

	/**
	 * Gets the entities filtered on title by User, Company, Group, classes, eventTypes and dateRange.
	 * 
	 * @param userId
	 *            the user id
	 * @param companyId
	 *            the company id [ When 0 is passed, there is no filtering on company ]
	 * @param groupId
	 *            the group id [ When 0 is passed, there is no filtering on group (community) ]
	 * @param entityClasses
	 *            the entity classes
	 * @param eventTypes
	 *            the event types
	 * @param maxDate
	 *            the date (most recent date of both dates, minDate < maxDate)
	 * @param minDate
	 *            the date (less recent date of both dates, minDate < maxDate)
	 * @param filter
	 *            the filter
	 * @param ctx
	 *            the ctx
	 * 
	 * @return SearchResult the filtered entries by user classes event types
	 * 
	 * @throws SystemException
	 *             the system exception
	 */
	public static List<PortfolioSearchResult> getFilteredEntitiesByUCGClassesEventTypesDateRange(final long userId,
			final long companyId, final long groupId, final List<String> entityClasses, final List<String> eventTypes,
			final Date minDate, final Date maxDate, final String filter, final ServiceContext ctx)
			throws SystemException {

		final List<PortfolioEntry> entries = JournalEntryFacade.getEntitiesByUCGClassesEventTypes(userId, companyId,
				groupId, entityClasses, eventTypes, minDate, maxDate);

		final List<PortfolioSearchResult> resultList = getFilteredPortfolioSearchResults(entries, filter, ctx);

		// Get current value of the entity
		for (final PortfolioSearchResult result : resultList) {
			final PortfolioEntry entry = result.getEntry();

			// For competence get current attained level
			if (entry.getEntityClass().equalsIgnoreCase(Competence.class.getName())) {
				try {
					final Competence competence = (Competence) getEntityFromJournalEntry(entry, ctx, false);
					final LevelValue level = PdpService.getAttainedLevelForUser(userId, competence, ctx);
					if (level != null) {
						entry.setValue(String.valueOf(level.getPrimaryKey()));
						entry.setEventType(EventType.ATTAINED_COMPETENCE_LEVEL.toString());
					}
				} catch (PortalException e) {
					LOG.debug(e);
				}
			}

			// For a LearningAction set eventtype to completed or subscribed.
			if (result.getObj() instanceof ILearningAction) {
				final ILearningAction action = (ILearningAction) result.getObj();

				if (action.isCompleted()) {
					entry.setEventType(EventType.COMPLETE.name());
				} else if (action.isSubscribed()) {
					entry.setEventType(EventType.SUBSCRIBE.name());
				}

				entry.setValue(StringPool.BLANK);
			}
		}

		return resultList;

	}

	/**
	 * Gets a list from specific entities from a list from journal entries.
	 * 
	 * @param <T>
	 *            Generic BaseModel<T> type
	 * 
	 * @param entries
	 *            The PortfolioEntries
	 * @param ctx
	 *            The ctx
	 * @param logInJournal
	 *            Log this action in the journal
	 * @param clazz
	 *            the clazz
	 * @return List<T>
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	@SuppressWarnings("unchecked")
	public static <T> List<T> getEntitiesFromJournalEntries(final List<PortfolioEntry> entries, final Class<T> clazz,
			final ServiceContext ctx, final boolean logInJournal) throws SystemException, PortalException {

		final String clazzName = clazz.getName();
		final List<Long> ids = JournalEntryFacade.convertToIdList(entries, clazzName);

		// TODO add more entities
		if (Goal.class == clazz) {
			return (List<T>) GoalFacade.getGoalsByIds(ids);
		} else if (LearningPath.class == clazz) {
			return (List<T>) LearningPathFacade.getLearningPathsByIds(ids, ctx);
		} else if (Competence.class == clazz) {
			return (List<T>) CompetenceFacade.getCompetenceByIds(ids, ctx);
		}

		return Collections.EMPTY_LIST;
	}

	private static boolean isBaseModelEntity(final String className) {
		return className.startsWith("org.tencompetence.generated.model");
	}

	private static boolean isLPElement(final String className) {

		return ILearningPlan.class.getName().equals(className) || ILearningPath.class.getName().equals(className)
				|| ILearningAction.class.getName().equals(className);
	}

	/**
	 * Get assessment test result for a user which is expressed as an int ranging from 0 to 8 corresponding to the EQF
	 * levels.
	 * 
	 * @param userId
	 *            userId (In case the current user gets a portfolio-entry for another user)
	 * @param assessmentId
	 *            the assessmentId for which the result is fetched
	 * 
	 * @param ctx
	 *            the ctx
	 * 
	 * @return test result of specified test
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static Integer getTestResultForUser(final long assessmentId, final long userId, final ServiceContext ctx)
			throws SystemException, PortalException {

		Integer result = null;

		checkInputValues(assessmentId, userId, ctx);

		final PortfolioEntry portfolioEntry = JournalEntryFacade.getLatestEntryByUCGClassEntityIdEventTypeValue(userId,
				0L, 0L, AssessmentTest.class.getName(), String.valueOf(assessmentId), EventType.ATTAINED_TEST.name(),
				null);

		if (portfolioEntry != null) {
			result = Integer.valueOf(portfolioEntry.getValue());
		}

		return result;
	}
}