/*
 * Copyright (c) 2009, TENCompetence Foundation All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice, this
 *     list of conditions and the following disclaimer.
 *   - Redistributions in binary
 *     form must reproduce the above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or other materials 
 *     provided with the distribution.
 *   - Neither the name of the TENCompetence Foundation nor
 *     the names of its contributors may be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package org.tencompetence.business.services;

import org.tencompetence.business.constants.EventType;
import org.tencompetence.business.services.PortfolioService.SubscribeType;
import org.tencompetence.business.utils.EntityUtils;
import org.tencompetence.generated.model.PortfolioEntry;

//import com.liferay.portal.PortalException;
//import com.liferay.portal.SystemException;

import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.exception.PortalException;

import com.liferay.portal.model.BaseModel;
import com.liferay.portal.service.ServiceContext;

/**
 * The class SubscribeService which defines all subscribe/unsubscribe related functionality..
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @version $Revision: 1.2 $, $Date: 2009/11/09 15:57:53 $
 */
public final class SubscribeService {
	private SubscribeService() {
		// private constructor
	}

	/**
	 * Checks if a user is subscribed to an entity.
	 * 
	 * @param userId
	 *            the user id
	 * @param className
	 *            the className
	 * @param entityId
	 *            the entity id
	 * @param ctx
	 *            the ctx
	 * @param value
	 *            the value
	 * 
	 * @return PortfolioEntry
	 * 
	 * @throws SystemException
	 *             the system exception
	 */
	public static PortfolioEntry getSubscribedFromEntity(final long userId, final String className,
			final String entityId, final String value, final ServiceContext ctx) throws SystemException {
		final PortfolioEntry subscribed = PortfolioService.getLatestEntryByUserEventType(userId, className, entityId,
				EventType.SUBSCRIBE, value);
		final PortfolioEntry unsubscribed = PortfolioService.getLatestEntryByUserEventType(userId, className, entityId,
				EventType.UNSUBSCRIBE, value);

		if (subscribed == null) {
			// If there is no subscribed event found, this user is not subscribed.
			return null;
		} else if (subscribed != null && unsubscribed == null) {
			// If there is a subscribed event found and there is no unsubscribe event found,
			// this user is subscribed.
			return subscribed;
		} else if (subscribed != null && unsubscribed != null) {
			// We have found a subscribe and an unsubscribed event, now check if
			// the subscribe is the most recent.
			if (subscribed.getJournalDate().after(unsubscribed.getJournalDate())) {
				return subscribed;
			}
		}

		return null;
	}

	/**
	 * Checks if a user is subscribed to an entity.
	 * 
	 * @param userId
	 *            the user id
	 * @param entity
	 *            the entity
	 * @param ctx
	 *            the ctx
	 * 
	 * @return PortfolioEntry
	 * 
	 * @throws SystemException
	 *             the system exception
	 */
	public static PortfolioEntry getSubscribedFromEntity(final long userId, final BaseModel<?> entity,
			final ServiceContext ctx) throws SystemException {
		return getSubscribedFromEntity(userId, EntityUtils.getInterfaceFromEntity(entity), entity.getPrimaryKeyObj()
				.toString(), null, ctx);
	}
	
	/**
	 * Checks if a user is subscribed to an entity.
	 * 
	 * @param userId
	 *            the user id
	 * @param entity
	 *            the entity
	 * @param ctx
	 *            the ctx
	 * 
	 * @return true, if user is subscribed to entity
	 * 
	 * @throws SystemException
	 *             the system exception
	 */
	public static boolean isSubscribedToEntity(final long userId, final BaseModel<?> entity, final ServiceContext ctx)
			throws SystemException {
		return getSubscribedFromEntity(userId, entity, ctx) != null;
	}

	/**
	 * Checks if a user is subscribed to an entity.
	 * 
	 * @param userId
	 *            the user id
	 * @param className
	 *            the class name
	 * @param entityId
	 *            the entity id
	 * @param ctx
	 *            the ctx
	 * @param value
	 *            the value
	 * 
	 * @return true, if user is subscribed to entity
	 * 
	 * @throws SystemException
	 *             the system exception
	 */
	public static boolean isSubscribedToEntity(final long userId, final String className, final String entityId,
			final String value, final ServiceContext ctx) throws SystemException {
		return getSubscribedFromEntity(userId, className, entityId, value, ctx) != null;
	}

	/**
	 * Subscribe to BaseModel entity (convenience method).
	 * 
	 * @param userId
	 *            the user id
	 * @param entity
	 *            the entity
	 * @param ctx
	 *            the ctx
	 * @param value
	 *            the value
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static void subscribeToEntity(final long userId, final BaseModel<?> entity, final String value,
			final ServiceContext ctx) throws SystemException, PortalException {
		subscribeToEntity(userId, entity, SubscribeType.ONCE, value, ctx);
	}

	/**
	 * Subscribe to entity (convenience method).
	 * 
	 * @param userId
	 *            the user id
	 * @param className
	 *            the className
	 * @param ctx
	 *            the ctx
	 * @param value
	 *            the value
	 * @param entityId
	 *            the entity id
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static void subscribeToEntity(final long userId, final String className, final String entityId,
			final String value, final ServiceContext ctx) throws SystemException, PortalException {
		subscribeToEntity(userId, className, entityId, SubscribeType.ONCE, value, ctx);
	}

	/**
	 * Subscribe to BaseModel entity.
	 * 
	 * @param userId
	 *            the user id
	 * @param entity
	 *            the entity
	 * @param ctx
	 *            the ctx
	 * @param subscribeType
	 *            the subscribe type
	 * @param value
	 *            the value
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static void subscribeToEntity(final long userId, final BaseModel<?> entity,
			final SubscribeType subscribeType, final String value, final ServiceContext ctx) throws SystemException,
			PortalException {
		subscribeToEntity(userId, EntityUtils.getInterfaceFromEntity(entity), entity.getPrimaryKeyObj().toString(),
				subscribeType, value, ctx);
	}

	/**
	 * Subscribe to an entity.
	 * 
	 * @param userId
	 *            the user id
	 * @param subscribeType
	 *            the subscribe type
	 * @param className
	 *            the class name
	 * @param entityId
	 *            the entity id
	 * @param value
	 *            the value
	 * @param ctx
	 *            the ctx
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static void subscribeToEntity(final long userId, final String className, final String entityId,
			final SubscribeType subscribeType, final String value, final ServiceContext ctx) throws SystemException,
			PortalException {
		final boolean isSubscribed = isSubscribedToEntity(userId, className, entityId, value, ctx);

		// MULTIPLE means that user can subscribe multiple times to this entity.
		// ONCE means that the user can only subscribe to this entity if not subscribed.
		if (subscribeType == SubscribeType.MULTIPLE || (subscribeType == SubscribeType.ONCE && !isSubscribed)) {
			PortfolioService.addJournalEntry(userId, className, entityId, EventType.SUBSCRIBE, value, ctx);
		}
	}

	/**
	 * Unsubscribe from BaseModel entity.
	 * 
	 * @param userId
	 *            the user id
	 * @param entity
	 *            the entity
	 * @param ctx
	 *            the ctx
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static void unsubscribeFromEntity(final long userId, final BaseModel<?> entity, final ServiceContext ctx)
			throws SystemException, PortalException {

		// Only possible to unsubscribe from an entity if user was first subscribed to it.
		if (isSubscribedToEntity(userId, entity, ctx)) {
			PortfolioService.addJournalEntry(entity, EventType.UNSUBSCRIBE, ctx);
		}
	}

	/**
	 * Unsubscribe from entity.
	 * 
	 * @param userId
	 *            the user id
	 * @param className
	 *            the class name
	 * @param entityId
	 *            the entity id
	 * @param ctx
	 *            the ctx
	 * @param value
	 *            the value
	 * 
	 * @throws SystemException
	 *             the system exception
	 * @throws PortalException
	 *             the portal exception
	 */
	public static void unsubscribeFromEntity(final long userId, final String className, final String entityId,
			final String value, final ServiceContext ctx) throws SystemException, PortalException {

		// Only possible to unsubscribe from an entity if user was first subscribed to it.
		if (isSubscribedToEntity(userId, className, entityId, value, ctx)) {
			PortfolioService.addJournalEntry(userId, className, entityId, EventType.UNSUBSCRIBE, null, ctx);
		}
	}
}
