/*
 * CCSI, a Learning Design service integration component
 * Copyright (C) 2005 Harrie Martens and Hubert Vogten
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program (/license.txt); if not,
 * write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *     Contact information:
 *     Open University of the Netherlands
 *     Valkenburgerweg 177 Heerlen
 *     PO Box 2960 6401 DL Heerlen
 *     e-mail: hubert.vogten@ou.nl or
 *             harrie.martens@ou.nl
 *
 *
 * Open Universiteit Nederland, hereby disclaims all copyright interest
 * in the program CCSI written by
 * Harrie Martens and Hubert Vogten
 *
 * prof.dr. Rob Koper,
 * director of learning technologies research and development
 *
 */
package adapters;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;
import java.util.Hashtable;
import java.util.List;
import java.util.Stack;
import java.util.Vector;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.helpers.DefaultHandler;

import apis.InvalidQTIv2Exception;
import apis.QTIv21test;
import apis.QTIv2item;
import apis.UserTestCase;

// deleteme
/**
 * This class parses any XML file containing IMS QTI v2 assessment items. These assessment items will be
 * rendered/transformed by APIS. All non IMS QTI v2 elements will be passed trough without modifications.
 * 
 * @author Harrie Martens
 * @author Hubert Vogten
 * @version $Revision: 1.1 $, $Date: 2009/11/25 13:44:45 $
 */
public class QtiParser extends DefaultHandler implements ContentHandler {

	/**
	 * ITEM_PREFIX may NOT contain underscores because APIS can't deal with them
	 */
	private final static String ITEMPREFIX = "item-";
	// UPF - Pau
	private final static String TESTPREFIX = "test-";
	/**
	 * This is the key that will be used to pass the original identifier of the item. This key is used as attribute name
	 * in the cc:qti tag as well as the key in the name value pair in the postStr. The attribute is needed because we
	 * will geneate our own item ids.
	 */
	private final static String ITEM_IDENTIFIER = "itemIdentifier";
	private final static String TEST_IDENTIFIER = "identifier";

	/**
	 * The level is incremented on each startElement and decremented on each endElement. This is used to output the
	 * indentation.
	 */
	private int level;

	/**
	 * It is possible that characters(...) will be called for adjacent pieces of text. Often this is due to entities in
	 * the text. This will allow for these to be joined back together.
	 */
	private boolean lookingForChars;

	/**
	 * One of the difficulties in SAX parsing is that it does not retain state. Even for an empty tag, it calls
	 * startElement and endElement. This allows for making empty elements to have the empty tag notation: &lt;tag/&gt;.
	 */
	private boolean pendingEndTag;

	private String sourceUrl = null;

	private int itemCounter = 0;

	private String namespaces = "";

	private Stack writerStack = new Stack();

	private String assessmentId = null;

	private String assessmentTestId = null;

	private String postStr = "";

	// private ApisAdapter owner;

	private String userId = null;

	private int runId;

	private String uolID;

	private String language;

	private String testTitle = "";

	boolean hasResponse = false;

	private String testDir = "";

	private String itemBase = "";

	private QTIv21test theTest;

	private Hashtable xml_container;

	/**
	 * This constructor should be called when processing the response of a user on a QTI assessment item.
	 * 
	 * @param userId
	 *            the id of the user for which the QTI items are rendered
	 * @param runId
	 *            the id of the run for which the QTI items are rendered
	 * @param owner
	 *            the ApisAdapter responsible for creating this object
	 * @param theWriter
	 *            a Writer in which the output of this parsing process will be written
	 * @param sourceUrl
	 *            the URL refererencing to the resource that is being parsed
	 * @param postStr
	 *            an URL encoded string containing key value pairs of the reponses generated by the user. This is
	 *            typically a response to a web form.
	 */
	public QtiParser(String userId, int runId, Writer theWriter, String sourceUrl, String postStr, String language,
			Hashtable xml_container_) {
		// reset the counter used to generate unique id for the assesement items
		itemCounter = 0;
		writerStack.push(theWriter == null ? new StringWriter() : theWriter);

		this.language = language;
		// this.owner = owner;
		this.sourceUrl = sourceUrl;
		this.postStr = postStr;
		this.userId = userId;
		this.runId = runId;
		this.xml_container = xml_container_;

		System.out.println("[***] Qti Parser Instantiated");

	}

	public QtiParser(String userId, int runId, Writer theWriter, String sourceUrl, String postStr, String language) {
		// reset the counter used to generate unique id for the assesement items
		itemCounter = 0;
		writerStack.push(theWriter == null ? new StringWriter() : theWriter);

		this.language = language;
		// this.owner = owner;
		this.sourceUrl = sourceUrl;
		this.postStr = postStr;
		this.userId = userId;
		this.runId = runId;
		this.xml_container = new Hashtable();

		System.out.println("[***] Qti Parser Instantiated");
	}

	/**
	 * This constructor should be called when initially parsing a resource containing assessment items.
	 * 
	 * @param userId
	 *            the id of the user for which the QTI items are rendered
	 * @param runId
	 *            the id of the run for which the QTI items are rendered
	 * @param owner
	 *            the ApisAdapter responsible for creating this object
	 * @param theWriter
	 *            a Writer in which the output of this parsing process will be written
	 * @param sourceUrl
	 *            the URL refererencing to the resource that is being parsed
	 */
	public QtiParser(String userId, int runId, Writer theWriter, String sourceUrl) {
		this(userId, runId, theWriter, sourceUrl, null, null, null);
	}

	/**
	 * Returns the String representation of the parsing results from the active writer. The active writer is the writer
	 * which is currently on top of the stack of writers.
	 * 
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return ((Stack) writerStack.peek()).toString();
	}

	/**
	 * Deal with the prefixes defined in the current context by storing them for later reference.
	 * 
	 * @see org.xml.sax.ContentHandler#startPrefixMapping(java.lang.String, java.lang.String)
	 */
	public void startPrefixMapping(String prefix, String uri) {
		namespaces += " xmlns" + ("".equals(prefix) ? "" : (":" + prefix)) + "=\"" + uri + "\"";
	}

	/**
	 * Process the start of a new XML element. QTI elements that need special threatment are filtered out. All other
	 * elements are directly copied to the active writer.
	 * 
	 * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String,
	 *      org.xml.sax.Attributes)
	 */
	public void startElement(String uri, String localname, String qname, Attributes attrs) {

		if (level > 0) {
			handlePending();
		}
		String templateValue = null;
		if (isQtiItem(uri, localname)) {
			writerStack.push(new StringWriter());

			// store assessment id for later usage
			assessmentId = attrs.getValue("identifier");
		}
		// UPF - Pau
		else if (isQtiTest(uri, localname)) {
			writerStack.push(new StringWriter());
			// store assessment id for later usage
			assessmentTestId = attrs.getValue("identifier");
			testTitle = attrs.getValue("title");
		} else if (isResponseProcessing(uri, localname)) {
			// check if a templateLocation was used. templateLocation attributes
			// will
			// be transformed to template attributes
			// by resolving the relative references into absolute ones.
			String templateLocation = attrs.getValue("templateLocation");
			if (templateLocation != null) {
				// convert the templateLocation with to the fully qualified
				// equivalent
				templateValue = escape(getBaseUrl(sourceUrl) + templateLocation);
			}
		}

		write(getTagStart());
		write(qname);

		// write the namespace declaration now
		write(namespaces);
		namespaces = "";

		if (isResponseProcessing(uri, localname) && templateValue != null) {
			// a template value attribute was used, we replace the
			// templateLocation
			// attribute
			// with a fully qualified template attribute
			for (int i = 0; i < attrs.getLength(); i++) {
				write(' ');

				if (attrs.getLocalName(i).equals("templateLocation")) {
					write("template='" + templateValue + "'");
				} else if (attrs.getLocalName(i).equals("template")) {
					// ignore the template attribute
				} else {
					write(("".equals(attrs.getQName(i)) ? attrs.getLocalName(i) : attrs.getQName(i)));
					write("='");
					write(escape(attrs.getValue(i)));
					write('\'');
				}
			}
		} else {
			for (int i = 0; i < attrs.getLength(); i++) {
				write(' ');
				write(("".equals(attrs.getQName(i)) ? attrs.getLocalName(i) : attrs.getQName(i)));
				write("='");
				write(escape(attrs.getValue(i)));
				write('\'');
			}
		}

		pendingEndTag = true;
		level++;
	}

	/**
	 * Processes the end of an XML element. For all non QTI assessment items this means that the closing tag is written
	 * to the writer. In case of an QTI item the rendering of that item will be done here.
	 * 
	 * @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
	 */
	public void endElement(String uri, String localname, String qname) {
		System.out.println("Empieza el festival...");
		level--;
		if (pendingEndTag) {

			write(getEmptyTagEnd());
		} else {
			write(getEndTagStart());
			write(qname);
			write(getTagEnd());
		}

		pendingEndTag = false;
		lookingForChars = false;

		if (isQtiItem(uri, localname)) {
			// now start replacing the assesementItem tag with the APIS content
			System.out.println("Estamos en el primer bucle...");
			
			Writer qtiItem = (Writer) writerStack.pop();
			String identifier = assessmentId;
			QTIv2item theItem = new QTIv2item(qtiItem.toString(), identifier);

			theItem.setMediaBase(getBaseUrl(sourceUrl));
			theItem.setAppletBase("/applets/");
			write("<cc:qti-item xmlns:cc=\"http://coppercore.org/xsd/gc\" url=\"" + sourceUrl + "\"" + " "
					+ ITEM_IDENTIFIER + "=\"" + assessmentId + "\">");

			hasResponse = this.hasItemResponse(postStr);
			theItem.setItemsToProcess(postStr);
			// theItem.setProcessedItems(processedItems);
			if (hasResponse) {
				theItem.readFormData(postStr);
			}

			/*
			 * UPF - Javi write(theItem.renderInfo());
			 */

			try {
				write(theItem.renderItem());
			} catch (InvalidQTIv2Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			if (hasResponse) {
				// process all QTI outcomes
				for (int i = 0; i < theItem.outcomeCount(); i++) {
					String propId = theItem.getOutcomeIdentifier(i);
					// because IMS-LD can't handle array type construction we
					// only process
					// the first value
					// of an outcome assuming a singular value response.
					String value = theItem.getOutcomeValue(i, 0);

					if (value != null) {
						// DELETED ADAPTER owner.postEvent(userId, runId,
						// assessmentId + "." + propId, value);
					}
				}
			}

			write("</cc:qti-item>");
		}
		// UPF - Pau
		else if (isQtiTest(uri, localname)) {
			// endelement in a whole test
			System.out.println("Estamos en el segundo bucle...");
			
			Writer qtiTest = (Writer) writerStack.pop();
			// String tidentifier = TESTPREFIX + testCounter++;
			itemBase = getBaseUrl(sourceUrl);
			String tmp = itemBase.substring(0, itemBase.length() - 1);

			uolID = tmp.substring(tmp.lastIndexOf("/") + 1, tmp.length());
			theTest = new QTIv21test(qtiTest.toString(), "a", "/", itemBase, testDir, "/", sourceUrl, assessmentTestId,
					testTitle, runId, userId, uolID, language, xml_container);

			setDBProperties(theTest);

			theTest.startParse();

			write("<imsld:qti-test xmlns:imsld=\"http://www.imsglobal.org/xsd/imsqti_v2p1\" url=\"" + sourceUrl
					+ "\" identifier=\"" + assessmentTestId + "\">");
			// ***********************************************************************************************/
			// This is where the bit that is similar to use of APIS in a service
			// etc. starts
			if (qtiTest != null) {
				// # At this point, the item needs appropriate rendering, which
				// means finding it's
				// # state, which requires sending any associated POST data to
				// the engine.

				theTest.getUser().updateLoginUserTestCase();

				theTest.getUser().checkUserTestCaseNotDone();

				hasResponse = this.hasItemResponse(postStr); // if the form
				// contains
				// itemIdentifier=ITEMID
				// the item will
				// be stored in
				// processed
				// items
				// the item struct will also get the user answer
				System.out.println("Selected User Name: " + theTest.getUser().getName());
				System.out.println(theTest.toString());
				
				try
				{
					theTest.setProcessedItems(theTest.getUser()); // user processed
					// items is
					// given to all
					// items so they
					// can set
					// themselve as the
					// db stored
					// state

					// get user context (post)

					theTest.updatePage(postStr);
				}
				catch(Exception ex)
				{
					System.out.println("El problema est aqu: " + ex.getMessage());
					ex.printStackTrace();
				}


				if (hasResponse) {
					theTest.setItemsToProcess(postStr); // if the form contains
					// itemIdentifier=ITEMID
					// the "to process" flag
					// in the item which contains this
					// ITEMID will be true
					theTest.readFormData(postStr); // form string is processed
					// and divided and given to
					// items which will store
					// (in hash formdata) its
					// parts and set their
					// "unckecked data" flag
				}
				try {
					write(theTest.render()); // render
				} catch (Exception e) {
					e.printStackTrace();
					write("Error - " + e);
				}
				theTest.getUser().setItems(); // store changes in DB

				if (theTest.getOutcomeVars() != null && theTest.showOutcome) {
					theTest.getUser().setUserTestCaseDone();
					handleCompletedTest(theTest);
				}
			}
			write("</imsld:qti-test>");
		}
	}

	/**
	 * Streams all encountered characters directly to the active writer.
	 * 
	 * @see org.xml.sax.ContentHandler#characters(char[], int, int)
	 */
	public void characters(char[] chars, int start, int length) {
		if (!lookingForChars) {
			handlePending();
		}

		String s = new String(chars, start, length);
		write(s);
		lookingForChars = true;
	}

	/**
	 * Streams all ignorable white spaces directly to the active writer.
	 * 
	 * @see org.xml.sax.ContentHandler#ignorableWhitespace(char[], int, int)
	 */
	public void ignorableWhitespace(char[] chars, int start, int length) {
		characters(chars, start, length);
	}

	/**
	 * Streams all processing istructions to the active writer.
	 * 
	 * @see org.xml.sax.ContentHandler#processingInstruction(java.lang.String, java.lang.String)
	 */
	public void processingInstruction(String target, String data) {
		handlePending();

		write(getPIStart());
		write(target);
		write(' ');
		write(data);
		write(getPIEnd());

	}

	private String getTagStart() {
		return "<";
	}

	private String getTagEnd() {
		return ">";
	}

	private String getEmptyTagEnd() {
		return "/>";
	}

	private String getEndTagStart() {
		return "</";
	}

	private String getPIStart() {
		return "<!";
	}

	private String getPIEnd() {
		return "!>";
	}

	private void write(String obj) {
		try {
			((Writer) writerStack.peek()).write(obj);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private void write(char obj) {
		try {
			((Writer) writerStack.peek()).write(obj);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private void handlePending() {
		if (pendingEndTag) {
			pendingEndTag = false;

			write(getTagEnd());

		}

		lookingForChars = false;
	}

	private String escape(String s) {
		if (s == null) {
			return s;
		}
		int len = s.length();
		StringBuffer str = new StringBuffer(len);

		for (int i = 0; i < len; i++) {
			char ch = s.charAt(i);
			switch (ch) {
			case '<':
				str.append("&lt;");
				break;

			case '>':
				str.append("&gt;");
				break;

			case '&':
				str.append("&amp;");
				break;

			case '"':
				str.append("&quot;");
				break;

			default:
				str.append(ch);
			}
		}

		return str.toString();
	}

	private boolean isQtiItem(String uri, String localname) {
		return ("http://www.imsglobal.org/xsd/imsqti_v2p0".equalsIgnoreCase(uri) && "assessmentItem"
				.equalsIgnoreCase(localname));
	}

	private boolean isQtiTest(String uri, String localname) {
		return ("http://www.imsglobal.org/xsd/imsqti_v2p1".equalsIgnoreCase(uri) && "assessmentTest"
				.equalsIgnoreCase(localname));
	}

	private boolean isResponseProcessing(String uri, String localname) {
		return ("http://www.imsglobal.org/xsd/imsqti_v2p0".equalsIgnoreCase(uri) && "responseProcessing"
				.equalsIgnoreCase(localname));
	}

	private boolean isTestResponseProcessing(String uri, String localname) {
		return ("http://www.imsglobal.org/xsd/imsqti_v2p1".equalsIgnoreCase(uri) && "responseProcessing"
				.equalsIgnoreCase(localname));
	}

	private boolean hasItemResponse(String postString) {
		boolean result = false;
		String item_id = "";
		Vector ans_set = new Vector();
		boolean open = false;

		System.out.println(postString);
		if (postString != null) {
			String params[] = postString.split("&");
			System.out.println(postString);
			for (int i = 0; i < params.length; i++) {
				if (params[i].startsWith(ITEM_IDENTIFIER)) {
					System.out.println(params[i]);
					item_id = params[i].split("=")[1].trim();
					result = true;
				}

				if (params[i].startsWith("apisa_RESPONSE") || params[i].startsWith("apisa_RD")) {
					System.out.println(params[i]);
					ans_set.add(params[i].split("=")[1].trim());
					result = true;
				}

				if (params[i].startsWith("open_ans")) {
					open = true;
				}
			}

			if (open) {// thoose answers that are unknow when instantiating an
				// item from the xml are the open ones such as the
				// content answered in a fillInTheBlank interactions
				theTest.getUser().setCloseAnswer((String) ans_set.firstElement(), (String) ans_set.firstElement());
			}

			if (item_id != "" && !ans_set.isEmpty()) {

				theTest.getUser().addToProcessItem(item_id);

				theTest.getUser().setItemCloseAnswer(item_id, ans_set);
			}
		}

		// System.out.println("if this is false, problem at test qtiPrser 373 "+
		// result);
		return result;
	}

	// UPF - Pau - method to load processedItems vector for a qtiv2.1 test
	private String getBaseUrl(String url) {
		int index = url.lastIndexOf("/");
		if (index != -1) {
			return url.substring(0, index + 1);
		}
		return url;
	}

	public UserTestCase getUser() {
		return theTest.getUser();
	}

	protected void handleCompletedTest(QTIv21test theTest_) {
		List result = theTest_.getPlainOutcomeVars();
	}

	public void setDBProperties(QTIv21test theTest_) {
		try {
			theTest_.linkTheTestWithDb("apis-adapter.properties");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}// you can overwrite this function with

	/*
	 * public void setDBPorperties(){ try{ theTest.linkTheTestWithDb(String driverClass,String databaseUrl,String
	 * databaseUsername,String databasePassword); }catch(Exception e){ e.printStackTrace(); }
	 * 
	 * }
	 */
}
