package apis;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.sql.SQLException;
import java.util.Vector;

import nb_utils.NBEasyXML;

/**
 * Class: assessmentItemRef
 *
 * Description :  Items are incorporated into the test by reference and not by direct aggregation.
 *
 * Attributes :	* identifier 
 * 					The identifier to be used to refer to this item within the test. The identifier 
 * 					must be unique within the test. Note that this identifier need not have any meaning 
 * 					outside the test. In particular it is not required to be unique in the context of 
 * 					any catalog, or be represented in the item's meta-data. The syntax of this identifier
 * 					is more restrictive than that of the identifier attribute of the assessmentItem itself.
 * 				* href
 *  				The uri used to refer to the item's file (e.g., elsewhere in the same content package).
 *  				There is no requirement that this be unique. A test may refer to the same item multiple 
 *  				times within a test. Note however that each reference must have a unique identifier.
 *  			* category
 *  				Items can optionally be assigned to one or more categories. Categories are used to allow 
 *  				custom sets of item outcomes to be aggregated during outcomes processing.
 *   
 * Contains :	* variableMapping [*]
 * 					Variable mappings are used to alter the name of an item outcome for the purposes of this
 * 				 	test.
 * 				* weight [*]
 * 				 Allow custom values to be defined for scaling an item's outcomes. 
 * 
 * */

public class QTIv21assessmentItemRef  extends QTIv21sectionPart
{
	
	
	//-- Attributes specification
    protected String itemIdentifier;			// identifier [1]: identifier
    protected String href;						// href [1]: uri 
    protected String category;					// category [*]: identifier 
    
    protected Vector varMappingPart;			// Vector where variableMappings are stored, if contained
    
    protected String provaRed;
    
    // -- constructor 
    public QTIv21assessmentItemRef(QTIv21test owner)
    {
    	super(owner);
    	itemIdentifier = "";
    	href = "";
    	category = "";
    }
    
    public QTIv21assessmentItemRef(String nItemXML)
    {
    	super(nItemXML);
    }    
    
    // -- Parse : assessmentItemRef parsing. 
    public int parse(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {
       	System.out.println("Assessment Item xml:" + itemXML.ExportBlock(idx));
       	provaRed=itemXML.ExportBlock(idx);
       	
    	super.parse(itemXML,idx);
        
    	//href + item loading
    	QTIv2item anItem = new QTIv2item(theTest);
   	 	String itemName;
   	 	String itemURL = itemXML.getAttribute(idx, "href"); 
   	 	int p = itemURL.lastIndexOf("/");
          if (p == -1)
              itemName = itemURL;
          else
              itemName = itemURL.substring(p + 1);
          
        setIdentifier(itemName);
        try {
			loadItem(itemName,anItem);
		} catch (URISyntaxException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

       	
       	//category 
        if(itemXML.getAttribute(idx,"category")!=null ) 
       	{
       		category= itemXML.getAttribute(idx,"category");  
       	}
       	
        // -- CONTAINS
        
        int cidx = itemXML.findChildElement(idx);
        while(cidx != -1)
        { 
            int id = QTIv2define.identify(itemXML.getName(cidx));    
            switch(id)
            {
             case QTIv2define.QTIv21_weight: 
            	 QTIv21weight weight = new QTIv21weight();
            	 cidx = weight.parse(itemXML,cidx);
            	 anItem.itemWeights.add(weight);
            	 break;                 
             case QTIv2define.QTIv21_variableMapping:
            	 QTIv21variableMapping aVarMap= new QTIv21variableMapping();
            	 cidx = aVarMap.parse(itemXML, cidx);
            	 anItem.itemVarMapping.add(aVarMap);
            	 break;
              default:    	 
            }	
        } 
       
        anItem.doVarMappings();
      	return itemXML.findNextPeer(idx);
    }
    
    // -- loadItem : loads and parses the referenced item
    protected boolean loadItem(String itemName,QTIv2item anItem) throws InvalidQTIv2Exception, URISyntaxException
    {

    	String test_name = theTest.file.substring(theTest.file.lastIndexOf('/')+1);
    	if (test_name.lastIndexOf('.') != -1) {
		   test_name = test_name.substring(0,test_name.lastIndexOf('.'));
		 }
    	String url = theTest.getItemBase()+ test_name + "/"+ itemName;
    	NBEasyXML itemRefXML; 
    	
        DAOAssessmentItem daoitem = new DAOAssessmentItem();
        try{
        	String anse = this.itemIdentifier.substring(0, this.itemIdentifier.length()-4);
        //	System.out.println();
        System.out.println("anse: " +anse);
        
        	daoitem.setItem(anse,"scoreable",this.theTest.getUser());
        }catch(SQLException e){
        		System.out.println("Error de BBDDs DAOAssessmentItem");
        		e.printStackTrace();
        		
        }
    	
    	try{
    		System.out.println("URL: "+ url + ", " + "XMLContainer"+theTest.getXmlContainer());
    		System.out.println("The Test: "+ theTest.toString());
    		itemRefXML = new NBEasyXML(new URI(url),theTest.getXmlContainer());
    		//Tiene que ser slo el mensaje XML
    		//itemRefXML = theTest.testXML; //new NBEasyXML(provaRed); //
    		
    	}catch(Exception e){
    	  	url = theTest.getItemBase()+ theTest.identifier+"/"+ itemName;
    		itemRefXML = new NBEasyXML(new URI(url));
    	}
        
        // find start of response processing
        int position = itemRefXML.root();
        if(position == -1)
            return false;
        // parse Item
        if(position != -1)
        {
            anItem.parse(itemRefXML, position);
            if(parts==null)
            	parts=new Vector();
            parts.add(anItem);         
        }
        else
            return false;

        return true;
    }
    
    public void setProcessedItems(UserTestCase processedItems)
    {
    	for(int i=0;i<parts.size();i++)
    		((QTIv2item)parts.get(i)).setProcessedItems(processedItems);
    }
    
    public QTIv2variableDeclaration getOutcomeValue(String identifier) throws InvalidQTIv2Exception
    {
    	return ((QTIv2item)parts.get(0)).getOutcomeValue(identifier);
    }

    public void setItemsToProcess(String postStr)
    {    	
    	for(int i=0;i < parts.size();i++)
    		((QTIv2item)parts.get(i)).setItemsToProcess(postStr);
    }

    public String getResultInfo()
    {    	
    	String resultInfo="";
    	for(int i=0;i < parts.size();i++)
    		resultInfo+=((QTIv2item)parts.get(i)).resultInfo;
    	
    	return resultInfo;
    }

    //  -- addFormDataValue : updates data from all items in this testPart marked to be processed.
    public void addFormDataValue(String name, String value)
    {
    	for(int i=0;i < parts.size();i++)
    	{
    		((QTIv2item)parts.get(i)).addFormDataValue(name,value);	
    	}
    }

    public boolean categoryCheck(String include, String exclude)
    {
    	String [] catParts = this.category.split(" ");
    	
    	for(int i = 0 ; i < catParts.length; i++)
    	{
    		if(exclude != null)
    			if (catParts[i].equals(exclude))
    				return false;    			
    	}
    	for(int i = 0 ; i < catParts.length; i++)
    	{
    		if(include!= null)
    			if (catParts[i].equals(include))
    				return true;    			
    	}
    	return false;
    }
    public void  findRelatedVar(String varIdentifier,String includeCategory,String excludeCategory, Vector relatedVarDec  )
    {
    	for(int i=0;i < parts.size();i++)
    	{
    		if(includeCategory !=null || excludeCategory !=null)
    		{	
    			if(categoryCheck(includeCategory, excludeCategory))
    			((QTIv2item)parts.get(i)).findRelatedVar(varIdentifier,relatedVarDec);			
    		}
    		else 
    			((QTIv2item)parts.get(i)).findRelatedVar(varIdentifier,relatedVarDec);			    			
    	}
    }
    
    public boolean findItem(String itemIdentifier, Vector item)
    { 
 	   for(int i=0;i < parts.size();i++)
 		   if(((QTIv2item)parts.get(i)).findItem(itemIdentifier,item))		
 			   return true;
 	   return false;
    }
    
    public int getNumberPresented() 
    {
    	return 1;
    }
    
    public int getNumberCorrect()
    {
    	if(((QTIv2item)parts.get(0)).getIsCorrect())		
    		return 1;
    	else return 0;
    }
    
    public int getNumberIncorrect()
    {
    	if(((QTIv2item)parts.get(0)).getIsIncorrect())		
    		return 1;
    	else return 0;
    }
    
    // render : renders items assessmentSections and rubricBlocks within this assessmentSection
    public String render() throws InvalidQTIv2Exception
    {	
        String out = "";
        for(int n=0; n < parts.size(); n++)
        		out+=((QTIv2item)parts.get(n)).render();
        
        return out;
    }
    
    // -- Getters & Setters
    public String getCategory() {
		return category;
	}
	public void setCategory(String category) {
		this.category = category;
	}
	public String getHref() {
		return href;
	}
	public void setHref(String href) {
		this.href = href;
	}
	public String getIdentifier() {
		return itemIdentifier;
	}
	public void setIdentifier(String identifier) {
		this.itemIdentifier = identifier;
	}
	
	public int answeredItems()
	 {
         if (((QTIv2item)parts.get(0)).isAnswered())
        	 return 1;
        return 0; 
	 }
}
