package apis;

import nb_utils.*;


import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.*;
/**
 * Class : AssessmentSection[1..*] (sectionPart)
 * Description : Items contained in each testPart are arranged into sections and subsections
 * Attributes:  * identifier
 * 					The identifier of the section must be unique within the test.
 *				* title 
 *					The title of the section is intended to enable the section to be selected in
 *					situations where the contents of the section are not available, for example
 *					when a candidate is browsing a test.
 * 			    * visible
 * 					A visible section is one that is identifiable by the candidate.  The visibility 
 * 					of each section is determined solely by the value of its own visible attribute.
 * 				* keepTogether 
 * 					An invisible section with a parent that is subject to shuffling can specify 
 * 					whether or not its children, which will appear to the candidate as if they were 
 * 					part of the parent, are shuffled as a block or mixed up with the other children 
 * 					of the parent section.
 * Contains :	* selection [0..1]
 * 					The rules used to select which children of the section are to be used for each 
 * 					instance of the test.
 * 				* ordering [0..1]
 * 					The rules used to determine the order in which the children of the section are to be
 *					arranged for each instance of the test. Each child section has its own selection and 
 *					ordering rules followed before those of its parent. A child section may shuffle the 
 *					order of its own children while still requiring that they are kept together when 
 *					shuffling the parent section.
 * 				* sectionPart ( assessmentSection  | assessmentItemRef ) [1..*]
 * 				* rubricBlock [0..1]				
 * 					Section rubric is presented to the candidate with each item contained (directly or
 * 					indirectly) by the section. As sections are nestable the rubric presented for each item 
 * 					is the concatenation of the rubric blocks from the top-most section down to the item's
 * 					immediately enclosing section.
**/	

public class QTIv21assessmentSection extends QTIv21sectionPart 
{
    protected QTIv2structuredBodyPart prompt;
    
    //  Specification Attributes 
    protected String identifier;		// identifier [1]: identifier 
    protected String title;				// title [1]: string
    protected boolean visible;			// visible [1]: boolean 
    protected boolean keepTogether;		// keepTogether [0..1]: boolean = true 
    protected Vector togetherIdentifier;    
    //  Contains
    protected QTIv21ordering anOrder ;
    protected QTIv21selection aSelection;
    

    
    
    //  Other

    protected Vector together;			// invisible assessmentSections that should be shuffled Together
    boolean shuffled;					// for shuffling just one time
    boolean printedResults;
    
    // -- constructor
    public QTIv21assessmentSection(QTIv21test owner)
    {
        super(owner);
        identifier = "";
        title = " untitled";
        visible = false;
        keepTogether = true;
        togetherIdentifier = new Vector();
        shuffled = false;
        printedResults=false;
        

        
    }
    
    // -- parse : assessmentSection elements' parsing 
    public int parse(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {    
    	together = new Vector();
    	parts = new Vector();
    	
    	System.out.println("Assessment Section xml:" + itemXML.ExportBlock(idx));
    	
    	super.parse(itemXML,idx);
    	
        //ATTRIBUTES
        if(itemXML.getAttribute(idx,"identifier")!= null)
        {
        	identifier=itemXML.getAttribute(idx,"identifier");
        }else 
        	throw new InvalidQTIv2Exception("identifer needed");

        if(itemXML.getAttribute(idx,"title")!= null)
        {
        	title=itemXML.getAttribute(idx,"title");
        }else 
        	throw new InvalidQTIv2Exception("title needed");

        if(itemXML.getAttribute(idx,"visible")!= null )
        {
        	if(itemXML.getAttribute(idx,"visible").equals("true"))
        	visible=true;
        }else 
        	throw new InvalidQTIv2Exception("visible attribute needed");
        
        if(itemXML.getAttribute(idx,"keepTogether")!= null && itemXML.getAttribute(idx,"keepTogether").equals("false") )
        {
        	keepTogether = false;
        }
        
        // CONTAINS
        int cidx = itemXML.findChildElement(idx);
        while(cidx != -1)
        { 
            int id = QTIv2define.identify(itemXML.getName(cidx));    
            switch(id)
            {
             case QTIv2define.QTIv21_assessmentItemRef: 
            	 QTIv21assessmentItemRef aiRef = new QTIv21assessmentItemRef(theTest);        	  
            	 cidx = aiRef.parse(itemXML,cidx);
                 if(parts == null)
                       parts = new Vector();
               	 parts.add(aiRef); 
             break;                 
             case QTIv2define.QTIv21_selection: 
            	 aSelection = new QTIv21selection(theTest);
            	 cidx = aSelection.parse(itemXML, cidx);
            	 break;
             case QTIv2define.QTIv21_ordering:
            	 anOrder = new QTIv21ordering(theTest);
            	 cidx = anOrder.parse(itemXML, cidx);
            	 break;
             case QTIv2define.QTIv21_assessmentSection:
            	 QTIv21assessmentSection as = new QTIv21assessmentSection(theTest);
             	 cidx =as.parse(itemXML,cidx );
             	 
             	 if(as.visible)
             		 parts.add(as);
             	 else
             	 {//invisible
             		 if (as.keepTogether) 
             		 {
             			 parts.add(as.identifier);	//for identifying keeptogether elements
                		 together.add(as);			// saving invisible assessmentSection
             		 }
             		 for(int k=0;k<as.parts.size();k++)	// adding to parts assementSection item's
	             	 {
	             		 parts.add(as.parts.get(k));
	             	 }
	            }
             	
	              break;
             case QTIv2define.QTIv2_rubricBlock:
            	 QTIv2rubricBlock rb = new QTIv2rubricBlock(theTest);
                 cidx = rb.parse(itemXML,cidx);                
                 if(parts == null)
                     parts = new Vector();
             	 parts.add(rb);
                 break;
             case QTIv2define.QTIv21_timeLimits:
             	 QTIv21timeLimits tl = new QTIv21timeLimits(this);
             	 cidx =tl.parse(itemXML,cidx );
                 break;
             default:    
            }	
        }
        
        return itemXML.findNextPeer(idx);
    }
    
    public void setProcessedItems(UserTestCase processedItems)
    {
    	
    	if(parts != null)
    	for(int i=0;i<parts.size();i++)
    		if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock") &&
    		   !(parts.get(i).getClass().getCanonicalName()).equals("java.lang.String"))	
    			((QTIv21sectionPart)parts.get(i)).setProcessedItems(processedItems);   
 
    	getNumberResponded();
    }
    public int answeredItems()
    {
    	int res = 0;
    	if (parts!= null)
    		for(int i=0;i < parts.size();i++)
    		if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock") &&
    	       !(parts.get(i).getClass().getCanonicalName()).equals("java.lang.String"))
    			res +=((QTIv21sectionPart)parts.get(i)).answeredItems();
    	return res;
    }
    
    // -- setItemsToProcess : marks items with unchecked data, items with data that shoul be checked
    public void setItemsToProcess(String postStr)
    { 
    	if (parts!= null)
    	for(int i=0;i < parts.size();i++)
    		if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock") &&
    	       !(parts.get(i).getClass().getCanonicalName()).equals("java.lang.String"))   
    			((QTIv21sectionPart)parts.get(i)).setItemsToProcess(postStr);
    }
   
    // -- addFormDataValue : updates data from all items in this testPart marked to be processed.
    public void addFormDataValue(String name, String value)
    {
    	
    	for(int i=0;i < parts.size();i++)
    	{
    		if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock") &&
    	   	   !(parts.get(i).getClass().getCanonicalName()).equals("java.lang.String"))
    			((QTIv21sectionPart)parts.get(i)).addFormDataValue(name,value);		
    	}
    }
 
    public void  findRelatedVar(String varIdentifier,String sectionIdentifier,String includeCategory,
    		String excludeCategory, Vector relatedVarDec )
    {
    	if(parts!=null)
    	for(int i=0;i < parts.size();i++)
    	{
    		if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock") &&
    	       !(parts.get(i).getClass().getCanonicalName()).equals("java.lang.String"))
    	   	{
    			if(sectionIdentifier != null)
    			{
    			if (this.identifier.equals(sectionIdentifier))
    					((QTIv21sectionPart)parts.get(i)).findRelatedVar(varIdentifier,includeCategory, excludeCategory,relatedVarDec);	
    				
    			}else
    				((QTIv21sectionPart)parts.get(i)).findRelatedVar(varIdentifier,includeCategory, excludeCategory,relatedVarDec);	

    		}
    	}
    }
    
    public boolean findItem(String itemIdentifier, Vector item)
    { 
 	   for(int i=0;i < parts.size();i++)
 	   {
 		  if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock") && 
 			 !(parts.get(i).getClass().getCanonicalName()).equals("java.lang.String"))
 	    	  if(((QTIv21sectionPart)parts.get(i)).findItem(itemIdentifier,item))
 	    		  return true;
 	   }
 	   return false;
    }
    // GET NUMBERS
    public int getNumberCorrect()
    {
    	int numberCorrect= 0;

    	for(int i=0;i < parts.size();i++)
    	{
    		if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock") &&
               !(parts.get(i).getClass().getCanonicalName()).equals("java.lang.String"))
    	    		numberCorrect+=((QTIv21sectionPart)parts.get(i)).getNumberCorrect();	
    	}

    	return numberCorrect;
    }
    public int getNumberIncorrect()
    {
    	int numberIncorrect= 0;
    	
    	for(int i=0;i < parts.size();i++)
    	{
    		if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock") &&
    		   !(parts.get(i).getClass().getCanonicalName()).equals("java.lang.String"))
    	    		numberIncorrect+=((QTIv21sectionPart)parts.get(i)).getNumberIncorrect();	
    	}
   	
    	return numberIncorrect;
    }
    
    public int getNumberPresented() 
    {
    	int numberPresented = 0;
    	for(int i=0;i < parts.size();i++)
    	{
    		if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock") &&
    	       !(parts.get(i).getClass().getCanonicalName()).equals("java.lang.String"))
    	    		numberPresented+=((QTIv21sectionPart)parts.get(i)).getNumberPresented();	
    	}
    	return numberPresented;
    }
    
    public int getNumberSelected() 
    {
    	int numberSelected= 0;
    	for(int i=0;i < parts.size();i++)
    	{
    		if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock") &&
    		   !(parts.get(i).getClass().getCanonicalName()).equals("java.lang.String"))	
    		{
    			if((parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv21assessmentSection"))
    			{	
    				if(((QTIv21assessmentSection)parts.get(i)).visible)
    					numberSelected+=((QTIv21sectionPart)parts.get(i)).getNumberPresented();	
    			}else
    				numberSelected+=((QTIv21sectionPart)parts.get(i)).getNumberPresented();	

    		}
    	}
    	return numberSelected;
    }
    
    public int getNumberResponded()
    { 
    	int numberResponded  = 0;
    	if (parts!= null)
    	for(int i=0;i < parts.size();i++)
    	{
    		if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock") &&
    	       !(parts.get(i).getClass().getCanonicalName()).equals("java.lang.String"))
   	    			numberResponded +=((QTIv21sectionPart)parts.get(i)).answeredItems();
    	}
    	return numberResponded;
    }
    
    public String getResultInfo()
    {
    	String rInfo="";
		if (parts!= null)
	    	for(int i=0;i < parts.size();i++)
	    	{
	    		if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock") &&
	    	       !(parts.get(i).getClass().getCanonicalName()).equals("java.lang.String"))
	   	    			rInfo+=((QTIv21sectionPart)parts.get(i)).getResultInfo();
	    	}

		return rInfo;
    }
    public void checkSelected()
    {
    	if (aSelection.select == getNumberResponded())	
    	{		
			endSection = true;
			finishingReason = "<h4> number of selectable questions accomplished </h4>";   	
    	}    	
    }
     public void checkKeepTogether()
     {
     	for(int n=0; n < parts.size(); n++)
      	{
      		if(!(parts.get(n).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock"))
  			 {
      			if((parts.get(n).getClass().getCanonicalName()).equals("apis.QTIv21assessmentSection"))
      			{	
      				if(!((QTIv21assessmentSection)parts.get(n)).keepTogether)
      				{
      				
      				}
      			}
  			 }
        }
     }
     public void checkOrdering()
     {
     	Vector toShuffle = new Vector();
 		//QTIv2structuredBodyPart shuffle  = new QTIv2structuredBodyPart(null); //for shuffling
 		int pos=0;
 		int posTogether=0;
 		
 		//variables to shuffle
       	for(int n=0; n < parts.size(); n++)
       	{
       		if(!(parts.get(n).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock"))
          	{	
       			//if there is any identifier of a keepTogether section
       			if((parts.get(n).getClass().getCanonicalName()).equals("java.lang.String")) 
       			{	
       				toShuffle.add(together.get(posTogether)); // adding the invisible section identifier
       				n += ((QTIv21assessmentSection)together.get(posTogether)).parts.size();		// avancem la n tantes posicions com ocupessin els assessmentsItems guardats per separat
       				posTogether++;
       			}
       			//fixed elements shouldn't be shuffled 
       			else if(!((QTIv21sectionPart)parts.get(n)).fixed)
       			{
       				toShuffle.add(((QTIv21sectionPart)parts.get(n)));
       			}
       		}
       	}
       	
       	//shuffle.doShuffle(toShuffle, new Random());	
       	
 		for(int i=0;i<parts.size();i++)
        	{
 			if(!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock") &&
 			   !(parts.get(i).getClass().getCanonicalName()).equals("java.lang.String"))   		
 			 {			
 				if(!(((QTIv21sectionPart)parts.get(i)).fixed))
 				 {
 					if((toShuffle.get(pos).getClass().getCanonicalName()).equals("apis.QTIv21assessmentSection"))
 					{	
 						QTIv21assessmentSection asTogether =(QTIv21assessmentSection)toShuffle.get(pos);
 						
 						//setting the elements keeped together during the suffling to the final ordered vector
 						if(asTogether.keepTogether && !asTogether.visible)
 						{
 							for(int j=0;j<asTogether.parts.size();j++)
 							{  	
 								if (!(parts.get(i).getClass().getCanonicalName()).equals("java.lang.String") &&
                                    (!(parts.get(i).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock")))
 								{
 									parts.setElementAt((QTIv21sectionPart)asTogether.parts.get(j),i);
 									i++;
 								}
 								else // if there is a rubrickBlock or a String we don't want to overwrite them, so just try next i with the same j
 								{	i++;
 									j--;
 								}
 							}
 							i--;
 							pos++;
 						}else
 						{		
 							parts.setElementAt(toShuffle.get(pos),i);
 							pos++;
 						}
 					}
 					else
 					{				
 						parts.setElementAt((QTIv21sectionPart)toShuffle.get(pos),i);
 						pos++;
 					}
 				 }
 			}
        	}
 		shuffled = true;
     }
         // render : renders items assessmentSections and rubricBlocks within this assessmentSection
    public String render() throws InvalidQTIv2Exception
    {	
	
        String out = "";
        
        if (parts!= null )
	    {
        	if( anOrder!= null && anOrder.shuffle)
        	{
        		if(!shuffled)
        			checkOrdering();
        	}
        	
        	page_num = theTest.theUser.getPage();
        	int total_pages = parts.size()/page_set_size + 1 ;
        	
        	if((total_pages-1)*page_set_size == parts.size())
        		total_pages--;
        	
        	if(page_num<0)
        		page_num = 0;
        	init_part = page_num * page_set_size;
        	fin_part = init_part + page_set_size;
        	if( fin_part > parts.size() )
        		fin_part = parts.size();
        	
            //out+="<table  width='90%' align='center' border ='0'><tr align = 'center'><td> <h3>  ASSESSMENT SECTION</h3></td></tr>";	      
           //out+="<table><tr><td> <h3></h3></td></tr>";	      
            
            out+="<tr><td>";		
 	       	if(aSelection!= null)
 	       	{
         	   out +=aSelection.render(); 
 	           checkSelected();
 	       	}
 	       	out+="</td></tr>";
            
 	        out+="<tr><td>";		
 	        
	       	for(int n=init_part; n < fin_part; n++)
	        {   
	       		if((parts.get(n).getClass().getCanonicalName()).equals("java.lang.String")) ; // do render nothing	
	       		else if((parts.get(n).getClass().getCanonicalName()).equals("apis.QTIv2rubricBlock"))
	                out+=((QTIv2rubricBlock)parts.get(n)).render();
	        	else 
	        		out +=((QTIv21sectionPart)parts.get(n)).render();
	        }	
	        out+="</td></tr>";
           out +="</table>";
           
           
        
           if(total_pages > 1){
        	   
        	   if( fin_part != parts.size() ){
        		   out+="<form/><form method=\"post\" action=\"QTIPost.do?url="+theTest.getFile()+"&amp;identifier="+theTest.getId()+"\">"; 
                   
        		   out+="<form method='GET'>";
	        	   out+="<input type='hidden' name='file' value='" + theTest.getFile()+"'>";
	        	   out+="</input>";
		           out+="<input type='hidden' name='identifier' value='" +
		                         theTest.getId() + "'>";
		           out+="</input>";
				   out+="<input type='hidden' name='PAGE' value='"+(page_num+1)+"'/>";
				   out+="<input type=\'submit\' value=\'Next\'>" +  "</input></form></form>";	
        	   }
        	   
        	   if( init_part != 0 ){
        		   out+="<form/><form method=\"post\" action=\"QTIPost.do?url="+theTest.getFile()+"&amp;identifier="+theTest.getId()+"\">"; 
        		   out+="<form method='GET'>";
	        	   out+="<input type='hidden' name='file' value='" + theTest.getFile()+"'>";
	        	   out+="</input>";
		           out+="<input type='hidden' name='identifier' value='" +
		                         theTest.getId() + "'>";
		           out+="</input>";
				   out+="<input type='hidden' name='PAGE' value='"+(page_num-1)+"'/>";
				   out+="<input type=\'submit\' value=\'Previous\'>" +  "</input></form></form>";	
        	   }
           }
			
	    }
        return out;
    }
}
