package apis;

import nb_utils.*;


/**
 *  Class : itemSessionControl
 *  
 *  Description :
 * 	When items are referenced as part of a test, the test may impose constraints on how many attempts, and which states
 *	are allowed. These constraints can be specified for individual items, for whole sections or for an entire testPart. 
 *	By default, a setting at testPart level affects all items in that part unless the setting is overridden at the 
 *	assessmentSection level or ultimately at the individual assessmentItemRef. The defaults given below are used only in 
 *	the absence of any applicable constraint.
 *
 *  Attributes :* maxAttempts 
 *		  			For non-adaptive items, maxAttempts controls the maximum number of attempts allowed in the given test 
 *  				context. Normally this is 1 as the scoring rules for non-adaptive items are the same for each attempt. 
 *  				A value of 0 indicates no limit. If it is unspecified it is treated as 1 for non-adaptive items. For 
 *  				adaptive items, the value of maxAttempts is ignored as the number of attempts is limited by the value 
 *  				of the completionStatus outcome variable.
 *				* showFeedback
 *					This constraint affects the visibility of feedback after the end of the last attempt. If it is false 
 *					then feedback is not shown. This includes both Modal Feedback and Integrated Feedback even if the 
 *					candidate has access to the review state. The default is false.
 * 				* allowReview [0..1]: boolean 
 * 					This constraint also applies only after the end of the last attempt. If set to true the itemSession is 
 * 					allowed to enter the review state during which the candidate can review the itemBody along with the 
 * 					responses they gave, but cannot update or resubmit them. If set to false the candidate can not review 
 * 					the itemBody or their responses once they have submitted their last attempt. The default is true. 
 * 				* showSolution 
 * 					This constraint controls whether or not the system may provide the candidate with a way of entering the 
 * 					solution state. The default is false.
 * 				* allowComment 
 * 					This constraint controls whether or not the candidate is allowed to provide a comment on the item during 
 * 					the session. 
 * */

public class QTIv21itemSessionControl
{   
    protected QTIv21test theTest;
    
    //--  Attributes
    protected String allowComment;				// allowComment [0..1]: string  
    protected boolean allowReview;				// allowReview [0..1]: boolean 
    protected int maxAttempts; 	 	 			// maxAttempts [0..1]: integer 
    protected boolean showFeedBack;				// showFeedback [0..1]: boolean 
    protected boolean showSolution;				// showSolution [0..1]: boolean 
    
    // -- constructor
    public QTIv21itemSessionControl(QTIv21test owner)
    {
    	theTest = owner;
    	allowComment = " ";
    	allowReview = false;
    	maxAttempts = 1;	
    	showFeedBack = false;
    	showSolution = false;
    }
    
    // -- parse : itemSessionControl content parsing
    public int parse(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
    {     
    	//ATTRIBUTES
        if (itemXML.getAttribute(idx,"allowComment")!=null)
        {
        	allowComment=itemXML.getAttribute(idx,"allowComment");
        }
        
        if ((itemXML.getAttribute(idx,"allowReview")!=null) && (itemXML.getAttribute(idx,"allowReview").equals("true")))
        {
        	allowReview=true;
        }
        if(itemXML.getAttribute(idx,"maxAttempts")!=null)
        {
        	maxAttempts = Integer.parseInt(itemXML.getAttribute(idx,"maxAttempts"));
        }
        if ((itemXML.getAttribute(idx,"showFeedBack")!=null) && (itemXML.getAttribute(idx,"showFeedBack").equals("true")))
        {
        	showFeedBack=true;
        }
        if ((itemXML.getAttribute(idx,"showSolution")!=null) && (itemXML.getAttribute(idx,"showSolution").equals("true")))
        {
        	showSolution=true;
        }
          	
        return itemXML.findNextPeer(idx);
    }
}
