package apis;

/**
 * Class : outcomeDeclaration (variableDeclaration)
 * 
 * Declaration : Outcome variables are declared by outcome declarations. Their value is set either from a 
 * 	default given in the declaration itself or by a responseRule during responseProcessing. 
 * 
 * Attributes : * interpretation 
 * 			 	A human interpretation of the variable's value.
 * 				* longInterpretation 
 * 				An optional link to an extended interpretation of the outcome variable's value.
 * 				* normalMaximum 
 * 				The normalMaximum attribute optionally defines the maximum magnitude of numeric outcome 
 * 				variables, it must be a positive value.
 * 				* normalMinimum 
 * 				The normalMinimum attribute optionally defines the minimum value of numeric outcome variables, 
 * 				it may be negative.
 * 				* masteryValue 
 * 				The masteryValue attribute optionally defines a value for numeric outcome variables above which 
 * 				the aspect being measured is considered to have been mastered by the candidate.
 * */

import java.io.Serializable;
import java.util.Vector;

import nb_utils.NBEasyXML;

public class QTIv2outcomeDeclaration extends QTIv2variableDeclaration implements Serializable 
{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	//-- Attributes specification
	protected String interpretation;			// interpretation [0..1]: string
	protected String longInterpretation;		// longInterpretation [0..1]: uri 
	protected float normalMaximum;				// normalMaximum [0..1]: float
	protected float normalMinimum;				// normalMinimum [0..1]: float
	protected float masteryValue;				// masteryValue [0..1]: float 
	protected Vector interTables;
	
	// -- contructor
	public QTIv2outcomeDeclaration () throws InvalidQTIv2Exception
	{
		identifier = "";
		cardinality = -1;
		//UPF - Javi
		cardinality = CARD_single;
		
		type = -1;
		interpretation = "";
		longInterpretation = "";
		normalMaximum = -1;
		normalMinimum = -1;
		masteryValue = -1;
	}
	
	// -- constructor
	public QTIv2outcomeDeclaration (String nIdentifier, int nCardinality,int baseType) throws InvalidQTIv2Exception
	{
		identifier = nIdentifier;
		 
		cardinality = nCardinality;
		if(cardinality == -1) // not defined in the XML, default to single
			cardinality = CARD_single;
		type = baseType;
		interpretation = "";
		longInterpretation = "";
		normalMaximum = -1;
		normalMinimum = -1;
		masteryValue = -1;
	}
	 
	// -- parse
	public int parse(int position, int decType, NBEasyXML itemXML) throws InvalidQTIv2Exception
	{
	    int type = QTIv2variableDeclaration.identifyType(itemXML.getAttribute(position, "baseType"));
	    String identifier = itemXML.getAttribute(position, "identifier");
	    int cardinality = QTIv2variableDeclaration.identifyCardinality(itemXML.getAttribute(position, "cardinality"));
	        
	    switch (type)
	    {
	        case QTIv2variableDeclaration.VTYPE_integer:
	        case QTIv2variableDeclaration.VTYPE_identifier:
	        case QTIv2variableDeclaration.VTYPE_float:
	        case QTIv2variableDeclaration.VTYPE_boolean:
	        case QTIv2variableDeclaration.VTYPE_string:
	        case QTIv2variableDeclaration.VTYPE_point:
	        case QTIv2variableDeclaration.VTYPE_uri:
	        case QTIv2variableDeclaration.VTYPE_pair:
	        case QTIv2variableDeclaration.VTYPE_directedPair:
	        case QTIv2variableDeclaration.VTYPE_duration:
	        case QTIv2variableDeclaration.VTYPE_outcomeValue:
	     	    this.identifier = identifier;
	   		 	this.cardinality = cardinality;
	   		 	this.type = type;
	   		 	parseVariableContents(position,itemXML);
	            break;
	         
	        case QTIv2variableDeclaration.VTYPE_file:
	        	throw new InvalidQTIv2Exception("Problem with variable "+identifier+" - Not yet implemented this variable's baseType");
	        default:
	            throw new InvalidQTIv2Exception("Unrecognised variable baseType");
       }
       return itemXML.findNextPeer(position);
	}
	 
	// -- parseVariableContents : parses the content of outcome variables (the defaultsValue)   
	protected void parseVariableContents(int position, NBEasyXML itemXML) throws InvalidQTIv2Exception
	{	   	
	   int p2 = itemXML.findChildElement(position);
	   while(p2 != -1)
	   {
	        int id = QTIv2define.identify(itemXML.getName(p2));
	        switch (id)
	        {
	        	case QTIv2define.QTIv2_defaultValue:
	        	   assert(null == value);
	       	       int p = itemXML.findChildElement(p2);
	       	       while (p != -1)
	       	       {
	       	    	   String strval = itemXML.getContent(p);
	       	    	   strval = strval.trim();
	       	           setValue(strval);
	       	           p = itemXML.findNextPeer(p);
	       	       }   
	               break;
	            case QTIv2define.QTIv21_interpolationTable:
	            	 int p3 = itemXML.findChildElement(p2);
            	   if (interTables == null){
            		   interTables = new Vector();
            	   }	            	   
            	   while (p3 != -1)
            	   {
	            	   QTIv21interpolationTable interTable = new QTIv21interpolationTable();
	            	   p3 = interTable.parse(p3, itemXML);
	            	   interTables.add(interTable);
            	   }
            	   break; 
	        }           
	        p2 = itemXML.findNextPeer(p2);
	   }
	   if(p2==-1) 	// default Value if not specified = 0
		   setValue(0);
	   }
}