package apis;

/**
 * Class : responseDeclaration (variableDeclaration)
 * 
 * Declaration : Response variables are declared by response declarations and bound to interactions in the itemBody. 
 * 
 * Contains :	* correctResponse [0..1]
 * 				 A response declaration may assign an optional correctResponse. This value may indicate the only possible 
 * 				value of the response variable to be considered correct or merely just a correct value. For responses that 
 * 				are being measured against a more complex scale than correct/incorrect this value should be set to the 
 * 				(or an) optimal value. Finally, for responses for which no such optimal value is defined the correctResponse 
 * 				must be omitted. If a delivery system supports the display of a solution then it should display the correct 
 * 				values of responses (where defined) to the candidate. When correct values are displayed they must be clearly 
 * 				distinguished from the candidate's own responses (which may be hidden completely if necessary).
 * 		 		* mapping [0..1]
 * 				The mapping provides a mapping from the set of base values to a set of numeric values for the purposes of 
 * 				response processing. 
 * 				* areaMapping [0..1]
 * 				The areaMapping, which may only be present in declarations of variables with baseType point, provides an 
 * 				alternative form of mapping which tests against areas of the coordinate space instead of mapping single values 
 * 				(i.e., single points).
 * */

import java.util.ArrayList;
import java.util.Vector;

import nb_utils.NBEasyXML;

public class QTIv2responseDeclaration extends QTIv2variableDeclaration 
{
	
	protected Vector correct;
	protected ArrayList mapKeys;
	protected ArrayList mapValues;
	protected Float mapDefault;

	
	 public QTIv2responseDeclaration () throws InvalidQTIv2Exception
	  {
		 identifier = "";
		 cardinality = -1;
		 //UPF - Javi
		 cardinality = CARD_single;
		 
		 type = -1;     
	  }
	 
	 public QTIv2responseDeclaration (String nIdentifier, int nCardinality,int baseType) throws InvalidQTIv2Exception
	  {
		 identifier = nIdentifier;
		 cardinality = nCardinality;
		 if(cardinality == -1) // not defined in the XML, default to single
	           cardinality = CARD_single;
		 type = baseType;
	  }	
	
	 public QTIv2variableDeclaration mapResponse() throws InvalidQTIv2Exception
	    {
	     	if(null==mapKeys)
	            throw new InvalidQTIv2Exception("mapResponse on variable "+identifier+" with no mapping defined");
	        QTIv2variableDeclaration out = new QTIv2responseDeclaration(null, CARD_single, VTYPE_float);
	        out.setValue(mapDefault.floatValue());
	        if((null==value)||(0==value.size()))
	            return out;
	        
	        int n=0;
	        float valor=0;
	        for(int i=0;i<value.size();i++)
	        {
	        	n=0;
	        	while(n<mapKeys.size())
	        	{	if(((valuePart)(value.get(i))).compare((valuePart)mapKeys.get(n)))
	        		{
	        			valor += ((Float) (mapValues.get(n))).floatValue();
	        			break;
	        		} 
	            n++;
	        	}
	        }
	        out.setValue(valor);
	        return out;	        
	    }
	 
	   public QTIv2responseDeclaration getCorrect()
	    {
		   if (correct == null)
	            return null;
		   String correcte = "";
	        try
	        {
	            QTIv2responseDeclaration tmp = new QTIv2responseDeclaration(identifier, cardinality, type);
	            	
	            for (int n = 0; n < correct.size(); n++){
	            	correcte += (String)correct.get(n);
	            	tmp.setValue(correcte);
	            }
	            return tmp;
	        }
	        catch (InvalidQTIv2Exception ex)
	        {
	        }
	        return null;
	    }
	   
	
	   //-- keeps in a vector the correct response/s
	   public void parseCorrectResponse(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
	    {
	        if(correct==null)
	        {
	            correct = new Vector();
	            int p = itemXML.findChildElement(idx);

	            while (p != -1)
	            {         
	                correct.add(itemXML.getContent(p));
	                p = itemXML.findNextPeer(p);            
	            }
	        }
	        else
	            throw new InvalidQTIv2Exception("More than one correctResponse in variable.");        
	    }
	   protected void parseVariableContents( int position,NBEasyXML itemXML) throws InvalidQTIv2Exception
	   {
	   	
	       int p2 = itemXML.findChildElement(position);
	       while(p2 != -1)
	       {
	           int id = QTIv2define.identify(itemXML.getName(p2));
	           switch (id)
	           {
	               case QTIv2define.QTIv2_defaultValue:
	                   parseDefaultValue(itemXML, p2);
	                   break;
	               case QTIv2define.QTIv2_correctResponse:
	            	   parseCorrectResponse(itemXML, p2);
	                   break;
	               case QTIv2define.QTIv2_mapping:
	                   parseMapping(itemXML, p2);
	                   break;
	               case QTIv2define.QTIv2_areaMapping:
	                   //throw new InvalidQTIv2Exception("Not yet implemented areaMapping"); // A IMPLEMENTAR!!!
	                   break;
	           }
	           p2 = itemXML.findNextPeer(p2);
	       }

	   }

	   
	    protected void parseMapping(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
	     {
	    	 if(null==mapKeys)
	         {
	             mapKeys = new ArrayList();
	             mapValues = new ArrayList();
	             mapDefault = new Float(itemXML.getAttribute(idx,"defaultValue"));
	             int p = itemXML.findChildElement(idx);
	             while (p != -1)
	             {
	                 if(!itemXML.getName(p).equals("mapEntry"))
	                     throw new InvalidQTIv2Exception("Element other than mapEntry inside mapping");
	                 String kstr = itemXML.getAttribute(p,"mapKey");
	                 valuePart k = valuePart.newValuePart(kstr,type);
	                 Float v = new Float(itemXML.getAttribute(p,"mappedValue"));
	                 mapKeys.add(k);
	                 mapValues.add(v);
	                 p = itemXML.findNextPeer(p);
	             }
	         }
	         else
	             throw new InvalidQTIv2Exception("More than one mapping in variable.");
	     }

	    public int parse(int position, int decType, NBEasyXML itemXML) throws InvalidQTIv2Exception
		 {
		       int type = QTIv2variableDeclaration.identifyType(itemXML.getAttribute(position, "baseType"));
		       String identifier = itemXML.getAttribute(position, "identifier");
		       int cardinality = QTIv2variableDeclaration.identifyCardinality(itemXML.getAttribute(position, "cardinality"));
		        
		       switch (type)
		       {
		           case QTIv2variableDeclaration.VTYPE_integer:
		           case QTIv2variableDeclaration.VTYPE_identifier:
		           case QTIv2variableDeclaration.VTYPE_float:
		           case QTIv2variableDeclaration.VTYPE_boolean:
		           case QTIv2variableDeclaration.VTYPE_string:
		           case QTIv2variableDeclaration.VTYPE_point:
		           case QTIv2variableDeclaration.VTYPE_uri:
		           case QTIv2variableDeclaration.VTYPE_pair:
		           case QTIv2variableDeclaration.VTYPE_directedPair:
		           case QTIv2variableDeclaration.VTYPE_duration:
		        	    this.identifier = identifier;
		      		 	this.cardinality = cardinality;
		      		 	this.type = type;
		      		 	parseVariableContents(position,itemXML);
		                break;
		           case QTIv2variableDeclaration.VTYPE_file:
		         throw new InvalidQTIv2Exception("Problem with variable "+identifier+" - Not yet implemented this variable's baseType");
		           default:
		               throw new InvalidQTIv2Exception("Unrecognised variable baseType");

		       }
		       return itemXML.findNextPeer(position);
		   }
	    public void parseDefaultValue(NBEasyXML itemXML, int idx) throws InvalidQTIv2Exception
	    {
	        // default value is set when the xml is parsed, so always set first.
	        // POST data must be parsed after the XML, so that the variables exist.
	        assert(null == value);
	        int p = itemXML.findChildElement(idx);
	        while (p != -1)
	        {
	            String strval = itemXML.getContent(p);
	            strval = strval.trim();
	            setValue(strval);
	            p = itemXML.findNextPeer(p);
	        }
	        
	       
	    }

}