package apis;

/**
 * Class : variableDeclaration
 * 
 * Description : Item variables are declared by variable declarations. All variables must be 
 *  declared except for the built-in session variables referred to below which are declared implicitly. 
 *  The purpose of the declaration is to associate an identifier with the variable and to identify the
 *  runtime type of the variable's value. 
 *  
 * Attributes :  * identifier [1]: identifier 
 * 				The identifiers of the built-in session variables are reserved. They are completionStatus 
 * 				and duration. All item variables declared in an item share the same namespace. Different 
 * 				items have different namespaces.
 * 				* cardinality [1]: cardinality 
 * 				Each variable is either single valued or multi-valued. Multi-valued variables are referred 
 * 				to as containers and come in ordered, unordered and record types. See cardinality for more 
 * 				information.
 * 				* baseType [0..1]: baseType 
 * 				The value space from which the variable's value can be drawn (or in the case of containers, 
 * 				from which the individual values are drawn) is identified with a baseType. The baseType selects 
 * 				one of a small set of predefined types that are considered to have atomic values within the 
 * 				runtime data model. Variables with record cardinality have no base-type.
 * 
 * Contains : 	* defaultValue [0..1]
 * 				 An optional default value for the variable. The point at which a variable is set to its default 
 * 				value varies depending on the type of item variable.
 **/

import java.io.Serializable;
import java.util.Vector;

public class QTIv2variableDeclaration implements Serializable
{
    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	public static final int CARD_single = 0;
    public static final int CARD_multiple = 1;
    public static final int CARD_ordered = 2;
    public static final int CARD_record = 3;

    
    public static final int VTYPE_boolean = 0;
    public static final int VTYPE_directedPair = 1;
    public static final int VTYPE_duration = 2;
    public static final int VTYPE_file = 3;
    public static final int VTYPE_float = 4;
    public static final int VTYPE_identifier = 5;
    public static final int VTYPE_integer = 6;
    public static final int VTYPE_pair = 7;
    public static final int VTYPE_point = 8;
    public static final int VTYPE_string = 9;
    public static final int VTYPE_uri = 10;
    public static final int VTYPE_outcomeValue = 11;

    protected String identifier;
    protected int cardinality;
    protected int type;
    protected Vector value; // always a vector, so single cardinality has one entry
    protected Vector relatedItemsVarDec; 
    
    //UPF - Javi
    public QTIv2variableDeclaration () throws InvalidQTIv2Exception
	  {
		 identifier = "";
		 cardinality = -1;
		 //UPF - Javi
		 cardinality = CARD_single;
		 
		 type = -1;     
		 relatedItemsVarDec = new Vector();
	  }
   
    public QTIv2variableDeclaration (String nIdentifier, int nCardinality,int baseType) throws InvalidQTIv2Exception
	  {
		 identifier = nIdentifier;
		 cardinality = nCardinality;
		 if(cardinality == -1) // not defined in the XML, default to single
	           cardinality = CARD_single;
		 type = baseType;
		 relatedItemsVarDec = new Vector();
	  }	
    
    public static int identifyType(String token)
    {
        if((token == null)||(token.length()<3))
            return -1;
        String search[] =
              {
              "boolean",
              "directedPair",
              "duration",
              "file",
              "float",
              "identifier",
              "integer",
              "pair",
              "point",
              "string",
              "uri",
              "outcomeValue"
        };
        int idx = 0;
       

        while (idx < search.length)
        {
            if (token.equals(search[idx]))
                return idx;
            idx++;
        }
        return -1;
    }

    public static int identifyCardinality(String token)
    {
        String search[] =
              {
              "single",
              "multiple",
              "ordered",
        };
        int idx = 0;

        while (idx < search.length)
        {
            if (token.equals(search[idx]))
                return idx;
            idx++;
        }
        return -1;
    }

    public void setValue(String nVal) throws InvalidQTIv2Exception
     {
    	 if(value == null)
             value = new Vector();
         if(cardinality == CARD_single)
             clear();
         switch (type)
         {
             case VTYPE_boolean:
                 value.add(new booleanPart(nVal));
                 break;
             case VTYPE_directedPair:
                 value.add(new directedPairPart(nVal));
                 break;
             case VTYPE_duration:
                 value.add(new durationPart(nVal));
                 break;
             case VTYPE_file:
                 value.add(new filePart(nVal));
                 break;
             case VTYPE_float:
                 value.add(new floatPart(nVal));
                 break;
             case VTYPE_identifier:
                 value.add(new identifierPart(nVal));
                 break;
             case VTYPE_integer:
                 value.add(new integerPart(nVal));
                 break;
             case VTYPE_pair:
                 value.add(new pairPart(nVal));
                 break;
             case VTYPE_point:
                 value.add(new pointPart(nVal));
                 break;
             case VTYPE_string:
                 value.add(new stringPart(nVal));
                 break;
             case VTYPE_uri:
                 value.add(new uriPart(nVal));
                 break;
         }
     }

     public void setValue(float nVal) throws InvalidQTIv2Exception
      {
          if(value == null)
              value = new Vector();
          if(cardinality == CARD_single)
              clear();
          switch (type)
          {
              case VTYPE_float:
                  value.add(new floatPart(nVal));
                  break;
              case VTYPE_integer:
                  Float nVal2 = new Float(nVal);
                  value.add(new integerPart(nVal2.intValue()));
                  break;
              case VTYPE_string:
                  value.add(new stringPart(String.valueOf(nVal)));
                  break;     
              case VTYPE_identifier:
                  value.add(new identifierPart(String.valueOf(nVal)));
                  break;     
              default:
                  throw new InvalidQTIv2Exception("Attempted to set inappropriate variable to a numeric value");
          }
      }

     public void setValue(QTIv2variableDeclaration nVal) throws InvalidQTIv2Exception
     {
         assert(type == nVal.getType());
         assert(nVal != this);
         if (value == null)
             value = new Vector();
         if (cardinality == CARD_single)
             clear();
        for(int n=0; n < nVal.getSize(); n++){
            value.add(nVal.value.get(n));
        }
    }


     public void setValue(boolean nVal) throws InvalidQTIv2Exception
      {
          if(value == null)
              value = new Vector();
          if (cardinality == CARD_single)
              clear();
          if (type == VTYPE_boolean)
              value.add(new booleanPart(nVal));
      }

      public void setValue(valuePart nVal) throws InvalidQTIv2Exception
       {
           if(value == null)
               value = new Vector();
           if (cardinality == CARD_single)
               clear();
           value.add(nVal);
       }

    void clear()
    {
        value.clear();
    }

    public boolean containsIdentifier(String id)
    {
        if(type != VTYPE_identifier)
            return false;
        if(value != null)
        {
            int n = 0;
            while (n < value.size())
            {
                if (((identifierPart)value.get(n)).getValue().equals(id))
                    return true;
                n++;
            }
        }
        return false;
    }
     	
	public Vector getRelatedItemsVarDec() {
		return relatedItemsVarDec;
	}
	
	public void setRelatedItemsVarDec(Vector relatedItemsVarDec) {
		this.relatedItemsVarDec = relatedItemsVarDec;
	}
	
    public boolean isTrue() throws InvalidQTIv2Exception
    {
        if(cardinality != CARD_single)
            throw new InvalidQTIv2Exception("isTrue called on variable with non-single cardinality.");
        if(value.size()==0)
            return false;
        switch (type)
        {
            case QTIv2variableDeclaration.VTYPE_boolean:
                return ((booleanPart) value.get(0)).getValue();
            case QTIv2variableDeclaration.VTYPE_float:
                if (((floatPart) value.get(0)).getValue() == 0)
                    return false;
                else
                    return true;
            case QTIv2variableDeclaration.VTYPE_integer:
                if (((integerPart) value.get(0)).getValue() == 0)
                    return false;
                else
                    return true;
            case QTIv2variableDeclaration.VTYPE_point:
                if ((((pointPart) value.get(0)).getValue()).equals("null"))
                    return false;
                else
                    return true;
       
            default:
                throw new InvalidQTIv2Exception(
                      "isTrue called on variable of inappropriate type.");
        }
    }
    	//-- per vars amb cardinalitat simple
    public Float toFloat() throws InvalidQTIv2Exception
    {    	
        if(cardinality != CARD_single)
            throw new InvalidQTIv2Exception("toFloat called on variable with non-single cardinality.");
        if(value.size()==0)
            return null;
        switch (type)
        {
            case QTIv2variableDeclaration.VTYPE_float:
                return new Float(((floatPart) value.get(0)).getValue());
            case QTIv2variableDeclaration.VTYPE_integer:
                return new Float(((integerPart) value.get(0)).getValue());
            default:
                throw new InvalidQTIv2Exception("toFloat called on variable of inappropriate type.");
        }
    }

    
    public QTIv2variableDeclaration getDuplicate()
	   {
	       QTIv2variableDeclaration dup = null;
	       
	       try
	       {
	           if(value != null)
	           {
	               dup = new QTIv2variableDeclaration(identifier, cardinality, type);
	               for (int n = 0; n < value.size(); n++)
	                   dup.setValue(((valuePart) value.get(n)).duplicate());
	               dup.setRelatedItemsVarDec(relatedItemsVarDec);
	           }
	       }
	       catch (Exception ex)
	       {
	           ex.printStackTrace();
	       }
	       return dup;
	   }

    public static QTIv2variableDeclaration newBooleanVar(boolean isTrue)
    {
        try
        {
            QTIv2responseDeclaration out = new QTIv2responseDeclaration(null, CARD_single, VTYPE_boolean);
            out.setValue(isTrue);
            return out;
        }
        catch (InvalidQTIv2Exception ex)
        {
            System.out.println("Arrrggghhh - this shouldn't have happened. (failed to create boolean var)");
        }
        return null;
    }

    public boolean match(QTIv2variableDeclaration compare) throws InvalidQTIv2Exception
    {
        int n,m,numEquals=0;
        
        if(compare==null)
            throw new InvalidQTIv2Exception("Comparason on null");
        if(type != compare.type)
            return false;
        if(cardinality != compare.cardinality)
            return false;
        if(value.size() != compare.value.size())
        	return false;
        switch(cardinality)
        {
            case CARD_single:
                 return ((valuePart) value.get(0)).compare(((valuePart) compare.value.get(0)));
            case CARD_multiple: //s'ha comparar tenint en compte que [A,B,C] es equivalent a [B,C,A] o sigui no importa l'ordre
                /*HeLeNa*/ 
            	boolean mt = false;
            	for (n = 0; n < value.size(); n++)
                {
                	for( m=0; m < value.size();m++)
                		if(((valuePart)value.get(n)).compare(((valuePart) compare.value.get(m))))
                			numEquals++;
                }
                if(numEquals == value.size())  mt = true;  // all the variables in value are 
                return mt;                

            case CARD_ordered:
                boolean ot = true;
                for (n = 0; n < value.size(); n++)
                {
                    if (!((valuePart) value.get(n)).compare(((valuePart) compare.value.get(n))))
                        ot = false;
                }
                return ot;
        }
        return false;
    }

    public boolean isNull()
    {
        if(value==null)
            return true;
        if(value.size()==0)
            return true;
        return false;
    }

    public int getType()
    {
        return type;
    }

    public int getSize()
    {
        if(value==null)
            return 0;
        else
            return value.size();
    }

    public String getStringValue(int pos) throws InvalidQTIv2Exception
    {
        if(null==value)
            return null;
        if(pos >= value.size())
            return null;
        return ((valuePart) value.get(pos)).getAsString();
    }

    public void deleteParts(QTIv2variableDeclaration del) throws InvalidQTIv2Exception
    {
        if(del==null)
            throw new InvalidQTIv2Exception("QTIv2Variable.deleteParts with null input");
        if(del.cardinality != CARD_single)
            throw new InvalidQTIv2Exception("QTIv2Variable.deleteParts with non single input");
        valuePart d2 = (valuePart)del.value.get(0);
        int n=0;
        while(n < value.size())
        {
            if(d2.compare((valuePart)value.get(n)))
                value.remove(n);
            else
                n++;
        }
    }

    public String getIdentifier()
    {
        return identifier;
    }

	public int getCardinality() {
		return cardinality;
	}

	public void setCardinality(int cardinality) {
		this.cardinality = cardinality;
	}

	public Vector getValue() {
		return value;
	}

	public void setValue(Vector value) {
		this.value = value;
	}

	public void setIdentifier(String identifier) {
		this.identifier = identifier;
	}

	public void setType(int type) {
		this.type = type;
	}
}

abstract class valuePart implements Serializable
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	public static valuePart newValuePart(String val, int type) throws InvalidQTIv2Exception
	{		
		switch (type)
		{
		case QTIv2variableDeclaration.VTYPE_boolean:
			return new booleanPart(val);
		case QTIv2variableDeclaration.VTYPE_directedPair:
			return new directedPairPart(val);
		case QTIv2variableDeclaration.VTYPE_duration:
			return new durationPart(val);
		case QTIv2variableDeclaration.VTYPE_file:
			return new filePart(val);
		case QTIv2variableDeclaration.VTYPE_float:
			return new floatPart(val);
		case QTIv2variableDeclaration.VTYPE_identifier:
			return new identifierPart(val);
		case QTIv2variableDeclaration.VTYPE_integer:
			return new integerPart(val);
		case QTIv2variableDeclaration.VTYPE_pair:
			return new pairPart(val);
		case QTIv2variableDeclaration.VTYPE_point:
			return new pointPart(val);
		case QTIv2variableDeclaration.VTYPE_string:
			return new stringPart(val);
		case QTIv2variableDeclaration.VTYPE_uri:
			return new uriPart(val);
		}
		return null;
	}
	
	//Els he posat tots abstractes pq cada tipus de body part l'implementi
    abstract boolean compare(valuePart comp) throws InvalidQTIv2Exception;
    abstract String getAsString() throws InvalidQTIv2Exception;
    abstract valuePart duplicate() throws Exception;
 }

// Generated by Niall's template filling wizard thingie...' wizard.
// With luck it's at http://www.nbsoftware.com/jtools/templatefill.php
class booleanPart extends valuePart
{
    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	boolean value;

    booleanPart(String val) throws InvalidQTIv2Exception
    {
        val = val.toLowerCase();
        val = val.trim();
        if((val.equals("true"))||(val.equals("yes"))||(val.equals("1")))
            value = true;
        else
            value = false;
    }

    booleanPart(boolean nVal) throws InvalidQTIv2Exception
    {
        value = nVal;
    }

    boolean getValue()
    {
        return value;
    }

    boolean compare(valuePart comp) throws InvalidQTIv2Exception
    {
    	/*HeLeNa*/
        booleanPart comp2 = (booleanPart)comp;
        if(value != comp2.value)
        	return false;
        else
        	return true;
    }

    String getAsString() throws InvalidQTIv2Exception
    {
        return String.valueOf(value);
    }

    valuePart duplicate() throws InvalidQTIv2Exception
    {
    	/*HeLeNa*/
        try
        {
            return new booleanPart(value);
        }
        catch (InvalidQTIv2Exception ex)
        {
            return null;
        }
    }
}

// represents a pair of identifiers corresponding to a directed association between two objects. The two 
// identifiers correspond to the source and destination objects.
 /*HeLeNa*/
class directedPairPart extends valuePart
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	identifierPart valueSource;
	identifierPart valueDest;
	String value;
	
	
    directedPairPart(String val) throws InvalidQTIv2Exception
    {
    	value=val;
    	String[] values = val.split(" "); //public String[] split(String regex,int limit) divideix segons un caracter
    
    	valueSource= new identifierPart(values[0]);
    	valueDest= new identifierPart(values[1]);
  	  }

    boolean compare(valuePart comp) throws InvalidQTIv2Exception
    {
    	directedPairPart comp2 = (directedPairPart)comp;
       
      	if (comp2.valueSource.compare(valueSource))
    	{
    		if(comp2.valueDest.compare(valueDest))
    			return true;
    		return false;
    	}
    	return false; 	
    }

    String getAsString() throws InvalidQTIv2Exception
    {
    	return value;
    }

    valuePart duplicate() throws InvalidQTIv2Exception
    {
    	 try
         {
             return new directedPairPart(value);
         }
         catch (InvalidQTIv2Exception ex)
         {
             return null;
         }
    }
}

//A duration value specifies  a distance(in time)between two points. In other words, a time period as defined by 
//[ISO8601]. Duration are measured in seconds ans may have a fractional part. 
////// de moment ho tractare com un float

class durationPart extends valuePart
{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	float value;

    durationPart(String val) throws InvalidQTIv2Exception
    {
        value = Float.parseFloat(val);
    }

    durationPart(float val) throws InvalidQTIv2Exception
    {
        value = val;
    }

    boolean compare(valuePart comp) throws InvalidQTIv2Exception
    {
        durationPart comp2 = (durationPart) comp;
        if(comp2.getValue()==value)
            return true;
        else
            return false;
        }

    float getValue()
    {
        return value;
    }

    String getAsString() throws InvalidQTIv2Exception
    {
        return String.valueOf(value);
    }

    valuePart duplicate() throws InvalidQTIv2Exception
    {
        try
        {
            return new durationPart(value);
        }
        catch (InvalidQTIv2Exception ex)
        {
            return null;
        } 
    }
}

//a file value is a sequence of octets(bytes) qualified by a content-type and an optional filename given
// to the file. The content-type of the file is one of the MIME types defined by [RFC2045]
class filePart extends valuePart
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	String filename="";
	
    filePart(String val) throws InvalidQTIv2Exception
    {
    	
        throw new InvalidQTIv2Exception("Unimplimented variable type (file)");
    }

    boolean compare(valuePart comp) throws InvalidQTIv2Exception
    {
 //   filePart comp2 = (filePart)comp;
        if(true) throw new InvalidQTIv2Exception("Unimplimented type comparison (file)");
        return false;
    }

    String getAsString() throws InvalidQTIv2Exception
    {
        if(true) throw new InvalidQTIv2Exception("Unimplimented type to string (file)");
        return null;
    }

    valuePart duplicate() throws InvalidQTIv2Exception
    {
        if (true)throw new InvalidQTIv2Exception("Unimplimented type duplicate (file)");
        return null;
    }
}


class floatPart extends valuePart
{
    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	float value;

    floatPart(String val) throws InvalidQTIv2Exception
    {
        value = Float.parseFloat(val);
       
    }

    floatPart(float val) throws InvalidQTIv2Exception
    {
        value = val;
    }

    float getValue()
    {
        return value;
    }

    boolean compare(valuePart comp) throws InvalidQTIv2Exception
    {
        floatPart comp2 = (floatPart) comp;
        if(comp2.getValue()==value)
            return true;
        else
            return false;
    }

    String getAsString() throws InvalidQTIv2Exception
    {
        return String.valueOf(value);
    }

    valuePart duplicate()
    {
        try
        {
            return new floatPart(value);
        }
        catch (InvalidQTIv2Exception ex)
        {
            return null;
        }
    }

}


class identifierPart extends valuePart
{
    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	String value;

    identifierPart(String val) throws InvalidQTIv2Exception
    {
        value = val;
    }

    boolean compare(valuePart comp) throws InvalidQTIv2Exception
    {
        identifierPart comp2 = (identifierPart) comp;
        return getAsString().equals(comp2.getAsString());
    }

    String getAsString() throws InvalidQTIv2Exception
    {
        return value;
    }

    public String toString(){
    	return value;
    }
    
    valuePart duplicate()
    {
        try
        {
            return new identifierPart(value);
        }
        catch (InvalidQTIv2Exception ex)
        {
            return null;
        }
    }
    String getValue()
    {
        return value;
    }
}

class integerPart extends valuePart
{
    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	int value;

    integerPart(String val) throws InvalidQTIv2Exception
    {
        value = Integer.parseInt(val);
    }
    
    integerPart(int nVal) throws InvalidQTIv2Exception
    {
        value = nVal;
    }

    int getValue()
    {
        return value;
    }
   

    boolean compare(valuePart comp) throws InvalidQTIv2Exception
    {
        integerPart comp2 = (integerPart) comp;
        if(comp2.getValue()==value)
            return true;
        else
            return false;
    }

    String getAsString() throws InvalidQTIv2Exception
    {
        return String.valueOf(value);
    }

    valuePart duplicate()
    {
        try
        {
            return new integerPart(value);
        }
        catch (InvalidQTIv2Exception ex)
        {
            return null;
        }
    }
}

/*HeLeNa*/
//A pair value represents a pair of identifiers corresponding to a directed association between
//two objects. The two identifiers correspond to the source destination objects  ex:  <value>118 184</value>
class pairPart extends valuePart
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	identifierPart value1;
	identifierPart value2;
	String value;

	
	
    pairPart(String val) throws InvalidQTIv2Exception
    { 
    	value=val;
    	String[] values = val.split(" "); //public String[] split(String regex,int limit) divideix segons un caracter
    	value1= new identifierPart(values[0]);
    	value2= new identifierPart(values[1]);
    }

    boolean compare(valuePart comp) throws InvalidQTIv2Exception
    {
    	pairPart comp2 = (pairPart)comp;
    	
    	if (comp2.value1.compare(value1))
    	{
    		if(comp2.value2.compare(value2))
    			return true;
    	}else if(comp2.value1.compare(value2))  // it's not important the order of the pair, it's not a directed pair
    	{
    		if(comp2.value2.compare(value1))
    			return true;
    	}
    	return false;
    }

    String getAsString() throws InvalidQTIv2Exception
    {
    	return value;
    }

    valuePart duplicate() throws InvalidQTIv2Exception
    {
    	   try
           {
               return new pairPart(value);
           }
           catch (InvalidQTIv2Exception ex)
           {
               return null;
           }
    }
}


class pointPart extends valuePart
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	int valueX;
	int valueY;
	String value;

	// hem de guardar els valors x i y del punt
	/*HeLeNa*/
	pointPart(String val) throws InvalidQTIv2Exception
    {
		value=val;
		String[] values = val.split(" "); //public String[] split(String regex,int limit) divideix segons un caracter
	       
		valueX = Integer.parseInt(values[0]);
        valueY = Integer.parseInt(values[1]);
    }
	
    boolean compare(valuePart comp) throws InvalidQTIv2Exception
    {

    	pointPart comp2 = (pointPart)comp;
    	if(comp2.valueX == valueX)
    	{
    		if(comp2.valueY == valueY)
    			return true;
    		return false;
    	}
    	return false;
    }

    String getAsString() throws InvalidQTIv2Exception
    {
    	return value;
    }

    String getValue() throws InvalidQTIv2Exception
    {
    	return value;
    }
    
    
    valuePart duplicate() throws InvalidQTIv2Exception
    {
    	  try
          {
              return new pointPart(value);
          }
          catch (InvalidQTIv2Exception ex)
          {
              return null;
          }
    }
}


class stringPart extends valuePart
{
    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	String value;

    stringPart(String val) throws InvalidQTIv2Exception
    {
        value = val;
    }

    boolean compare(valuePart comp) throws InvalidQTIv2Exception
    {
        stringPart comp2 = (stringPart) comp;
        return getAsString().equals(comp2.getAsString());
    }

    String getAsString() throws InvalidQTIv2Exception
    {
        return value;
    }

    valuePart duplicate()
    {
        try
        {
            return new stringPart(value);
        }
        catch (InvalidQTIv2Exception ex)
        {
            return null;
        }
    }
    
    public String toString(){
    	return value;
    }

}

//a URI value is a Uniform Resource Identifier
/*HeLeNa*/
class uriPart extends valuePart
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	String value = "";
	
    uriPart(String val) throws InvalidQTIv2Exception
    {
    	value=val;
    }

    boolean compare(valuePart comp) throws InvalidQTIv2Exception
    {
    	uriPart comp2 = (uriPart)comp;
    	if(comp2.equals(value))
    		return true;
    	else 
    		return false;
    }

    String getAsString() throws InvalidQTIv2Exception
    {
        return value;
    }

    valuePart duplicate() throws InvalidQTIv2Exception
    {
    	  try
          {
              return new uriPart(value);
          }
          catch (InvalidQTIv2Exception ex)
          {
              return null;
          }
    }
}
